#include <stdlib.h>
#include <stdio.h>
#include <fcntl.h>
#include <unistd.h>
#include <string.h>
#include "provision.h"

#define ENC_DATA_SIZE       32

//#define DEBUG 1
/* Because /dev/mtd doesn't have image header, while proc/provision/partition has only the image header, 
   we need two parameters */
int main( int argc, char ** argv )
{
	int image_size;
    int header_at_end;
	char * proc_image_name;
    char * mtd_image_name;
	int mtd_fd, proc_fd;
//	unsigned short words[8192];
	unsigned short words[ENC_DATA_SIZE << 3];
	unsigned short check_sum;
	int read_total;
	int read_now;
	int i;
	ImageHeader hdr;

	if ( argc < 2 )
	{
		return -1;
	}
    
    mtd_image_name = argv[1];
	proc_image_name = argv[2];

#ifdef DEBUG
	printf( "Proc Image: %s\n", proc_image_name );
#endif

	proc_fd = open( proc_image_name, O_RDONLY );
	if ( proc_fd < 0 )
	{
		perror("");
		return -1;
	}
    
    mtd_fd = open( mtd_image_name, O_RDONLY );
	if ( mtd_fd < 0 )
	{
		perror("");
		return -1;
	}

    image_size = lseek( mtd_fd, 0, SEEK_END );

#ifdef DEBUG
    printf("Image Size On Disk: %d\n", image_size);
#endif

 	if ( read( proc_fd, &hdr, sizeof ( hdr ) ) != sizeof ( hdr ) )
	{
		perror("");
		return -1;
	}

#ifdef DEBUG
	printf("Image Size Reported: %d\n", gs_endian_conv32(hdr.Size) );
	printf("Image Size Used Reported: %d\n", gs_endian_conv32(hdr.SizeUsed) );
#endif


    if ( lseek( mtd_fd, 0, SEEK_SET ) < 0 )
    {
         perror("");
         return -1;
    }

	read_total = 0;
	check_sum = 0;
	while ( read_total < gs_endian_conv32(hdr.SizeUsed) )
	{
		read_now = read( mtd_fd, &words, sizeof ( words ) );
		if ( read_now < 0 )
		{
			perror("");
			close( mtd_fd );
			return -1;
		}
		if ( read_now + read_total > gs_endian_conv32(hdr.SizeUsed) )
		{
			read_now = gs_endian_conv32(hdr.SizeUsed) - read_total;
		}
		for ( i = 0; i < ( read_now >> 1 ); i++ )
		{
			check_sum += gs_endian_conv16(words[i]);
		}
		read_total += read_now;
		if ( read_now < sizeof ( words ) )
		{
			break;
		}
	}
	
	if ( read_total != gs_endian_conv32(hdr.SizeUsed) )
	{
#ifdef DEBUG
        printf("read_total is %d\n", read_total);
		printf("File ended early\n");
#endif
		return -1;
	}
	close( mtd_fd );
	close( proc_fd );

#ifdef DEBUG
	printf("Calculated Check Sum: %04x\n", ( 0xFFFF - check_sum ) + 1);
	printf("Reported Check Sum: %04x\n", gs_endian_conv16(hdr.Checksum));
	printf("Sum: %04x\n", (unsigned short)( check_sum + gs_endian_conv16(hdr.Checksum) ) );
#endif

	if ( 0 == (unsigned short)( check_sum + gs_endian_conv16(hdr.Checksum) ) )
	{
		return 0;
	}
	return -1;
}


