/*
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 *   Copyright (C) 2004 btxu Generate from INCA-IP project
 *   Copyright (C) 2005 Jin-Sze.Sow Comments edited
 *   Copyright (C) 2006 Huang Xiaogang Modification & verification on Danube chip
 *   Copyright (C) 2007 John Crispin <blogic@openwrt.org>
 *   Copyright (C) 2010 Jevgenijs.Grigorjevs@lantiq.com adaptation for xT16 project
 */

#include <linux/autoconf.h>

#ifdef CONFIG_GENERIC_GPIO
#include <linux/module.h>
#include <linux/types.h>
#include <linux/errno.h>
#include <linux/proc_fs.h>
#include <linux/init.h>
#include <linux/ioctl.h>
#include <linux/timer.h>
#include <linux/interrupt.h>
#include <linux/kthread.h>
#include <linux/io.h>
#include <linux/kobject.h>
#include <linux/workqueue.h>
#include <linux/skbuff.h>
#include <linux/netlink.h>
#include <linux/platform_device.h>
#include <net/sock.h>
#include <linux/uaccess.h>
#include <linux/version.h>
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 26))
#include <linux/semaphore.h>
#else
#include <asm/semaphore.h>
#endif
#include <asm/ifxmips/ifxmips.h>
#include <linux/reboot.h>
#ifdef CONFIG_SVIP
#include <asm/ifx/mux.h>
#include <asm/ifxmips/ifxmips_irq.h>
#endif /* CONFIG_SVIP */

typedef struct {
   u32 base;
   u32 pins;
} ifxmips_port_base_t;

/* Base addresses for ports */
static const ifxmips_port_base_t ifxmips_port_bases[] = {
#ifdef CONFIG_SVIP
   {IFXMIPS_GPIO_P0_BASE, 20},
   {IFXMIPS_GPIO_P1_BASE, 20},
   {IFXMIPS_GPIO_P2_BASE, 19},
   {IFXMIPS_GPIO_P3_BASE, 20},
   {IFXMIPS_GPIO_P4_BASE, 24}
#else
   {IFXMIPS_GPIO_P0_BASE, 16},
   {IFXMIPS_GPIO_P1_BASE, 16},
#endif /* CONFIG_SVIP */
};

#define MAX_PORTS    (sizeof (ifxmips_port_bases) / sizeof (ifxmips_port_bases[0]))
#define PINS_PER_PORT(port) (ifxmips_port_bases[port].pins)

static struct timer_list wd_timer;
static unsigned int wd_port = 0;
#ifdef CONFIG_GXW42XX_BUILD_48_96
static unsigned int wd_pin = 11;
#else
static unsigned int wd_pin = 5;
#endif
unsigned int wdt_feed = 1;

#define RST_USE_INTRRUPT             1

#define GPIO_CHECK_INTERVAL_MS         100
#define FACTORY_RESET_INTERVAL_S       5 /* 5S */
#define FACTORY_RESET_INTERVAL_COUNT   (FACTORY_RESET_INTERVAL_S * (1000 / GPIO_CHECK_INTERVAL_MS) )
#define MACHINE_RESET_INTERVAL_MS      300
#define MACHINE_RESET_INTERVAL_COUNT   (MACHINE_RESET_INTERVAL_MS / GPIO_CHECK_INTERVAL_MS)

#ifdef CONFIG_IFXMIPS_GPIO_RST_BTN

#ifdef CONFIG_IFX_BOARD_GS_GXW
#if RST_USE_INTRRUPT
#define RST_EXINT_MODE  2 /* Interrupt on falling edge */
#define RST_EXINT3      3 /* EXINT3 for reset button*/
#define IFXMIPS_RST_IRQ (160+8+15)
#endif
unsigned int rst_port = 0;
unsigned int rst_pin = 10;

extern int nvram_eraseall(void);
extern int nvram_commit(void);
extern char *nvram_get(const char *key);
extern void start_half_led_blinking(void);
extern void stop_half_led_blinking(void);
extern int gxw42xx_kp_key_generate_event (unsigned long data); 
#define EVENT_NOTIFY                    0x2000
#define EVENT_NOTIFY_RESET1				(EVENT_NOTIFY   | 0x01)

static struct semaphore* sem_reset_pressed;
#define     KTHREAD_NAME_LEN    16
/* a structure to store all information we need
   for our thread */
typedef struct kthread_struct
{
   /* Completion struct for thread completion detection */
   struct completion    thrCompletion;
   /* A name of the kernel thread */
   char                 name[KTHREAD_NAME_LEN];
   /* "Thread is running" flag */
   int                  running;
   /* "Terminate thread" signalling flag */
   int                  terminate;
   /* Kernel thread process ID */
   int                  pid;
   /* Function activated on thread creation */
   void                 (*function)(struct kthread_struct *kthread);
   /* Argument for the "function" */
   void                 *arg;
} kthread_t;

static kthread_t         s_reset_handle_thread;

#else
unsigned int rst_port = 1;
unsigned int rst_pin = 15;
#endif

static struct timer_list rst_button_timer;

//extern struct sock *uevent_sock;
//extern u64 uevent_next_seqnum(void);
//static unsigned long seen;
static int pressed;
static int reset_val = 0;
static int fact_reset_count = 0;
static int reset_button_pressed = 0;

struct event_t {
	struct work_struct wq;
	int set;
	unsigned long jiffies;
};
#endif

#ifdef CONFIG_IFXMIPS_GPIO_RST_BTN
/* initialize new created thread. Called by the new thread. */
static void init_kthread(kthread_t *kthread)
{
    /* initialise termination flag */
    kthread->terminate = 0;

    /* Enable signals in Kernel >= 2.6 */
    allow_signal(SIGKILL);
    allow_signal(SIGINT);
    allow_signal(SIGTERM);
}

/* private functions */
static void kthread_launcher(void *data)
{
   kthread_t *kthread               = data;
   int retVal                       = -1;

   if(kthread == NULL)
   {
      return;
   }

   /* do LINUX specific setup */
   daemonize(kthread->name);

   kthread->running = 1;
   kthread->function((void*)kthread);
   kthread->running = 0;

   /* When kthread->function exits (upon a check
    * kthread->terminate == 1 is TRUE),
    * thread gets completed.
    */
   complete_and_exit(&kthread->thrCompletion, (long)retVal);
}

/* create a new kernel thread. Called by the creator. */
static void start_kthread(void (*func)(kthread_t *), kthread_t *kthread,  char *name)
{
   if (kthread == NULL)
   {
      return;
   }

   /* Set the name of the future thread */
   strncpy(kthread->name, name, KTHREAD_NAME_LEN - 1);
   kthread->name[KTHREAD_NAME_LEN - 1] = '\0';

   /* Initialise termination flag */
   kthread->terminate = 0;

   /* Initialise thread function pointer */
   kthread->function = func;

   /* Initialise the completion structure*/
   init_completion(&kthread->thrCompletion);

   /* start kernel thread via the wrapper function */
   kthread->pid = kernel_thread( (int (*)(void *))kthread_launcher,
                  (void *)kthread,
                  (CLONE_FS | CLONE_FILES));

}
#endif

#define IFXMIPS_GPIO_SANITY \
   {if (port < 0 || port > MAX_PORTS || pin < 0 || pin > PINS_PER_PORT(port)) return -EINVAL; }

#define IFXMIPS_GPIO_PIN_SET(offset) { \
      volatile u32 *addr = (volatile u32*)(ifxmips_port_bases[port].base + (offset)); \
	   ifxmips_w32 (ifxmips_r32 (addr) | (1 << pin), addr); \
   }

#define IFXMIPS_GPIO_PIN_CLEAR(offset) { \
      volatile u32 *addr = (volatile u32*)(ifxmips_port_bases[port].base + (offset)); \
	   ifxmips_w32 (ifxmips_r32 (addr) & ~(1 << pin), addr); \
   }

#define IFXMIPS_GPIO_PIN_GET(offset) \
      (ifxmips_r32((volatile u32*)(ifxmips_port_bases[port].base + (offset))) & (1 << pin))

/***************************************************************************/
/**                                                                       **/
/** External interrupts                                                   **/
/**                                                                       **/
/***************************************************************************/

int ifxmips_port_reserve_pin(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
	printk(KERN_INFO "%s : call to obseleted function\n", __func__);
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_reserve_pin);

int ifxmips_port_free_pin(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
	printk(KERN_INFO "%s : call to obseleted function\n", __func__);
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_free_pin);

int ifxmips_port_set_exintcr0(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
#ifdef IFXMIPS_GPIO_EXINTCR0_REG
   IFXMIPS_GPIO_PIN_SET (IFXMIPS_GPIO_EXINTCR0_REG);
#else
	printk(KERN_INFO "%s : call to unsupported function\n", __func__);
#endif /* IFXMIPS_GPIO_EXINTCR0_REG */
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_set_exintcr0);

int ifxmips_port_clear_exintcr0(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
#ifdef IFXMIPS_GPIO_EXINTCR0_REG
   IFXMIPS_GPIO_PIN_CLEAR (IFXMIPS_GPIO_EXINTCR0_REG);
#else
	printk(KERN_INFO "%s : call to unsupported function\n", __func__);
#endif /* IFXMIPS_GPIO_EXINTCR0_REG */
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_clear_exintcr0);

int ifxmips_port_set_exintcr1(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
#ifdef IFXMIPS_GPIO_EXINTCR1_REG
   IFXMIPS_GPIO_PIN_SET (IFXMIPS_GPIO_EXINTCR1_REG);
#else
	printk(KERN_INFO "%s : call to unsupported function\n", __func__);
#endif /* IFXMIPS_GPIO_EXINTCR1_REG */
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_set_exintcr1);

int ifxmips_port_clear_exintcr1(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
#ifdef IFXMIPS_GPIO_EXINTCR1_REG
   IFXMIPS_GPIO_PIN_CLEAR (IFXMIPS_GPIO_EXINTCR1_REG);
#else
	printk(KERN_INFO "%s : call to unsupported function\n", __func__);
#endif /* IFXMIPS_GPIO_EXINTCR1_REG */
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_clear_exintcr1);

int ifxmips_port_set_irncfg(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
#ifdef IFXMIPS_GPIO_IRNCFG_REG
   IFXMIPS_GPIO_PIN_SET (IFXMIPS_GPIO_IRNCFG_REG);
#else
	printk(KERN_INFO "%s : call to unsupported function\n", __func__);
#endif /* IFXMIPS_GPIO_IRNCFG_REG */
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_set_irncfg);

int ifxmips_port_clear_irncfg(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
#ifdef IFXMIPS_GPIO_IRNCFG_REG
   IFXMIPS_GPIO_PIN_CLEAR (IFXMIPS_GPIO_IRNCFG_REG);
#else
	printk(KERN_INFO "%s : call to unsupported function\n", __func__);
#endif /* IFXMIPS_GPIO_IRNCFG_REG */
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_clear_irncfg);

int ifxmips_port_set_irnen(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
#ifdef IFXMIPS_GPIO_IRNENSET_REG
   IFXMIPS_GPIO_PIN_SET (IFXMIPS_GPIO_IRNENSET_REG);
#else
	printk(KERN_INFO "%s : call to unsupported function\n", __func__);
#endif /* IFXMIPS_GPIO_IRNENSET_REG */
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_set_irnen);

int ifxmips_port_clear_irnen(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
#ifdef IFXMIPS_GPIO_IRNENCLR_REG
   IFXMIPS_GPIO_PIN_SET (IFXMIPS_GPIO_IRNENCLR_REG);
#else
	printk(KERN_INFO "%s : call to unsupported function\n", __func__);
#endif /* IFXMIPS_GPIO_IRNENCLR_REG */
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_clear_irnen);

int ifxmips_port_set_open_drain(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
#ifdef IFXMIPS_GPIO_OD_REG
   IFXMIPS_GPIO_PIN_SET (IFXMIPS_GPIO_OD_REG);
#else
	printk(KERN_INFO "%s : call to unsupported function\n", __func__);
#endif /* IFXMIPS_GPIO_OD_REG */
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_set_open_drain);

int ifxmips_port_clear_open_drain(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
#ifdef IFXMIPS_GPIO_OD_REG
   IFXMIPS_GPIO_PIN_CLEAR (IFXMIPS_GPIO_OD_REG);
#else
	printk(KERN_INFO "%s : call to unsupported function\n", __func__);
#endif /* IFXMIPS_GPIO_OD_REG */
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_clear_open_drain);

int ifxmips_port_set_pudsel(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
#ifdef IFXMIPS_GPIO_PUDSEL_REG
   IFXMIPS_GPIO_PIN_SET (IFXMIPS_GPIO_PUDSEL_REG);
#else
	printk(KERN_INFO "%s : call to unsupported function\n", __func__);
#endif /* IFXMIPS_GPIO_PUDSEL_REG */
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_set_pudsel);

int ifxmips_port_clear_pudsel(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
#ifdef IFXMIPS_GPIO_PUDSEL_REG
   IFXMIPS_GPIO_PIN_CLEAR (IFXMIPS_GPIO_PUDSEL_REG);
#else
	printk(KERN_INFO "%s : call to unsupported function\n", __func__);
#endif /* IFXMIPS_GPIO_PUDSEL_REG */
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_clear_pudsel);

int ifxmips_port_set_puden(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
#ifdef IFXMIPS_GPIO_PUDEN_REG
   IFXMIPS_GPIO_PIN_SET (IFXMIPS_GPIO_PUDEN_REG);
#else
	printk(KERN_INFO "%s : call to unsupported function\n", __func__);
#endif /* IFXMIPS_GPIO_PUDSEL_REG */
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_set_puden);

int ifxmips_port_clear_puden(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
#ifdef IFXMIPS_GPIO_PUDEN_REG
   IFXMIPS_GPIO_PIN_CLEAR (IFXMIPS_GPIO_PUDEN_REG);
#else
	printk(KERN_INFO "%s : call to unsupported function\n", __func__);
#endif /* IFXMIPS_GPIO_PUDEN_REG */
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_clear_puden);

int ifxmips_port_set_stoff(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
#ifdef IFXMIPS_GPIO_STOFF_REG
   IFXMIPS_GPIO_PIN_SET (IFXMIPS_GPIO_STOFF_REG);
#else
	printk(KERN_INFO "%s : call to unsupported function\n", __func__);
#endif /* IFXMIPS_GPIO_STOFF_REG */
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_set_stoff);

int ifxmips_port_clear_stoff(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
#ifdef IFXMIPS_GPIO_STOFF_REG
   IFXMIPS_GPIO_PIN_CLEAR (IFXMIPS_GPIO_STOFF_REG);
#else
	printk(KERN_INFO "%s : call to unsupported function\n", __func__);
#endif /* IFXMIPS_GPIO_STOFF_REG */
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_clear_stoff);

int ifxmips_port_set_dir_out(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
#ifdef IFXMIPS_GPIO_DIR_REG
   IFXMIPS_GPIO_PIN_SET (IFXMIPS_GPIO_DIR_REG);
#else
	printk(KERN_INFO "%s : call to unsupported function\n", __func__);
#endif /* IFXMIPS_GPIO_DIR_REG */
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_set_dir_out);

int ifxmips_port_set_dir_in(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
#ifdef IFXMIPS_GPIO_DIR_REG
   IFXMIPS_GPIO_PIN_CLEAR (IFXMIPS_GPIO_DIR_REG);
#else
	printk(KERN_INFO "%s : call to unsupported function\n", __func__);
#endif /* IFXMIPS_GPIO_DIR_REG */
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_set_dir_in);

int ifxmips_port_set_output(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
#ifdef IFXMIPS_GPIO_OUT_REG
   IFXMIPS_GPIO_PIN_SET (IFXMIPS_GPIO_OUT_REG);
#else
	printk(KERN_INFO "%s : call to unsupported function\n", __func__);
#endif /* IFXMIPS_GPIO_OUT_REG */
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_set_output);

int ifxmips_port_clear_output(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
#ifdef IFXMIPS_GPIO_OUT_REG
   IFXMIPS_GPIO_PIN_CLEAR (IFXMIPS_GPIO_OUT_REG);
#else
	printk(KERN_INFO "%s : call to unsupported function\n", __func__);
#endif /* IFXMIPS_GPIO_OUT_REG */
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_clear_output);

int ifxmips_port_get_input(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
#ifdef IFXMIPS_GPIO_IN_REG
   if (IFXMIPS_GPIO_PIN_GET (IFXMIPS_GPIO_IN_REG))
		return 1;   //return 0; SXIE: don't know why invert the value here. This is causing gpio read return reverted value
	else
		return 0;   //return 1;
#else
	printk(KERN_INFO "%s : call to unsupported function\n", __func__);
   return 0;
#endif /* IFXMIPS_GPIO_IN_REG */
}
EXPORT_SYMBOL(ifxmips_port_get_input);

int ifxmips_port_set_altsel0(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
#ifdef IFXMIPS_GPIO_ALTSEL0_REG
   IFXMIPS_GPIO_PIN_SET (IFXMIPS_GPIO_ALTSEL0_REG);
#else
	printk(KERN_INFO "%s : call to unsupported function\n", __func__);
#endif /* IFXMIPS_GPIO_ALTSEL0_REG */
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_set_altsel0);

int ifxmips_port_clear_altsel0(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
#ifdef IFXMIPS_GPIO_ALTSEL0_REG
   IFXMIPS_GPIO_PIN_CLEAR (IFXMIPS_GPIO_ALTSEL0_REG);
#else
	printk(KERN_INFO "%s : call to unsupported function\n", __func__);
#endif /* IFXMIPS_GPIO_ALTSEL0_REG */
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_clear_altsel0);

int ifxmips_port_set_altsel1(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
#ifdef IFXMIPS_GPIO_ALTSEL1_REG
   IFXMIPS_GPIO_PIN_SET (IFXMIPS_GPIO_ALTSEL1_REG);
#else
	printk(KERN_INFO "%s : call to unsupported function\n", __func__);
#endif /* IFXMIPS_GPIO_ALTSEL1_REG */
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_set_altsel1);

int ifxmips_port_clear_altsel1(unsigned int port, unsigned int pin)
{
	IFXMIPS_GPIO_SANITY;
#ifdef IFXMIPS_GPIO_ALTSEL1_REG
   IFXMIPS_GPIO_PIN_CLEAR (IFXMIPS_GPIO_ALTSEL1_REG);
#else
	printk(KERN_INFO "%s : call to unsupported function\n", __func__);
#endif /* IFXMIPS_GPIO_ALTSEL1_REG */
	return 0;
}
EXPORT_SYMBOL(ifxmips_port_clear_altsel1);

#ifdef CONFIG_IFXMIPS_GPIO_RST_BTN
static void reset_button_timer_func(unsigned long unused)
{
    up(sem_reset_pressed);
}

extern int gs_erase_partition(char * name);
static void reset_button_handle(unsigned long unused)
{
    int timer_started = 0;
    static int led_pattern_started = 0;

    reset_val = !ifxmips_port_get_input(rst_port, rst_pin);

//    printk("reset_val = %d, fact_reset_count = %d\n", reset_val, fact_reset_count);

    /* reset button pressed */
    if (reset_val == 1)
    {
        /* first transition to pressed */
        if ( reset_button_pressed == 0 )
        {
            char reset_lock = 0;
            char *ptr = nvram_get(":reset_lock");
//FIXME: Add nvram_get_safe implementation
//            char reset_lock[16];
//            nvram_get_safe(":reset_lock", reset_lock, sizeof ( reset_lock ) );
//            if ( reset_lock[0] == '1' )
            if ( ptr )
                reset_lock = ptr[0];

            if ( reset_lock == '1' )
            {
                printk("Skipping reset while provisioning\n");
                goto done;
            }           
            else        
            {           
                printk("Reset button pressed\n");
                reset_button_pressed = 1;
                /* restart timer*/
                timer_started = 1;
            }           

            fact_reset_count = 0;
            goto done;
        }       
        else    
        {       
            fact_reset_count ++;
            /* restart timer*/
            timer_started = 1;
        }
    }

    /* Transition for releasing the reset button */
    if ( 1 == reset_button_pressed && (reset_val == 0 || fact_reset_count > FACTORY_RESET_INTERVAL_COUNT) )
    {
        /* After the reset button has been pressed for long enaough time to trigger the factory reset */
        if ( reset_val == 0 && fact_reset_count > FACTORY_RESET_INTERVAL_COUNT)
        {
            char provider_lock = 0;
            char *ptr = nvram_get("88");
//FIXME: Add nvram_get_safe implementation
            if ( ptr )
                provider_lock = ptr[0];

            if ( provider_lock == '1' )
            {
                printk("Keypad locked, factory reset disabled\n");
                /* make shure we only set led once */
                if ( 1 == led_pattern_started )
                {
                    /* half LED bliking, different from provisioning */
                    stop_half_led_blinking() ;
                    led_pattern_started = 0 ;
                }
            }
            else if ( provider_lock == '2' )
            {
                printk("Basic reset triggered.\n");

                printk("FIXME: Add LED factory reset pattern for basic pvalues reset\n");
                printk("FIXME: Add support for basic value reset only.\n");
                gxw42xx_kp_key_generate_event ( EVENT_NOTIFY_RESET1 ) ;

                /* Give some time for the LED pattern to cycle through */
//                msleep( 5 * 1000 );

                //machine_restart(NULL);
            }
            else
            {
                /* factory reset */
                printk("Factory reset triggered.\n");

                gs_erase_partition("data");
                nvram_eraseall();
                nvram_commit();

                /* Give some time for the LED pattern to cycle through */
//                msleep( 5 * 1000 );

                /* factory reset and reboot */
                printk("Rebooting...\n");
                machine_restart(NULL);
            }

            fact_reset_count = 0;
            led_pattern_started = 0;

        }
        /* press reset button long enough and release it to trigger LED pattern */
        else if( reset_val == 1 && fact_reset_count > FACTORY_RESET_INTERVAL_COUNT)
        {
        	/* make shure we only set led once */
            if ( 0 == led_pattern_started )
            {
                /* half LED bliking, different from provisioning */
                start_half_led_blinking();
                led_pattern_started = 1;
            }
        }
        else if( reset_val == 0 && fact_reset_count > MACHINE_RESET_INTERVAL_COUNT )
        {
            /* normal reset */
            printk("Normal reset\n");
            printk("Rebooting...\n");
            machine_restart(NULL);
        }
    }

    if (reset_val == 0 && reset_button_pressed)
    {
        printk("Reset button released\n");
        reset_button_pressed = 0;
        fact_reset_count = 0;
    }

done: 
#if RST_USE_INTRRUPT
    if ( timer_started == 0 )
        /* reenable irq */
        enable_irq(IFXMIPS_RST_IRQ);
   else
#endif
    {
        /* restart timer*/
        rst_button_timer.expires = jiffies + msecs_to_jiffies(GPIO_CHECK_INTERVAL_MS);
        add_timer(&rst_button_timer);
    }
}


static void reset_button_thread_func(kthread_t *pThread)
{
    init_kthread(pThread);

    while ((pThread->terminate != 1) &&
        (down_interruptible(sem_reset_pressed) == 0))
    {
        reset_button_handle(0);
    }
    /* infinite looping, no thread exit */
}

#if RST_USE_INTRRUPT     
static irqreturn_t rst_interrupt(int irq, void *dev_id)
{
    disable_irq_nosync(irq);

    reset_val = 1;

    up(sem_reset_pressed);

    return IRQ_HANDLED;
}
#endif

#if 0
static inline void add_msg(struct sk_buff *skb, char *msg)
{
	char *scratch;
	scratch = skb_put(skb, strlen(msg) + 1);
	sprintf(scratch, msg);
}

static void hotplug_button(struct work_struct *wq)
{
	struct sk_buff *skb;
	struct event_t *event;
	size_t len;
	char *scratch, *s;
	char buf[128];

	event = container_of(wq, struct event_t, wq);
	if (!uevent_sock)
		goto done;

	s = event->set ? "pressed" : "released";
	len = strlen(s) + 2;
	skb = alloc_skb(len + 2048, GFP_KERNEL);
	if (!skb)
		goto done;

	scratch = skb_put(skb, len);
	sprintf(scratch, "%s@", s);
	add_msg(skb, "HOME=/");
	add_msg(skb, "PATH=/sbin:/bin:/usr/sbin:/usr/bin");
	add_msg(skb, "SUBSYSTEM=button");
	add_msg(skb, "BUTTON=reset");
	add_msg(skb, (event->set ? "ACTION=pressed" : "ACTION=released"));
	sprintf(buf, "SEEN=%ld", (event->jiffies - seen)/HZ);
	add_msg(skb, buf);
	snprintf(buf, 128, "SEQNUM=%llu", uevent_next_seqnum());
	add_msg(skb, buf);

	NETLINK_CB(skb).dst_group = 1;
	netlink_broadcast(uevent_sock, skb, 0, 1, GFP_KERNEL);
done:
	kfree(event);
}

static void reset_button_poll(unsigned long unused)
{
	struct event_t *event;

	rst_button_timer.expires = jiffies + (HZ / 4);
	add_timer(&rst_button_timer);

	if (pressed != ifxmips_port_get_input(rst_port, rst_pin)) {
		if (pressed)
			pressed = 0;
		else
			pressed = 1;
		event = kzalloc(sizeof(struct event_t), GFP_ATOMIC);
		if (!event) {
			printk(KERN_INFO "Could not alloc hotplug event\n");
			return;
		}
		event->set = pressed;
		event->jiffies = jiffies;
		INIT_WORK(&event->wq, (void *)(void *)hotplug_button);
		schedule_work(&event->wq);
		seen = jiffies;
	}
}
#endif
#endif

void feed_wdt( void )
{
	static int h_or_l = 1;

    if(h_or_l)
    {
        h_or_l = 0;
        ifxmips_port_clear_output(wd_port, wd_pin);
    }
    else
    {
        h_or_l = 1;
        ifxmips_port_set_output(wd_port, wd_pin);
    }
}

static void kick_watchdog(unsigned long unused)
{
    wd_timer.expires = jiffies + HZ;
    add_timer(&wd_timer);
    if ( wdt_feed )
    {
        feed_wdt();
    }
    else
        printk("WDT auto feed disabled\n");
}

static int ifxmips_gpio_probe(struct platform_device *dev)
{
	int retval = 0;

#ifdef CONFIG_IFXMIPS_GPIO_RST_BTN
#if 0
	rst_port = dev->resource[0].start;
	rst_pin = dev->resource[0].end;
	ifxmips_port_set_open_drain(rst_port, rst_pin);
	ifxmips_port_clear_altsel0(rst_port, rst_pin);
	ifxmips_port_clear_altsel1(rst_port, rst_pin);
	ifxmips_port_set_dir_in(rst_port, rst_pin);
	seen = jiffies;
	init_timer(&rst_button_timer);
	rst_button_timer.function = reset_button_poll;
	rst_button_timer.expires = jiffies + HZ;
	add_timer(&rst_button_timer);
#endif
#endif
    printk("ifx gpio probe\n");
    ifxmips_port_set_dir_out(wd_port, wd_pin);
    ifxmips_port_set_output(wd_port, wd_pin);

    init_timer(&wd_timer);
    wd_timer.function = kick_watchdog;
    wd_timer.expires = jiffies + HZ;
    add_timer(&wd_timer);

/* PR:
   Moved all GPIO initialization to gpio proc fs init inside drivers/mtd/nand/nand_gs.c
*/

#if 0
    /* slave 1 svip reset */
    ifxmips_port_set_dir_out(0, 12);
    ifxmips_port_clear_output(0, 12);
    
#ifdef CONFIG_GXW42XX_BUILD_48_96
	/* slave 2 */
    ifxmips_port_set_dir_out(0, 13);
    ifxmips_port_clear_output(0, 13);

    /* slave 3 */
    ifxmips_port_set_dir_out(0, 3);
    ifxmips_port_clear_output(0, 3);

    /* slave 4 */
    ifxmips_port_set_dir_out(0, 4);
    ifxmips_port_clear_output(0, 4);

    /* slave 5 */
    ifxmips_port_set_dir_out(0, 5);
    ifxmips_port_clear_output(0, 5);

    /* reset i2c_gpio chips */
    ifxmips_port_set_dir_out(1, 14);
    ifxmips_port_clear_output(1, 14);
#endif

    msleep(200);   

#ifdef CONFIG_GXW42XX_BUILD_48_96
    ifxmips_port_set_output(1, 14);
#endif
#endif

	return retval;
}

int gs_boardreset_init(void)
{
#ifdef CONFIG_IFXMIPS_GPIO_RST_BTN
    ifxmips_port_set_dir_in(rst_port, rst_pin);

    init_timer(&rst_button_timer);
    rst_button_timer.function = reset_button_timer_func;
    rst_button_timer.expires = jiffies + msecs_to_jiffies(GPIO_CHECK_INTERVAL_MS);

    /* initialize mutex (locked) */
    if ( ((sem_reset_pressed)=kmalloc(sizeof(struct semaphore), GFP_KERNEL)) != NULL )\
    {
        sema_init((sem_reset_pressed), 1);
    }
    down(sem_reset_pressed);

    /* start the thread calling the interrupt handler  */
    start_kthread(reset_button_thread_func, &s_reset_handle_thread, "reset_button service thread");

#if RST_USE_INTRRUPT     
    if (request_irq(IFXMIPS_RST_IRQ,rst_interrupt,0 /*IRQF_DISABLED*/, "rst_button",(void*)0) != 0)
    {
        printk("error, cannot get rst_irq!\n");
        return -EFAULT;
    }
    if (ifx_enable_external_int(RST_EXINT3, RST_EXINT_MODE) != 0)
        printk("error, enable EXINT failed\n");
#else
    add_timer(&rst_button_timer);
#endif
#endif
    return 0;
}

static int ifxmips_gpio_remove(struct platform_device *pdev)
{
#ifdef CONFIG_IFXMIPS_GPIO_RST_BTN
	del_timer_sync(&rst_button_timer);
#endif
	return 0;
}

static struct platform_driver ifxmips_gpio_driver = {
	.probe = ifxmips_gpio_probe,
	.remove = ifxmips_gpio_remove,
	.driver = {
		.name = "ifxmips_gpio",
		.owner = THIS_MODULE,
	},
};

int __init ifxmips_gpio_init(void)
{
	int ret = platform_driver_register(&ifxmips_gpio_driver);
	if (ret)
		printk(KERN_INFO "ifxmips_gpio : Error registering platfom driver!");
	return ret;
}

void __exit ifxmips_gpio_exit(void)
{
	platform_driver_unregister(&ifxmips_gpio_driver);
}

module_init(ifxmips_gpio_init);
module_exit(ifxmips_gpio_exit);

#endif /* GENERIC_GPIO */
