/*
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 *   Copyright (C) 2005 Wu Qi Ming infineon
 *   Copyright (C) 2007 John Crispin <blogic@openwrt.org>
 *   Copyright (C) 2010 Jevgenijs.Grigorjevs@lantiq.com adaptation for xT16 project
 */

#ifdef CONFIG_CPU_MIPSR2_IRQ_VI /* debug RMi */
#error CONFIG_CPU_MIPSR2_IRQ_VI /* debug RMi */
#endif


#include <linux/init.h>
#include <linux/sched.h>
#include <linux/slab.h>
#include <linux/interrupt.h>
#include <linux/kernel_stat.h>
#include <linux/module.h>
#include <linux/version.h>
#include <asm/bootinfo.h>
#include <asm/irq.h>
#include <asm/ifxmips/ifxmips.h>
#include <asm/ifxmips/ifxmips_irq.h>
#include <asm/irq_cpu.h>
#ifdef CONFIG_SVIP
#include <asm/ifxmips/ifxmips_gpio.h>
#include <asm/ifx/svip_mux.h>
#include <asm/ifx/ifx_reg_base.h>
#define IM_NUM 6
#else
#define IM_NUM 5
#endif

void ifxmips_disable_irq(unsigned int irq_nr)
{
#ifndef CONFIG_SVIP
	int i;
	u32 *ifxmips_ier = IFXMIPS_ICU_IM0_IER;

	irq_nr -= INT_NUM_IRQ0;
	for (i = 0; i <= 4; i++) {
		if (irq_nr < INT_NUM_IM_OFFSET) {
			ifxmips_w32(ifxmips_r32(ifxmips_ier) & ~(1 << irq_nr),
				ifxmips_ier);
			return;
		}
		ifxmips_ier += IFXMIPS_ICU_OFFSET;
		irq_nr -= INT_NUM_IM_OFFSET;
	}
#else /* CONFIG_SVIP */
	/* have to access the correct register here */
	if (irq_nr <= (INT_NUM_IM0_IRL0 + 31) && irq_nr >= INT_NUM_IM0_IRL0)
	{
		/* access IM0 */
      *IFX_ICU_IM0_IER &= ~IFX_ICU_IM0_IR(irq_nr - INT_NUM_IRQ0);
	}
	if (irq_nr <= (INT_NUM_IM1_IRL0 + 31) && irq_nr >= INT_NUM_IM1_IRL0)
	{
		/* access IM1 */
      *IFX_ICU_IM1_IER &= ~IFX_ICU_IM1_IR(irq_nr - INT_NUM_IRQ0 - 32);
	}
	if (irq_nr <= (INT_NUM_IM2_IRL0 + 31) && irq_nr >= INT_NUM_IM2_IRL0)
	{
		/* access IM2 */
      *IFX_ICU_IM2_IER &= ~IFX_ICU_IM2_IR(irq_nr - INT_NUM_IRQ0 - 64);
	}
	if (irq_nr <= (INT_NUM_IM3_IRL0 + 31) && irq_nr >= INT_NUM_IM3_IRL0)
	{
		/* access IM3 */
      *IFX_ICU_IM3_IER &= ~IFX_ICU_IM3_IR(irq_nr - INT_NUM_IRQ0 - 96);
	}
	if (irq_nr <= (INT_NUM_IM4_IRL0 + 31) && irq_nr >= INT_NUM_IM4_IRL0)
	{
		/* access IM4 */
      *IFX_ICU_IM4_IER &= ~IFX_ICU_IM4_IR(irq_nr - INT_NUM_IRQ0 - 128);
	}
	if (irq_nr <= (INT_NUM_IM5_IRL0 + 31) && irq_nr >= INT_NUM_IM5_IRL0)
	{
		/* access IM5 */
      *IFX_ICU_IM5_IER &= ~IFX_ICU_IM5_IR(irq_nr - INT_NUM_IRQ0 - 160);
	}
#endif
}
EXPORT_SYMBOL(ifxmips_disable_irq);

void ifxmips_mask_and_ack_irq(unsigned int irq_nr)
{
#ifndef CONFIG_SVIP
	int i;
	u32 *ifxmips_ier = IFXMIPS_ICU_IM0_IER;
	u32 *ifxmips_isr = IFXMIPS_ICU_IM0_ISR;

	irq_nr -= INT_NUM_IRQ0;
	for (i = 0; i <= 4; i++) {
		if (irq_nr < INT_NUM_IM_OFFSET) {
			ifxmips_w32(ifxmips_r32(ifxmips_ier) & ~(1 << irq_nr),
				ifxmips_ier);
			ifxmips_w32((1 << irq_nr), ifxmips_isr);
			return;
		}
		ifxmips_ier += IFXMIPS_ICU_OFFSET;
		ifxmips_isr += IFXMIPS_ICU_OFFSET;
		irq_nr -= INT_NUM_IM_OFFSET;
	}
#else /* CONFIG_SVIP */
   unsigned int irq_bit;
	/* have to access the correct register here */
	if (irq_nr <= (INT_NUM_IM0_IRL0 + 31) && irq_nr >= INT_NUM_IM0_IRL0)
	{
      irq_bit = irq_nr - INT_NUM_IRQ0;
		/* access IM0 */
		/* mask */
      *IFX_ICU_IM0_IER &= ~IFX_ICU_IM0_IR(irq_bit);
		/* ack */
      *IFX_ICU_IM0_ISR = IFX_ICU_IM0_IR(irq_bit);
      return;
	}
	if (irq_nr <= (INT_NUM_IM1_IRL0 + 31) && irq_nr >= INT_NUM_IM1_IRL0)
	{
      irq_bit = irq_nr - INT_NUM_IRQ0 - 32;
		/* access IM1 */
		/* mask */
      *IFX_ICU_IM1_IER &= ~IFX_ICU_IM1_IR(irq_bit);
		/* ack */
      *IFX_ICU_IM1_ISR = IFX_ICU_IM1_IR(irq_bit);
      return;
	}
	if (irq_nr <= (INT_NUM_IM2_IRL0 + 31) && irq_nr >= INT_NUM_IM2_IRL0)
	{
      irq_bit = irq_nr - INT_NUM_IRQ0 - 64;
		/* access IM2 */
		/* mask */
      *IFX_ICU_IM2_IER &= ~IFX_ICU_IM2_IR(irq_bit);
	   /* ack */
      *IFX_ICU_IM2_ISR = IFX_ICU_IM2_IR(irq_bit);
      return;
	}
	if (irq_nr <= (INT_NUM_IM3_IRL0 + 31) && irq_nr >= INT_NUM_IM3_IRL0)
	{
      irq_bit = irq_nr - INT_NUM_IRQ0 - 96;
		/* access IM3 */
		/* mask */
      *IFX_ICU_IM3_IER &= ~IFX_ICU_IM3_IR(irq_bit);
		/* ack */
      *IFX_ICU_IM3_ISR = IFX_ICU_IM3_IR(irq_bit);
      return;
	}
	if (irq_nr <= (INT_NUM_IM4_IRL0 + 31) && irq_nr >= INT_NUM_IM4_IRL0)
	{
      irq_bit = irq_nr - INT_NUM_IRQ0 - 128;
		/* access IM4 */
		/* mask */
      *IFX_ICU_IM4_IER &= ~IFX_ICU_IM4_IR(irq_bit);
		/* ack */
      *IFX_ICU_IM4_ISR = IFX_ICU_IM4_IR(irq_bit);
      return;
	}
	if (irq_nr <= (INT_NUM_IM5_IRL0 + 31) && irq_nr >= INT_NUM_IM5_IRL0)
	{
      irq_bit = irq_nr - INT_NUM_IRQ0 - 160;
		/* access IM5 */
		/* mask */
      *IFX_ICU_IM5_IER &= ~IFX_ICU_IM5_IR(irq_bit);
		/* ack */
      *IFX_ICU_IM5_ISR = IFX_ICU_IM5_IR(irq_bit);
	}
#endif
}
EXPORT_SYMBOL(ifxmips_mask_and_ack_irq);

void ifxmips_enable_irq(unsigned int irq_nr)
{
#ifndef CONFIG_SVIP
	int i;
	u32 *ifxmips_ier = IFXMIPS_ICU_IM0_IER;

	irq_nr -= INT_NUM_IRQ0;
	for (i = 0; i <= 4; i++) {
		if (irq_nr < INT_NUM_IM_OFFSET) {
			ifxmips_w32(ifxmips_r32(ifxmips_ier) | (1 << irq_nr),
				ifxmips_ier);
			return;
		}
		ifxmips_ier += IFXMIPS_ICU_OFFSET;
		irq_nr -= INT_NUM_IM_OFFSET;
	}
#else /* CONFIG_SVIP */
	/* have to access the correct register here */
	if (irq_nr <= (INT_NUM_IM0_IRL0 + 31) && irq_nr >= INT_NUM_IM0_IRL0)
	{
		/* access IM0 */
      *IFX_ICU_IM0_IER |= IFX_ICU_IM0_IR(irq_nr - INT_NUM_IRQ0);
	}
	if (irq_nr <= (INT_NUM_IM1_IRL0 + 31) && irq_nr >= INT_NUM_IM1_IRL0)
	{
		/* access IM1 */
      *IFX_ICU_IM1_IER |= IFX_ICU_IM1_IR(irq_nr - INT_NUM_IRQ0 - 32);
	}
	if (irq_nr <= (INT_NUM_IM2_IRL0 + 31) && irq_nr >= INT_NUM_IM2_IRL0)
	{
		/* access IM2 */
      *IFX_ICU_IM2_IER |= IFX_ICU_IM2_IR(irq_nr - INT_NUM_IRQ0 - 64);
	}
	if (irq_nr <= (INT_NUM_IM3_IRL0 + 31) && irq_nr >= INT_NUM_IM3_IRL0)
	{
		/* access IM3 */
      *IFX_ICU_IM3_IER |= IFX_ICU_IM3_IR(irq_nr - INT_NUM_IRQ0 - 96);
	}
	if (irq_nr <= (INT_NUM_IM4_IRL0 + 31) && irq_nr >= INT_NUM_IM4_IRL0)
	{
		/* access IM4 */
      *IFX_ICU_IM4_IER |= IFX_ICU_IM4_IR(irq_nr - INT_NUM_IRQ0 - 128);
	}
	if (irq_nr <= (INT_NUM_IM5_IRL0 + 31) && irq_nr >= INT_NUM_IM5_IRL0)
	{
		/* access IM5 */
      *IFX_ICU_IM5_IER |= IFX_ICU_IM5_IR(irq_nr - INT_NUM_IRQ0 - 160);
	}
#endif
}
EXPORT_SYMBOL(ifxmips_enable_irq);

static unsigned int ifxmips_startup_irq(unsigned int irq)
{
	ifxmips_enable_irq(irq);
	return 0;
}

static void ifxmips_end_irq(unsigned int irq)
{
	if (!(irq_desc[irq].status & (IRQ_DISABLED | IRQ_INPROGRESS)))
		ifxmips_enable_irq(irq);
}

static struct hw_interrupt_type ifxmips_irq_type = {
	"IFXMIPS",
	.startup = ifxmips_startup_irq,
	.enable = ifxmips_enable_irq,
	.disable = ifxmips_disable_irq,
	.unmask = ifxmips_enable_irq,
	.ack = ifxmips_end_irq,
	.mask = ifxmips_disable_irq,
	.mask_ack = ifxmips_mask_and_ack_irq,
	.end = ifxmips_end_irq,
};

static inline int ls1bit32(unsigned long x)
{
	__asm__ (
		".set push \n"
		".set mips32 \n"
		"clz %0, %1 \n"
		".set pop \n"
		: "=r" (x)
		: "r" (x));
	return 31 - x;
}

void ifxmips_hw_irqdispatch(int module)
{
#ifndef CONFIG_SVIP
	u32 irq;

	irq = ifxmips_r32(IFXMIPS_ICU_IM0_IOSR + (module * IFXMIPS_ICU_OFFSET));

	if (irq == 0)
		return;

	/* we need to do this due to a silicon bug */
	irq = ls1bit32(irq);
	do_IRQ((int)irq + INT_NUM_IM0_IRL0 + (INT_NUM_IM_OFFSET * module));

	if ((irq == 22) && (module == 0))
		ifxmips_w32(ifxmips_r32(IFXMIPS_EBU_PCC_ISTAT) | 0x10,
			IFXMIPS_EBU_PCC_ISTAT);
#else /* CONFIG_SVIP */
   int irq = 0;

   switch (module)
   {
      case 0:
         irq = *IFX_ICU_IM0_CLO - 1;
         if (irq < 0)
            return;
         irq += INT_NUM_IRQ0;
         break;
      case 1:
         irq = *IFX_ICU_IM1_CLO - 1;
         if (irq < 0)
            return;
         irq += INT_NUM_IRQ0 + 32;
         break;
      case 2:
         irq = *IFX_ICU_IM2_CLO - 1;
         if (irq < 0)
            return;
         irq += INT_NUM_IRQ0 + 64;
         break;
      case 3:
         irq = *IFX_ICU_IM3_CLO - 1;
         if (irq < 0)
            return;
         irq += INT_NUM_IRQ0 + 96;
         break;
      case 4:
         irq = *IFX_ICU_IM4_CLO - 1;
         if (irq < 0)
            return;
         irq += INT_NUM_IRQ0 + 128;
         break;
      case 5:
      {
	      unsigned long int_status;

         int_status = *IFX_ICU_IM5_IOSR;

	      /* if int_status == 0, then the interrupt has already been cleared */
         if (int_status == 0)
		      return;

         if (int_status & IFX_ICU_IM5_IR(2))
         {
            irq = MIPS_CPU_TIMER_IRQ;
         }
         else
         {
            irq = ls1bit32(int_status) + INT_NUM_IRQ0 + 160;
         }

         break;
      }
   }
	if (irq == 0)
		return;

   do_IRQ(irq);
#endif
}

#ifdef CONFIG_CPU_MIPSR2_IRQ_VI
#define DEFINE_HWx_IRQDISPATCH(x) \
static void ifxmips_hw ## x ## _irqdispatch(void)\
{\
	ifxmips_hw_irqdispatch(x); \
}
static void ifxmips_hw5_irqdispatch(void)
{
	do_IRQ(MIPS_CPU_TIMER_IRQ);
}
DEFINE_HWx_IRQDISPATCH(0)
DEFINE_HWx_IRQDISPATCH(1)
DEFINE_HWx_IRQDISPATCH(2)
DEFINE_HWx_IRQDISPATCH(3)
DEFINE_HWx_IRQDISPATCH(4)
/*DEFINE_HWx_IRQDISPATCH(5)*/
#endif /* #ifdef CONFIG_CPU_MIPSR2_IRQ_VI */

#ifdef CONFIG_SVIP
/**
 * Convert interrupt number to corresponding port/pin pair
 * Returns the port/pin pair serving the selected external interrupt; needed since mapping
 * not linear.
 *
 * \param exint     External interrupt number
 * \param port      Pointer for resulting port
 * \param pin       Pointer for resutling pin
 * \return -EINVAL  Invalid exint
 * \return 0        port/pin updated
 * \ingroup API
 */
static int ifx_exint2port (u32 exint, int *port, int *pin)
{
   if ((exint >= 0) && (exint <= 10))
   {
      *port = 0;
      *pin  = exint + 7;
   }
   else if ((exint >= 11) && (exint <= 14))
   {
      *port = 1;
      *pin  = 18 - (exint - 11) ;
   }
   else if (exint == 15)
   {
      *port = 1;
      *pin  = 19;
   }
   else if (exint == 16)
   {
      *port = 0;
      *pin  = 19;
   }
   else
   {
      return -EINVAL;
   }
   return 0;
}
#endif /* CONFIG_SVIP */

/**
 * Enable external interrupt.
 * This function enables an external interrupt and sets the given mode.
 * valid values for mode are:
 *   - 0 = Interrupt generation disabled
 *   - 1 = Interrupt on rising edge
 *   - 2 = Interrupt on falling edge
 *   - 3 = Interrupt on rising and falling edge
 *   - 5 = Interrupt on high level detection
 *   - 6 = Interrupt on low level detection
 *
 * \param   exint - Number of external interrupt
 * \param   mode  - Trigger mode
 * \return  0 on success
 * \ingroup API
 */
int ifx_enable_external_int(u32 exint, u32 mode)
{
#ifdef CONFIG_SVIP
   int port;
   int pin;

   if ((ifx_mux_exin_pin(exint)) == 0)
      return -EINVAL;

   if ((mode < 0) || (mode > 6))
      return -EINVAL;

   if (ifx_exint2port(exint, &port, &pin))
      return -EINVAL;

   ifxmips_port_clear_exintcr0 (port, pin);
   ifxmips_port_clear_exintcr1 (port, pin);
   ifxmips_port_clear_irncfg (port, pin);

   if (mode & 0x1)
      ifxmips_port_set_exintcr0 (port, pin);
   if (mode & 0x2)
      ifxmips_port_set_exintcr1 (port, pin);
   if (mode & 0x4)
      ifxmips_port_set_irncfg (port, pin);

   return ifxmips_port_set_irnen (port, pin);
#else
   return -EPERM;
#endif /* CONFIG_SVIP */
}
EXPORT_SYMBOL(ifx_enable_external_int);

/**
 * Disable external interrupt.
 * This function disables an external interrupt and sets mode to 0x00.
 *
 * \param   exint - Number of external interrupt
 * \return  0 on success
 * \ingroup API
 */
int ifx_disable_external_int(u32 exint)
{
#ifdef CONFIG_SVIP
   int port;
   int pin;

   if ((ifx_mux_exin_pin(exint)) == 0)
      return -EINVAL;

   if (ifx_exint2port (exint, &port, &pin))
      return -EINVAL;

   return ifxmips_port_clear_irnen (port, pin);
#else
   return -EPERM;
#endif /* CONFIG_SVIP */
}
EXPORT_SYMBOL(ifx_disable_external_int);

/**
 * Get level of external interrupt line.
 * This function returns the level of an external interrupt line.
 *
 * \param   exint - Number of external interrupt
 * \return  Level of external interrupt line.
 * \return  0 or 1 on success
 * \ingroup API
 */
int ifx_external_int_level(u32 exint)
{
#ifdef COONFIG_SVIP
   int port;
   int pin;

   if ((ifx_mux_exin_pin(exint)) == 0)
      return -EINVAL;

   if (ifx_exint2port (exint, &port, &pin))
      return -EINVAL;

   /* TODO: check whether this works while int is enabled... */
   return ifxmips_port_get_input (port, pin);
#else
   return -EPERM;
#endif /* CONFIG_SVIP */
}

asmlinkage void plat_irq_dispatch(void)
{
	unsigned int pending = read_c0_status() & read_c0_cause() & ST0_IM;
	unsigned int i;

#ifndef CONFIG_SVIP
	if (pending & CAUSEF_IP7) {
		do_IRQ(MIPS_CPU_TIMER_IRQ);
		goto out;
	}
   else {
#endif
		for (i = 0; i <= IM_NUM; i++) {
			if (pending & (CAUSEF_IP2 << i)) {
				ifxmips_hw_irqdispatch(i);
				goto out;
			}
		}
#ifndef CONFIG_SVIP
	}
#endif
	printk(KERN_ALERT "Spurious IRQ: CAUSE=0x%08x\n", read_c0_status());

out:
	return;
}

static struct irqaction cascade = {
	.handler = no_action,
	.flags = IRQF_DISABLED,
	.name = "cascade",
};

#ifdef CONFIG_MIPS_MT_SMP

#define MIPS_CPU_IPI_RESCHED_IRQ 0	/* SW int 0 for resched */
#define MIPS_CPU_IPI_CALL_IRQ 1		/* SW int 1 for resched */
static int cpu_ipi_resched_irq, cpu_ipi_call_irq;

static void ipi_resched_dispatch(void)
{
	do_IRQ(MIPS_CPU_IRQ_BASE + MIPS_CPU_IPI_RESCHED_IRQ);
}

static void ipi_call_dispatch(void)
{
	do_IRQ(MIPS_CPU_IRQ_BASE + MIPS_CPU_IPI_CALL_IRQ);
}

static irqreturn_t ipi_resched_interrupt(int irq, void *dev_id)
{
	return IRQ_HANDLED;
}

static irqreturn_t ipi_call_interrupt(int irq, void *dev_id)
{
	smp_call_function_interrupt();

	return IRQ_HANDLED;
}

static struct irqaction irq_resched = {
	.handler	= ipi_resched_interrupt,
	.flags		= IRQF_DISABLED|IRQF_PERCPU,
	.name		= "IPI_resched"
};

static struct irqaction irq_call = {
	.handler	= ipi_call_interrupt,
	.flags		= IRQF_DISABLED|IRQF_PERCPU,
	.name		= "IPI_call"
};

#endif /* CONFIG_MIPS_MT_SMP */

void __init arch_init_irq(void)
{
	int i;

#ifndef  CONFIG_SVIP
   for (i = 0; i < 5; i++)
		ifxmips_w32(0, IFXMIPS_ICU_IM0_IER + (i * IFXMIPS_ICU_OFFSET));
#else
	for (i = 0; i <= 5; i++)
   {
      if (i > 1 && i < 3)
		   ifxmips_w32(0, IFX_ICU_IM2_IER + (i * IFXMIPS_ICU_OFFSET));
      else
		   ifxmips_w32(0, IFX_ICU_IM0_IER + (i * IFXMIPS_ICU_OFFSET));
   }
#endif

#if (LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 26))
	mips_cpu_irq_init();
#else
	mips_cpu_irq_init(0);
#endif

	for (i = 2; i <= 6; i++)
		setup_irq(i, &cascade);

#ifdef CONFIG_CPU_MIPSR2_IRQ_VI
	if (cpu_has_vint) {
		printk(KERN_INFO "Setting up vectored interrupts\n");
		set_vi_handler(2, ifxmips_hw0_irqdispatch);
		set_vi_handler(3, ifxmips_hw1_irqdispatch);
		set_vi_handler(4, ifxmips_hw2_irqdispatch);
		set_vi_handler(5, ifxmips_hw3_irqdispatch);
		set_vi_handler(6, ifxmips_hw4_irqdispatch);
		set_vi_handler(7, ifxmips_hw5_irqdispatch);
	}
#endif /* CONFIG_CPU_MIPSR2_IRQ_VI */

	for (i = INT_NUM_IRQ0; i < (INT_NUM_IRQ0 + (6 * INT_NUM_IM_OFFSET)); i++)
   {
		set_irq_chip_and_handler(i, &ifxmips_irq_type, handle_level_irq);
   }
#if 0
	#if !defined(CONFIG_MIPS_MT_SMP) && !defined(CONFIG_MIPS_MT_SMTC)
	set_c0_status(IE_IRQ0 | IE_IRQ1 | IE_IRQ2 |
		IE_IRQ3 | IE_IRQ4 | IE_IRQ5);
	#else
	set_c0_status(IE_SW0 | IE_SW1 | IE_IRQ0 | IE_IRQ1 |
		IE_IRQ2 | IE_IRQ3 | IE_IRQ4 | IE_IRQ5);
	#endif
#endif

   set_c0_status(IE_IRQ0 | IE_IRQ1 | IE_IRQ2 | IE_IRQ3 | IE_IRQ4 | IE_IRQ5);

#ifdef CONFIG_MIPS_MT_SMP
	if (cpu_has_mipsmt) {
		/* set up ipi interrupts */
		if (cpu_has_vint) {
			set_vi_handler (MIPS_CPU_IPI_RESCHED_IRQ, ipi_resched_dispatch);
			set_vi_handler (MIPS_CPU_IPI_CALL_IRQ, ipi_call_dispatch);
		}
		cpu_ipi_resched_irq = MIPS_CPU_IRQ_BASE + MIPS_CPU_IPI_RESCHED_IRQ;
		cpu_ipi_call_irq = MIPS_CPU_IRQ_BASE + MIPS_CPU_IPI_CALL_IRQ;

		setup_irq(cpu_ipi_resched_irq, &irq_resched);
		setup_irq(cpu_ipi_call_irq, &irq_call);

		set_irq_handler(cpu_ipi_resched_irq, handle_percpu_irq);
		set_irq_handler(cpu_ipi_call_irq, handle_percpu_irq);
	}
#endif /* CONFIG_MIPS_MT_SMP */

}

#if (LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 26))
void __cpuinit arch_fixup_c0_irqs(void)
{
	/* FIXME: check for CPUID and only do fix for specific chips/versions */
#ifndef CONFIG_SVIP
	cp0_compare_irq = CP0_LEGACY_COMPARE_IRQ;
	cp0_perfcount_irq = CP0_LEGACY_PERFCNT_IRQ;
#else
	cp0_compare_irq = MIPS_CPU_TIMER_IRQ;
	cp0_perfcount_irq = MIPS_CPU_TIMER_IRQ;
#endif
}
#endif
