/************************************************************************
 *
 * Copyright (c) 2007
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/

/* Group definitions for Doxygen */
/** \addtogroup API API-Functions */
/** \addtogroup Internal Internally used functions */

#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <linux/init.h>
#include <asm/addrspace.h>
#include <asm/ifx/ifx_reg_base.h>
#include <asm/ifx/ifx_reg_port.h>
#include <asm/ifx/svip_mux.h>

static u32 mux_ebucs = 0;
static u32 mux_ebuaddr = 0;
static u32 mux_exin = 0;
static u32 mux_ssc = 0;
static u32 mux_gp[5]      = {0,0,0,0,0};
#if 0
static u32 mux_puden = 0;
#endif /* 0 */

static u32 mux_dir_cfgmask[5] = {0,0,0,0,0};
static u32 mux_dir[5]     = {0,0,0,0,0};

static u32 mux_puen_cfgmask[5] = {0,0,0,0,0};
static u32 mux_puen[5]    = {0,0,0,0,0};

static u32 mux_alt0_cfgmask[5] = {0,0,0,0,0};
static u32 mux_altsel0[5] = {0,0,0,0,0};

static u32 mux_alt1_cfgmask[5] = {0,0,0,0,0};
static u32 mux_altsel1[5] = {0,0,0,0,0};

#define PUEN_SET(port, pin) \
    mux_puen_cfgmask[(port)] |= (1 << (pin)); \
    mux_puen[(port)] |= (1 << (pin))

#define DIR_OUT(port, pin) \
    mux_dir_cfgmask[(port)] |= (1 << (pin)); \
    mux_dir[(port)] |= (1 << (pin))
#define DIR_IN(port, pin) \
    mux_dir_cfgmask[(port)] |= (1 << (pin)); \
    mux_dir[(port)] &=~(1 << (pin))

#define ALTSEL0_SET(port, pin) \
    mux_alt0_cfgmask[(port)] |= (1 << (pin)); \
    mux_altsel0[(port)] |= (1 << (pin))
#define ALTSEL0_CLEAR(port, pin) \
    mux_alt0_cfgmask[(port)] |= (1 << (pin)); \
    mux_altsel0[(port)] &=~(1 << (pin))

#define ALTSEL1_SET(port, pin) \
    mux_alt1_cfgmask[(port)] |= (1 << (pin)); \
    mux_altsel1[(port)] |= (1 << (pin))
#define ALTSEL1_CLEAR(port, pin) \
    mux_alt1_cfgmask[(port)] |= (1 << (pin)); \
    mux_altsel1[(port)] &=~(1 << (pin))


static u32 port_bases[] = {
   IFX_PORT_P0_BASE,
   IFX_PORT_P1_BASE,
   IFX_PORT_P2_BASE,
   IFX_PORT_P3_BASE,
   IFX_PORT_P4_BASE
};

#ifdef CONFIG_IFX_PMS_CLKENR_PORT0
   #define IFX_PMS_PORT0_STATE 1
#else /* CONFIG_IFX_PMS_CLKENR_PORT0 */
   #define IFX_PMS_PORT0_STATE 0
#endif /* CONFIG_IFX_PMS_CLKENR_PORT0 */
#ifdef CONFIG_IFX_PMS_CLKENR_PORT1
   #define IFX_PMS_PORT1_STATE 1
#else /* CONFIG_IFX_PMS_CLKENR_PORT1 */
   #define IFX_PMS_PORT1_STATE 0
#endif /* CONFIG_IFX_PMS_CLKENR_PORT1 */
#ifdef CONFIG_IFX_PMS_CLKENR_PORT2
   #define IFX_PMS_PORT2_STATE 1
#else /* CONFIG_IFX_PMS_CLKENR_PORT2 */
   #define IFX_PMS_PORT2_STATE 0
#endif /* CONFIG_IFX_PMS_CLKENR_PORT2 */
#ifdef CONFIG_IFX_PMS_CLKENR_PORT3
   #define IFX_PMS_PORT3_STATE 1
#else /* CONFIG_IFX_PMS_CLKENR_PORT3 */
   #define IFX_PMS_PORT3_STATE 0
#endif /* CONFIG_IFX_PMS_CLKENR_PORT3 */
#ifdef CONFIG_IFX_PMS_CLKENR_PORT4
   #define IFX_PMS_PORT4_STATE 1
#else /* CONFIG_IFX_PMS_CLKENR_PORT4 */
   #define IFX_PMS_PORT4_STATE 0
#endif /* CONFIG_IFX_PMS_CLKENR_PORT4 */

static u32 port_enabled[] = {
   IFX_PMS_PORT0_STATE,
   IFX_PMS_PORT1_STATE,
   IFX_PMS_PORT2_STATE,
   IFX_PMS_PORT3_STATE,
   IFX_PMS_PORT4_STATE
};

/**
 * Init external interrupts.
 * This functions initializes bitfield of available external interrups.
 * \ingroup Internal
 */
void ifx_mux_exin_init(void)
{
#ifdef CONFIG_IFX_MUX_EXINT16
    mux_exin |= IFX_MUX_BIT_EXINT16;

    DIR_IN (0, 19);
    ALTSEL0_SET (0, 19);
    ALTSEL1_CLEAR (0, 19);
#endif
#ifdef CONFIG_IFX_MUX_EXINT15
    mux_exin |= IFX_MUX_BIT_EXINT15;

    DIR_IN (1, 19);
    ALTSEL0_SET (1, 19);
    ALTSEL1_CLEAR (1, 19);
#endif
#ifdef CONFIG_IFX_MUX_EXINT14
    mux_exin |= IFX_MUX_BIT_EXINT14;

    DIR_IN (1, 15);
    ALTSEL0_SET (1, 15);
    ALTSEL1_CLEAR (1, 15);
#endif
#ifdef CONFIG_IFX_MUX_EXINT13
    mux_exin |= IFX_MUX_BIT_EXINT13;

    DIR_IN (1, 16);
    ALTSEL0_SET (1, 16);
    ALTSEL1_CLEAR (1, 16);
#endif
#ifdef CONFIG_IFX_MUX_EXINT12
    mux_exin |= IFX_MUX_BIT_EXINT12;

    DIR_IN (1, 17);
    ALTSEL0_SET (1, 17);
    ALTSEL1_CLEAR (1, 17);
#endif
#ifdef CONFIG_IFX_MUX_EXINT11
    mux_exin |= IFX_MUX_BIT_EXINT11;

    DIR_IN (1, 18);
    ALTSEL0_SET (1, 18);
    ALTSEL1_CLEAR (1, 18);
#endif
#ifdef CONFIG_IFX_MUX_EXINT10
    mux_exin |= IFX_MUX_BIT_EXINT10;

    DIR_IN (0, 17);
    ALTSEL0_CLEAR (0, 17);
    ALTSEL1_CLEAR (0, 17);
#endif
#ifdef CONFIG_IFX_MUX_EXINT9
    mux_exin |= IFX_MUX_BIT_EXINT9;

    DIR_IN (0, 16);
    ALTSEL0_CLEAR (0, 16);
    ALTSEL1_CLEAR (0, 16);
#endif
#ifdef CONFIG_IFX_MUX_EXINT8
    mux_exin |= IFX_MUX_BIT_EXINT8;

    DIR_IN (0, 15);
    ALTSEL0_CLEAR (0, 15);
    ALTSEL1_CLEAR (0, 15);
#endif
#ifdef CONFIG_IFX_MUX_EXINT7
    mux_exin |= IFX_MUX_BIT_EXINT7;

    DIR_IN (0, 14);
    ALTSEL0_CLEAR (0, 14);
    ALTSEL1_CLEAR (0, 14);
#endif
#ifdef CONFIG_IFX_MUX_EXINT6
    mux_exin |= IFX_MUX_BIT_EXINT6;

    DIR_IN (0, 13);
    ALTSEL0_SET (0, 13);
    ALTSEL1_CLEAR (0, 13);
#endif
#ifdef CONFIG_IFX_MUX_EXINT5
    mux_exin |= IFX_MUX_BIT_EXINT5;

    DIR_IN (0, 12);
    ALTSEL0_SET (0, 12);
    ALTSEL1_CLEAR (0, 12);
#endif
#ifdef CONFIG_IFX_MUX_EXINT4
    mux_exin |= IFX_MUX_BIT_EXINT4;

    DIR_IN (0, 11);
    ALTSEL0_SET (0, 11);
    ALTSEL1_CLEAR (0, 11);
#endif
#ifdef CONFIG_IFX_MUX_EXINT3
    mux_exin |= IFX_MUX_BIT_EXINT3;

    DIR_IN (0, 10);
    ALTSEL0_SET (0, 10);
    ALTSEL1_CLEAR (0, 10);
#endif
#ifdef CONFIG_IFX_MUX_EXINT2
    mux_exin |= IFX_MUX_BIT_EXINT2;

    DIR_IN (0, 9);
    ALTSEL0_SET (0, 9);
    ALTSEL1_CLEAR (0, 9);
#endif
#ifdef CONFIG_IFX_MUX_EXINT1
    mux_exin |= IFX_MUX_BIT_EXINT1;

    DIR_IN (0, 8);
    ALTSEL0_SET (0, 8);
    ALTSEL1_CLEAR (0, 8);
#endif
#ifdef CONFIG_IFX_MUX_EXINT0
    mux_exin |= IFX_MUX_BIT_EXINT0;

    DIR_IN (0, 7);
    ALTSEL0_SET (0, 7);
    ALTSEL1_CLEAR (0, 7);
#endif
}

/**
 * Init SSC
 * This functions initializes altsel and direction values for SSCs.
 * \ingroup Internal
 */
void ifx_mux_ssc_init(void)
{
#ifdef CONFIG_IFX_MUX_SSC0_MTSR
    DIR_OUT (0, 0);
    ALTSEL0_CLEAR (0, 0);
    ALTSEL1_CLEAR (0, 0);
# ifdef CONFIG_IFX_MUX_SSC0_MRST
    DIR_IN (0, 1);
    ALTSEL0_CLEAR (0, 1);
    ALTSEL1_CLEAR (0, 1);
#  ifdef CONFIG_IFX_MUX_SSC0_SCLK
    DIR_OUT (0, 2);
    ALTSEL0_CLEAR (0, 2);
    ALTSEL1_CLEAR (0, 2);
    /* if the three above pins are available, SSC0 is usable */
    mux_ssc |= IFX_MUX_BIT_SSC0;
#  endif
# endif
#endif
#ifdef CONFIG_IFX_MUX_SSC0_CS0
    DIR_OUT (0, 6);
    PUEN_SET (0, 6);
    ALTSEL0_CLEAR (0, 6);
    ALTSEL1_CLEAR (0, 6);
#endif
#ifdef CONFIG_IFX_MUX_SSC0_CS1
    DIR_OUT (0, 7);
    PUEN_SET (0, 7);
    ALTSEL0_CLEAR (0, 7);
    ALTSEL1_CLEAR (0, 7);
#endif
#ifdef CONFIG_IFX_MUX_SSC0_CS2
    DIR_OUT (0, 8);
    PUEN_SET (0, 8);
    ALTSEL0_CLEAR (0, 8);
    ALTSEL1_CLEAR (0, 8);
#endif
#ifdef CONFIG_IFX_MUX_SSC0_CS3
    DIR_OUT (0, 9);
    PUEN_SET (0, 9);
    ALTSEL0_CLEAR (0, 9);
    ALTSEL1_CLEAR (0, 9);
#endif
#ifdef CONFIG_IFX_MUX_SSC0_CS4
    DIR_OUT (0, 10);
    PUEN_SET (0, 10);
    ALTSEL0_CLEAR (0, 10);
    ALTSEL1_CLEAR (0, 10);
#endif
#ifdef CONFIG_IFX_MUX_SSC0_CS5
    DIR_OUT (0, 11);
    PUEN_SET (0, 11);
    ALTSEL0_CLEAR (0, 11);
    ALTSEL1_CLEAR (0, 11);
#endif
#ifdef CONFIG_IFX_MUX_SSC0_CS6
    DIR_OUT (0, 12);
    PUEN_SET (0, 12);
    ALTSEL0_CLEAR (0, 12);
    ALTSEL1_CLEAR (0, 12);
#endif
#ifdef CONFIG_IFX_MUX_SSC0_CS7
    DIR_OUT (0, 13);
    PUEN_SET (0, 13);
    ALTSEL0_CLEAR (0, 13);
    ALTSEL1_CLEAR (0, 13);
#endif

#ifdef CONFIG_IFX_MUX_SSC1_MTSR
    DIR_OUT (0, 3);
    ALTSEL0_CLEAR (0, 3);
    ALTSEL1_CLEAR (0, 3);
# ifdef CONFIG_IFX_MUX_SSC1_MRST
    DIR_IN (0, 4);
    ALTSEL0_CLEAR (0, 4);
    ALTSEL1_CLEAR (0, 4);
#  ifdef CONFIG_IFX_MUX_SSC1_SCLK
    DIR_OUT (0, 5);
    ALTSEL0_CLEAR (0, 5);
    ALTSEL1_CLEAR (0, 5);
    /* if the three above pins are available, SSC1 is usable */
    mux_ssc |= IFX_MUX_BIT_SSC1;
#  endif
# endif
#endif
#ifdef CONFIG_IFX_MUX_SSC1_CS0
    DIR_OUT (0, 6);
    ALTSEL0_CLEAR (0, 6);
    ALTSEL1_SET (0, 6);
#endif
#ifdef CONFIG_IFX_MUX_SSC1_CS1
    DIR_OUT (0, 7);
    ALTSEL0_CLEAR (0, 7);
    ALTSEL1_SET (0, 7);
#endif
#ifdef CONFIG_IFX_MUX_SSC1_CS2
    DIR_OUT (0, 8);
    ALTSEL0_CLEAR (0, 8);
    ALTSEL1_SET (0, 8);
#endif
#ifdef CONFIG_IFX_MUX_SSC1_CS3
    DIR_OUT (0, 9);
    ALTSEL0_CLEAR (0, 9);
    ALTSEL1_SET (0, 9);
#endif
#ifdef CONFIG_IFX_MUX_SSC1_CS4
    DIR_OUT (0, 10);
    ALTSEL0_CLEAR (0, 10);
    ALTSEL1_SET (0, 10);
#endif
#ifdef CONFIG_IFX_MUX_SSC1_CS5
    DIR_OUT (0, 11);
    ALTSEL0_CLEAR (0, 11);
    ALTSEL1_SET (0, 11);
#endif
#ifdef CONFIG_IFX_MUX_SSC1_CS6
    DIR_OUT (0, 12);
    ALTSEL0_CLEAR (0, 12);
    ALTSEL1_SET (0, 12);
#endif
#ifdef CONFIG_IFX_MUX_SSC1_CS7
    DIR_OUT (0, 13);
    ALTSEL0_CLEAR (0, 13);
    ALTSEL1_SET (0, 13);
#endif
}

/**
 * Init ASC.
 * This functions initializes altsel and direction values for ASCs.
 * \ingroup Internal
 */
void ifx_mux_asc_init(void)
{
#if 0
#if defined(CONFIG_IFX_MUX_TXD0)
    mux_altsel23 = 0;
    mux_dir |= 0x00800000;
    mux_dir_mask |= 0x00800000;
#endif
#if defined(CONFIG_IFX_MUX_RXD0)
    mux_altsel24 = 0;
    mux_dir_mask |= 0x01000000;
#endif
#if defined(CONFIG_IFX_MUX_TXD1)
    mux_altsel17 = 3;
    mux_dir |= 0x00020000;
    mux_dir_mask |= 0x00020000;
#endif
#if defined(CONFIG_IFX_MUX_RXD1)
    mux_altsel18 = 3;
    mux_dir_mask |= 0x00040000;
#endif
#endif
}

/**
 * Init PCM.
 * This functions initializes altsel values for PCM.
 * \ingroup Internal
 */
void ifx_mux_pcm_init(void)
{
#if 0
#ifdef CONFIG_IFX_MUX_RES
    mux_altsel5 = 1;
    mux_dir |= 0x00000020;
    mux_dir_mask |= 0x00000020;
#endif
#ifdef CONFIG_IFX_MUX_TC
    mux_altsel4 = 1;
    mux_dir |= 0x00000010;
#endif
#ifdef CONFIG_IFX_MUX_DO
    mux_altsel3 = 1;
    mux_dir |= 0x00000008;
    mux_dir_mask |= 0x00000008;
    mux_puden |= 0x00000008;
#endif
#ifdef CONFIG_IFX_MUX_DI
    mux_altsel2 = 1;
    mux_dir_mask |= 0x00000004;
    mux_puden |= 0x00000004;
#endif
#ifdef CONFIG_IFX_MUX_DCL
    mux_altsel1 = 1;
    mux_dir |= 0x00000002;
    mux_dir_mask |= 0x00000002;
#endif
#ifdef CONFIG_IFX_MUX_DCL_SLAVE
    mux_altsel1 = 1;
    mux_dir_mask |= 0x00000002;
#endif
#ifdef CONFIG_IFX_MUX_FSC
    mux_altsel0 = 1;
    mux_dir |= 0x00000001;
    mux_dir_mask |= 0x00000001;
#endif
#ifdef CONFIG_IFX_MUX_FSC_SLAVE
    mux_altsel0 = 1;
    mux_dir_mask |= 0x00000001;
#endif
#endif
}

/**
 * Init EBU.
 * This functions initializes bitfield of available chip selects and address pins.
 * \ingroup Internal
 */
void ifx_mux_ebu_init(void)
{
   /*  \note:
       The  direction  is  automatically  configured  to  Input  or Output.
    */
#ifdef CONFIG_IFX_MUX_EBU_BC1
    ALTSEL0_CLEAR (2, 18);
    ALTSEL1_CLEAR (2, 18);
#endif /* CONFIG_IFX_MUX_EBU_BC1 */
#ifdef CONFIG_IFX_MUX_EBU_BC0
    ALTSEL0_CLEAR (2, 17);
    ALTSEL1_CLEAR (2, 17);
#endif /* CONFIG_IFX_MUX_EBU_BC0 */
#ifdef CONFIG_IFX_MUX_EBU_RDBY
    ALTSEL0_CLEAR (2, 16);
    ALTSEL1_CLEAR (2, 16);
#endif /* CONFIG_IFX_MUX_EBU_RDBY */
#ifdef CONFIG_IFX_MUX_EBU_WAIT
    ALTSEL0_CLEAR (2, 15);
    ALTSEL1_CLEAR (2, 15);
#endif /* CONFIG_IFX_MUX_EBU_WAIT */
#ifdef CONFIG_IFX_MUX_EBU_ALE
    ALTSEL0_CLEAR (2, 14);
    ALTSEL1_CLEAR (2, 14);
#endif /* CONFIG_IFX_MUX_EBU_ALE */
#ifdef CONFIG_IFX_MUX_EBU_WR
    ALTSEL0_CLEAR (2, 13);
    ALTSEL1_CLEAR (2, 13);
#endif /* CONFIG_IFX_MUX_EBU_WR */
#ifdef CONFIG_IFX_MUX_EBU_RD
    ALTSEL0_CLEAR (2, 12);
    ALTSEL1_CLEAR (2, 12);
#endif /* CONFIG_IFX_MUX_EBU_RD */
#ifdef CONFIG_IFX_MUX_EBU_A11
    mux_ebuaddr |= IFX_MUX_BIT_EBU_A11;
    ALTSEL0_CLEAR (2, 11);
    ALTSEL1_CLEAR (2, 11);
#endif /* CONFIG_IFX_MUX_EBU_A11 */
#ifdef CONFIG_IFX_MUX_EBU_A10
    mux_ebuaddr |= IFX_MUX_BIT_EBU_A10;
    ALTSEL0_CLEAR (2, 10);
    ALTSEL1_CLEAR (2, 10);
#endif /* CONFIG_IFX_MUX_EBU_A10 */
#ifdef CONFIG_IFX_MUX_EBU_A9
    mux_ebuaddr |= IFX_MUX_BIT_EBU_A9;
    ALTSEL0_CLEAR (2, 9);
    ALTSEL1_CLEAR (2, 9);
#endif /* CONFIG_IFX_MUX_EBU_A9 */
#ifdef CONFIG_IFX_MUX_EBU_A8
    mux_ebuaddr |= IFX_MUX_BIT_EBU_A8;
    ALTSEL0_CLEAR (2, 8);
    ALTSEL1_CLEAR (2, 8);
#endif /* CONFIG_IFX_MUX_EBU_A8 */
#ifdef CONFIG_IFX_MUX_EBU_A7
    mux_ebuaddr |= IFX_MUX_BIT_EBU_A7;
    ALTSEL0_CLEAR (2, 7);
    ALTSEL1_CLEAR (2, 7);
#endif /* CONFIG_IFX_MUX_EBU_A7 */
#ifdef CONFIG_IFX_MUX_EBU_A6
    mux_ebuaddr |= IFX_MUX_BIT_EBU_A6;
    ALTSEL0_CLEAR (2, 6);
    ALTSEL1_CLEAR (2, 6);
#endif /* CONFIG_IFX_MUX_EBU_A6 */
#ifdef CONFIG_IFX_MUX_EBU_A5
    mux_ebuaddr |= IFX_MUX_BIT_EBU_A5;
    ALTSEL0_CLEAR (2, 5);
    ALTSEL1_CLEAR (2, 5);
#endif /* CONFIG_IFX_MUX_EBU_A5 */
#ifdef CONFIG_IFX_MUX_EBU_A4
    mux_ebuaddr |= IFX_MUX_BIT_EBU_A4;
    ALTSEL0_CLEAR (2, 4);
    ALTSEL1_CLEAR (2, 4);
#endif /* CONFIG_IFX_MUX_EBU_A4 */
#ifdef CONFIG_IFX_MUX_EBU_A3
    mux_ebuaddr |= IFX_MUX_BIT_EBU_A3;
    ALTSEL0_CLEAR (2, 3);
    ALTSEL1_CLEAR (2, 3);
#endif /* CONFIG_IFX_MUX_EBU_A3 */
#ifdef CONFIG_IFX_MUX_EBU_A2
    mux_ebuaddr |= IFX_MUX_BIT_EBU_A2;
    ALTSEL0_CLEAR (2, 2);
    ALTSEL1_CLEAR (2, 2);
#endif /* CONFIG_IFX_MUX_EBU_A2 */
#ifdef CONFIG_IFX_MUX_EBU_A1
    mux_ebuaddr |= IFX_MUX_BIT_EBU_A1;
    ALTSEL0_CLEAR (2, 1);
    ALTSEL1_CLEAR (2, 1);
#endif /* CONFIG_IFX_MUX_EBU_A1 */
#ifdef CONFIG_IFX_MUX_EBU_A0
    mux_ebuaddr |= IFX_MUX_BIT_EBU_A0;
    ALTSEL0_CLEAR (2, 0);
    ALTSEL1_CLEAR (2, 0);
#endif /* CONFIG_IFX_MUX_EBU_A0 */
#ifdef CONFIG_IFX_MUX_EBU_CS3
    mux_ebucs |= IFX_MUX_BIT_EBU_CS3;
    ALTSEL0_CLEAR (3, 19);
    ALTSEL1_CLEAR (3, 19);
#endif /* CONFIG_IFX_MUX_EBU_CS3 */
#ifdef CONFIG_IFX_MUX_EBU_CS2
    mux_ebucs |= IFX_MUX_BIT_EBU_CS2;
    ALTSEL0_CLEAR (3, 18);
    ALTSEL1_CLEAR (3, 18);
#endif /* CONFIG_IFX_MUX_EBU_CS2 */
#ifdef CONFIG_IFX_MUX_EBU_CS1
    mux_ebucs |= IFX_MUX_BIT_EBU_CS1;
    ALTSEL0_CLEAR (3, 17);
    ALTSEL1_CLEAR (3, 17);
#endif /* CONFIG_IFX_MUX_EBU_CS1 */
#ifdef CONFIG_IFX_MUX_EBU_CS0
    mux_ebucs |= IFX_MUX_BIT_EBU_CS0;
    ALTSEL0_CLEAR (3, 16);
    ALTSEL1_CLEAR (3, 16);
#endif /* CONFIG_IFX_MUX_EBU_CS0 */
#ifdef CONFIG_IFX_MUX_EBU_AD15
    ALTSEL0_CLEAR (3, 15);
    ALTSEL1_CLEAR (3, 15);
#endif /* CONFIG_IFX_MUX_EBU_AD15 */
#ifdef CONFIG_IFX_MUX_EBU_AD14
    ALTSEL0_CLEAR (3, 14);
    ALTSEL1_CLEAR (3, 14);
#endif /* CONFIG_IFX_MUX_EBU_AD14 */
#ifdef CONFIG_IFX_MUX_EBU_AD13
    ALTSEL0_CLEAR (3, 13);
    ALTSEL1_CLEAR (3, 13);
#endif /* CONFIG_IFX_MUX_EBU_AD13 */
#ifdef CONFIG_IFX_MUX_EBU_AD12
    ALTSEL0_CLEAR (3, 12);
    ALTSEL1_CLEAR (3, 12);
#endif /* CONFIG_IFX_MUX_EBU_AD12 */
#ifdef CONFIG_IFX_MUX_EBU_AD11
    ALTSEL0_CLEAR (3, 11);
    ALTSEL1_CLEAR (3, 11);
#endif /* CONFIG_IFX_MUX_EBU_AD11 */
#ifdef CONFIG_IFX_MUX_EBU_AD10
    ALTSEL0_CLEAR (3, 10);
    ALTSEL1_CLEAR (3, 10);
#endif /* CONFIG_IFX_MUX_EBU_AD10 */
#ifdef CONFIG_IFX_MUX_EBU_AD9
    ALTSEL0_CLEAR (3, 9);
    ALTSEL1_CLEAR (3, 9);
#endif /* CONFIG_IFX_MUX_EBU_AD9 */
#ifdef CONFIG_IFX_MUX_EBU_AD8
    ALTSEL0_CLEAR (3, 8);
    ALTSEL1_CLEAR (3, 8);
#endif /* CONFIG_IFX_MUX_EBU_AD8 */
#ifdef CONFIG_IFX_MUX_EBU_AD7
    ALTSEL0_CLEAR (3, 7);
    ALTSEL1_CLEAR (3, 7);
#endif /* CONFIG_IFX_MUX_EBU_AD7 */
#ifdef CONFIG_IFX_MUX_EBU_AD6
    ALTSEL0_CLEAR (3, 6);
    ALTSEL1_CLEAR (3, 6);
#endif /* CONFIG_IFX_MUX_EBU_AD6 */
#ifdef CONFIG_IFX_MUX_EBU_AD5
    ALTSEL0_CLEAR (3, 5);
    ALTSEL1_CLEAR (3, 5);
#endif /* CONFIG_IFX_MUX_EBU_AD5 */
#ifdef CONFIG_IFX_MUX_EBU_AD4
    ALTSEL0_CLEAR (3, 4);
    ALTSEL1_CLEAR (3, 4);
#endif /* CONFIG_IFX_MUX_EBU_AD4 */
#ifdef CONFIG_IFX_MUX_EBU_AD3
    ALTSEL0_CLEAR (3, 3);
    ALTSEL1_CLEAR (3, 3);
#endif /* CONFIG_IFX_MUX_EBU_AD3 */
#ifdef CONFIG_IFX_MUX_EBU_AD2
    ALTSEL0_CLEAR (3, 2);
    ALTSEL1_CLEAR (3, 2);
#endif /* CONFIG_IFX_MUX_EBU_AD2 */
#ifdef CONFIG_IFX_MUX_EBU_AD1
    ALTSEL0_CLEAR (3, 1);
    ALTSEL1_CLEAR (3, 1);
#endif /* CONFIG_IFX_MUX_EBU_AD1 */
#ifdef CONFIG_IFX_MUX_EBU_AD0
    ALTSEL0_CLEAR (3, 0);
    ALTSEL1_CLEAR (3, 0);
#endif /* CONFIG_IFX_MUX_EBU_AD0 */
}

/**
 * Init GP bitfield.
 * This functions initializes bitfield of available general purpose pins.
 * \ingroup Internal
 */
void ifx_mux_gp_init(void)
{
	/* PORT P0 */
#ifdef CONFIG_IFX_MUX_P0_19
    mux_gp[0]      |= (1 << 19);
    ALTSEL0_SET (0, 19);
    ALTSEL1_CLEAR (0, 19);
#endif
#ifdef CONFIG_IFX_MUX_P0_18
    mux_gp[0]      |= (1 << 18);
    ALTSEL0_SET (0, 18);
    ALTSEL1_CLEAR (0, 18);
#endif
#ifdef CONFIG_IFX_MUX_P0_17
    mux_gp[0]      |= (1 << 17);
    ALTSEL0_CLEAR (0, 17);
    ALTSEL1_CLEAR (0, 17);
#endif
#ifdef CONFIG_IFX_MUX_P0_16
    mux_gp[0]      |= (1 << 16);
    ALTSEL0_CLEAR (0, 16);
    ALTSEL1_CLEAR (0, 16);
#endif
#ifdef CONFIG_IFX_MUX_P0_15
    mux_gp[0]      |= (1 << 15);
    ALTSEL0_CLEAR (0, 15);
    ALTSEL1_CLEAR (0, 15);
#endif
#ifdef CONFIG_IFX_MUX_P0_14
    mux_gp[0]      |= (1 << 14);
    ALTSEL0_CLEAR (0, 14);
    ALTSEL1_CLEAR (0, 14);
#endif
#ifdef CONFIG_IFX_MUX_P0_13
    mux_gp[0]      |= (1 << 13);
    DIR_OUT (0, 13);
    ALTSEL0_SET (0, 13);
    ALTSEL1_CLEAR (0, 13);
#endif
#ifdef CONFIG_IFX_MUX_P0_12
    mux_gp[0]      |= (1 << 12);
    DIR_OUT (0, 12);
    ALTSEL0_SET (0, 12);
    ALTSEL1_CLEAR (0, 12);
#endif
#ifdef CONFIG_IFX_MUX_P0_11
    mux_gp[0]      |= (1 << 11);
    ALTSEL0_SET (0, 11);
    ALTSEL1_CLEAR (0, 11);
#endif
#ifdef CONFIG_IFX_MUX_P0_10
    mux_gp[0]      |= (1 << 10);
    ALTSEL0_SET (0, 10);
    ALTSEL1_CLEAR (0, 10);
#endif
#ifdef CONFIG_IFX_MUX_P0_9
    mux_gp[0]      |= (1 << 9);
    ALTSEL0_SET (0, 9);
    ALTSEL1_CLEAR (0, 9);
#endif
#ifdef CONFIG_IFX_MUX_P0_8
    mux_gp[0]      |= (1 << 8);
    ALTSEL0_SET (0, 8);
    ALTSEL1_CLEAR (0, 8);
#endif
#ifdef CONFIG_IFX_MUX_P0_7
    mux_gp[0]      |= (1 << 7);
    ALTSEL0_SET (0, 7);
    ALTSEL1_CLEAR (0, 7);
#endif
#ifdef CONFIG_IFX_MUX_P0_6
    mux_gp[0]      |= (1 << 6);
    ALTSEL0_SET (0, 6);
    ALTSEL1_CLEAR (0, 6);
#endif
#ifdef CONFIG_IFX_MUX_P0_5
    mux_gp[0]      |= (1 << 5);
    ALTSEL0_SET (0, 5);
    ALTSEL1_CLEAR (0, 5);
#endif
#ifdef CONFIG_IFX_MUX_P0_4
    mux_gp[0]      |= (1 << 4);
    ALTSEL0_SET (0, 4);
    ALTSEL1_CLEAR (0, 4);
#endif
#ifdef CONFIG_IFX_MUX_P0_3
    mux_gp[0]      |= (1 << 3);
    ALTSEL0_SET (0, 3);
    ALTSEL1_CLEAR (0, 3);
#endif
#ifdef CONFIG_IFX_MUX_P0_2
    mux_gp[0]      |= (1 << 2);
    ALTSEL0_SET (0, 2);
    ALTSEL1_CLEAR (0, 2);
#endif
#ifdef CONFIG_IFX_MUX_P0_1
    mux_gp[0]      |= (1 << 1);
    ALTSEL0_SET (0, 1);
    ALTSEL1_CLEAR (0, 1);
#endif
#ifdef CONFIG_IFX_MUX_P0_0
    mux_gp[0]      |= (1 << 0);
    ALTSEL0_SET (0, 0);
    ALTSEL1_CLEAR (0, 0);
#endif
	/* PORT P1 */
#ifdef CONFIG_IFX_MUX_P1_19
    mux_gp[1]      |= (1 << 19);
    ALTSEL0_SET (1, 19);
    ALTSEL1_CLEAR (1, 19);
#endif
#ifdef CONFIG_IFX_MUX_P1_18
    mux_gp[1]      |= (1 << 18);
    ALTSEL0_SET (1, 18);
    ALTSEL1_CLEAR (1, 18);
#endif
#ifdef CONFIG_IFX_MUX_P1_17
    mux_gp[1]      |= (1 << 17);
    ALTSEL0_SET (1, 17);
    ALTSEL1_CLEAR (1, 17);
#endif
#ifdef CONFIG_IFX_MUX_P1_16
    mux_gp[1]      |= (1 << 16);
    ALTSEL0_SET (1, 16);
    ALTSEL1_CLEAR (1, 16);
#endif
#ifdef CONFIG_IFX_MUX_P1_15
    mux_gp[1]      |= (1 << 15);
    ALTSEL0_SET (1, 15);
    ALTSEL1_CLEAR (1, 15);
#endif
#ifdef CONFIG_IFX_MUX_P1_14
    mux_gp[1]      |= (1 << 14);
    ALTSEL0_SET (1, 14);
    ALTSEL1_CLEAR (1, 14);
#endif
#ifdef CONFIG_IFX_MUX_P1_13
    mux_gp[1]      |= (1 << 13);
    ALTSEL0_SET (1, 13);
    ALTSEL1_CLEAR (1, 13);
#endif
#ifdef CONFIG_IFX_MUX_P1_12
    mux_gp[1]      |= (1 << 12);
    ALTSEL0_SET (1, 12);
    ALTSEL1_CLEAR (1, 12);
#endif
#ifdef CONFIG_IFX_MUX_P1_11
    mux_gp[1]      |= (1 << 11);
    ALTSEL0_SET (1, 11);
    ALTSEL1_CLEAR (1, 11);
#endif
#ifdef CONFIG_IFX_MUX_P1_10
    mux_gp[1]      |= (1 << 10);
    ALTSEL0_SET (1, 10);
    ALTSEL1_CLEAR (1, 10);
#endif
#ifdef CONFIG_IFX_MUX_P1_9
    mux_gp[1]      |= (1 << 9);
    ALTSEL0_SET (1, 9);
    ALTSEL1_CLEAR (1, 9);
#endif
#ifdef CONFIG_IFX_MUX_P1_8
    mux_gp[1]      |= (1 << 8);
    ALTSEL0_SET (1, 8);
    ALTSEL1_CLEAR (1, 8);
#endif
#ifdef CONFIG_IFX_MUX_P1_7
    mux_gp[1]      |= (1 << 7);
    ALTSEL0_SET (1, 7);
    ALTSEL1_CLEAR (1, 7);
#endif
#ifdef CONFIG_IFX_MUX_P1_6
    mux_gp[1]      |= (1 << 6);
    ALTSEL0_SET (1, 6);
    ALTSEL1_CLEAR (1, 6);
#endif
#ifdef CONFIG_IFX_MUX_P1_5
    mux_gp[1]      |= (1 << 5);
    ALTSEL0_SET (1, 5);
    ALTSEL1_CLEAR (1, 5);
#endif
#ifdef CONFIG_IFX_MUX_P1_4
    mux_gp[1]      |= (1 << 4);
    ALTSEL0_SET (1, 4);
    ALTSEL1_CLEAR (1, 4);
#endif
#ifdef CONFIG_IFX_MUX_P1_3
    mux_gp[1]      |= (1 << 3);
    ALTSEL0_SET (1, 3);
    ALTSEL1_CLEAR (1, 3);
#endif
#ifdef CONFIG_IFX_MUX_P1_2
    mux_gp[1]      |= (1 << 2);
    ALTSEL0_SET (1, 2);
    ALTSEL1_CLEAR (1, 2);
#endif
#ifdef CONFIG_IFX_MUX_P1_1
    mux_gp[1]      |= (1 << 1);
    ALTSEL0_SET (1, 1);
    ALTSEL1_CLEAR (1, 1);
#endif
#ifdef CONFIG_IFX_MUX_P1_0
    mux_gp[1]      |= (1 << 0);
    ALTSEL0_SET (1, 0);
    ALTSEL1_CLEAR (1, 0);
#endif
	/* PORT P2 */
#ifdef CONFIG_IFX_MUX_P2_18
    mux_gp[2]      |= (1 << 18);
    ALTSEL0_SET (2, 18);
    ALTSEL1_CLEAR (2, 18);
#endif
#ifdef CONFIG_IFX_MUX_P2_17
    mux_gp[2]      |= (1 << 17);
    ALTSEL0_SET (2, 17);
    ALTSEL1_CLEAR (2, 17);
#endif
#ifdef CONFIG_IFX_MUX_P2_16
    mux_gp[2]      |= (1 << 16);
    ALTSEL0_SET (2, 16);
    ALTSEL1_CLEAR (2, 16);
#endif
#ifdef CONFIG_IFX_MUX_P2_15
    mux_gp[2]      |= (1 << 15);
    ALTSEL0_SET (2, 15);
    ALTSEL1_CLEAR (2, 15);
#endif
#ifdef CONFIG_IFX_MUX_P2_14
    mux_gp[2]      |= (1 << 14);
    ALTSEL0_SET (2, 14);
    ALTSEL1_CLEAR (2, 14);
#endif
#ifdef CONFIG_IFX_MUX_P2_13
    mux_gp[2]      |= (1 << 13);
    ALTSEL0_SET (2, 13);
    ALTSEL1_CLEAR (2, 13);
#endif
#ifdef CONFIG_IFX_MUX_P2_12
    mux_gp[2]      |= (1 << 12);
    ALTSEL0_SET (2, 12);
    ALTSEL1_CLEAR (2, 12);
#endif
#ifdef CONFIG_IFX_MUX_P2_11
    mux_gp[2]      |= (1 << 11);
    ALTSEL0_SET (2, 11);
    ALTSEL1_CLEAR (2, 11);
#endif
#ifdef CONFIG_IFX_MUX_P2_10
    mux_gp[2]      |= (1 << 10);
    ALTSEL0_SET (2, 10);
    ALTSEL1_CLEAR (2, 10);
#endif
#ifdef CONFIG_IFX_MUX_P2_9
    mux_gp[2]      |= (1 << 9);
    ALTSEL0_SET (2, 9);
    ALTSEL1_CLEAR (2, 9);
#endif
#ifdef CONFIG_IFX_MUX_P2_8
    mux_gp[2]      |= (1 << 8);
    ALTSEL0_SET (2, 8);
    ALTSEL1_CLEAR (2, 8);
#endif
#ifdef CONFIG_IFX_MUX_P2_7
    mux_gp[2]      |= (1 << 7);
    ALTSEL0_SET (2, 7);
    ALTSEL1_CLEAR (2, 7);
#endif
#ifdef CONFIG_IFX_MUX_P2_6
    mux_gp[2]      |= (1 << 6);
    ALTSEL0_SET (2, 6);
    ALTSEL1_CLEAR (2, 6);
#endif
#ifdef CONFIG_IFX_MUX_P2_5
    mux_gp[2]      |= (1 << 5);
    ALTSEL0_SET (2, 5);
    ALTSEL1_CLEAR (2, 5);
#endif
#ifdef CONFIG_IFX_MUX_P2_4
    mux_gp[2]      |= (1 << 4);
    ALTSEL0_SET (2, 4);
    ALTSEL1_CLEAR (2, 4);
#endif
#ifdef CONFIG_IFX_MUX_P2_3
    mux_gp[2]      |= (1 << 3);
    ALTSEL0_SET (2, 3);
    ALTSEL1_CLEAR (2, 3);
#endif
#ifdef CONFIG_IFX_MUX_P2_2
    mux_gp[2]      |= (1 << 2);
    ALTSEL0_SET (2, 2);
    ALTSEL1_CLEAR (2, 2);
#endif
#ifdef CONFIG_IFX_MUX_P2_1
    mux_gp[2]      |= (1 << 1);
    ALTSEL0_SET (2, 1);
    ALTSEL1_CLEAR (2, 1);
#endif
#ifdef CONFIG_IFX_MUX_P2_0
    mux_gp[2]      |= (1 << 0);
    ALTSEL0_SET (2, 0);
    ALTSEL1_CLEAR (2, 0);
#endif
	/* PORT P3 */
#ifdef CONFIG_IFX_MUX_P3_19
    mux_gp[3]      |= (1 << 19);
    ALTSEL0_SET (3, 19);
    ALTSEL1_CLEAR (3, 19);
#endif
#ifdef CONFIG_IFX_MUX_P3_18
    mux_gp[3]      |= (1 << 18);
    ALTSEL0_SET (3, 18);
    ALTSEL1_CLEAR (3, 18);
#endif
#ifdef CONFIG_IFX_MUX_P3_17
    mux_gp[3]      |= (1 << 17);
    ALTSEL0_SET (3, 17);
    ALTSEL1_CLEAR (3, 17);
#endif
#ifdef CONFIG_IFX_MUX_P3_16
    mux_gp[3]      |= (1 << 16);
    ALTSEL0_SET (3, 16);
    ALTSEL1_CLEAR (3, 16);
#endif
#ifdef CONFIG_IFX_MUX_P3_15
    mux_gp[3]      |= (1 << 15);
    ALTSEL0_SET (3, 15);
    ALTSEL1_CLEAR (3, 15);
#endif
#ifdef CONFIG_IFX_MUX_P3_14
    mux_gp[3]      |= (1 << 14);
    ALTSEL0_SET (3, 14);
    ALTSEL1_CLEAR (3, 14);
#endif
#ifdef CONFIG_IFX_MUX_P3_13
    mux_gp[3]      |= (1 << 13);
    ALTSEL0_SET (3, 13);
    ALTSEL1_CLEAR (3, 13);
#endif
#ifdef CONFIG_IFX_MUX_P3_12
    mux_gp[3]      |= (1 << 12);
    ALTSEL0_SET (3, 12);
    ALTSEL1_CLEAR (3, 12);
#endif
#ifdef CONFIG_IFX_MUX_P3_11
    mux_gp[3]      |= (1 << 11);
    ALTSEL0_SET (3, 11);
    ALTSEL1_CLEAR (3, 11);
#endif
#ifdef CONFIG_IFX_MUX_P3_10
    mux_gp[3]      |= (1 << 10);
    ALTSEL0_SET (3, 10);
    ALTSEL1_CLEAR (3, 10);
#endif
#ifdef CONFIG_IFX_MUX_P3_9
    mux_gp[3]      |= (1 << 9);
    ALTSEL0_SET (3, 9);
    ALTSEL1_CLEAR (3, 9);
#endif
#ifdef CONFIG_IFX_MUX_P3_8
    mux_gp[3]      |= (1 << 8);
    ALTSEL0_SET (3, 8);
    ALTSEL1_CLEAR (3, 8);
#endif
#ifdef CONFIG_IFX_MUX_P3_7
    mux_gp[3]      |= (1 << 7);
    ALTSEL0_SET (3, 7);
    ALTSEL1_CLEAR (3, 7);
#endif
#ifdef CONFIG_IFX_MUX_P3_6
    mux_gp[3]      |= (1 << 6);
    ALTSEL0_SET (3, 6);
    ALTSEL1_CLEAR (3, 6);
#endif
#ifdef CONFIG_IFX_MUX_P3_5
    mux_gp[3]      |= (1 << 5);
    ALTSEL0_SET (3, 5);
    ALTSEL1_CLEAR (3, 5);
#endif
#ifdef CONFIG_IFX_MUX_P3_4
    mux_gp[3]      |= (1 << 4);
    ALTSEL0_SET (3, 4);
    ALTSEL1_CLEAR (3, 4);
#endif
#ifdef CONFIG_IFX_MUX_P3_3
    mux_gp[3]      |= (1 << 3);
    ALTSEL0_SET (3, 3);
    ALTSEL1_CLEAR (3, 3);
#endif
#ifdef CONFIG_IFX_MUX_P3_2
    mux_gp[3]      |= (1 << 2);
    ALTSEL0_SET (3, 2);
    ALTSEL1_CLEAR (3, 2);
#endif
#ifdef CONFIG_IFX_MUX_P3_1
    mux_gp[3]      |= (1 << 1);
    ALTSEL0_SET (3, 1);
    ALTSEL1_CLEAR (3, 1);
#endif
#ifdef CONFIG_IFX_MUX_P3_0
    mux_gp[3]      |= (1 << 0);
    ALTSEL0_SET (3, 0);
    ALTSEL1_CLEAR (3, 0);
#endif
	/* PORT P4 */
#ifdef CONFIG_IFX_MUX_P4_23
    mux_gp[4]      |= (1 << 23);
    ALTSEL0_SET (4, 23);
    ALTSEL1_CLEAR (4, 23);
#endif
#ifdef CONFIG_IFX_MUX_P4_22
    mux_gp[4]      |= (1 << 22);
    ALTSEL0_SET (4, 22);
    ALTSEL1_CLEAR (4, 22);
#endif
#ifdef CONFIG_IFX_MUX_P4_21
    mux_gp[4]      |= (1 << 21);
    ALTSEL0_SET (4, 21);
    ALTSEL1_CLEAR (4, 21);
#endif
#ifdef CONFIG_IFX_MUX_P4_20
    mux_gp[4]      |= (1 << 20);
    ALTSEL0_SET (4, 20);
    ALTSEL1_CLEAR (4, 20);
#endif
#ifdef CONFIG_IFX_MUX_P4_19
    mux_gp[4]      |= (1 << 19);
    ALTSEL0_SET (4, 19);
    ALTSEL1_CLEAR (4, 19);
#endif
#ifdef CONFIG_IFX_MUX_P4_18
    mux_gp[4]      |= (1 << 18);
    ALTSEL0_SET (4, 18);
    ALTSEL1_CLEAR (4, 18);
#endif
#ifdef CONFIG_IFX_MUX_P4_17
    mux_gp[4]      |= (1 << 17);
    ALTSEL0_SET (4, 17);
    ALTSEL1_CLEAR (4, 17);
#endif
#ifdef CONFIG_IFX_MUX_P4_16
    mux_gp[4]      |= (1 << 16);
    ALTSEL0_SET (4, 16);
    ALTSEL1_CLEAR (4, 16);
#endif
#ifdef CONFIG_IFX_MUX_P4_15
    mux_gp[4]      |= (1 << 15);
    ALTSEL0_SET (4, 15);
    ALTSEL1_CLEAR (4, 15);
#endif
#ifdef CONFIG_IFX_MUX_P4_14
    mux_gp[4]      |= (1 << 14);
    ALTSEL0_SET (4, 14);
    ALTSEL1_CLEAR (4, 14);
#endif
#ifdef CONFIG_IFX_MUX_P4_13
    mux_gp[4]      |= (1 << 13);
    ALTSEL0_SET (4, 13);
    ALTSEL1_CLEAR (4, 13);
#endif
#ifdef CONFIG_IFX_MUX_P4_12
    mux_gp[4]      |= (1 << 12);
    ALTSEL0_SET (4, 12);
    ALTSEL1_CLEAR (4, 12);
#endif
#ifdef CONFIG_IFX_MUX_P4_11
    mux_gp[4]      |= (1 << 11);
    ALTSEL0_SET (4, 11);
    ALTSEL1_CLEAR (4, 11);
#endif
#ifdef CONFIG_IFX_MUX_P4_10
    mux_gp[4]      |= (1 << 10);
    ALTSEL0_SET (4, 10);
    ALTSEL1_CLEAR (4, 10);
#endif
#ifdef CONFIG_IFX_MUX_P4_9
    mux_gp[4]      |= (1 << 9);
    ALTSEL0_SET (4, 9);
    ALTSEL1_CLEAR (4, 9);
#endif
#ifdef CONFIG_IFX_MUX_P3_8
    mux_gp[4]      |= (1 << 8);
    ALTSEL0_SET (4, 8);
    ALTSEL1_CLEAR (4, 8);
#endif
#ifdef CONFIG_IFX_MUX_P4_7
    mux_gp[4]      |= (1 << 7);
    ALTSEL0_SET (4, 7);
    ALTSEL1_CLEAR (4, 7);
#endif
#ifdef CONFIG_IFX_MUX_P4_6
    mux_gp[4]      |= (1 << 6);
    ALTSEL0_SET (4, 6);
    ALTSEL1_CLEAR (4, 6);
#endif
#ifdef CONFIG_IFX_MUX_P4_5
    mux_gp[4]      |= (1 << 5);
    ALTSEL0_SET (4, 5);
    ALTSEL1_CLEAR (4, 5);
#endif
#ifdef CONFIG_IFX_MUX_P4_4
    mux_gp[4]      |= (1 << 4);
    ALTSEL0_SET (4, 4);
    ALTSEL1_CLEAR (4, 4);
#endif
#ifdef CONFIG_IFX_MUX_P4_3
    mux_gp[4]      |= (1 << 3);
    ALTSEL0_SET (4, 3);
    ALTSEL1_CLEAR (4, 3);
#endif
#ifdef CONFIG_IFX_MUX_P4_2
    mux_gp[4]      |= (1 << 2);
    ALTSEL0_SET (4, 2);
    ALTSEL1_CLEAR (4, 2);
#endif
#ifdef CONFIG_IFX_MUX_P4_1
    mux_gp[4]      |= (1 << 1);
    ALTSEL0_SET (4, 1);
    ALTSEL1_CLEAR (4, 1);
#endif
#ifdef CONFIG_IFX_MUX_P4_0
    mux_gp[4]      |= (1 << 0);
    ALTSEL0_SET (4, 0);
    ALTSEL1_CLEAR (4, 0);
#endif

}

/**
 * Availability of SSC Ports.
 * This functions reports which SSC ports are available.
 *
 * \return mux_ssc bitfield of available SSC ports
 * \ingroup API
 */
u32 ifx_mux_ssc_pins(void)
{
    return mux_ssc;
}

/**
 * Availability of ASC1.
 * This functions reports whether support for ASC1 is enabled in the multiplexer
 * settings.
 *
 * \return 1 ASC1 is available
 * \return 0 ASC1 is not available.
 * \ingroup API
 */
u32 ifx_mux_asc1_pins(void)
{
#if defined(CONFIG_IFX_MUX_ASC1_TXD) && defined(CONFIG_IFX_MUX_ASC1_RXD)
    return 1;
#else
    return 0;
#endif
}

/**
 * Availability of ASC0.
 * This functions reports whether support for ASC0 is enabled in the multiplexer
 * settings.
 *
 * \return 1 ASC0 is available
 * \return 0 ASC0 is not available.
 * \ingroup API
 */
u32 ifx_mux_asc0_pins(void)
{
#if defined(CONFIG_IFX_MUX_ASC0_TXD) && defined(CONFIG_IFX_MUX_ASC0_RXD)
    return 1;
#else
    return 0;
#endif
}

/**
 * Number of EXIN pins.
 * This functions returns the bitfield of available external interrupt pins
 *
 * \return mux_exin bitfield of available external interrupts pins
 * \ingroup API
 */
u32 ifx_mux_exin_pins(void)
{
    return mux_exin;
}

/**
 * Retrive EXIN pin by EXIN number.
 *
 * \return mux_exin bitfield for special external interrupts number
 * \ingroup API
 */
u32 ifx_mux_exin_pin(int exin)
{
   u32 exin_mask = 0;
   switch (exin) {
      case  0:  exin_mask = IFX_MUX_BIT_EXINT0;   break;
      case  1:  exin_mask = IFX_MUX_BIT_EXINT1;   break;
      case  2:  exin_mask = IFX_MUX_BIT_EXINT2;   break;
      case  3:  exin_mask = IFX_MUX_BIT_EXINT3;   break;
      case  4:  exin_mask = IFX_MUX_BIT_EXINT4;   break;
      case  5:  exin_mask = IFX_MUX_BIT_EXINT5;   break;
      case  6:  exin_mask = IFX_MUX_BIT_EXINT6;   break;
      case  7:  exin_mask = IFX_MUX_BIT_EXINT7;   break;
      case  8:  exin_mask = IFX_MUX_BIT_EXINT8;   break;
      case  9:  exin_mask = IFX_MUX_BIT_EXINT9;   break;
      case 10:  exin_mask = IFX_MUX_BIT_EXINT10;  break;
      case 11:  exin_mask = IFX_MUX_BIT_EXINT11;  break;
      case 12:  exin_mask = IFX_MUX_BIT_EXINT12;  break;
      case 13:  exin_mask = IFX_MUX_BIT_EXINT13;  break;
      case 14:  exin_mask = IFX_MUX_BIT_EXINT14;  break;
      case 15:  exin_mask = IFX_MUX_BIT_EXINT15;  break;
      case 16:  exin_mask = IFX_MUX_BIT_EXINT16;  break;
   }
   return (mux_exin & exin_mask);
}

/**
 * Number of EBU chip select pins.
 * This functions returns the bitfield of available EBU chip select pins
 *
 * \return mux_ebucs bitfield of available EBU chip select pins
 * \ingroup API
 */
u32 ifx_mux_ebucs_pins(void)
{
    return mux_ebucs;
}

/**
 * Number of EBU address pins.
 * This functions returns the bitfield of available EBU address pins
 *
 * \return mux_ebuaddr bitfield of available EBU address pins
 * \ingroup API
 */
u32 ifx_mux_ebuaddr_pins(void)
{
    return mux_ebuaddr;
}

/**
 * Number of GP pins.
 * This functions returns the bitfield of available general purpose pins
 *
 * \return mux_gp bitfield of available general purpose pins
 * \ingroup API
 */
u32 ifx_mux_gp_pins(int port)
{
    return mux_gp[port];
}

/**
 * Initialize multiplex unit.
 * This function sets a static multiplex configuration
 * \ingroup Internal
 */
void ifx_mux_init(void)
{
    int i;

    ifx_mux_exin_init();
    ifx_mux_ssc_init();
    ifx_mux_gp_init();
    ifx_mux_ebu_init();
    ifx_mux_asc_init();
    ifx_mux_pcm_init();

   for (i=0; i<(sizeof(port_bases)/sizeof(port_bases[0])); i++)
   {
      ifx_port_reg* port = (volatile ifx_port_reg*)port_bases[i];

      if (0 == port_enabled[i])
         continue;

      if (mux_dir_cfgmask[i])
         port->dir = (port->dir & ~mux_dir_cfgmask[i]) | mux_dir[i];

      if (mux_puen_cfgmask[i])
         port->puen = (port->puen & ~mux_puen_cfgmask[i]) | mux_puen[i];

      if (mux_alt0_cfgmask[i])
         port->altsel0 = (port->altsel0 & ~mux_alt0_cfgmask[i]) | mux_altsel0[i];

      if (mux_alt1_cfgmask[i])
         port->altsel1 = (port->altsel1 & ~mux_alt1_cfgmask[i]) | mux_altsel1[i];
   }
}

/**
 * Create proc file output.
 * This function creates the output for the multiplexer proc file, by
 * reading all appropriate registers and variables and displaying the
 * content as a table.
 *
 * \param   buf         Buffer to write the string to
 * \param   start   not used (Linux internal)
 * \param   offset  not used (Linux internal)
 * \param   count   not used (Linux internal)
 * \param   eof         Set to 1 when all data is stored in buffer
 * \param   data    not used (Linux internal)
 * \return len  Lenght of data in buffer
 * \ingroup Internal
 */
int ifx_mux_read_procmem(char *buf, char **start, off_t offset, int count, int *eof, void *data)
{
   int len = 0;
   int t=0, i=0;

#define PROC_PRINT(fmt, args...) ({ \
   int c_len = 0; \
   c_len = snprintf (buf + len, count - len, fmt, ## args); \
   if (c_len <= 0) goto out; \
   if (c_len >= (count - len)) { \
      len += (count - len); \
      goto out; \
   } \
   len += c_len; \
   if (offset > 0) \
   { \
      if (len > offset) \
      { \
         len -= offset; \
         memmove (buf, buf + offset, len); \
         offset = 0; \
      } \
      else \
      { \
         offset -= len; \
         len = 0; \
      } \
   } \
})


   PROC_PRINT ("\nVINETIC-SVIP Multiplex Settings\n");
   PROC_PRINT ("              3         2         1         0\n");
   PROC_PRINT ("             10987654321098765432109876543210\n");
   PROC_PRINT ("             --------------------------------\n");
   for (i=0; i<(sizeof(port_bases)/sizeof(port_bases[0])); i++)
   {
      u32 bit=0;
      u32 reg_tmp,bits_tmp;

      ifx_port_reg* port = (ifx_port_reg*)port_bases[i];

      if (0 == port_enabled[i])
         continue;

      PROC_PRINT ("P%d.%-10s", i, "OUT:");
      reg_tmp = port->out;
      bit = 0x80000000;
      for (t = 31; t != -1; t--) {
         bits_tmp = (reg_tmp & bit) >> (t);
         PROC_PRINT ("%d",bits_tmp);
         bit = bit >> 1;
      }
      PROC_PRINT ("\n");

      PROC_PRINT ("P%d.%-10s", i, "IN:");
      reg_tmp = port->in;
      bit = 0x80000000;
      for (t = 31; t != -1; t--) {
         bits_tmp = (reg_tmp & bit) >> (t);
         PROC_PRINT ("%d",bits_tmp);
         bit = bit >> 1;
      }
      PROC_PRINT ("\n");

      PROC_PRINT ("P%d.%-10s", i, "DIR:");
      reg_tmp = port->dir;
      bit = 0x80000000;
      for (t = 31; t != -1; t--) {
         bits_tmp = (reg_tmp & bit) >> (t);
         PROC_PRINT ("%d",bits_tmp);
         bit = bit >> 1;
      }
      PROC_PRINT ("\n");

      PROC_PRINT ("P%d.%-10s", i, "PUEN:");
      reg_tmp = port->puen;
      bit = 0x80000000;
      for (t = 31; t != -1; t--) {
         bits_tmp = (reg_tmp & bit) >> (t);
         PROC_PRINT ("%d",bits_tmp);
         bit = bit >> 1;
      }
      PROC_PRINT ("\n");


      PROC_PRINT ("P%d.%-10s", i, "ALTSEL0:");
      reg_tmp = port->altsel0;
      bit = 0x80000000;
      for (t = 31; t != -1; t--) {
         bits_tmp = (reg_tmp & bit) >> (t);
         PROC_PRINT ("%d",bits_tmp);
         bit = bit >> 1;
      }
      PROC_PRINT ("\n");

      PROC_PRINT ("P%d.%-10s", i, "ALTSEL1:");
      reg_tmp = port->altsel1;
      bit = 0x80000000;
      for (t = 31; t != -1; t--) {
         bits_tmp = (reg_tmp & bit) >> (t);
         PROC_PRINT ("%d",bits_tmp);
         bit = bit >> 1;
      }
      PROC_PRINT ("\n\n");
   }

   PROC_PRINT ("ASC0  : %u\n",ifx_mux_asc0_pins());
   PROC_PRINT ("ASC1  : %u\n",ifx_mux_asc1_pins());
   PROC_PRINT ("EXINT : 0x%08x\n",ifx_mux_exin_pins());
   PROC_PRINT ("EBUAD : 0x%08x\n",ifx_mux_ebuaddr_pins());
   PROC_PRINT ("EBUCS : 0x%08x\n",ifx_mux_ebucs_pins());
   PROC_PRINT ("GPIO0 : 0x%08x\n",ifx_mux_gp_pins(0));
   PROC_PRINT ("GPIO1 : 0x%08x\n",ifx_mux_gp_pins(1));
   PROC_PRINT ("GPIO2 : 0x%08x\n",ifx_mux_gp_pins(2));
   PROC_PRINT ("GPIO3 : 0x%08x\n",ifx_mux_gp_pins(3));
   PROC_PRINT ("SSC   : 0x%08x\n",ifx_mux_ssc_pins());

   PROC_PRINT ("\n");

out:

   if (len < 0)
   {
      len = 0;
      *eof = 1;
   }
   else if (len < count)
   {
      *eof = 1;
   }
   else
   {
      len = count;
   }

   *start = buf;

   return len;
}


/**
 *  Initialize mux proc file.
 *  This function will create the proc file entry.
 *
 * \return 0    OK
 * \ingroup Internal
 */
int __init ifx_mux_init_proc(void)
{

    create_proc_read_entry("driver/ifx_mux", 0, NULL, ifx_mux_read_procmem, NULL);
    return 0;
}

module_init(ifx_mux_init_proc);

EXPORT_SYMBOL(ifx_mux_asc0_pins);
EXPORT_SYMBOL(ifx_mux_asc1_pins);
EXPORT_SYMBOL(ifx_mux_ebuaddr_pins);
EXPORT_SYMBOL(ifx_mux_ebucs_pins);
EXPORT_SYMBOL(ifx_mux_exin_pins);
EXPORT_SYMBOL(ifx_mux_gp_pins);
EXPORT_SYMBOL(ifx_mux_ssc_pins);
EXPORT_SYMBOL(ifx_mux_exin_pin);

