/************************************************************************
 *
 * Copyright (c) 2007
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/

/* Group definitions for Doxygen */
/** \addtogroup API API-Functions */
/** \addtogroup Internal Internally used functions */

#include <linux/module.h>
#include <linux/types.h>
#include <linux/kernel.h>
#include <linux/proc_fs.h>
#include <linux/init.h>
#include <asm/addrspace.h>
#include <asm/ifx/ifx_reg_base.h>
#include <asm/ifx/ifx_reg_sys1.h>
#include <asm/ifx/ifx_reg_sys2.h>

//#define PMS_DEBUG

/**
 * Initialize power managment unit.
 * This function sets a static power managment configuration
 * \ingroup Internal  
 */
void ifx_pms_init(void)
{
    u32 sys1 = 0;
    u32 sys2 = 0;
	u32 reg;

#ifndef CONFIG_IFX_PMS_USE_BOOT_SETTINGS
	/* SYS1 CLKENR register content... */
#ifdef CONFIG_IFX_PMS_CLKENR_L2C
	sys1 |= IFX_SYS1_CLKENR_L2C;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_DDR2
	sys1 |= IFX_SYS1_CLKENR_DDR2;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_SMI2
	sys1 |= IFX_SYS1_CLKENR_SMI2;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_SMI1
	sys1 |= IFX_SYS1_CLKENR_SMI1;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_SMI0
	sys1 |= IFX_SYS1_CLKENR_SMI0;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_FMI0
	sys1 |= IFX_SYS1_CLKENR_FMI0;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_PORT0
	sys1 |= IFX_SYS1_CLKENR_PORT0;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_PCM3
	sys1 |= IFX_SYS1_CLKENR_PCM3;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_PCM2
	sys1 |= IFX_SYS1_CLKENR_PCM2;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_PCM1
	sys1 |= IFX_SYS1_CLKENR_PCM1;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_PCM0
	sys1 |= IFX_SYS1_CLKENR_PCM0;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_ASC1
	sys1 |= IFX_SYS1_CLKENR_ASC1;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_ASC0
	sys1 |= IFX_SYS1_CLKENR_ASC0;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_SSC2
	sys1 |= IFX_SYS1_CLKENR_SSC2;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_SSC1
	sys1 |= IFX_SYS1_CLKENR_SSC1;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_SSC0
	sys1 |= IFX_SYS1_CLKENR_SSC0;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_GPTC
	sys1 |= IFX_SYS1_CLKENR_GPTC;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_DMA
	sys1 |= IFX_SYS1_CLKENR_DMA;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_FSCT
	sys1 |= IFX_SYS1_CLKENR_FSCT;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_ETHSW
	sys1 |= IFX_SYS1_CLKENR_ETHSW;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_EBU
	sys1 |= IFX_SYS1_CLKENR_EBU;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_TRNG
	sys1 |= IFX_SYS1_CLKENR_TRNG;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_DEU
	sys1 |= IFX_SYS1_CLKENR_DEU;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_PORT3
	sys1 |= IFX_SYS1_CLKENR_PORT3;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_PORT2
	sys1 |= IFX_SYS1_CLKENR_PORT2;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_PORT1
	sys1 |= IFX_SYS1_CLKENR_PORT1;
#endif

	/* SYS2 CLKENR register content... */
#ifdef CONFIG_IFX_PMS_CLKENR_PORT4
	sys2 |= IFX_SYS2_CLKENR_PORT4;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_HWSYNC
	sys2 |= IFX_SYS2_CLKENR_HWSYNC;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_MBS
	sys2 |= IFX_SYS2_CLKENR_MBS;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_SWINT
	sys2 |= IFX_SYS2_CLKENR_SWINT;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_HWACC3
	sys2 |= IFX_SYS2_CLKENR_HWACC3;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_HWACC2
	sys2 |= IFX_SYS2_CLKENR_HWACC2;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_HWACC1
	sys2 |= IFX_SYS2_CLKENR_HWACC1;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_HWACC0
	sys2 |= IFX_SYS2_CLKENR_HWACC0;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_SIF7
	sys2 |= IFX_SYS2_CLKENR_SIF7;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_SIF6
	sys2 |= IFX_SYS2_CLKENR_SIF6;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_SIF5
	sys2 |= IFX_SYS2_CLKENR_SIF5;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_SIF4
	sys2 |= IFX_SYS2_CLKENR_SIF4;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_SIF3
	sys2 |= IFX_SYS2_CLKENR_SIF3;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_SIF2
	sys2 |= IFX_SYS2_CLKENR_SIF2;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_SIF1
	sys2 |= IFX_SYS2_CLKENR_SIF1;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_SIF0
	sys2 |= IFX_SYS2_CLKENR_SIF0;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_DFEV7
	sys2 |= IFX_SYS2_CLKENR_DFEV7;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_DFEV6
	sys2 |= IFX_SYS2_CLKENR_DFEV6;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_DFEV5
	sys2 |= IFX_SYS2_CLKENR_DFEV5;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_DFEV4
	sys2 |= IFX_SYS2_CLKENR_DFEV4;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_DFEV3
	sys2 |= IFX_SYS2_CLKENR_DFEV3;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_DFEV2
	sys2 |= IFX_SYS2_CLKENR_DFEV2;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_DFEV1
	sys2 |= IFX_SYS2_CLKENR_DFEV1;
#endif
#ifdef CONFIG_IFX_PMS_CLKENR_DFEV0
	sys2 |= IFX_SYS2_CLKENR_DFEV0;
#endif

#ifdef PMS_DEBUG
	asm("sdbbp;nop;");
    printk("IFX_SYS1_CLKENR = 0x%08x\n", sys1);
    printk("IFX_SYS2_CLKENR = 0x%08x\n", sys2);
	asm("sdbbp;nop;");
#endif

    *IFX_SYS1_CLKENR = sys1;
	asm("sync;");
    *IFX_SYS2_CLKENR = sys2;
	asm("sync;");

#endif

#ifdef PMS_DEBUG
	/* TODO: crashes at this point - why?? just reading and printk... */
	asm("sdbbp;nop;");
	reg = *IFX_SYS1_CLKSR;
    printk("IFX_SYS1_CLKSR = 0x%08x\n", reg);
	reg = *IFX_SYS2_CLKSR;
    printk("IFX_SYS2_CLKSR = 0x%08x\n", reg);
	asm("sdbbp;nop;");
#endif
}

/**
 * Create proc file output.
 * This function creates the output for the pms proc file, by 
 * reading all appropriate registers and variables and displaying the 
 * content as a table.
 *
 * \param   buf         Buffer to write the string to
 * \param   start   not used (Linux internal)
 * \param   offset  not used (Linux internal)
 * \param   count   not used (Linux internal)
 * \param   eof         Set to 1 when all data is stored in buffer
 * \param   data    not used (Linux internal)
 * \return len  Lenght of data in buffer 
 * \ingroup Internal  
 */
int ifx_pms_read_procmem(char *buf, char **start, off_t offset, int count, int *eof, void *data)
{
    long    len = 0;
    int     t=0;
    u32 bit=0;
    u32 reg_tmp,bits_tmp;

    len = sprintf(buf,"\nSVIP PMS Settings\n");
#ifdef CONFIG_IFX_PMS_USE_BOOT_SETTINGS
    len=len+sprintf(buf+len, "(using boot settings)\n");
#endif
    len=len+sprintf(buf+len,"              3         2         1         0\n");
    len=len+sprintf(buf+len,"            210987654321098765432109876543210\n");
    len=len+sprintf(buf+len,"---------------------------------------------\n");
    len=len+sprintf(buf+len,"SYS1_CLKSR: ");
    reg_tmp = *IFX_SYS1_CLKSR;
    bit = 0x80000000;
    for (t = 31; t != -1; t--) {
        bits_tmp = (reg_tmp & bit) >> t;
        len=len+sprintf(buf+len,"%d",bits_tmp);
        bit = bit >> 1;
    }
    len=len+sprintf(buf+len,"\n\n");
    len=len+sprintf(buf+len,"SYS2_CLKSR: ");
    reg_tmp = *IFX_SYS2_CLKSR;
    bit = 0x80000000;
    for (t = 31; t != -1; t--) {
        bits_tmp = (reg_tmp & bit) >> t;
        len=len+sprintf(buf+len,"%d",bits_tmp);
        bit = bit >> 1;
    }
    len=len+sprintf(buf+len,"\n\n");

    *eof = 1;   // No more data available

    return len;
}


/**
 *  Initialize pms proc file.
 *  This function will create the proc file entry.
 *
 * \return 0    OK
 * \ingroup Internal  
 */
int __init ifx_pms_init_proc(void)
{
    
    create_proc_read_entry("driver/ifx_pms", 0, NULL, ifx_pms_read_procmem, NULL);
    return 0;
}

module_init(ifx_pms_init_proc);

//EXPORT_SYMBOL(ifx_pms_status);

