/*
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 *   Copyright (C) 2007 John Crispin <blogic@openwrt.org>
 */

#include <linux/kernel.h>
#include <linux/pm.h>
#include <linux/io.h>
#include <linux/delay.h>
#include <linux/sched.h>
#include <asm/reboot.h>
#include <asm/system.h>
#include <asm/ifxmips/ifxmips.h>
#ifdef CONFIG_SVIP
#include <asm/ifx/ifx_reg_base.h>
#include <asm/ifx/ifx_reg_sys1.h>
#include <asm/ifx/ifx_reg_boot.h>
#if defined(CONFIG_IFX_EASY336) || defined(CONFIG_IFX_EASY33016) || defined(CONFIG_IFX_EASY3201EVS)
#include <asm/ifx/ifx_reg_ebu.h>
#endif
#include <asm/ifx/ifx_reg_dma.h>
#endif

#ifdef CONFIG_SVIP
extern void switchip_reset(void);
#if defined(CONFIG_IFX_EASY336) || defined(CONFIG_IFX_EASY33016) || defined(CONFIG_IFX_EASY3201EVS)
#define CPLD_CMDREG3  ((volatile unsigned char*)(KSEG1 + 0x120000f3))
#endif
#endif

static void ifxmips_machine_restart(char *command)
{
	printk(KERN_NOTICE "System restart\n");
	local_irq_disable();

#ifndef CONFIG_SVIP /* non SVIP systems */
	ifxmips_w32(ifxmips_r32(IFXMIPS_RCU_RST) | IFXMIPS_RCU_RST_ALL,
		IFXMIPS_RCU_RST);
#else /* SVIP systems */
#if defined(CONFIG_IFX_EASY336) || defined (CONFIG_IFX_EASY33016)
   /* We just use the CPLD function to reset the entire system as a
      workaround for the switch reset problem */
   local_irq_disable();
   *IFX_EBU_ADDR_SEL_2 = 0x120000f1;
   *IFX_EBU_CON_2 = 0x404027ff;
#if CONFIG_IFX_EASY336
   /* set bit 0 to reset SVIP */
   *CPLD_CMDREG3 = (1<<0);
#else
   /* set bit 7 to reset SVIP, set bit 3 to reset xT */
   *CPLD_CMDREG3 = (1<<7) | (1<<3);
#endif
#else
   yield();
   *BOOT_RVEC(0) = 0;
   *IFX_SYS1_RREQR   = 0x00043F3E; /* reset all except PER, SUBSYS and CPU0 */
   *IFX_SYS1_RRLSR   = 0x00000100; /* release WDT0 reset */
   *IFX_SYS1_CLKCLR  = ~0x0c000040; /* restore reset value for clock enables */
	*IFX_SYS1_RBTR    = 0x00030001; /* reset SUBSYS (incl. DDR2) and CPU0 */
#endif
#endif /* CONFIG_SVIP */

   for (;;)
      ;
}

static void ifxmips_machine_halt(void)
{
	printk(KERN_NOTICE "System halted.\n");
	local_irq_disable();
	for (;;)
		;
}

static void ifxmips_machine_power_off(void)
{
	printk(KERN_NOTICE "Please turn off the power now.\n");
	local_irq_disable();
	for (;;)
		;
}

void ifxmips_reboot_setup(void)
{
	_machine_restart = ifxmips_machine_restart;
	_machine_halt = ifxmips_machine_halt;
	pm_power_off = ifxmips_machine_power_off;
}
