/*
 *   This program is free software; you can redistribute it and/or modify
 *   it under the terms of the GNU General Public License as published by
 *   the Free Software Foundation; either version 2 of the License, or
 *   (at your option) any later version.
 *
 *   This program is distributed in the hope that it will be useful,
 *   but WITHOUT ANY WARRANTY; without even the implied warranty of
 *   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *   GNU General Public License for more details.
 *
 *   You should have received a copy of the GNU General Public License
 *   along with this program; if not, write to the Free Software
 *   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307, USA.
 *
 *   Copyright (C) 2004 peng.liu@infineon.com
 *   Copyright (C) 2007 John Crispin <blogic@openwrt.org>
 */

#include <linux/init.h>


#include <linux/cpu.h>

#include <asm/time.h>
#include <asm/traps.h>
#include <asm/irq.h>
#include <asm/bootinfo.h>

#include <asm/ifxmips/ifxmips.h>
#include <asm/ifxmips/ifxmips_irq.h>
#include <asm/ifxmips/ifxmips_pmu.h>
#include <asm/ifxmips/ifxmips_cgu.h>
#include <asm/ifxmips/ifxmips_prom.h>
#ifdef CONFIG_SVIP
#include <asm/ifx/ifx_reg_sys1.h>
#if defined(CONFIG_IFX_EASY336) || defined(CONFIG_IFX_EASY33016) || defined(CONFIG_IFX_EASY3201EVS) || defined(CONFIG_IFX_BOARD_GS_GXW)
#include <asm/ifx/svip_reg_base.h>
#include <asm/ifx/ifx_reg_ebu.h>
#endif
extern void ifx_pms_init(void);
extern void ifx_mux_init(void);
extern void ifx_reboot_setup(void);
extern unsigned int ifxmips_get_cpu_hz(void);
#if defined(CONFIG_IFX_EASY336) || defined(CONFIG_IFX_EASY33016) || defined(CONFIG_IFX_EASY3201EVS) || defined(CONFIG_IFX_BOARD_GS_GXW)
void ifx_ebu_setup(void);
#endif
#endif /* CONFIG_SVIP */

static unsigned int r4k_offset;
static unsigned int r4k_cur;

/* required in arch/mips/kernel/kspd.c */
unsigned long cpu_khz;

extern void ifxmips_reboot_setup(void);

unsigned int ifxmips_get_cpu_ver(void)
{
	return (ifxmips_r32(IFXMIPS_MPS_CHIPID) & 0xF0000000) >> 28;
}
EXPORT_SYMBOL(ifxmips_get_cpu_ver);

static inline u32 ifxmips_get_counter_resolution(void)
{
	u32 res;
	__asm__ __volatile__(
		".set   push\n"
		".set   mips32r2\n"
		".set   noreorder\n"
		"rdhwr  %0, $3\n"
		"ehb\n"
		".set pop\n"
		: "=&r" (res)
		: /* no input */
		: "memory");
	instruction_hazard();
	return res;
}

void __init plat_time_init(void)
#ifndef CONFIG_SVIP
{
	mips_hpt_frequency = ifxmips_get_cpu_hz() / ifxmips_get_counter_resolution();
	r4k_cur = (read_c0_count() + r4k_offset);
	write_c0_compare(r4k_cur);

	ifxmips_pmu_enable(IFXMIPS_PMU_PWDCR_GPT | IFXMIPS_PMU_PWDCR_FPI);
	ifxmips_w32(0x100, IFXMIPS_GPTU_GPT_CLC); /* set clock divider to 1 */
	cpu_khz = ifxmips_get_cpu_hz();
}
#else /* CONFIG_SVIP */
{
   mips_hpt_frequency = ifxmips_get_cpu_hz() / ifxmips_get_counter_resolution();
   write_c0_count(0);
   write_c0_compare(mips_hpt_frequency / HZ);
   cpu_khz = ifxmips_get_cpu_hz() / 1000;

   enable_irq(MIPS_CPU_TIMER_IRQ);
}
#endif

void __init plat_mem_setup(void)
{
	u32 status;
	prom_printf("This %s system has a cpu rev of %d\n", get_system_type(), ifxmips_get_cpu_ver());

	/* make sure to have no "reverse endian" for user mode! */
	status = read_c0_status();
	status &= (~(1<<25));
	write_c0_status(status);

	ifxmips_reboot_setup();
#ifdef CONFIG_SVIP
   ifx_pms_init();
   ifx_mux_init();
#if defined(CONFIG_IFX_EASY336) || defined(CONFIG_IFX_EASY33016) || defined(CONFIG_IFX_EASY3201EVS) || defined(CONFIG_IFX_BOARD_GS_GXW)
    ifx_ebu_setup();
#endif
#endif

	ioport_resource.start = IOPORT_RESOURCE_START;
	ioport_resource.end = IOPORT_RESOURCE_END;
	iomem_resource.start = IOMEM_RESOURCE_START;
	iomem_resource.end = IOMEM_RESOURCE_END;
}


#if (LINUX_VERSION_CODE < KERNEL_VERSION(2, 6, 21))
static irqreturn_t mips_timer_interrupt(int irq, void *dev_id)
{
	int cpu = smp_processor_id();
#ifdef CONFIG_MIPS_MT_SMTC
	/*
	 *  In an SMTC system, one Count/Compare set exists per VPE.
	 *  Which TC within a VPE gets the interrupt is essentially
	 *  random - we only know that it shouldn't be one with
	 *  IXMT set. Whichever TC gets the interrupt needs to
	 *  send special interprocessor interrupts to the other
	 *  TCs to make sure that they schedule, etc.
	 *
	 *  That code is specific to the SMTC kernel, not to
	 *  the a particular platform, so it's invoked from
	 *  the general MIPS timer_interrupt routine.
	 */
	/*
	 * We could be here due to timer interrupt,
	 * perf counter overflow, or both.
	 */
	if (read_c0_cause() & (1 << 30)) {
		/*
		 * There are things we only want to do once per tick
		 * in an "MP" system.   One TC of each VPE will take
		 * the actual timer interrupt.  The others will get
		 * timer broadcast IPIs. We use whoever it is that takes
		 * the tick on VPE 0 to run the full timer_interrupt().
		 */
		if (cpu_data[cpu].vpe_id == 0) {
			timer_interrupt(irq, NULL);
		} else {
			write_c0_compare(read_c0_count() + r4k_offset);
			local_timer_interrupt(irq, dev_id);
		}
		smtc_timer_broadcast(cpu_data[cpu].vpe_id);
	}
#else /* CONFIG_MIPS_MT_SMTC */
	int r2 = cpu_has_mips_r2;

	if (r2 && ((read_c0_cause() & (1 << 30)) == 0))
		goto out;

	if (cpu == 0) {
		/*
		 * CPU 0 handles the global timer interrupt job and process
		 * accounting resets count/compare registers to trigger next
		 * timer int.
		 */
		timer_interrupt(irq, NULL);
	} else {
		/* Everyone else needs to reset the timer int here as
		 ll_local_timer_interrupt doesn't */
		/*
		 * FIXME: need to cope with counter underflow.
		 * More support needs to be added to kernel/time for
		 * counter/timer interrupts on multiple CPU's
		 */
		write_c0_compare(read_c0_count() + r4k_offset);
		/*
		 * Other CPUs should do profiling and process accounting
		 */
		local_timer_interrupt(irq, dev_id);
	}
out:
#endif /* CONFIG_MIPS_MT_SMTC */
	return IRQ_HANDLED;
}


/*
 * The CPU counter for System timer, set to HZ
 * GPTU Timer 6 for high resolution timer, set to hr_time_resolution
 * Also misuse this routine to print out the CPU type and clock.
 */
void __init plat_timer_setup(struct irqaction *irq)
{
	plat_time_init();

	/* cpu counter for timer interrupts */
#ifdef CONFIG_MIPS_MT_SMTC
	irq->handler = mips_timer_interrupt;/* Or use timer_irqaction->timer_interrupt defined in arch/mips/kernel/time.c */
	setup_irq_smtc(MIPS_CPU_TIMER_IRQ, irq, 0x8000); /* irq_hwmask is not used in our platforms */
#else
	setup_irq(MIPS_CPU_TIMER_IRQ, irq);

#ifndef CONFIG_SVIP
	/* to generate the first timer interrupt */
	write_c0_compare(read_c0_count() + r4k_offset);
#else /* CONFIG_SVIP */
   /* Enable timer interrupt in IM5 */
   enable_irq(MIPS_CPU_TIMER_IRQ);

   write_c0_count(0);
   /* to generate the first timer interrupt */
   r4k_cur = (read_c0_count() + r4k_offset);
   write_c0_compare(r4k_cur);

   set_c0_status(IE_IRQ0 | IE_IRQ1 | IE_IRQ2 | IE_IRQ3 | IE_IRQ4 | IE_IRQ5);
#endif
#endif
}
#endif

#if defined(CONFIG_IFX_EASY336) || defined(CONFIG_IFX_EASY33016) || defined(CONFIG_IFX_EASY3201EVS) || defined(CONFIG_IFX_BOARD_GS_GXW)
void ifx_ebu_setup(void)
{
   *IFX_EBU_ADDR_SEL_2 = 0x120000f1;
   *IFX_EBU_CON_2 = /*0x404027ff;*/
               IFX_EBU_CON_0_ADSWP | IFX_EBU_CON_0_SETUP |
               IFX_EBU_CON_0_BCGEN_VAL(0x02) | IFX_EBU_CON_0_WAITWRC_VAL(7) |
               IFX_EBU_CON_0_WAITRDC_VAL(3) | IFX_EBU_CON_0_HOLDC_VAL(3) |
               IFX_EBU_CON_0_RECOVC_VAL(3) | IFX_EBU_CON_0_CMULT_VAL(3);

}
#endif

