/******************************************************************************
     Copyright (c) 2007, Infineon Technologies.  All rights reserved.

                               No Warranty
   Because the program is licensed free of charge, there is no warranty for
   the program, to the extent permitted by applicable law.  Except when
   otherwise stated in writing the copyright holders and/or other parties
   provide the program "as is" without warranty of any kind, either
   expressed or implied, including, but not limited to, the implied
   warranties of merchantability and fitness for a particular purpose. The
   entire risk as to the quality and performance of the program is with
   you.  should the program prove defective, you assume the cost of all
   necessary servicing, repair or correction.

   In no event unless required by applicable law or agreed to in writing
   will any copyright holder, or any other party who may modify and/or
   redistribute the program as permitted above, be liable to you for
   damages, including any general, special, incidental or consequential
   damages arising out of the use or inability to use the program
   (including but not limited to loss of data or data being rendered
   inaccurate or losses sustained by you or third parties or a failure of
   the program to operate with any other programs), even if such holder or
   other party has been advised of the possibility of such damages.
 ******************************************************************************
   Module      : spi_setup.c
   Date        : 2010-04-30
   Description : Basic setup of SPI bus
   Remarks: Jevgenijs.Grigorjevs@lantiq.com

 *****************************************************************************/
#include <linux/module.h>
#include <linux/init.h>

#include <asm/ifx/ifx_reg_base.h>
#include <linux/platform_device.h>
#include <asm/ifxmips/ifxmips_irq.h>
#include <linux/spi/spi.h>
#include <linux/mtd/mtd.h>
#include <linux/mtd/partitions.h>
#include <linux/spi/flash.h>
#include <linux/err.h>

static int __init easy336_ssc_init (int bus_num, unsigned long base, int irq_rx,
   int irq_tx, int irq_err, int irq_frm)
{
   struct platform_device *pdev;

   struct resource res[] = {
         {
            .name   = "regs",
            .start  = base,
            .end    = base + 0x20 - 1,
            .flags  = IORESOURCE_MEM,
         }, {
            .name   = "rx",
            .start  = irq_rx,
            .flags  = IORESOURCE_IRQ,
         }, {
            .name   = "tx",
            .start  = irq_tx,
            .flags  = IORESOURCE_IRQ,
         }, {
            .name   = "err",
            .start  = irq_err,
            .flags  = IORESOURCE_IRQ,
         }, {
            .name   = "frm",
            .start  = irq_frm,
            .flags  = IORESOURCE_IRQ,
         },
      };

   pdev = platform_device_register_simple("ifx_ssc",
      bus_num, res, ARRAY_SIZE(res));

   if (IS_ERR(pdev))
      return PTR_ERR(pdev);

   return 0;
}

/* If the SPI flash MTD partitioning is given per Linux command line,
 * the following partition settings are ignored.
 */
static struct mtd_partition easy336_sflash_partitions[] = {
      {
         .name		= "SPI flash",
         .size		= MTDPART_SIZ_FULL,
         .offset		= 0,
      },
   };

static struct flash_platform_data easy336_sflash_data = {
      .name = "m25p32",
      .parts = (void *)&easy336_sflash_partitions,
      .nr_parts = ARRAY_SIZE(easy336_sflash_partitions),
      .type = "m25p32",
   };

static struct spi_board_info  bdinfo[] __initdata = {
      {
         .modalias = "m25p80",
         .platform_data = &easy336_sflash_data,
         .mode = SPI_MODE_0,
         .irq = -1,
         .max_speed_hz = 25000000,
         .bus_num = 0,
         .chip_select = 0,
      },
      {
#ifdef CONFIG_SPI_SPIDEV
         .modalias = "spidev",
#else
#ifdef CONFIG_IFX_EASY3201EVS
         .modalias = "duslicxt",
#else
         .modalias = "xt16",
#endif /* ifdef CONFIG_IFX_EASY3201EVS */
#endif /* CONFIG_SPI_SPIDEV */
         .mode = SPI_MODE_3,
         .irq = INT_NUM_IM5_IRL0 + 28,
#ifdef CONFIG_IFX_EASY3201EVS
         .max_speed_hz = 2048000,
#else
         .max_speed_hz = 1000000,
#endif /* ifdef CONFIG_IFX_EASY3201EVS */
         .bus_num = 0,
         .chip_select = 1,
      },
#ifndef CONFIG_IFX_EASY3201EVS
      {
#ifdef CONFIG_SPI_SPIDEV
         .modalias = "spidev",
#else
         .modalias = "xt16",
#endif /* CONFIG_SPI_SPIDEV */
         .mode = SPI_MODE_3,
         .irq = INT_NUM_IM5_IRL0 + 19,
         .max_speed_hz = 1000000,
         .bus_num = 0,
         .chip_select = 2,
      },
#endif /* CONFIG_IFX_EASY3201EVS */
      {
         .modalias = "loop",
         .mode = SPI_MODE_0 | SPI_LOOP,
         .irq = -1,
         .max_speed_hz = 10000000,
         .bus_num = 0,
         .chip_select = 3,
      },
      {
         .modalias = "uc1701fb",
         .mode = SPI_MODE_3,
         .irq = -1,
         .max_speed_hz = 250000,
         .bus_num = 0,
         .chip_select = 5,
//       .platform_data = &uc1698_platdata,

      },
   };

static int __init spi_board_init(void)
{
   spi_register_board_info(bdinfo, ARRAY_SIZE (bdinfo));

   easy336_ssc_init (0, IFX_SSC0_BASE, INT_NUM_IM1_IRL0 + 6,
      INT_NUM_IM1_IRL0 + 7, INT_NUM_IM1_IRL0 + 8, INT_NUM_IM1_IRL0 + 9);

   easy336_ssc_init (1, IFX_SSC1_BASE, INT_NUM_IM1_IRL0 + 10,
      INT_NUM_IM1_IRL0 + 11, INT_NUM_IM1_IRL0 + 12, INT_NUM_IM1_IRL0 + 13);

   return 0;
}

__initcall(spi_board_init);

EXPORT_SYMBOL(spi_board_init);


