/******************************************************************************

                            Copyright (c) 2007
                         Infineon Technologies AG
               St. Martin Strasse 53; 81669 Munich, Germany

  Any use of this Software is subject to the conclusion of a respective 
  License Agreement. Without such a License Agreement no rights to the
  Software are granted.

******************************************************************************/


#ifndef __IFX_PMS_H
#define __IFX_PMS_H


/*******************************************************************************
 * PMS
 ******************************************************************************/
typedef struct _ifx_pms_reg {
	volatile unsigned long  sr0;  /*  0x0000 */
	volatile unsigned long  clr0;  /*  0x0004 */
	volatile unsigned long  enr0;  /*  0x0008 */
	volatile unsigned long  reserved0;  /*  0x0c */
	volatile unsigned long  sr1;  /*  0x0010 */
	volatile unsigned long  clr1;  /*  0x0014 */
	volatile unsigned long  enr1;  /*  0x0018 */
	volatile unsigned long  reserved1;  /*  0x1c */
	volatile unsigned long  cpu0srssr;  /*  0x0020 */
	volatile unsigned long  reserved2[5];  /*  0x24 */
	volatile unsigned long  cpu0wrssr;  /*  0x0038 */
	volatile unsigned long  reserved3[5];  /*  0x3c */
	volatile unsigned long  perwrssr;  /*  0x0050 */
} ifx_pms_reg;

/*******************************************************************************
 * PMS
 ******************************************************************************/
#define IFX_PMS_SR0   ((volatile unsigned int*)(IFX_PMS_BASE + 0x0000))
#define IFX_PMS_CLR0   ((volatile unsigned int*)(IFX_PMS_BASE + 0x0004))
#define IFX_PMS_ENR0   ((volatile unsigned int*)(IFX_PMS_BASE + 0x0008))
#define IFX_PMS_SR1   ((volatile unsigned int*)(IFX_PMS_BASE + 0x0010))
#define IFX_PMS_CLR1   ((volatile unsigned int*)(IFX_PMS_BASE + 0x0014))
#define IFX_PMS_ENR1   ((volatile unsigned int*)(IFX_PMS_BASE + 0x0018))
#define IFX_PMS_CPU0SRSSR   ((volatile unsigned int*)(IFX_PMS_BASE + 0x0020))
#define IFX_PMS_CPU0WRSSR   ((volatile unsigned int*)(IFX_PMS_BASE + 0x0038))
#define IFX_PMS_PERWRSSR   ((volatile unsigned int*)(IFX_PMS_BASE + 0x0050))

/*******************************************************************************
 * PMS Status Register 0
 ******************************************************************************/

/* CPU5 Status (5) */
#define IFX_PMS_SR0_CPU5   (0x1 << 5)
#define IFX_PMS_SR0_CPU5_GET(val)   ((((val) & IFX_PMS_SR0_CPU5) >> 5) & 0x1)
/* CPU4 Status (4) */
#define IFX_PMS_SR0_CPU4   (0x1 << 4)
#define IFX_PMS_SR0_CPU4_GET(val)   ((((val) & IFX_PMS_SR0_CPU4) >> 4) & 0x1)
/* CPU3 Status (3) */
#define IFX_PMS_SR0_CPU3   (0x1 << 3)
#define IFX_PMS_SR0_CPU3_GET(val)   ((((val) & IFX_PMS_SR0_CPU3) >> 3) & 0x1)
/* CPU2 Status (2) */
#define IFX_PMS_SR0_CPU2   (0x1 << 2)
#define IFX_PMS_SR0_CPU2_GET(val)   ((((val) & IFX_PMS_SR0_CPU2) >> 2) & 0x1)
/* CPU1 Status (1) */
#define IFX_PMS_SR0_CPU1   (0x1 << 1)
#define IFX_PMS_SR0_CPU1_GET(val)   ((((val) & IFX_PMS_SR0_CPU1) >> 1) & 0x1)
/* CPU0 Status (0) */
#define IFX_PMS_SR0_CPU0   (0x1)
#define IFX_PMS_SR0_CPU0_GET(val)   ((((val) & IFX_PMS_SR0_CPU0) >> 0) & 0x1)

/*******************************************************************************
 * PMS Clear Request Register 0
 ******************************************************************************/

/* CPU5 Clear Request (5) */
#define IFX_PMS_CLR0_CPU5   (0x1 << 5)
#define IFX_PMS_CLR0_CPU5_VAL(val)   (((val) & 0x1) << 5)
#define IFX_PMS_CLR0_CPU5_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR0_CPU5) | (val) & 1) << 5)
/* CPU4 Clear Request (4) */
#define IFX_PMS_CLR0_CPU4   (0x1 << 4)
#define IFX_PMS_CLR0_CPU4_VAL(val)   (((val) & 0x1) << 4)
#define IFX_PMS_CLR0_CPU4_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR0_CPU4) | (val) & 1) << 4)
/* CPU3 Clear Request (3) */
#define IFX_PMS_CLR0_CPU3   (0x1 << 3)
#define IFX_PMS_CLR0_CPU3_VAL(val)   (((val) & 0x1) << 3)
#define IFX_PMS_CLR0_CPU3_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR0_CPU3) | (val) & 1) << 3)
/* CPU2 Clear Request (2) */
#define IFX_PMS_CLR0_CPU2   (0x1 << 2)
#define IFX_PMS_CLR0_CPU2_VAL(val)   (((val) & 0x1) << 2)
#define IFX_PMS_CLR0_CPU2_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR0_CPU2) | (val) & 1) << 2)
/* CPU1 Clear Request (1) */
#define IFX_PMS_CLR0_CPU1   (0x1 << 1)
#define IFX_PMS_CLR0_CPU1_VAL(val)   (((val) & 0x1) << 1)
#define IFX_PMS_CLR0_CPU1_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR0_CPU1) | (val) & 1) << 1)
/* CPU0 Clear Request (0) */
#define IFX_PMS_CLR0_CPU0   (0x1)
#define IFX_PMS_CLR0_CPU0_VAL(val)   (((val) & 0x1) << 0)
#define IFX_PMS_CLR0_CPU0_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR0_CPU0) | (val) & 1) << 0)

/*******************************************************************************
 * PMS Enable Request Register 0
 ******************************************************************************/

/* CPU5 Enable Request (5) */
#define IFX_PMS_ENR0_CPU5   (0x1 << 5)
#define IFX_PMS_ENR0_CPU5_VAL(val)   (((val) & 0x1) << 5)
#define IFX_PMS_ENR0_CPU5_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR0_CPU5) | (val) & 1) << 5)
/* CPU4 Enable Request (4) */
#define IFX_PMS_ENR0_CPU4   (0x1 << 4)
#define IFX_PMS_ENR0_CPU4_VAL(val)   (((val) & 0x1) << 4)
#define IFX_PMS_ENR0_CPU4_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR0_CPU4) | (val) & 1) << 4)
/* CPU3 Enable Request (3) */
#define IFX_PMS_ENR0_CPU3   (0x1 << 3)
#define IFX_PMS_ENR0_CPU3_VAL(val)   (((val) & 0x1) << 3)
#define IFX_PMS_ENR0_CPU3_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR0_CPU3) | (val) & 1) << 3)
/* CPU2 Enable Request (2) */
#define IFX_PMS_ENR0_CPU2   (0x1 << 2)
#define IFX_PMS_ENR0_CPU2_VAL(val)   (((val) & 0x1) << 2)
#define IFX_PMS_ENR0_CPU2_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR0_CPU2) | (val) & 1) << 2)
/* CPU1 Enable Request (1) */
#define IFX_PMS_ENR0_CPU1   (0x1 << 1)
#define IFX_PMS_ENR0_CPU1_VAL(val)   (((val) & 0x1) << 1)
#define IFX_PMS_ENR0_CPU1_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR0_CPU1) | (val) & 1) << 1)
/* CPU0 Enable Request (0) */
#define IFX_PMS_ENR0_CPU0   (0x1)
#define IFX_PMS_ENR0_CPU0_VAL(val)   (((val) & 0x1) << 0)
#define IFX_PMS_ENR0_CPU0_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR0_CPU0) | (val) & 1) << 0)

/*******************************************************************************
 * PMS Status Register 1
 ******************************************************************************/

/* L2$ Controller Status (31) */
#define IFX_PMS_SR1_L2C   (0x1 << 31)
#define IFX_PMS_SR1_L2C_GET(val)   ((((val) & IFX_PMS_SR1_L2C) >> 31) & 0x1)
/* DDR2 SDRAM Controller Status (30) */
#define IFX_PMS_SR1_DDR2   (0x1 << 30)
#define IFX_PMS_SR1_DDR2_GET(val)   ((((val) & IFX_PMS_SR1_DDR2) >> 30) & 0x1)
/* SRAM Bus Master Interface SMI1 Status (29) */
#define IFX_PMS_SR1_SMI1   (0x1 << 29)
#define IFX_PMS_SR1_SMI1_GET(val)   ((((val) & IFX_PMS_SR1_SMI1) >> 29) & 0x1)
/* SRAM Bus Master Interface SMI0 Status (28) */
#define IFX_PMS_SR1_SMI0   (0x1 << 28)
#define IFX_PMS_SR1_SMI0_GET(val)   ((((val) & IFX_PMS_SR1_SMI0) >> 28) & 0x1)
/* SRAM Bus Master Interface SMI2 Status (27) */
#define IFX_PMS_SR1_SMI2   (0x1 << 27)
#define IFX_PMS_SR1_SMI2_GET(val)   ((((val) & IFX_PMS_SR1_SMI2) >> 27) & 0x1)
/* FPI Bus Master Interface FMI0 Status (26) */
#define IFX_PMS_SR1_FMI0   (0x1 << 26)
#define IFX_PMS_SR1_FMI0_GET(val)   ((((val) & IFX_PMS_SR1_FMI0) >> 26) & 0x1)
/* PCM3 Status (19) */
#define IFX_PMS_SR1_PCM3   (0x1 << 19)
#define IFX_PMS_SR1_PCM3_GET(val)   ((((val) & IFX_PMS_SR1_PCM3) >> 19) & 0x1)
/* PCM2 Status (18) */
#define IFX_PMS_SR1_PCM2   (0x1 << 18)
#define IFX_PMS_SR1_PCM2_GET(val)   ((((val) & IFX_PMS_SR1_PCM2) >> 18) & 0x1)
/* PCM1 Status (17) */
#define IFX_PMS_SR1_PCM1   (0x1 << 17)
#define IFX_PMS_SR1_PCM1_GET(val)   ((((val) & IFX_PMS_SR1_PCM1) >> 17) & 0x1)
/* PCM0 Status (16) */
#define IFX_PMS_SR1_PCM0   (0x1 << 16)
#define IFX_PMS_SR1_PCM0_GET(val)   ((((val) & IFX_PMS_SR1_PCM0) >> 16) & 0x1)
/* ASC1 Status (15) */
#define IFX_PMS_SR1_ASC1   (0x1 << 15)
#define IFX_PMS_SR1_ASC1_GET(val)   ((((val) & IFX_PMS_SR1_ASC1) >> 15) & 0x1)
/* ASC0 Status (14) */
#define IFX_PMS_SR1_ASC0   (0x1 << 14)
#define IFX_PMS_SR1_ASC0_GET(val)   ((((val) & IFX_PMS_SR1_ASC0) >> 14) & 0x1)
/* SSC2 Status (13) */
#define IFX_PMS_SR1_SSC2   (0x1 << 13)
#define IFX_PMS_SR1_SSC2_GET(val)   ((((val) & IFX_PMS_SR1_SSC2) >> 13) & 0x1)
/* SSC1 Status (12) */
#define IFX_PMS_SR1_SSC1   (0x1 << 12)
#define IFX_PMS_SR1_SSC1_GET(val)   ((((val) & IFX_PMS_SR1_SSC1) >> 12) & 0x1)
/* SSC0 Status (11) */
#define IFX_PMS_SR1_SSC0   (0x1 << 11)
#define IFX_PMS_SR1_SSC0_GET(val)   ((((val) & IFX_PMS_SR1_SSC0) >> 11) & 0x1)
/* GPTC Status (10) */
#define IFX_PMS_SR1_GPTC   (0x1 << 10)
#define IFX_PMS_SR1_GPTC_GET(val)   ((((val) & IFX_PMS_SR1_GPTC) >> 10) & 0x1)
/* DMA Status (9) */
#define IFX_PMS_SR1_DMA   (0x1 << 9)
#define IFX_PMS_SR1_DMA_GET(val)   ((((val) & IFX_PMS_SR1_DMA) >> 9) & 0x1)
/* FSC Timer Status (8) */
#define IFX_PMS_SR1_FSCT   (0x1 << 8)
#define IFX_PMS_SR1_FSCT_GET(val)   ((((val) & IFX_PMS_SR1_FSCT) >> 8) & 0x1)
/* Ethernet Switch Status (7) */
#define IFX_PMS_SR1_ETHSW   (0x1 << 7)
#define IFX_PMS_SR1_ETHSW_GET(val)   ((((val) & IFX_PMS_SR1_ETHSW) >> 7) & 0x1)
/* EBU Status (6) */
#define IFX_PMS_SR1_EBU   (0x1 << 6)
#define IFX_PMS_SR1_EBU_GET(val)   ((((val) & IFX_PMS_SR1_EBU) >> 6) & 0x1)
/* TRNG Status (5) */
#define IFX_PMS_SR1_TRNG   (0x1 << 5)
#define IFX_PMS_SR1_TRNG_GET(val)   ((((val) & IFX_PMS_SR1_TRNG) >> 5) & 0x1)
/* DEU Status (4) */
#define IFX_PMS_SR1_DEU   (0x1 << 4)
#define IFX_PMS_SR1_DEU_GET(val)   ((((val) & IFX_PMS_SR1_DEU) >> 4) & 0x1)
/* GPIO Port P3 Status (3) */
#define IFX_PMS_SR1_PORT3   (0x1 << 3)
#define IFX_PMS_SR1_PORT3_GET(val)   ((((val) & IFX_PMS_SR1_PORT3) >> 3) & 0x1)
/* GPIO Port P2 Status (2) */
#define IFX_PMS_SR1_PORT2   (0x1 << 2)
#define IFX_PMS_SR1_PORT2_GET(val)   ((((val) & IFX_PMS_SR1_PORT2) >> 2) & 0x1)
/* GPIO Port P1 Status (1) */
#define IFX_PMS_SR1_PORT1   (0x1 << 1)
#define IFX_PMS_SR1_PORT1_GET(val)   ((((val) & IFX_PMS_SR1_PORT1) >> 1) & 0x1)
/* GPIO Port P0 Status (0) */
#define IFX_PMS_SR1_PORT0   (0x1)
#define IFX_PMS_SR1_PORT0_GET(val)   ((((val) & IFX_PMS_SR1_PORT0) >> 0) & 0x1)

/*******************************************************************************
 * PMS Clear Request Register 1
 ******************************************************************************/

/* L2$ Controller Clear Request (31) */
#define IFX_PMS_CLR1_L2C   (0x1 << 31)
#define IFX_PMS_CLR1_L2C_VAL(val)   (((val) & 0x1) << 31)
#define IFX_PMS_CLR1_L2C_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_L2C) | (val) & 1) << 31)
/* DDR2 SDRAM Controller Clear Request (30) */
#define IFX_PMS_CLR1_DDR2   (0x1 << 30)
#define IFX_PMS_CLR1_DDR2_VAL(val)   (((val) & 0x1) << 30)
#define IFX_PMS_CLR1_DDR2_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_DDR2) | (val) & 1) << 30)
/* SRAM Bus Master Interface SMI1 Clear Request (29) */
#define IFX_PMS_CLR1_SMI1   (0x1 << 29)
#define IFX_PMS_CLR1_SMI1_VAL(val)   (((val) & 0x1) << 29)
#define IFX_PMS_CLR1_SMI1_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_SMI1) | (val) & 1) << 29)
/* SRAM Bus Master Interface SMI0 Clear Request (28) */
#define IFX_PMS_CLR1_SMI0   (0x1 << 28)
#define IFX_PMS_CLR1_SMI0_VAL(val)   (((val) & 0x1) << 28)
#define IFX_PMS_CLR1_SMI0_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_SMI0) | (val) & 1) << 28)
/* SRAM Bus Master Interface SMI2 Clear Request (27) */
#define IFX_PMS_CLR1_SMI2   (0x1 << 27)
#define IFX_PMS_CLR1_SMI2_VAL(val)   (((val) & 0x1) << 27)
#define IFX_PMS_CLR1_SMI2_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_SMI2) | (val) & 1) << 27)
/* FPI Bus Master Interface FMI0 Clear Request (26) */
#define IFX_PMS_CLR1_FMI0   (0x1 << 26)
#define IFX_PMS_CLR1_FMI0_VAL(val)   (((val) & 0x1) << 26)
#define IFX_PMS_CLR1_FMI0_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_FMI0) | (val) & 1) << 26)
/* PCM3 Clear Request (19) */
#define IFX_PMS_CLR1_PCM3   (0x1 << 19)
#define IFX_PMS_CLR1_PCM3_VAL(val)   (((val) & 0x1) << 19)
#define IFX_PMS_CLR1_PCM3_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_PCM3) | (val) & 1) << 19)
/* PCM2 Clear Request (18) */
#define IFX_PMS_CLR1_PCM2   (0x1 << 18)
#define IFX_PMS_CLR1_PCM2_VAL(val)   (((val) & 0x1) << 18)
#define IFX_PMS_CLR1_PCM2_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_PCM2) | (val) & 1) << 18)
/* PCM1 Clear Request (17) */
#define IFX_PMS_CLR1_PCM1   (0x1 << 17)
#define IFX_PMS_CLR1_PCM1_VAL(val)   (((val) & 0x1) << 17)
#define IFX_PMS_CLR1_PCM1_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_PCM1) | (val) & 1) << 17)
/* PCM0 Clear Request (16) */
#define IFX_PMS_CLR1_PCM0   (0x1 << 16)
#define IFX_PMS_CLR1_PCM0_VAL(val)   (((val) & 0x1) << 16)
#define IFX_PMS_CLR1_PCM0_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_PCM0) | (val) & 1) << 16)
/* ASC1 Clear Request (15) */
#define IFX_PMS_CLR1_ASC1   (0x1 << 15)
#define IFX_PMS_CLR1_ASC1_VAL(val)   (((val) & 0x1) << 15)
#define IFX_PMS_CLR1_ASC1_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_ASC1) | (val) & 1) << 15)
/* ASC0 Clear Request (14) */
#define IFX_PMS_CLR1_ASC0   (0x1 << 14)
#define IFX_PMS_CLR1_ASC0_VAL(val)   (((val) & 0x1) << 14)
#define IFX_PMS_CLR1_ASC0_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_ASC0) | (val) & 1) << 14)
/* SSC2 Clear Request (13) */
#define IFX_PMS_CLR1_SSC2   (0x1 << 13)
#define IFX_PMS_CLR1_SSC2_VAL(val)   (((val) & 0x1) << 13)
#define IFX_PMS_CLR1_SSC2_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_SSC2) | (val) & 1) << 13)
/* SSC1 Clear Request (12) */
#define IFX_PMS_CLR1_SSC1   (0x1 << 12)
#define IFX_PMS_CLR1_SSC1_VAL(val)   (((val) & 0x1) << 12)
#define IFX_PMS_CLR1_SSC1_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_SSC1) | (val) & 1) << 12)
/* SSC0 Clear Request (11) */
#define IFX_PMS_CLR1_SSC0   (0x1 << 11)
#define IFX_PMS_CLR1_SSC0_VAL(val)   (((val) & 0x1) << 11)
#define IFX_PMS_CLR1_SSC0_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_SSC0) | (val) & 1) << 11)
/* GPTC Clear Request (10) */
#define IFX_PMS_CLR1_GPTC   (0x1 << 10)
#define IFX_PMS_CLR1_GPTC_VAL(val)   (((val) & 0x1) << 10)
#define IFX_PMS_CLR1_GPTC_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_GPTC) | (val) & 1) << 10)
/* DMA Clear Request (9) */
#define IFX_PMS_CLR1_DMA   (0x1 << 9)
#define IFX_PMS_CLR1_DMA_VAL(val)   (((val) & 0x1) << 9)
#define IFX_PMS_CLR1_DMA_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_DMA) | (val) & 1) << 9)
/* FSC Timer Clear Request (8) */
#define IFX_PMS_CLR1_FSCT   (0x1 << 8)
#define IFX_PMS_CLR1_FSCT_VAL(val)   (((val) & 0x1) << 8)
#define IFX_PMS_CLR1_FSCT_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_FSCT) | (val) & 1) << 8)
/* Ethernet Switch Clear Request (7) */
#define IFX_PMS_CLR1_ETHSW   (0x1 << 7)
#define IFX_PMS_CLR1_ETHSW_VAL(val)   (((val) & 0x1) << 7)
#define IFX_PMS_CLR1_ETHSW_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_ETHSW) | (val) & 1) << 7)
/* EBU Clear Request (6) */
#define IFX_PMS_CLR1_EBU   (0x1 << 6)
#define IFX_PMS_CLR1_EBU_VAL(val)   (((val) & 0x1) << 6)
#define IFX_PMS_CLR1_EBU_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_EBU) | (val) & 1) << 6)
/* TRNG Clear Request (5) */
#define IFX_PMS_CLR1_TRNG   (0x1 << 5)
#define IFX_PMS_CLR1_TRNG_VAL(val)   (((val) & 0x1) << 5)
#define IFX_PMS_CLR1_TRNG_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_TRNG) | (val) & 1) << 5)
/* DEU Clear Request (4) */
#define IFX_PMS_CLR1_DEU   (0x1 << 4)
#define IFX_PMS_CLR1_DEU_VAL(val)   (((val) & 0x1) << 4)
#define IFX_PMS_CLR1_DEU_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_DEU) | (val) & 1) << 4)
/* GPIO Port P3 Clear Request (3) */
#define IFX_PMS_CLR1_PORT3   (0x1 << 3)
#define IFX_PMS_CLR1_PORT3_VAL(val)   (((val) & 0x1) << 3)
#define IFX_PMS_CLR1_PORT3_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_PORT3) | (val) & 1) << 3)
/* GPIO Port P2 Clear Request (2) */
#define IFX_PMS_CLR1_PORT2   (0x1 << 2)
#define IFX_PMS_CLR1_PORT2_VAL(val)   (((val) & 0x1) << 2)
#define IFX_PMS_CLR1_PORT2_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_PORT2) | (val) & 1) << 2)
/* GPIO Port P1 Clear Request (1) */
#define IFX_PMS_CLR1_PORT1   (0x1 << 1)
#define IFX_PMS_CLR1_PORT1_VAL(val)   (((val) & 0x1) << 1)
#define IFX_PMS_CLR1_PORT1_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_PORT1) | (val) & 1) << 1)
/* GPIO Port P0 Clear Request (0) */
#define IFX_PMS_CLR1_PORT0   (0x1)
#define IFX_PMS_CLR1_PORT0_VAL(val)   (((val) & 0x1) << 0)
#define IFX_PMS_CLR1_PORT0_SET(reg,val) (reg) = (((reg & ~IFX_PMS_CLR1_PORT0) | (val) & 1) << 0)

/*******************************************************************************
 * PMS Enable Request Register 1
 ******************************************************************************/

/* L2$ Controller Enable Request (31) */
#define IFX_PMS_ENR1_L2C   (0x1 << 31)
#define IFX_PMS_ENR1_L2C_VAL(val)   (((val) & 0x1) << 31)
#define IFX_PMS_ENR1_L2C_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_L2C) | (val) & 1) << 31)
/* DDR2 SDRAM Controller Enable Request (30) */
#define IFX_PMS_ENR1_DDR2   (0x1 << 30)
#define IFX_PMS_ENR1_DDR2_VAL(val)   (((val) & 0x1) << 30)
#define IFX_PMS_ENR1_DDR2_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_DDR2) | (val) & 1) << 30)
/* SRAM Bus Master Interface SMI1 Enable Request (29) */
#define IFX_PMS_ENR1_SMI1   (0x1 << 29)
#define IFX_PMS_ENR1_SMI1_VAL(val)   (((val) & 0x1) << 29)
#define IFX_PMS_ENR1_SMI1_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_SMI1) | (val) & 1) << 29)
/* SRAM Bus Master Interface SMI0 Enable Request (28) */
#define IFX_PMS_ENR1_SMI0   (0x1 << 28)
#define IFX_PMS_ENR1_SMI0_VAL(val)   (((val) & 0x1) << 28)
#define IFX_PMS_ENR1_SMI0_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_SMI0) | (val) & 1) << 28)
/* SRAM Bus Master Interface SMI2 Enable Request (27) */
#define IFX_PMS_ENR1_SMI2   (0x1 << 27)
#define IFX_PMS_ENR1_SMI2_VAL(val)   (((val) & 0x1) << 27)
#define IFX_PMS_ENR1_SMI2_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_SMI2) | (val) & 1) << 27)
/* FPI Bus Master Interface FMI0 Enable Request (26) */
#define IFX_PMS_ENR1_FMI0   (0x1 << 26)
#define IFX_PMS_ENR1_FMI0_VAL(val)   (((val) & 0x1) << 26)
#define IFX_PMS_ENR1_FMI0_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_FMI0) | (val) & 1) << 26)
/* PCM3 Enable Request (19) */
#define IFX_PMS_ENR1_PCM3   (0x1 << 19)
#define IFX_PMS_ENR1_PCM3_VAL(val)   (((val) & 0x1) << 19)
#define IFX_PMS_ENR1_PCM3_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_PCM3) | (val) & 1) << 19)
/* PCM2 Enable Request (18) */
#define IFX_PMS_ENR1_PCM2   (0x1 << 18)
#define IFX_PMS_ENR1_PCM2_VAL(val)   (((val) & 0x1) << 18)
#define IFX_PMS_ENR1_PCM2_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_PCM2) | (val) & 1) << 18)
/* PCM1 Enable Request (17) */
#define IFX_PMS_ENR1_PCM1   (0x1 << 17)
#define IFX_PMS_ENR1_PCM1_VAL(val)   (((val) & 0x1) << 17)
#define IFX_PMS_ENR1_PCM1_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_PCM1) | (val) & 1) << 17)
/* PCM0 Enable Request (16) */
#define IFX_PMS_ENR1_PCM0   (0x1 << 16)
#define IFX_PMS_ENR1_PCM0_VAL(val)   (((val) & 0x1) << 16)
#define IFX_PMS_ENR1_PCM0_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_PCM0) | (val) & 1) << 16)
/* ASC1 Enable Request (15) */
#define IFX_PMS_ENR1_ASC1   (0x1 << 15)
#define IFX_PMS_ENR1_ASC1_VAL(val)   (((val) & 0x1) << 15)
#define IFX_PMS_ENR1_ASC1_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_ASC1) | (val) & 1) << 15)
/* ASC0 Enable Request (14) */
#define IFX_PMS_ENR1_ASC0   (0x1 << 14)
#define IFX_PMS_ENR1_ASC0_VAL(val)   (((val) & 0x1) << 14)
#define IFX_PMS_ENR1_ASC0_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_ASC0) | (val) & 1) << 14)
/* SSC2 Enable Request (13) */
#define IFX_PMS_ENR1_SSC2   (0x1 << 13)
#define IFX_PMS_ENR1_SSC2_VAL(val)   (((val) & 0x1) << 13)
#define IFX_PMS_ENR1_SSC2_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_SSC2) | (val) & 1) << 13)
/* SSC1 Enable Request (12) */
#define IFX_PMS_ENR1_SSC1   (0x1 << 12)
#define IFX_PMS_ENR1_SSC1_VAL(val)   (((val) & 0x1) << 12)
#define IFX_PMS_ENR1_SSC1_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_SSC1) | (val) & 1) << 12)
/* SSC0 Enable Request (11) */
#define IFX_PMS_ENR1_SSC0   (0x1 << 11)
#define IFX_PMS_ENR1_SSC0_VAL(val)   (((val) & 0x1) << 11)
#define IFX_PMS_ENR1_SSC0_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_SSC0) | (val) & 1) << 11)
/* GPTC Enable Request (10) */
#define IFX_PMS_ENR1_GPTC   (0x1 << 10)
#define IFX_PMS_ENR1_GPTC_VAL(val)   (((val) & 0x1) << 10)
#define IFX_PMS_ENR1_GPTC_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_GPTC) | (val) & 1) << 10)
/* DMA Enable Request (9) */
#define IFX_PMS_ENR1_DMA   (0x1 << 9)
#define IFX_PMS_ENR1_DMA_VAL(val)   (((val) & 0x1) << 9)
#define IFX_PMS_ENR1_DMA_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_DMA) | (val) & 1) << 9)
/* FSC Timer Enable Request (8) */
#define IFX_PMS_ENR1_FSCT   (0x1 << 8)
#define IFX_PMS_ENR1_FSCT_VAL(val)   (((val) & 0x1) << 8)
#define IFX_PMS_ENR1_FSCT_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_FSCT) | (val) & 1) << 8)
/* Ethernet Switch Enable Request (7) */
#define IFX_PMS_ENR1_ETHSW   (0x1 << 7)
#define IFX_PMS_ENR1_ETHSW_VAL(val)   (((val) & 0x1) << 7)
#define IFX_PMS_ENR1_ETHSW_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_ETHSW) | (val) & 1) << 7)
/* EBU Enable Request (6) */
#define IFX_PMS_ENR1_EBU   (0x1 << 6)
#define IFX_PMS_ENR1_EBU_VAL(val)   (((val) & 0x1) << 6)
#define IFX_PMS_ENR1_EBU_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_EBU) | (val) & 1) << 6)
/* TRNG Enable Request (5) */
#define IFX_PMS_ENR1_TRNG   (0x1 << 5)
#define IFX_PMS_ENR1_TRNG_VAL(val)   (((val) & 0x1) << 5)
#define IFX_PMS_ENR1_TRNG_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_TRNG) | (val) & 1) << 5)
/* DEU Enable Request (4) */
#define IFX_PMS_ENR1_DEU   (0x1 << 4)
#define IFX_PMS_ENR1_DEU_VAL(val)   (((val) & 0x1) << 4)
#define IFX_PMS_ENR1_DEU_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_DEU) | (val) & 1) << 4)
/* GPIO Port P3 Enable Request (3) */
#define IFX_PMS_ENR1_PORT3   (0x1 << 3)
#define IFX_PMS_ENR1_PORT3_VAL(val)   (((val) & 0x1) << 3)
#define IFX_PMS_ENR1_PORT3_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_PORT3) | (val) & 1) << 3)
/* GPIO Port P2 Enable Request (2) */
#define IFX_PMS_ENR1_PORT2   (0x1 << 2)
#define IFX_PMS_ENR1_PORT2_VAL(val)   (((val) & 0x1) << 2)
#define IFX_PMS_ENR1_PORT2_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_PORT2) | (val) & 1) << 2)
/* GPIO Port P1 Enable Request (1) */
#define IFX_PMS_ENR1_PORT1   (0x1 << 1)
#define IFX_PMS_ENR1_PORT1_VAL(val)   (((val) & 0x1) << 1)
#define IFX_PMS_ENR1_PORT1_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_PORT1) | (val) & 1) << 1)
/* GPIO Port P0 Enable Request (0) */
#define IFX_PMS_ENR1_PORT0   (0x1)
#define IFX_PMS_ENR1_PORT0_VAL(val)   (((val) & 0x1) << 0)
#define IFX_PMS_ENR1_PORT0_SET(reg,val) (reg) = (((reg & ~IFX_PMS_ENR1_PORT0) | (val) & 1) << 0)

/*******************************************************************************
 * PMS CPU Sleep Request Source Select Register
 ******************************************************************************/

/* Sleep Request Source Select (1) */
#define IFX_PMS_CPU0SRSSR_RP   (0x1 << 1)
#define IFX_PMS_CPU0SRSSR_RP_VAL(val)   (((val) & 0x1) << 1)
#define IFX_PMS_CPU0SRSSR_RP_GET(val)   ((((val) & IFX_PMS_CPU0SRSSR_RP) >> 1) & 0x1)
#define IFX_PMS_CPU0SRSSR_RP_SET(reg,val) (reg) = ((reg & ~IFX_PMS_CPU0SRSSR_RP) | (((val) & 0x1) << 1))
/* Sleep Request Source Select (0) */
#define IFX_PMS_CPU0SRSSR_SLEEP   (0x1)
#define IFX_PMS_CPU0SRSSR_SLEEP_VAL(val)   (((val) & 0x1) << 0)
#define IFX_PMS_CPU0SRSSR_SLEEP_GET(val)   ((((val) & IFX_PMS_CPU0SRSSR_SLEEP) >> 0) & 0x1)
#define IFX_PMS_CPU0SRSSR_SLEEP_SET(reg,val) (reg) = ((reg & ~IFX_PMS_CPU0SRSSR_SLEEP) | (((val) & 0x1) << 0))

/*******************************************************************************
 * PMS CPU Wake-up Request Source Select Register
 ******************************************************************************/

/* Wake-up Request Source Select (1) */
#define IFX_PMS_CPU0WRSSR_PMAC   (0x1 << 1)
#define IFX_PMS_CPU0WRSSR_PMAC_VAL(val)   (((val) & 0x1) << 1)
#define IFX_PMS_CPU0WRSSR_PMAC_GET(val)   ((((val) & IFX_PMS_CPU0WRSSR_PMAC) >> 1) & 0x1)
#define IFX_PMS_CPU0WRSSR_PMAC_SET(reg,val) (reg) = ((reg & ~IFX_PMS_CPU0WRSSR_PMAC) | (((val) & 0x1) << 1))
/* Wake-up Request Source Select (0) */
#define IFX_PMS_CPU0WRSSR_ICU   (0x1)
#define IFX_PMS_CPU0WRSSR_ICU_VAL(val)   (((val) & 0x1) << 0)
#define IFX_PMS_CPU0WRSSR_ICU_GET(val)   ((((val) & IFX_PMS_CPU0WRSSR_ICU) >> 0) & 0x1)
#define IFX_PMS_CPU0WRSSR_ICU_SET(reg,val) (reg) = ((reg & ~IFX_PMS_CPU0WRSSR_ICU) | (((val) & 0x1) << 0))

/*******************************************************************************
 * PMS Peripheral Wake-up Request Source Select Register
 ******************************************************************************/

/* Peripheral Wake-Up Request Source Select (31) */
#define IFX_PMS_PERWRSSR_PMAC   (0x1 << 31)
#define IFX_PMS_PERWRSSR_PMAC_VAL(val)   (((val) & 0x1) << 31)
#define IFX_PMS_PERWRSSR_PMAC_GET(val)   ((((val) & IFX_PMS_PERWRSSR_PMAC) >> 31) & 0x1)
#define IFX_PMS_PERWRSSR_PMAC_SET(reg,val) (reg) = ((reg & ~IFX_PMS_PERWRSSR_PMAC) | (((val) & 0x1) << 31))
/* Peripheral Wake-Up Request Source Select (29) */
#define IFX_PMS_PERWRSSR_ICU5   (0x1 << 29)
#define IFX_PMS_PERWRSSR_ICU5_VAL(val)   (((val) & 0x1) << 29)
#define IFX_PMS_PERWRSSR_ICU5_GET(val)   ((((val) & IFX_PMS_PERWRSSR_ICU5) >> 29) & 0x1)
#define IFX_PMS_PERWRSSR_ICU5_SET(reg,val) (reg) = ((reg & ~IFX_PMS_PERWRSSR_ICU5) | (((val) & 0x1) << 29))
/* Peripheral Wake-Up Request Source Select (28) */
#define IFX_PMS_PERWRSSR_ICU4   (0x1 << 28)
#define IFX_PMS_PERWRSSR_ICU4_VAL(val)   (((val) & 0x1) << 28)
#define IFX_PMS_PERWRSSR_ICU4_GET(val)   ((((val) & IFX_PMS_PERWRSSR_ICU4) >> 28) & 0x1)
#define IFX_PMS_PERWRSSR_ICU4_SET(reg,val) (reg) = ((reg & ~IFX_PMS_PERWRSSR_ICU4) | (((val) & 0x1) << 28))
/* Peripheral Wake-Up Request Source Select (27) */
#define IFX_PMS_PERWRSSR_ICU3   (0x1 << 27)
#define IFX_PMS_PERWRSSR_ICU3_VAL(val)   (((val) & 0x1) << 27)
#define IFX_PMS_PERWRSSR_ICU3_GET(val)   ((((val) & IFX_PMS_PERWRSSR_ICU3) >> 27) & 0x1)
#define IFX_PMS_PERWRSSR_ICU3_SET(reg,val) (reg) = ((reg & ~IFX_PMS_PERWRSSR_ICU3) | (((val) & 0x1) << 27))
/* Peripheral Wake-Up Request Source Select (26) */
#define IFX_PMS_PERWRSSR_ICU2   (0x1 << 26)
#define IFX_PMS_PERWRSSR_ICU2_VAL(val)   (((val) & 0x1) << 26)
#define IFX_PMS_PERWRSSR_ICU2_GET(val)   ((((val) & IFX_PMS_PERWRSSR_ICU2) >> 26) & 0x1)
#define IFX_PMS_PERWRSSR_ICU2_SET(reg,val) (reg) = ((reg & ~IFX_PMS_PERWRSSR_ICU2) | (((val) & 0x1) << 26))
/* Peripheral Wake-Up Request Source Select (25) */
#define IFX_PMS_PERWRSSR_ICU1   (0x1 << 25)
#define IFX_PMS_PERWRSSR_ICU1_VAL(val)   (((val) & 0x1) << 25)
#define IFX_PMS_PERWRSSR_ICU1_GET(val)   ((((val) & IFX_PMS_PERWRSSR_ICU1) >> 25) & 0x1)
#define IFX_PMS_PERWRSSR_ICU1_SET(reg,val) (reg) = ((reg & ~IFX_PMS_PERWRSSR_ICU1) | (((val) & 0x1) << 25))
/* Peripheral Wake-Up Request Source Select (24) */
#define IFX_PMS_PERWRSSR_ICU0   (0x1 << 24)
#define IFX_PMS_PERWRSSR_ICU0_VAL(val)   (((val) & 0x1) << 24)
#define IFX_PMS_PERWRSSR_ICU0_GET(val)   ((((val) & IFX_PMS_PERWRSSR_ICU0) >> 24) & 0x1)
#define IFX_PMS_PERWRSSR_ICU0_SET(reg,val) (reg) = ((reg & ~IFX_PMS_PERWRSSR_ICU0) | (((val) & 0x1) << 24))
/* Peripheral Wake-Up Request Source Select (21) */
#define IFX_PMS_PERWRSSR_ERL5   (0x1 << 21)
#define IFX_PMS_PERWRSSR_ERL5_VAL(val)   (((val) & 0x1) << 21)
#define IFX_PMS_PERWRSSR_ERL5_GET(val)   ((((val) & IFX_PMS_PERWRSSR_ERL5) >> 21) & 0x1)
#define IFX_PMS_PERWRSSR_ERL5_SET(reg,val) (reg) = ((reg & ~IFX_PMS_PERWRSSR_ERL5) | (((val) & 0x1) << 21))
/* Peripheral Wake-Up Request Source Select (20) */
#define IFX_PMS_PERWRSSR_ERL4   (0x1 << 20)
#define IFX_PMS_PERWRSSR_ERL4_VAL(val)   (((val) & 0x1) << 20)
#define IFX_PMS_PERWRSSR_ERL4_GET(val)   ((((val) & IFX_PMS_PERWRSSR_ERL4) >> 20) & 0x1)
#define IFX_PMS_PERWRSSR_ERL4_SET(reg,val) (reg) = ((reg & ~IFX_PMS_PERWRSSR_ERL4) | (((val) & 0x1) << 20))
/* Peripheral Wake-Up Request Source Select (19) */
#define IFX_PMS_PERWRSSR_ERL3   (0x1 << 19)
#define IFX_PMS_PERWRSSR_ERL3_VAL(val)   (((val) & 0x1) << 19)
#define IFX_PMS_PERWRSSR_ERL3_GET(val)   ((((val) & IFX_PMS_PERWRSSR_ERL3) >> 19) & 0x1)
#define IFX_PMS_PERWRSSR_ERL3_SET(reg,val) (reg) = ((reg & ~IFX_PMS_PERWRSSR_ERL3) | (((val) & 0x1) << 19))
/* Peripheral Wake-Up Request Source Select (18) */
#define IFX_PMS_PERWRSSR_ERL2   (0x1 << 18)
#define IFX_PMS_PERWRSSR_ERL2_VAL(val)   (((val) & 0x1) << 18)
#define IFX_PMS_PERWRSSR_ERL2_GET(val)   ((((val) & IFX_PMS_PERWRSSR_ERL2) >> 18) & 0x1)
#define IFX_PMS_PERWRSSR_ERL2_SET(reg,val) (reg) = ((reg & ~IFX_PMS_PERWRSSR_ERL2) | (((val) & 0x1) << 18))
/* Peripheral Wake-Up Request Source Select (17) */
#define IFX_PMS_PERWRSSR_ERL1   (0x1 << 17)
#define IFX_PMS_PERWRSSR_ERL1_VAL(val)   (((val) & 0x1) << 17)
#define IFX_PMS_PERWRSSR_ERL1_GET(val)   ((((val) & IFX_PMS_PERWRSSR_ERL1) >> 17) & 0x1)
#define IFX_PMS_PERWRSSR_ERL1_SET(reg,val) (reg) = ((reg & ~IFX_PMS_PERWRSSR_ERL1) | (((val) & 0x1) << 17))
/* Peripheral Wake-Up Request Source Select (16) */
#define IFX_PMS_PERWRSSR_ERL0   (0x1 << 16)
#define IFX_PMS_PERWRSSR_ERL0_VAL(val)   (((val) & 0x1) << 16)
#define IFX_PMS_PERWRSSR_ERL0_GET(val)   ((((val) & IFX_PMS_PERWRSSR_ERL0) >> 16) & 0x1)
#define IFX_PMS_PERWRSSR_ERL0_SET(reg,val) (reg) = ((reg & ~IFX_PMS_PERWRSSR_ERL0) | (((val) & 0x1) << 16))
/* Peripheral Wake-Up Request Source Select (13) */
#define IFX_PMS_PERWRSSR_EXL5   (0x1 << 13)
#define IFX_PMS_PERWRSSR_EXL5_VAL(val)   (((val) & 0x1) << 13)
#define IFX_PMS_PERWRSSR_EXL5_GET(val)   ((((val) & IFX_PMS_PERWRSSR_EXL5) >> 13) & 0x1)
#define IFX_PMS_PERWRSSR_EXL5_SET(reg,val) (reg) = ((reg & ~IFX_PMS_PERWRSSR_EXL5) | (((val) & 0x1) << 13))
/* Peripheral Wake-Up Request Source Select (12) */
#define IFX_PMS_PERWRSSR_EXL4   (0x1 << 12)
#define IFX_PMS_PERWRSSR_EXL4_VAL(val)   (((val) & 0x1) << 12)
#define IFX_PMS_PERWRSSR_EXL4_GET(val)   ((((val) & IFX_PMS_PERWRSSR_EXL4) >> 12) & 0x1)
#define IFX_PMS_PERWRSSR_EXL4_SET(reg,val) (reg) = ((reg & ~IFX_PMS_PERWRSSR_EXL4) | (((val) & 0x1) << 12))
/* Peripheral Wake-Up Request Source Select (11) */
#define IFX_PMS_PERWRSSR_EXL3   (0x1 << 11)
#define IFX_PMS_PERWRSSR_EXL3_VAL(val)   (((val) & 0x1) << 11)
#define IFX_PMS_PERWRSSR_EXL3_GET(val)   ((((val) & IFX_PMS_PERWRSSR_EXL3) >> 11) & 0x1)
#define IFX_PMS_PERWRSSR_EXL3_SET(reg,val) (reg) = ((reg & ~IFX_PMS_PERWRSSR_EXL3) | (((val) & 0x1) << 11))
/* Peripheral Wake-Up Request Source Select (10) */
#define IFX_PMS_PERWRSSR_EXL2   (0x1 << 10)
#define IFX_PMS_PERWRSSR_EXL2_VAL(val)   (((val) & 0x1) << 10)
#define IFX_PMS_PERWRSSR_EXL2_GET(val)   ((((val) & IFX_PMS_PERWRSSR_EXL2) >> 10) & 0x1)
#define IFX_PMS_PERWRSSR_EXL2_SET(reg,val) (reg) = ((reg & ~IFX_PMS_PERWRSSR_EXL2) | (((val) & 0x1) << 10))
/* Peripheral Wake-Up Request Source Select (9) */
#define IFX_PMS_PERWRSSR_EXL1   (0x1 << 9)
#define IFX_PMS_PERWRSSR_EXL1_VAL(val)   (((val) & 0x1) << 9)
#define IFX_PMS_PERWRSSR_EXL1_GET(val)   ((((val) & IFX_PMS_PERWRSSR_EXL1) >> 9) & 0x1)
#define IFX_PMS_PERWRSSR_EXL1_SET(reg,val) (reg) = ((reg & ~IFX_PMS_PERWRSSR_EXL1) | (((val) & 0x1) << 9))
/* Peripheral Wake-Up Request Source Select (8) */
#define IFX_PMS_PERWRSSR_EXL0   (0x1 << 8)
#define IFX_PMS_PERWRSSR_EXL0_VAL(val)   (((val) & 0x1) << 8)
#define IFX_PMS_PERWRSSR_EXL0_GET(val)   ((((val) & IFX_PMS_PERWRSSR_EXL0) >> 8) & 0x1)
#define IFX_PMS_PERWRSSR_EXL0_SET(reg,val) (reg) = ((reg & ~IFX_PMS_PERWRSSR_EXL0) | (((val) & 0x1) << 8))

#endif /* __IFX_PMS_H */
