/*
   Copyright (c) 2006 - 2008, Infineon Technologies AG

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This file contains the internal definitions of the Managed Entity Interface
   driver.
   The MEI driver is the interface to the DSL PHY mailbox. The upper layer
   interface of this driver is used by the DSL CPE API driver for communication
   with the DSL PHY firmware.
*/
#ifndef _IFXMIPS_MEI_BSP_H_
#define _IFXMIPS_MEI_BSP_H_

/* Register address offsets, relative to MEI_SPACE_ADDRESS */
#define MEI_DATA_XFR_OFFSET            (0x0000)
#define MEI_VERSION_OFFSET             (0x0004)
#define MEI_ARC_GP_STAT_OFFSET         (0x0008)
#define MEI_DATA_XFR_STAT_OFFSET       (0x000C)
#define MEI_XFR_ADDR_OFFSET            (0x0010)
#define MEI_MAX_WAIT_OFFSET            (0x0014)
#define MEI_TO_ARC_INT_OFFSET          (0x0018)
#define ARC_TO_MEI_INT_OFFSET          (0x001C)
#define ARC_TO_MEI_INT_MASK_OFFSET     (0x0020)
#define MEI_DEBUG_WAD_OFFSET           (0x0024)
#define MEI_DEBUG_RAD_OFFSET           (0x0028)
#define MEI_DEBUG_DATA_OFFSET          (0x002C)
#define MEI_DEBUG_DEC_OFFSET           (0x0030)
#define MEI_CONFIG_OFFSET              (0x0034)
#define MEI_RST_CONTROL_OFFSET         (0x0038)
#define MEI_DBG_MASTER_OFFSET          (0x003C)
#define MEI_CLK_CONTROL_OFFSET         (0x0040)
#define MEI_BIST_CONTROL_OFFSET        (0x0044)
#define MEI_BIST_STAT_OFFSET           (0x0048)
#define MEI_XDATA_BASE_SH_OFFSET       (0x004c)
#define MEI_XDATA_BASE_OFFSET          (0x0050)
#define MEI_XMEM_BAR_BASE_OFFSET       (0x0054)
#define MEI_XMEM_BAR0_OFFSET           (0x0054)
#define MEI_XMEM_BAR1_OFFSET           (0x0058)
#define MEI_XMEM_BAR2_OFFSET           (0x005C)
#define MEI_XMEM_BAR3_OFFSET           (0x0060)
#define MEI_XMEM_BAR4_OFFSET           (0x0064)
#define MEI_XMEM_BAR5_OFFSET           (0x0068)
#define MEI_XMEM_BAR6_OFFSET           (0x006C)
#define MEI_XMEM_BAR7_OFFSET           (0x0070)
#define MEI_XMEM_BAR8_OFFSET           (0x0074)
#define MEI_XMEM_BAR9_OFFSET           (0x0078)
#define MEI_XMEM_BAR10_OFFSET          (0x007C)
#define MEI_XMEM_BAR11_OFFSET          (0x0080)
#define MEI_XMEM_BAR12_OFFSET          (0x0084)
#define MEI_XMEM_BAR13_OFFSET          (0x0088)
#define MEI_XMEM_BAR14_OFFSET          (0x008C)
#define MEI_XMEM_BAR15_OFFSET          (0x0090)
#define MEI_XMEM_BAR16_OFFSET          (0x0094)

/** maximum number of read operation until the ARC should be ready */
#define WHILE_DELAY                    20000

/** maximum swap page size, 8k * (32bits) */
#define MAXSWAPSIZE                    (8 * 1024)

/* Mailboxes */
#define YES_REPLY                      1
#define NO_REPLY                       0
/* jiffies */
#define CMV_TIMEOUT                    1000

/* Block size per BAR */
#define SDRAM_SEGMENT_SIZE             (64*1024)
/* Number of Bar registers
 * BAR0 to BAR15 are 64KB
 * BAR16 is 1MB
 * BAR15 is XDATA_BASE
 * total number of BARs = 17
 * total number of 64KB segments =
 */
#define MAX_BAR_REGISTERS              16
#define BAR_COUNT                      (MAX_BAR_REGISTERS + 1)
/* XMEM_BAR16 is 1MB. 1024 / 64 = 16 */
#define BAR_PAGE_CNT                   ((BAR_COUNT - 1) + 16)
#define XDATA_REGISTER                 15

/* ARC register addresss */
#define ARC_STATUS                     0x0
#define ARC_LP_START                   0x2
#define ARC_LP_END                     0x3
#define ARC_DEBUG                      0x5
#define ARC_INT_MASK                   0x10A

#define IRAM0_BASE                     0x00000
#define IRAM1_BASE                     0x04000
#define BRAM_BASE                      0x0A000

#define ADSL_BASE                      0x20000
#define CRI_BASE                       (ADSL_BASE + 0x11F00)
#define CRI_CCR0                       (CRI_BASE + 0x00)
#define CRI_RST                        (CRI_BASE + 0x04*4)
#define ADSL_DILV_BASE                 (ADSL_BASE+0x20000)

#define AUX_ARC_GPIO_CTRL              0x010C
#define AUX_ARC_GPIO_DATA              0x010D

#define IRAM0_ADDR_BIT_MASK            0xFFF
#define IRAM1_ADDR_BIT_MASK            0xFFF
#define BRAM_ADDR_BIT_MASK             0xFFF
#define RX_DILV_ADDR_BIT_MASK          0x1FFF

/*  Bit definitions */
#define BITPOS(pos)                    (1<<(pos))

#define ARC_AUX_HALT                   BITPOS(25)
#define ARC_DEBUG_HALT                 BITPOS(1)

/* CRI_CCR0 Register definitions */
#define CLK_2M_MODE_ENABLE             BITPOS(6)
#define ACL_CLK_MODE_ENABLE            BITPOS(4)
#define FDF_CLK_MODE_ENABLE            BITPOS(2)
#define STM_CLK_MODE_ENABLE            BITPOS(0)

/* CRI_RST Register definitions */
#define FDF_SRST                       BITPOS(3)
#define MTE_SRST                       BITPOS(2)
#define FCI_SRST                       BITPOS(1)
#define AAI_SRST                       BITPOS(0)

/* MEI_TO_ARC_INTERRUPT Register definitions */
#define MEI_TO_ARC_INT1                BITPOS(3)
#define MEI_TO_ARC_INT0                BITPOS(2)
#define MEI_TO_ARC_CS_DONE             BITPOS(1)
#define MEI_TO_ARC_MSGAV               BITPOS(0)

/* ARC_TO_MEI_INTERRUPT Register definitions */
#define ARC_TO_MEI_INT1                BITPOS(8)
#define ARC_TO_MEI_INT0                BITPOS(7)
#define ARC_TO_MEI_CS_REQ              BITPOS(6)
#define ARC_TO_MEI_DBG_DONE            BITPOS(5)
#define ARC_TO_MEI_MSGACK              BITPOS(4)
#define ARC_TO_MEI_NO_ACCESS           BITPOS(3)
#define ARC_TO_MEI_CHECK_AAITX         BITPOS(2)
#define ARC_TO_MEI_CHECK_AAIRX         BITPOS(1)
#define ARC_TO_MEI_MSGAV               BITPOS(0)

/* ARC_TO_MEI_INTERRUPT_MASK Register definitions */
#define GP_INT1_EN                     BITPOS(8)
#define GP_INT0_EN                     BITPOS(7)
#define CS_REQ_EN                      BITPOS(6)
#define DBG_DONE_EN                    BITPOS(5)
#define MSGACK_EN                      BITPOS(4)
#define NO_ACC_EN                      BITPOS(3)
#define AAITX_EN                       BITPOS(2)
#define AAIRX_EN                       BITPOS(1)
#define MSGAV_EN                       BITPOS(0)
#define MEI_SOFT_RESET                 BITPOS(0)

#define HOST_MSTR                      BITPOS(0)

#define JTAG_MASTER_MODE               0x0
#define MEI_MASTER_MODE                HOST_MSTR

/* MEI_DEBUG_DECODE Register definitions */
#define MEI_DEBUG_DEC_MASK             0x3
#define MEI_DEBUG_DEC_AUX_MASK         0x0
#define MEI_DEBUG_DEC_DMP1_MASK        0x1
#define MEI_DEBUG_DEC_DMP2_MASK        0x2
#define MEI_DEBUG_DEC_CORE_MASK        0x3

#define AUX_STATUS                     0x0

/* ARC_TO_MEI_MAILBOX[11] is a special location used to indicate
   page swap requests. */
#define MEI_TO_ARC_MAILBOX             0xDFD0
#define MEI_TO_ARC_MAILBOXR            (MEI_TO_ARC_MAILBOX + 0x2C)

#define ARC_TO_MEI_MAILBOX             0xDFA0
#define ARC_MEI_MAILBOXR               (ARC_TO_MEI_MAILBOX + 0x2C)

/** Codeswap request messages are indicated by setting BIT31 */
#define OMB_CODESWAP_MESSAGE_MSG_TYPE_MASK     BITPOS(31)
/** Clear Eoc messages received are indicated by setting BIT17 */
#define OMB_CLEAREOC_INTERRUPT_CODE    BITPOS(17)
/** Reboot request is indicated by bit 18 */
#define OMB_REBOOT_INTERRUPT_CODE      BITPOS(18)

/* Swap page header */
/* Page must be loaded at boot time if size field has BIT31 set */
#define BOOT_FLAG                      BITPOS(31)
#define BOOT_FLAG_MASK                 (~BOOT_FLAG)

#define FREE_RELOAD                    1
#define FREE_SHOWTIME                  2
#define FREE_ALL                       3

/* helper marcos */
#define IFX_MEI_WRITE_REGISTER_L(data, addr) (*((volatile u32*)(addr)) = (u32)(data))
#define IFX_MEI_READ_REGISTER_L(addr)  (*((volatile u32*)(addr)))

#define SET_BITFIELD(reg, mask, off, val)    {reg &= (~mask); reg |= (val << off); }
#define SET_BIT(reg, mask)             (reg |= (mask))
#define CLEAR_BIT(reg, mask)           (reg &= (~mask))
#define CLEAR_BITS(reg, mask)          CLEAR_BIT(reg, mask)
#define SET_BITS(reg, mask)            SET_BIT(reg, mask)

/* swap marco */
#define MEI_HALF_WORD_SWAP(data) {data = ((data & 0xffff)<<16) + ((data & 0xffff0000)>>16); }

#define BSP_DFEIR                      0
#define BSP_DYING_GASP                 1

#define MEI_CODESWAP_COMPLETE          0x0002

#define IFX_MEI_EMSG(fmt, args...)     printk(KERN_ERR  "[%s %d]: " fmt, __func__, __LINE__, ##args)
#define IFX_MEI_DMSG(fmt, args...)     printk(KERN_INFO "[%s %d]: " fmt, __func__, __LINE__, ##args)

#define NUM_OF_REG_ENTRY               (sizeof(regs[0])/sizeof(reg_entry_t))

/** Swap page header describes size in 32-bit words, load location, and image offset
   for program and/or data segments */
typedef struct _arc_swp_page_hdr {
	u32 p_offset;	/**< Offset bytes of progseg from beginning of image */
	u32 p_dest;		/**< Destination addr of progseg on processor */
	u32 p_size;		/**< Size in 32-bitwords of program segment */
	u32 d_offset;	/**< Offset bytes of dataseg from beginning of image */
	u32 d_dest;		/**< Destination addr of dataseg on processor */
	u32 d_size;		/**< Size in 32-bitwords of data segment */
} ARC_SWP_PAGE_HDR;

/*	Swap image header */
/** Flag used for program mem segment */
#define GET_PROG                       0
/** Flag used for data mem segment */
#define GET_DATA                       1

/** Image header contains size of image, checksum for image, and count of page
   headers. Following that are 'count' page headers followed by the code and/or
   data segments to be loaded */
typedef struct _arc_img_hdr {
	/** size of binary image in bytes */
	u32 size;
	/** checksum for image */
	u32 checksum;
	/** count of swap pages in image */
	u32 count;
	/** Should be "count" pages - '1' to make compiler happy */
	ARC_SWP_PAGE_HDR page[1];
} ARC_IMG_HDR;

typedef struct smmu_mem_info {
	int type;
	unsigned long nCopy;
	unsigned long size;
	unsigned char *address;
	unsigned char *org_address;
} smmu_mem_info_t;

#ifdef __KERNEL__
typedef struct ifx_mei_device_private {
	int modem_ready;
	int arcmsgav;
	int cmv_reply;
	int cmv_waiting;
	/* Mei to ARC CMV count, reply count, ARC Indicator count */
	int modem_ready_cnt;
	int cmv_count;
	int reply_count;
	unsigned long image_size;
	int nBar;
	u16 Recent_indicator[MEI_MSG_LENGTH];
	u16 CMV_RxMsg[MEI_MSG_LENGTH] __attribute__ ((aligned (4)));
	smmu_mem_info_t adsl_mem_info[BAR_COUNT];
	ARC_IMG_HDR *img_hdr;
	/* to wait for arc cmv reply, sleep on wait_queue_arcmsgav; */
	wait_queue_head_t wait_queue_arcmsgav;
	wait_queue_head_t wait_queue_modemready;
	struct mutex mei_cmv_mutex;
} ifx_mei_device_private_t;
#define DSL_DEV_PRIVATE(dev)  ((ifx_mei_device_private_t *)(dev->pPriv))
#endif


/** Defines all possible CMV groups */
typedef enum {
   DSL_CMV_GROUP_CNTL = 1,
   DSL_CMV_GROUP_STAT = 2,
   DSL_CMV_GROUP_INFO = 3,
   DSL_CMV_GROUP_TEST = 4,
   DSL_CMV_GROUP_OPTN = 5,
   DSL_CMV_GROUP_RATE = 6,
   DSL_CMV_GROUP_PLAM = 7,
   DSL_CMV_GROUP_CNFG = 8
} DSL_CmvGroup_t;

/** Defines all opcode types */
typedef enum {
   H2D_CMV_READ = 0x00,
   H2D_CMV_WRITE = 0x04,
   H2D_CMV_INDICATE_REPLY = 0x10,
   H2D_ERROR_OPCODE_UNKNOWN = 0x20,
   H2D_ERROR_CMV_UNKNOWN = 0x30,

   D2H_CMV_READ_REPLY = 0x01,
   D2H_CMV_WRITE_REPLY = 0x05,
   D2H_CMV_INDICATE = 0x11,
   D2H_ERROR_OPCODE_UNKNOWN = 0x21,
   D2H_ERROR_CMV_UNKNOWN = 0x31,
   D2H_ERROR_CMV_READ_NOT_AVAILABLE = 0x41,
   D2H_ERROR_CMV_WRITE_ONLY = 0x51,
   D2H_ERROR_CMV_READ_ONLY = 0x61,

   H2D_DEBUG_READ_DM = 0x02,
   H2D_DEBUG_READ_PM = 0x06,
   H2D_DEBUG_WRITE_DM = 0x0a,
   H2D_DEBUG_WRITE_PM = 0x0e,

   D2H_DEBUG_READ_DM_REPLY = 0x03,
   D2H_DEBUG_READ_FM_REPLY = 0x07,
   D2H_DEBUG_WRITE_DM_REPLY = 0x0b,
   D2H_DEBUG_WRITE_FM_REPLY = 0x0f,
   D2H_ERROR_ADDR_UNKNOWN = 0x33,

   D2H_AUTONOMOUS_MODEM_READY_MSG = 0xf1
} DSL_CmvOpcode_t;

#ifdef CONFIG_PROC_FS
typedef struct {
	int *flag;
	char name[30];
	char description[100];
	unsigned long low_ino;
} reg_entry_t;
#endif

#endif /* _IFXMIPS_MEI_BSP_H_ */
