/*
   Copyright (c) 2006 - 2008, Infineon Technologies AG

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This file contains the internal definitions of the Managed Entity Interface
   driver.
   The MEI driver is the interface to the DSL PHY mailbox. 
   This file represents the upper layer to be used by the DSL CPE API driver 
   for communication with the DSL PHY firmware.
*/
#ifndef _IFXMIPS_MEI_INTERFACE_H_
#define _IFXMIPS_MEI_INTERFACE_H_

#define IFXMIPS_MEI_DEVNAME				"ifx_mei"
#define MEI_DIRNAME							"ifx_mei"

#ifndef BSP_MAX_DEVICES
#	define BSP_MAX_DEVICES					1
#endif
#ifndef IFX_MEI_MAJOR_NUMBER
#	define IFX_MEI_MAJOR_NUMBER			105
#endif
#ifdef CONFIG_PROC_FS
#	define PROC_ITEMS							12
#endif
#ifdef CONFIG_AMAZON_SE
#  define FPI_BUG_FIX	/* todo, take ASE into acoount within ifxmips */
#endif


/*******************************************************************************
* DSL CPE API Driver Stack Interface Definitions
* *****************************************************************************/

/** IOCTL codes for bsp driver */
#define DSL_IOC_MEI_BSP_MAGIC          's'

#define DSL_FIO_BSP_DSL_START          _IO(DSL_IOC_MEI_BSP_MAGIC, 0)
#define DSL_FIO_BSP_RUN                _IO(DSL_IOC_MEI_BSP_MAGIC, 1)
#define DSL_FIO_BSP_FREE_RESOURCE      _IO(DSL_IOC_MEI_BSP_MAGIC, 2)
#define DSL_FIO_BSP_RESET              _IO(DSL_IOC_MEI_BSP_MAGIC, 3)
#define DSL_FIO_BSP_REBOOT             _IO(DSL_IOC_MEI_BSP_MAGIC, 4)
#define DSL_FIO_BSP_HALT               _IO(DSL_IOC_MEI_BSP_MAGIC, 5)
#define DSL_FIO_BSP_BOOTDOWNLOAD       _IO(DSL_IOC_MEI_BSP_MAGIC, 6)
#define DSL_FIO_BSP_JTAG_ENABLE        _IO(DSL_IOC_MEI_BSP_MAGIC, 7)
#define DSL_FIO_FREE_RESOURCE          _IO(DSL_IOC_MEI_BSP_MAGIC, 8)
#define DSL_FIO_ARC_MUX_TEST           _IO(DSL_IOC_MEI_BSP_MAGIC, 9)
#define DSL_FIO_BSP_REMOTE             _IOW(DSL_IOC_MEI_BSP_MAGIC, 10, u32)
#define DSL_FIO_BSP_GET_BASE_ADDRESS   _IOR(DSL_IOC_MEI_BSP_MAGIC, 11, u32)
#define DSL_FIO_BSP_IS_MODEM_READY     _IOR(DSL_IOC_MEI_BSP_MAGIC, 12, u32)
#define DSL_FIO_BSP_GET_VERSION        _IOR(DSL_IOC_MEI_BSP_MAGIC, 13, DSL_DEV_Version_t)
#define DSL_FIO_BSP_CMV_WINHOST        _IOWR(DSL_IOC_MEI_BSP_MAGIC, 14, DSL_DEV_WinHost_Message_t) 
#define DSL_FIO_BSP_CMV_READ           _IOWR(DSL_IOC_MEI_BSP_MAGIC, 15, DSL_DEV_MeiReg_t)
#define DSL_FIO_BSP_CMV_WRITE          _IOW(DSL_IOC_MEI_BSP_MAGIC, 16, DSL_DEV_MeiReg_t)
#define DSL_FIO_BSP_DEBUG_READ         _IOWR(DSL_IOC_MEI_BSP_MAGIC, 17, DSL_DEV_MeiDebug_t)
#define DSL_FIO_BSP_DEBUG_WRITE        _IOWR(DSL_IOC_MEI_BSP_MAGIC, 18, DSL_DEV_MeiDebug_t)
#define DSL_FIO_BSP_GET_CHIP_INFO      _IOR(DSL_IOC_MEI_BSP_MAGIC, 19, DSL_DEV_HwVersion_t)

#define MEI_DEBUG_BUFFER_SIZES         512

#define MEI_MSG_LENGTH                 16

typedef struct
{
	unsigned int iAddress;
	unsigned int iCount;
	unsigned int buffer[MEI_DEBUG_BUFFER_SIZES];
} DSL_DEV_MeiDebug_t;

/**
	Structure is used for debug access only.
	Refer to configure option INCLUDE_ADSL_WINHOST_DEBUG */
typedef struct
{
	/** Specifies that address for debug access */
	unsigned long iAddress;
	/** Specifies the pointer to the data that has to be written or returns a
		pointer to the data that has been read out*/
	unsigned long iData;
} DSL_DEV_MeiReg_t;

typedef struct
{
	int nInUse;                /* modem state, update by bsp driver, */
	void *pPriv;
	unsigned int base_address;       /* mei base address */
	int nIrq[2];               /* irq number */
	DSL_DEV_MeiDebug_t lop_debugwr;  /* dying gasp */
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,6,0))
	struct module *owner;
#endif
} DSL_DEV_Device_t;

typedef struct {
	unsigned long major;
	unsigned long minor;
	unsigned long revision;
} DSL_DEV_Version_t;

typedef struct {
	unsigned long major;
	unsigned long minor;
} DSL_DEV_HwVersion_t;

typedef struct {
	unsigned char dummy;
} DSL_DEV_DeviceConfig_t;

/** error code definitions */
typedef enum {
	DSL_DEV_MEI_ERR_SUCCESS = 0,
	DSL_DEV_MEI_ERR_FAILURE = -1,
	DSL_DEV_MEI_ERR_MAILBOX_FULL = -2,
	DSL_DEV_MEI_ERR_MAILBOX_EMPTY = -3,
	DSL_DEV_MEI_ERR_MAILBOX_TIMEOUT = -4
} DSL_DEV_MeiError_t;

typedef enum {
	DSL_BSP_MEMORY_READ=0,
	DSL_BSP_MEMORY_WRITE,
} DSL_BSP_MemoryAccessType_t;

typedef enum {
	DSL_LED_LINK_ID=0,
	DSL_LED_DATA_ID
} DSL_DEV_LedId_t;

typedef enum {
	DSL_LED_LINK_TYPE=0,
	DSL_LED_DATA_TYPE
} DSL_DEV_LedType_t;

typedef enum {
	DSL_LED_HD_CPU=0,
	DSL_LED_HD_FW
} DSL_DEV_LedHandler_t;

typedef enum {
	DSL_LED_ON=0,
	DSL_LED_OFF,
	DSL_LED_FLASH,
} DSL_DEV_LedMode_t;

typedef enum {
	DSL_CPU_HALT=0,
	DSL_CPU_RUN,
	DSL_CPU_RESET,
} DSL_DEV_CpuMode_t;

#if 0
typedef enum {
	DSL_BSP_EVENT_DYING_GASP = 0,
	DSL_BSP_EVENT_CEOC_IRQ,
} DSL_BSP_Event_id_t;

typedef union DSL_BSP_CB_Param
{
	unsigned int nIrqMessage;
} DSL_BSP_CB_Param_t;

typedef struct DSL_BSP_CB_Event
{
	DSL_BSP_Event_id_t nID;
	DSL_DEV_Device_t *pDev;
	DSL_BSP_CB_Param_t *pParam;
} DSL_BSP_CB_Event_t;
#endif

typedef struct winhost_message {
	union {
		u16 RxMessage[MEI_MSG_LENGTH] __attribute__ ((aligned (4)));
		u16 TxMessage[MEI_MSG_LENGTH] __attribute__ ((aligned (4)));
	} msg;
} DSL_DEV_WinHost_Message_t;

/**
* Dummy structure by now to show mechanism of extended data that will be
* provided within event callback itself.
* */
typedef struct
{
	/** Dummy value */
	unsigned int nDummy1;
} DSL_BSP_CB_Event1DataDummy_t;

/**
* Dummy structure by now to show mechanism of extended data that will be
* provided within event callback itself.
* */
typedef struct {
	/** Dummy value */
	unsigned int nDummy2;
} DSL_BSP_CB_Event2DataDummy_t;

/**
* encapsulate all data structures that are necessary for status event callbacks.
* */
typedef union {
	DSL_BSP_CB_Event1DataDummy_t dataEvent1;
	DSL_BSP_CB_Event2DataDummy_t dataEvent2;
} DSL_BSP_CB_DATA_Union_t;

typedef enum {
	/**
	* Informs the upper layer driver (DSL CPE API) about a reboot request from the
	* firmware.
	* \note This event does NOT include any additional data.
	*       More detailed information upon reboot reason has to be requested from
	*       upper layer software via CMV (INFO 109) if necessary. */
	DSL_BSP_CB_FIRST = 0,
		DSL_BSP_CB_DYING_GASP,
	DSL_BSP_CB_CEOC_IRQ,
	DSL_BSP_CB_FIRMWARE_REBOOT,
	/** Delimiter only */
	DSL_BSP_CB_LAST
} DSL_BSP_CB_Type_t;

/**
* Specifies the common event type that has to be used for registering and
* signalling of interrupts/autonomous status events from MEI BSP Driver.
*
* \param pDev
* Context pointer from MEI BSP Driver.
* \param IFX_ADSL_BSP_CallbackType_t
* Specifies the event callback type (reason of callback). Regrading to the
* setting of this value the data which is included in the following union
* might have different meanings.
* Please refer to the description of the union to get information about the
* meaning of the included data.
* \param pData
* Data according to \ref DSL_BSP_CB_DATA_Union_t.
* If this pointer is NULL there is no additional data available.
*
* \return depending on event
*/
typedef int (*DSL_BSP_EventCallback_t)
(
	DSL_DEV_Device_t *pDev,
	DSL_BSP_CB_Type_t nCallbackType,
	DSL_BSP_CB_DATA_Union_t *pData
);

typedef struct {
		DSL_BSP_EventCallback_t function;
		DSL_BSP_CB_Type_t       event;
		DSL_BSP_CB_DATA_Union_t *pData;
} DSL_BSP_EventCallBack_t;

extern int DSL_BSP_EventCBRegister(DSL_BSP_EventCallBack_t *);
extern int DSL_BSP_EventCBUnregister(DSL_BSP_EventCallBack_t *);

/** Modem states */
#define DSL_DEV_STAT_InitState				0x0000
#define DSL_DEV_STAT_ReadyState				0x0001
#define DSL_DEV_STAT_FailState				0x0002
#define DSL_DEV_STAT_IdleState				0x0003
#define DSL_DEV_STAT_QuietState				0x0004
#define DSL_DEV_STAT_GhsState					0x0005
#define DSL_DEV_STAT_FullInitState			0x0006
#define DSL_DEV_STAT_ShowTimeState			0x0007
#define DSL_DEV_STAT_FastRetrainState		0x0008
#define DSL_DEV_STAT_LoopDiagMode			0x0009
#define DSL_DEV_STAT_ShortInit				0x000A     /* Bis short initialization */

/* external functions (from the BSP Driver) */
extern DSL_DEV_Device_t* DSL_BSP_DriverHandleGet(int, int);
extern int DSL_BSP_DriverHandleDelete(DSL_DEV_Device_t *);
extern int DSL_BSP_ATMLedCBRegister( int (*ifx_adsl_ledcallback)(void));
extern volatile DSL_DEV_Device_t *adsl_dev;

DSL_DEV_MeiError_t DSL_BSP_FWDownload(DSL_DEV_Device_t *, const char *, unsigned long, long *, long *);
DSL_DEV_MeiError_t DSL_BSP_Showtime(DSL_DEV_Device_t *, unsigned int, unsigned int);
DSL_DEV_MeiError_t DSL_BSP_AdslLedInit(DSL_DEV_Device_t *, DSL_DEV_LedId_t, DSL_DEV_LedType_t, DSL_DEV_LedHandler_t);
DSL_DEV_MeiError_t DSL_BSP_AdslLedSet(DSL_DEV_Device_t *, DSL_DEV_LedId_t, DSL_DEV_LedMode_t);
DSL_DEV_MeiError_t DSL_BSP_MemoryDebugAccess(DSL_DEV_Device_t *, DSL_BSP_MemoryAccessType_t, unsigned int, unsigned int*, unsigned int);
DSL_DEV_MeiError_t DSL_BSP_SendCMV(DSL_DEV_Device_t *, u16 *, int, u16 *);
DSL_DEV_MeiError_t DSL_BSP_AdslLedTest(DSL_DEV_Device_t *pDev);
int DSL_BSP_KernelIoctls(DSL_DEV_Device_t *, unsigned int, unsigned long);

void IFX_MEI_ARC_MUX_Test(void);

#endif /* _IFXMIPS_MEI_INTERFACE_H_ */
