/*
   Copyright (c) 2006 - 2008, Infineon Technologies AG

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.
*/
#ifndef _IFXMIPS_MEI_LINUX_H_
#define _IFXMIPS_MEI_LINUX_H_

#include <linux/kernel.h>
#include <linux/module.h>
#include <linux/version.h>
#include <linux/types.h>
#include <linux/fs.h>
#include <linux/mm.h>
#include <linux/errno.h>
#include <linux/interrupt.h>
#include <linux/netdevice.h>
#include <linux/etherdevice.h>
#include <linux/version.h>
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 26))
#include <linux/semaphore.h>
#else
#include <asm/semaphore.h>
#endif
#include <linux/init.h>
#include <linux/ioport.h>
#include <linux/uaccess.h>

#ifdef CONFIG_DEVFS_FS
#include <linux/devfs_fs_kernel.h>
#endif
#ifdef CONFIG_PROC_FS
#include <linux/proc_fs.h>
#endif
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2,6,17))
#include <linux/utsrelease.h>
#endif

#include <linux/list.h>
#include <linux/delay.h>
#define __LINUX__

#include <asm/ifxmips/ifxmips.h>
#include <asm/ifxmips/ifxmips_irq.h>
#include <asm/ifxmips/ifxmips_gpio.h>
#include <asm/ifxmips/ifxmips_led.h>
#include <asm/ifxmips/ifxmips_irq.h>
#include <asm/ifxmips/ifxmips_pmu.h>

#define MEI_LOCKINT(var) local_irq_save(var)
#define MEI_UNLOCKINT(var) local_irq_restore(var)

#define MEI_MUTEX_INIT(mutex) mutex_init(&mutex)
#define MEI_MUTEX_LOCK(mutex) mutex_lock(&mutex)
#define MEI_MUTEX_UNLOCK(mutex) mutex_unlock(&mutex)

#define MEI_WAIT(ms) \
        {\
                set_current_state(TASK_INTERRUPTIBLE);\
                schedule_timeout(ms);\
        }

#define MEI_INIT_WAKELIST(name,queue) \
        init_waitqueue_head(&queue)

/* wait for an event, timeout is measured in ms */
#define MEI_WAIT_EVENT_TIMEOUT(ev,condition,timeout)\
        wait_event_interruptible_timeout(ev,condition, timeout * HZ / 1000)

#define MEI_WAKEUP_EVENT(ev)\
        wake_up_interruptible(&ev)

typedef struct semaphore MEI_mutex_t;

#endif /* _IFXMIPS_MEI_LINUX_H_ */
