/*
   Copyright (c) 2006 - 2008, Infineon Technologies AG

   This program is free software; you can redistribute it and/or modify
   it under the terms of the GNU General Public License as published by
   the Free Software Foundation; either version 2 of the License, or
   (at your option) any later version.

   This version is based on the version 3.08.00 / 04/08/2008 KH Teh.
   Changes:
   - code cleanup
   - removed several test routines, redundant code fragments
*******************************************************************************/

#include <linux/proc_fs.h>
#include <asm/ifxmips/ifxmips_mei_linux.h>
#include <asm/ifxmips/ifxmips_mei_interface.h>
#include <asm/ifxmips/ifxmips_mei_bsp.h>
#include <asm/ifxmips/ifxmips_irq.h>

/* todo: check if this hack is needed */
void ifx_usb_enable_afe_oc(void);

DSL_DEV_Version_t bsp_mei_version = {
	.major = 3,
	.minor = 8,
	.revision = 0
};

DSL_DEV_HwVersion_t bsp_chip_info;

int (*IFX_ADSL_LEDHandler)(void) = NULL;

static DSL_DEV_MeiError_t IFX_MEI_RunAdslModem(DSL_DEV_Device_t *);
static DSL_DEV_MeiError_t IFX_MEI_CpuModeSet(DSL_DEV_Device_t *, DSL_DEV_CpuMode_t);
static DSL_DEV_MeiError_t IFX_MEI_DownloadBootCode(DSL_DEV_Device_t *);
static DSL_DEV_MeiError_t IFX_MEI_ArcJtagEnable(DSL_DEV_Device_t *, int);
static DSL_DEV_MeiError_t IFX_MEI_AdslMailboxIRQEnable(DSL_DEV_Device_t *, int);
static int IFX_MEI_GetPage(DSL_DEV_Device_t *, u32, u32, u32, u32 *, u32 *);
static int IFX_MEI_BarUpdate(DSL_DEV_Device_t *, int);
static ssize_t IFX_MEI_Write(struct file *, const char *, size_t, loff_t *);
static int IFX_MEI_UserIoctls(struct inode *, struct file *, unsigned int, unsigned long);
static int IFX_MEI_Open(struct inode *, struct file *);
static int IFX_MEI_Release(struct inode *, struct file *);
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 0))
static irqreturn_t IFX_MEI_IrqHandle(int int1, void *void0);
#else
static void IFX_MEI_IrqHandle(int int1, void *void0, struct pt_regs *regs);
#endif

void (*IFX_MEI_set_cell_rate)(int, u32);
static u32 *mei_arc_swap_buff = NULL;	/*   holding swap pages */
struct kmem_cache *mei_cache = NULL;
char *mei_cache_bar[BAR_PAGE_CNT] = {NULL};

static int dev_major = IFX_MEI_MAJOR_NUMBER;
static int ifx_mei_debug = 0;

static struct file_operations bsp_mei_operations = {
		.owner = THIS_MODULE,
		.open = IFX_MEI_Open,
		.release = IFX_MEI_Release,
		.write = IFX_MEI_Write,
		.ioctl = IFX_MEI_UserIoctls,
};

#ifdef CONFIG_PROC_FS
static int IFX_MEI_ProcRead(struct file *, char *, size_t, loff_t *);
static ssize_t IFX_MEI_ProcWrite(struct file *, const char *, size_t, loff_t *);
#endif

#ifdef CONFIG_DEVFS_FS
static devfs_handle_t ifx_mei_devfs_handle[BSP_MAX_DEVICES];
#endif

#ifdef CONFIG_PROC_FS
static struct proc_dir_entry *meidir;
static struct file_operations IFX_MEI_ProcOperations = {
		.read = IFX_MEI_ProcRead,
		.write = IFX_MEI_ProcWrite,
};
/* total items to be monitored by /proc/mei */
static reg_entry_t regs[BSP_MAX_DEVICES][PROC_ITEMS];
#endif

static DSL_DEV_Device_t         dsl_devices[BSP_MAX_DEVICES];
static ifx_mei_device_private_t IFXMIPS_MEI_Private[BSP_MAX_DEVICES];

static DSL_BSP_EventCallBack_t dsl_bsp_event_callback[DSL_BSP_CB_LAST + 1];
DSL_BSP_CB_Type_t dsl_bsp_callback_event;

/**
	Write a value to register.
	This function writes a value to MEI register
	\param   ul_address  The address to write
	\param   ul_data     The value to write
 */
static void IFX_MEI_LongWordWrite(u32 ul_address, u32 ul_data)
{
	IFX_MEI_WRITE_REGISTER_L(ul_data, ul_address);
	wmb();
//printk("W%08x: %08x\n", ul_address, ul_data);
}

/**
	Write a value to register
	This function writes a value to danube register
	\param   pDev        The device pointer
	\param   ul_address  The address to write
	\param   ul_data     The value to write
 */
static void IFX_MEI_LongWordWriteOffset(DSL_DEV_Device_t *pDev, u32 ul_address, u32 ul_data)
{
	IFX_MEI_WRITE_REGISTER_L(ul_data, pDev->base_address + ul_address);
	wmb();
//printk("W%08x: %08x\n", ul_address, ul_data);
}

/**
	Read the MEI register
	This function read the value from MEI register
	\param   ul_address  The address to write
	\param   pul_data    Pointer to the data
*/
static void IFX_MEI_LongWordRead(u32 ul_address, u32 *pul_data)
{
	*pul_data = IFX_MEI_READ_REGISTER_L(ul_address);
	rmb();
//printk("R%08x: %08x\n", ul_address, *pul_data);
}

/**
	Read the MEI register
	This function read the value from MEI register

	\param   pDev        the device pointer
	\param   ul_address  The address to write
	\param   pul_data    Pounsigned intinter to the data
*/
static void IFX_MEI_LongWordReadOffset(DSL_DEV_Device_t *pDev, u32 ul_address, u32 *pul_data)
{
	*pul_data = IFX_MEI_READ_REGISTER_L(pDev->base_address + ul_address);
	rmb();
//printk("R%08x: %08x\n", ul_address, *pul_data);
}

/**
	Write several DWORD datas to ARC memory via ARC DMA interface
	This function writes several DWORD datas to ARC memory via DMA interface.
	\param   pDev            the device pointer
	\param   destaddr        The address to write
	\param   databuff        Pointer to the data buffer
	\param   databuffsize    Number of DWORDs to write
	\return
	- DSL_DEV_MEI_ERR_SUCCESS or
	- DSL_DEV_MEI_ERR_FAILURE
 */
static DSL_DEV_MeiError_t IFX_MEI_DMAWrite(DSL_DEV_Device_t *pDev, u32 destaddr, u32 *databuff, u32 databuffsize)
{
	u32 *p = databuff;
	u32 temp;

	if (destaddr & 3)
		return DSL_DEV_MEI_ERR_FAILURE;

	/*  Set the write transfer address */
	IFX_MEI_LongWordWriteOffset(pDev, MEI_XFR_ADDR_OFFSET, destaddr);

	/*  Write the data pushed across DMA */
	while (databuffsize--) {
		temp = *p;
		if (destaddr == MEI_TO_ARC_MAILBOX)
			MEI_HALF_WORD_SWAP (temp);
		IFX_MEI_LongWordWriteOffset(pDev, (u32) MEI_DATA_XFR_OFFSET, temp);
		p++;
	}

	return DSL_DEV_MEI_ERR_SUCCESS;
}

/**
	Read several DWORD datas from ARC memory via ARC DMA interface
	This function reads several DWORD datas from ARC memory via DMA interface.
	\param   pDev           the device pointer
	\param   srcaddr        the address to read
	\param   databuff       pointer to the data buffer
	\param   databuffsize   number of DWORDs to read
	\return
	- DSL_DEV_MEI_ERR_SUCCESS or
	- DSL_DEV_MEI_ERR_FAILURE
 */
static DSL_DEV_MeiError_t IFX_MEI_DMARead(DSL_DEV_Device_t *pDev, u32 srcaddr, u32 *databuff, u32 databuffsize)
{
	u32 *p = databuff;
	u32 temp;
	if (srcaddr & 3)
		return DSL_DEV_MEI_ERR_FAILURE;

	/*  Set the read transfer address */
	IFX_MEI_LongWordWriteOffset(pDev, (u32) MEI_XFR_ADDR_OFFSET, srcaddr);

	/*  Read the data popped across DMA */
	while (databuffsize--) {
		IFX_MEI_LongWordReadOffset(pDev, (u32) MEI_DATA_XFR_OFFSET, &temp);
		/*  swap half word */
		if (databuff == (u32 *) DSL_DEV_PRIVATE(pDev)->CMV_RxMsg)
			MEI_HALF_WORD_SWAP (temp);
		*p = temp;
		p++;
	}

	return DSL_DEV_MEI_ERR_SUCCESS;
}

/**
	Switch the ARC control mode
	This function switchs the ARC control mode to JTAG or MEI mode.
	\param   pDev  the device pointer
	\param   mode  new ARC mode:
				- JTAG_MASTER_MODE or
				- MEI_MASTER_MODE.
 */
static void IFX_MEI_ControlModeSet(DSL_DEV_Device_t *pDev, int mode)
{
	u32 temp = 0x0;
	IFX_MEI_LongWordReadOffset(pDev, (u32) MEI_DBG_MASTER_OFFSET, &temp);
	switch (mode) {
	case JTAG_MASTER_MODE:
		temp &= ~(HOST_MSTR);
		break;
	case MEI_MASTER_MODE:
		temp |= (HOST_MSTR);
		break;
	default:
	IFX_MEI_EMSG("IFX_MEI_ControlModeSet: unknown mode [%d]\n", mode);
		return;
	}
	IFX_MEI_LongWordWriteOffset(pDev, (u32) MEI_DBG_MASTER_OFFSET, temp);
}

/**
	Disable ARC to MEI interrupt
	\param pDev    the device pointer
*/
static void IFX_MEI_IRQDisable(DSL_DEV_Device_t *pDev)
{
	IFX_MEI_LongWordWriteOffset(pDev, (u32) ARC_TO_MEI_INT_MASK_OFFSET, 0x0);
}

/**
	Enable ARC to MEI interrupt
	\param pDev    the device pointer
*/
static void IFX_MEI_IRQEnable(DSL_DEV_Device_t *pDev)
{
	IFX_MEI_LongWordWriteOffset(pDev, (u32)ARC_TO_MEI_INT_MASK_OFFSET, MSGAV_EN);
}

/**
	Poll for transaction complete signal
	This function polls and waits for transaction complete signal.
	\param pDev    the device pointer
	\return
	- DSL_DEV_MEI_ERR_SUCCESS or
	- DSL_DEV_MEI_ERR_FAILURE
 */
static DSL_DEV_MeiError_t meiPollForDbgDone(DSL_DEV_Device_t *pDev)
{
	u32 query = 0;
	int i;
	for (i = 0; i < WHILE_DELAY && !(query & ARC_TO_MEI_DBG_DONE); i++)
		IFX_MEI_LongWordReadOffset(pDev, (u32) ARC_TO_MEI_INT_OFFSET, &query);
	if (i >= WHILE_DELAY) {
		IFX_MEI_EMSG("PollforDbg fail!\n");
		return DSL_DEV_MEI_ERR_FAILURE;
	} else {
		IFX_MEI_LongWordWriteOffset(pDev, (u32) ARC_TO_MEI_INT_OFFSET, ARC_TO_MEI_DBG_DONE);
		return DSL_DEV_MEI_ERR_SUCCESS;
		}
	}

/**
	ARC Debug Memory Access for a single DWORD reading.
	This function used for direct, address-based access to ARC memory.
	\param   pDev        the device pointer
	\param   DEC_mode    ARC memory space to used
	\param   address     address to read
	\param   data        pointer to data
	\return
	- DSL_DEV_MEI_ERR_SUCCESS or
	- DSL_DEV_MEI_ERR_FAILURE
 */
static DSL_DEV_MeiError_t _IFX_MEI_DBGLongWordRead(DSL_DEV_Device_t *pDev, u32 DEC_mode, u32 address, u32 *data)
{
	IFX_MEI_LongWordWriteOffset(pDev, (u32) MEI_DEBUG_DEC_OFFSET, DEC_mode);
	IFX_MEI_LongWordWriteOffset(pDev, (u32) MEI_DEBUG_RAD_OFFSET, address);
	if (meiPollForDbgDone(pDev) == DSL_DEV_MEI_ERR_SUCCESS) {
		IFX_MEI_LongWordReadOffset(pDev, (u32) MEI_DEBUG_DATA_OFFSET, data);
		return DSL_DEV_MEI_ERR_SUCCESS;
	} else {
printk("_IFX_MEI_DBGLongWordRead failed\n");
		return DSL_DEV_MEI_ERR_FAILURE;
	}
}

/**
	ARC Debug Memory Access for a single DWORD writing.
	This function used for direct, address-based access to ARC memory.
	\param   pDev        the device pointer
	\param   DEC_mode    ARC memory space to used
	\param   address     the address to write
	\param   data        the data to write
	\return
	- DSL_DEV_MEI_ERR_SUCCESS or
	- DSL_DEV_MEI_ERR_FAILURE
 */
static DSL_DEV_MeiError_t _IFX_MEI_DBGLongWordWrite(DSL_DEV_Device_t *pDev, u32 DEC_mode, u32 address, u32 data)
{
	IFX_MEI_LongWordWriteOffset(pDev, (u32) MEI_DEBUG_DEC_OFFSET, DEC_mode);
	IFX_MEI_LongWordWriteOffset(pDev, (u32) MEI_DEBUG_WAD_OFFSET, address);
	IFX_MEI_LongWordWriteOffset(pDev, (u32) MEI_DEBUG_DATA_OFFSET, data);
	return meiPollForDbgDone(pDev);
}

/**
	ARC Debug Memory Access for writing.
	This function used for direct, address-based access to ARC memory.
	\param   pDev           the device pointer
	\param   destaddr       the address to read
	\param   databuffer     pointer to data
	\param   databuffsize   the number of DWORDs to read
	\return
	- DSL_DEV_MEI_ERR_SUCCESS or
	- DSL_DEV_MEI_ERR_FAILURE
 */
static DSL_DEV_MeiError_t IFX_MEI_DebugWrite(DSL_DEV_Device_t *pDev, u32 destaddr, u32 *databuff, u32 databuffsize)
{
	u32 i;
	u32 temp = 0x0;
	u32 address = 0x0;
	u32 *buffer = 0x0;

	/*  Open the debug port before DMP memory write */
	IFX_MEI_ControlModeSet(pDev, MEI_MASTER_MODE);

	IFX_MEI_LongWordWriteOffset(pDev, (u32) MEI_DEBUG_DEC_OFFSET, MEI_DEBUG_DEC_DMP1_MASK);

	/*  For the requested length, write the address and write the data */
	address = destaddr;
	buffer = databuff;
	for (i = 0; i < databuffsize; i++) {
		temp = *buffer;
		_IFX_MEI_DBGLongWordWrite(pDev, MEI_DEBUG_DEC_DMP1_MASK, address, temp);
		address += 4;
		buffer++;
	}

	/*  Close the debug port after DMP memory write */
	IFX_MEI_ControlModeSet(pDev, JTAG_MASTER_MODE);

	return DSL_DEV_MEI_ERR_SUCCESS;
}

/**
	ARC Debug Memory Access for reading.
	This function used for direct, address-based access to ARC memory.
	\param   pDev           the device pointer
	\param   srcaddr        the address to read
	\param   databuffer     Pointer to data
	\param   databuffsize   The number of DWORDs to read
	\return
	- DSL_DEV_MEI_ERR_SUCCESS or
	- DSL_DEV_MEI_ERR_FAILURE
 */
static DSL_DEV_MeiError_t IFX_MEI_DebugRead(DSL_DEV_Device_t *pDev, u32 srcaddr, u32 *databuff, u32 databuffsize)
{
	u32 i;
	u32 temp = 0x0;
	u32 address = 0x0;
	u32 *buffer = 0x0;

	/* Open the debug port before DMP memory read */
	IFX_MEI_ControlModeSet(pDev, MEI_MASTER_MODE);

	IFX_MEI_LongWordWriteOffset(pDev, (u32) MEI_DEBUG_DEC_OFFSET, MEI_DEBUG_DEC_DMP1_MASK);

	/* For the requested length, write the address and read the data */
	address = srcaddr;
	buffer = databuff;
	for (i = 0; i < databuffsize; i++) {
		_IFX_MEI_DBGLongWordRead(pDev, MEI_DEBUG_DEC_DMP1_MASK, address, &temp);
		*buffer = temp;
		address += 4;
		buffer++;
	}

	/* Close the debug port after DMP memory read */
	IFX_MEI_ControlModeSet(pDev, JTAG_MASTER_MODE);

	return DSL_DEV_MEI_ERR_SUCCESS;
}

/**
	Send a message to ARC MailBox.
	This function sends a message to ARC Mailbox via ARC DMA interface.
	\param   pDev           the device pointer
	\param   msgsrcbuffer   Pointer to message.
	\param   msgsize        The number of words to write.
	\return
	- DSL_DEV_MEI_ERR_SUCCESS or
	- DSL_DEV_MEI_ERR_FAILURE
 */
static DSL_DEV_MeiError_t IFX_MEI_MailboxWrite(DSL_DEV_Device_t *pDev, u16 *msgsrcbuffer, u16 msgsize)
{
	int i;
	u32 temp, arc_mailbox_status = 0x0;
	DSL_DEV_MeiError_t meiMailboxError = DSL_DEV_MEI_ERR_SUCCESS;

	/* Write to mailbox */
	meiMailboxError =
		IFX_MEI_DMAWrite(pDev, MEI_TO_ARC_MAILBOX, (u32 *) msgsrcbuffer, msgsize / 2);
	meiMailboxError =
		IFX_MEI_DMAWrite(pDev, MEI_TO_ARC_MAILBOXR, (u32 *) (&temp), 1);

	/* Notify arc that mailbox write completed */
	DSL_DEV_PRIVATE(pDev)->cmv_waiting = 1;
	IFX_MEI_LongWordWriteOffset(pDev, (u32)MEI_TO_ARC_INT_OFFSET, MEI_TO_ARC_MSGAV);

	i = 0;
	while (i++ < WHILE_DELAY) {   /* wait for ARC to clear the bit */
		IFX_MEI_LongWordReadOffset(pDev, (u32) MEI_TO_ARC_INT_OFFSET, &arc_mailbox_status);
		if ((arc_mailbox_status & MEI_TO_ARC_MSGAV) != MEI_TO_ARC_MSGAV)
			break;
		}
	if (i >= WHILE_DELAY) {
		IFX_MEI_EMSG(">>> Timeout waiting for ARC to clear MEI_TO_ARC_MSGAV!!! "\
		"MEI_TO_ARC message size = %d DWORDs <<<\n", msgsize/2);
		meiMailboxError = DSL_DEV_MEI_ERR_FAILURE;
	}
	return meiMailboxError;
}

/**
	Read a message from ARC MailBox.
	This function reads a message from ARC Mailbox via ARC DMA interface.
	\param   pDev           the device pointer
	\param   msgsrcbuffer   Pointer to message.
	\param   msgsize        the number of words to read
	\return
	- DSL_DEV_MEI_ERR_SUCCESS or
	- DSL_DEV_MEI_ERR_FAILURE
*/
static DSL_DEV_MeiError_t IFX_MEI_MailboxRead(DSL_DEV_Device_t *pDev, u16 *msgdestbuffer, u16 msgsize)
{
	DSL_DEV_MeiError_t meiMailboxError = DSL_DEV_MEI_ERR_SUCCESS;
	/* Read from mailbox */
	meiMailboxError = IFX_MEI_DMARead(pDev, ARC_TO_MEI_MAILBOX, (u32 *) msgdestbuffer, msgsize / 2);
	/* Notify arc that mailbox read completed */
	IFX_MEI_LongWordWriteOffset(pDev, (u32) ARC_TO_MEI_INT_OFFSET, ARC_TO_MEI_MSGAV);
	return meiMailboxError;
}

/**
	Download boot pages to ARC.
	This function downloads boot pages to ARC.
	\param   pDev  the device pointer
	\return
	- DSL_DEV_MEI_ERR_SUCCESS or
	- DSL_DEV_MEI_ERR_FAILURE
 */
static DSL_DEV_MeiError_t IFX_MEI_DownloadBootPages(DSL_DEV_Device_t *pDev)
{
	int boot_loop;
	int page_size;
	u32 dest_addr;

	/* DMA the boot code page(s) */
	for (boot_loop = 1; boot_loop < DSL_DEV_PRIVATE(pDev)->img_hdr->count; boot_loop++) {
		if ((DSL_DEV_PRIVATE(pDev)->img_hdr->page[boot_loop].p_size) & BOOT_FLAG) {
			page_size = IFX_MEI_GetPage(pDev, boot_loop, GET_PROG, MAXSWAPSIZE, mei_arc_swap_buff, &dest_addr);		
			if (page_size > 0) {
				IFX_MEI_DMAWrite(pDev, dest_addr, mei_arc_swap_buff, page_size);
			}
		}
		if ((DSL_DEV_PRIVATE(pDev)->img_hdr->page[boot_loop].d_size) & BOOT_FLAG) {
			page_size = IFX_MEI_GetPage(pDev, boot_loop, GET_DATA, MAXSWAPSIZE, mei_arc_swap_buff, &dest_addr);
			if (page_size > 0) {
				IFX_MEI_DMAWrite(pDev, dest_addr, mei_arc_swap_buff, page_size);
			}
		}
	}
	return DSL_DEV_MEI_ERR_SUCCESS;
}

/**
	Initial efuse rar.
	\param   pDev  the device pointer
*/
static void IFX_MEI_FuseInit(DSL_DEV_Device_t *pDev)
{
	u32 data = 0;
	IFX_MEI_DMAWrite(pDev, IRAM0_BASE, &data, 1);
	IFX_MEI_DMAWrite(pDev, IRAM0_BASE + 4, &data, 1);
	IFX_MEI_DMAWrite(pDev, IRAM1_BASE, &data, 1);
	IFX_MEI_DMAWrite(pDev, IRAM1_BASE + 4, &data, 1);
	IFX_MEI_DMAWrite(pDev, BRAM_BASE, &data, 1);
	IFX_MEI_DMAWrite(pDev, BRAM_BASE + 4, &data, 1);
	IFX_MEI_DMAWrite(pDev, ADSL_DILV_BASE, &data, 1);
	IFX_MEI_DMAWrite(pDev, ADSL_DILV_BASE + 4, &data, 1);
}

/**
	efuse rar program
	\todo fix the 0x4000, it's depending on the core speed
 **/
static void IFX_MEI_FuseProg(DSL_DEV_Device_t *pDev)
{
	u32 reg_data, fuse_value;
	int i = 0;
	IFX_MEI_LongWordRead((u32) IFXMIPS_RCU_RST, &reg_data);
	while ((reg_data & 0x10000000) == 0) {
		IFX_MEI_LongWordRead((u32) IFXMIPS_RCU_RST, &reg_data);
		/* add a watchdog */
		i++;
		/* 0x4000 translate to  about 16 ms@111M, so should be enough */
		if (i == 0x4000)
			return;
	}
	/* STEP a: Prepare memory for external accesses */
	/* Write fuse_en bit24 */
	IFX_MEI_LongWordRead((u32) IFXMIPS_RCU_RST, &reg_data);
	IFX_MEI_LongWordWrite((u32) IFXMIPS_RCU_RST, reg_data | (1 << 24));

	IFX_MEI_FuseInit(pDev);
	for (i = 0; i < 4; i++) {
		IFX_MEI_LongWordRead((u32) (IFXMIPS_FUSE_BASE_ADDR) + i * 4, &fuse_value);
		switch (fuse_value & 0xF0000) {
		case 0x80000:
			reg_data = ((fuse_value & RX_DILV_ADDR_BIT_MASK) | (RX_DILV_ADDR_BIT_MASK + 0x1));
			IFX_MEI_DMAWrite(pDev, ADSL_DILV_BASE, &reg_data, 1);
			break;
		case 0x90000:
			reg_data = ((fuse_value & RX_DILV_ADDR_BIT_MASK) | (RX_DILV_ADDR_BIT_MASK + 0x1));
			IFX_MEI_DMAWrite(pDev, ADSL_DILV_BASE + 4, &reg_data, 1);
			break;
		case 0xA0000:
			reg_data = ((fuse_value & IRAM0_ADDR_BIT_MASK) | (IRAM0_ADDR_BIT_MASK + 0x1));
			IFX_MEI_DMAWrite(pDev, IRAM0_BASE, &reg_data, 1);
			break;
		case 0xB0000:
			reg_data = ((fuse_value & IRAM0_ADDR_BIT_MASK) | (IRAM0_ADDR_BIT_MASK + 0x1));
			IFX_MEI_DMAWrite(pDev, IRAM0_BASE + 4, &reg_data, 1);
			break;
		case 0xC0000:
			reg_data = ((fuse_value & IRAM1_ADDR_BIT_MASK) | (IRAM1_ADDR_BIT_MASK + 0x1));
			IFX_MEI_DMAWrite(pDev, IRAM1_BASE, &reg_data, 1);
			break;
		case 0xD0000:
			reg_data = ((fuse_value & IRAM1_ADDR_BIT_MASK) | (IRAM1_ADDR_BIT_MASK + 0x1));
			IFX_MEI_DMAWrite(pDev, IRAM1_BASE + 4, &reg_data, 1);
			break;
		case 0xE0000:
			reg_data = ((fuse_value & BRAM_ADDR_BIT_MASK) | (BRAM_ADDR_BIT_MASK + 0x1));
			IFX_MEI_DMAWrite(pDev, BRAM_BASE, &reg_data, 1);
			break;
		case 0xF0000:
			reg_data = ((fuse_value & BRAM_ADDR_BIT_MASK) | (BRAM_ADDR_BIT_MASK + 0x1));
			IFX_MEI_DMAWrite(pDev, BRAM_BASE + 4, &reg_data, 1);
			break;
		default: /* PPE efuse */
			break;
		}
	}
	IFX_MEI_LongWordRead((u32) IFXMIPS_RCU_RST, &reg_data);
	IFX_MEI_LongWordWrite((u32) IFXMIPS_RCU_RST, reg_data & ~(1 << 24));
	IFX_MEI_LongWordRead((u32) IFXMIPS_RCU_RST, &reg_data);
}

/**
	Enable DFE Clock
	This function enables DFE Clock
	\param pDev    the device pointer
	\return
	- DSL_DEV_MEI_ERR_SUCCESS or
	- DSL_DEV_MEI_ERR_FAILURE
 */
static DSL_DEV_MeiError_t IFX_MEI_EnableCLK(DSL_DEV_Device_t *pDev)
{
	u32 arc_debug_data = 0;
	IFX_MEI_ControlModeSet(pDev, MEI_MASTER_MODE);
	/* enable ac_clk signal */
	_IFX_MEI_DBGLongWordRead(pDev, MEI_DEBUG_DEC_DMP1_MASK, CRI_CCR0, &arc_debug_data);
	arc_debug_data |= ACL_CLK_MODE_ENABLE;
	_IFX_MEI_DBGLongWordWrite(pDev, MEI_DEBUG_DEC_DMP1_MASK, CRI_CCR0, arc_debug_data);
	IFX_MEI_ControlModeSet(pDev, JTAG_MASTER_MODE);
	return DSL_DEV_MEI_ERR_SUCCESS;
}

/**
	Halt the ARC.
	This function halts the ARC.
	\param   pDev     the device pointer
	\return
	- DSL_DEV_MEI_ERR_SUCCESS or
	- DSL_DEV_MEI_ERR_FAILURE
 */
static DSL_DEV_MeiError_t IFX_MEI_HaltArc(DSL_DEV_Device_t *pDev)
{
	u32 arc_debug_data = 0x0;

	/* Switch arc control from JTAG mode to MEI mode */
	IFX_MEI_ControlModeSet(pDev, MEI_MASTER_MODE);
	_IFX_MEI_DBGLongWordRead(pDev, MEI_DEBUG_DEC_AUX_MASK, ARC_DEBUG, &arc_debug_data);
	arc_debug_data |= ARC_DEBUG_HALT;
	_IFX_MEI_DBGLongWordWrite(pDev, MEI_DEBUG_DEC_AUX_MASK, ARC_DEBUG, arc_debug_data);
	/* Switch arc control from MEI mode to JTAG mode */
	IFX_MEI_ControlModeSet(pDev, JTAG_MASTER_MODE);

	MEI_WAIT (10);

	return DSL_DEV_MEI_ERR_SUCCESS;
}

/**
	Run the ARC.
	This function runs the ARC.
	\param   pDev  the device pointer
	\return
	- DSL_DEV_MEI_ERR_SUCCESS or
	- DSL_DEV_MEI_ERR_FAILURE
 */
static DSL_DEV_MeiError_t IFX_MEI_RunArc(DSL_DEV_Device_t *pDev)
{
	u32 arc_debug_data = 0x0;

	/* Switch arc control from JTAG mode to MEI mode- write '1' to bit0 */
	IFX_MEI_ControlModeSet(pDev, MEI_MASTER_MODE);
	_IFX_MEI_DBGLongWordRead(pDev, MEI_DEBUG_DEC_AUX_MASK, AUX_STATUS, &arc_debug_data);
	/* Write debug data reg with content ANDd with 0xFDFFFFFF (halt bit cleared) */
	arc_debug_data &= ~ARC_AUX_HALT;
	_IFX_MEI_DBGLongWordWrite(pDev, MEI_DEBUG_DEC_AUX_MASK, AUX_STATUS, arc_debug_data);
	/* Switch arc control from MEI mode to JTAG mode- write '0' to bit0 */
	IFX_MEI_ControlModeSet(pDev, JTAG_MASTER_MODE);
	/* Enable mask for arc codeswap interrupts */
	IFX_MEI_IRQEnable(pDev);

	return DSL_DEV_MEI_ERR_SUCCESS;
}

/**
	Reset the ARC.
	This function resets the ARC.
	\param   pDev     the device pointer
	\return
	- DSL_DEV_MEI_ERR_SUCCESS or
	- DSL_DEV_MEI_ERR_FAILURE
 */
static DSL_DEV_MeiError_t IFX_MEI_ResetARC(DSL_DEV_Device_t *pDev)
{
	u32 arc_debug_data = 0;
#ifdef FPI_BUG_FIX
	u32 bar, i, k=0;
#endif

	IFX_MEI_HaltArc(pDev);
#ifdef FPI_BUG_FIX
	for (i = 0; i < 5; i++) {
		IFX_MEI_LongWordRead((u32) IFXMIPS_RCU_RST, &arc_debug_data);
printk("IFXMIPS_RCU_RST %x\n", arc_debug_data);
		IFX_MEI_LongWordWrite((u32) IFXMIPS_RCU_RST, arc_debug_data | IFXMIPS_RCU_RST_REQ_DFE | IFXMIPS_RCU_RST_REQ_AFE);
		do {
			IFX_MEI_LongWordRead((u32) IFXMIPS_RCU_RST, &arc_debug_data);
		} while ((k++ < 10) && (arc_debug_data & (IFXMIPS_RCU_RST_REQ_DFE | IFXMIPS_RCU_RST_REQ_AFE)) != (IFXMIPS_RCU_RST_REQ_DFE | IFXMIPS_RCU_RST_REQ_AFE));
		IFX_MEI_LongWordReadOffset(pDev, (u32) MEI_XMEM_BAR_BASE_OFFSET, &bar);
printk("MEI_XMEM_BAR_BASE_OFFSET %x\n", bar);
		if (bar == 0x1FFFFFE0)
			break;
	}
	if (i >= 5)
		IFX_MEI_EMSG("IFX_MEI_ResetARC failed!\n");
#else
	IFX_MEI_LongWordRead((u32) IFXMIPS_RCU_RST, &arc_debug_data);
	IFX_MEI_LongWordWrite((u32) IFXMIPS_RCU_RST, arc_debug_data | IFXMIPS_RCU_RST_REQ_DFE | IFXMIPS_RCU_RST_REQ_AFE);
#endif
	/* reset ARC */
	IFX_MEI_LongWordWriteOffset(pDev, (u32) MEI_RST_CONTROL_OFFSET, MEI_SOFT_RESET);
	IFX_MEI_LongWordWriteOffset(pDev, (u32) MEI_RST_CONTROL_OFFSET, 0);

	IFX_MEI_IRQDisable(pDev);
	IFX_MEI_EnableCLK(pDev);

	DSL_DEV_PRIVATE(pDev)->modem_ready = 0; /* IFX_MEI_ResetARC */

	return DSL_DEV_MEI_ERR_SUCCESS;
}

/**
	Set the data rate upon entering showtime.

	\param   pDev        the device pointer
	\param   rate_fast   data rate / fast channel
	\param   rate_intl   data rate / interleaved channel
	\return
	- DSL_DEV_MEI_ERR_SUCCESS or
	- DSL_DEV_MEI_ERR_FAILURE
 */
DSL_DEV_MeiError_t DSL_BSP_Showtime(DSL_DEV_Device_t *pDev, unsigned int rate_fast, unsigned int rate_intl)
{
	if (IFX_MEI_set_cell_rate == NULL)
		return DSL_DEV_MEI_ERR_FAILURE;

	IFX_MEI_DMSG("Datarate US intl = %d, fast = %d\n", rate_intl, rate_fast);
	if (rate_intl && rate_fast) {
		IFX_MEI_set_cell_rate(0, rate_fast / (53 * 8));
		IFX_MEI_set_cell_rate(1, rate_intl / (53 * 8));
	} else if (rate_fast) {
		IFX_MEI_set_cell_rate(0, rate_fast / (53 * 8));
	} else if (rate_intl) {
		IFX_MEI_set_cell_rate(0, rate_intl / (53 * 8));
	} else {
		IFX_MEI_EMSG("Got rate fail.\n");
	}
	return DSL_DEV_MEI_ERR_SUCCESS;
};

/**
	Reset/halt/run the DFE.
	This function provide operations to reset/halt/run the DFE.
	\param   pDev     the device pointer
	\param   mode     which operation want to do
	\return
	- DSL_DEV_MEI_ERR_SUCCESS or
	- DSL_DEV_MEI_ERR_FAILURE
*/
static DSL_DEV_MeiError_t IFX_MEI_CpuModeSet(DSL_DEV_Device_t *pDev, DSL_DEV_CpuMode_t mode)
	{
	DSL_DEV_MeiError_t err_ret = DSL_DEV_MEI_ERR_FAILURE;
	switch (mode) {
	case DSL_CPU_HALT:
		err_ret = IFX_MEI_HaltArc(pDev);
		break;
	case DSL_CPU_RUN:
		err_ret = IFX_MEI_RunArc(pDev);
		break;
	case DSL_CPU_RESET:
		err_ret = IFX_MEI_ResetARC(pDev);
		break;
	default:
		break;
	}
	return err_ret;
	}

/**
	Accress DFE memory.
	This function provide a way to access DFE memory;
	\param   pDev           the device pointer
	\param   type           read or write
	\param   destaddr       destination address
	\param   databuff       pointer to hold data
	\param   databuffsize   size want to read/write
	\return
	- DSL_DEV_MEI_ERR_SUCCESS or
	- DSL_DEV_MEI_ERR_FAILURE
*/
DSL_DEV_MeiError_t DSL_BSP_MemoryDebugAccess(DSL_DEV_Device_t *pDev, DSL_BSP_MemoryAccessType_t type, unsigned int destaddr, unsigned int *databuff, unsigned int databuffsize)
{
	DSL_DEV_MeiError_t meierr = DSL_DEV_MEI_ERR_SUCCESS;
	switch (type) {
	case DSL_BSP_MEMORY_READ:
		meierr = IFX_MEI_DebugRead(pDev, destaddr, databuff, databuffsize);
		break;
	case DSL_BSP_MEMORY_WRITE:
		meierr = IFX_MEI_DebugWrite(pDev, destaddr, databuff, databuffsize);
		break;
	}
	return DSL_DEV_MEI_ERR_SUCCESS;
};

/**
	Download boot code to ARC.
	This function downloads boot code to ARC.
	\param   pDev     the device pointer
	\return
	- DSL_DEV_MEI_ERR_SUCCESS or
	- DSL_DEV_MEI_ERR_FAILURE
*/
static DSL_DEV_MeiError_t IFX_MEI_DownloadBootCode(DSL_DEV_Device_t *pDev)
{
	IFX_MEI_IRQDisable(pDev);
	if (IFX_MEI_EnableCLK(pDev) == DSL_DEV_MEI_ERR_SUCCESS) {
		IFX_MEI_FuseProg(pDev);   /* program fuse rar */
		IFX_MEI_DownloadBootPages(pDev);
		return DSL_DEV_MEI_ERR_FAILURE;
	}
	return DSL_DEV_MEI_ERR_SUCCESS;
};
 
/**
	Enable Jtag debugger interface
	This function setups mips gpio to enable jtag debugger
	\param   pDev     the device pointer
	\param   enable   enable or disable
	\return
	- DSL_DEV_MEI_ERR_SUCCESS or
	- DSL_DEV_MEI_ERR_FAILURE
*/
static DSL_DEV_MeiError_t IFX_MEI_ArcJtagEnable (DSL_DEV_Device_t *dev, int enable)
{
	int meierr;
	u32 reg_data;

	switch (enable) {
	case 1:
		/* reserve gpio 9, 10, 11, 14, 19 for ARC JTAG */
		meierr = ifxmips_port_reserve_pin(0, 9);
		if (meierr < 0) {
			IFX_MEI_EMSG("Reserve GPIO 9 Fail.\n");
			goto jtag_end;
		}
		meierr = ifxmips_port_reserve_pin(0, 10);
		if (meierr < 0) {
			IFX_MEI_EMSG("Reserve GPIO 10 Fail.\n");
			goto jtag_end;
		}
		meierr = ifxmips_port_reserve_pin(0, 11);
		if (meierr < 0) {
			IFX_MEI_EMSG("Reserve GPIO 11 Fail.\n");
			goto jtag_end;
		}
		meierr = ifxmips_port_reserve_pin(0, 14);
		if (meierr < 0) {
			IFX_MEI_EMSG("Reserve GPIO 14 Fail.\n");
			goto jtag_end;
		}
		meierr = ifxmips_port_reserve_pin(1, 3);
		if (meierr < 0) {
			IFX_MEI_EMSG("Reserve GPIO 19 Fail.\n");
			goto jtag_end;
		}
		/* set gpio11 to input */
		*(IFXMIPS_GPIO_P0_DIR) = (*IFXMIPS_GPIO_P0_DIR) & (~0x800);
		*(IFXMIPS_GPIO_P0_ALTSEL0) = ((*IFXMIPS_GPIO_P0_ALTSEL0) & (~0x800));
		*(IFXMIPS_GPIO_P0_ALTSEL1) = ((*IFXMIPS_GPIO_P0_ALTSEL1) & (~0x800));
		*(IFXMIPS_GPIO_P0_OD) = (*IFXMIPS_GPIO_P0_OD) | 0x800;

		/* enable ARC JTAG */
		IFX_MEI_LongWordRead((u32) IFXMIPS_RCU_RST, &reg_data);
		IFX_MEI_LongWordWrite((u32) IFXMIPS_RCU_RST, reg_data | IFXMIPS_RCU_RST_REQ_ARC_JTAG);
		break;
	case 0:
	default:
#if defined(__LINUX__)
		/* reserve gpio 9, 10, 11, 14, 19 for ARC JTAG */
		meierr = ifxmips_port_free_pin(0, 9);
		if (meierr < 0) {
			IFX_MEI_EMSG("Reserve GPIO 9 Fail.\n");
			goto jtag_end;
		}
		meierr = ifxmips_port_free_pin(0, 10);
		if (meierr < 0) {
			IFX_MEI_EMSG("Reserve GPIO 10 Fail.\n");
			goto jtag_end;
		}
		meierr = ifxmips_port_free_pin(0, 11);
		if (meierr < 0) {
			IFX_MEI_EMSG("Reserve GPIO 11 Fail.\n");
			goto jtag_end;
		}
		meierr = ifxmips_port_free_pin(0, 14);
		if (meierr < 0) {
			IFX_MEI_EMSG("Reserve GPIO 14 Fail.\n");
			goto jtag_end;
		}
		meierr = ifxmips_port_free_pin(1, 3);
		if (meierr < 0) {
			IFX_MEI_EMSG("Reserve GPIO 19 Fail.\n");
			goto jtag_end;
		}
#endif
		break;
	}
jtag_end:
	if (meierr)
		return DSL_DEV_MEI_ERR_FAILURE;
	return DSL_DEV_MEI_ERR_SUCCESS;
};

/**
	Enable DFE to MIPS interrupt
	This function enable DFE to MIPS interrupt
	\param   pDev     the device pointer
	\param   enable   enable or disable
	\return
	- DSL_DEV_MEI_ERR_SUCCESS or
	- DSL_DEV_MEI_ERR_FAILURE
*/
static DSL_DEV_MeiError_t IFX_MEI_AdslMailboxIRQEnable(DSL_DEV_Device_t *pDev, int enable)
{
	DSL_DEV_MeiError_t meierr;
	switch (enable) {
	case 0:
		meierr = DSL_DEV_MEI_ERR_SUCCESS;
		IFX_MEI_IRQDisable(pDev);
		break;
	case 1:
		IFX_MEI_IRQEnable(pDev);
		meierr = DSL_DEV_MEI_ERR_SUCCESS;
		break;
	default:
		meierr = DSL_DEV_MEI_ERR_FAILURE;
		break;
	}
	return meierr;
}

/**
	Get the modem status
	This function return the modem status
	\param   pDev     the device pointer
	\return
	- 1: modem ready
	- 0: not ready
*/
static inline int IFX_MEI_ModemReady(DSL_DEV_Device_t *pDev)
{
	return DSL_DEV_PRIVATE(pDev)->modem_ready;
}

/**
	Initialize LED handling.
	\param   pDev           the device pointer
	\param   led_number     DSL_LED_LINK_ID, DSL_LED_DATA_ID
	\param   type           DSL_LED_LINK_TYPE, DSL_LED_DATA_TYPE
	\param   handler        DSL_LED_HD_CPU
	\return
	- DSL_DEV_MEI_ERR_SUCCESS or
	- DSL_DEV_MEI_ERR_FAILURE
*/
DSL_DEV_MeiError_t DSL_BSP_AdslLedInit(DSL_DEV_Device_t *pDev, DSL_DEV_LedId_t led_number, DSL_DEV_LedType_t type, DSL_DEV_LedHandler_t handler)
{
	const char *adsl_led[2] = {"Link", "Data"};
	const char *adsl_led_handler[2] = {"Host", "Firmware"};
	IFX_MEI_DMSG("led_number %s, type %s, handler %s\n",
				adsl_led[led_number], adsl_led[type], adsl_led_handler[handler]);
	if ((led_number == DSL_LED_LINK_ID)
					&& (type == DSL_LED_LINK_TYPE)
					&& (handler == DSL_LED_HD_CPU)) {
		ifxmips_led_blink_set(1);
	} else if ((led_number == DSL_LED_DATA_ID)
					&& (type == DSL_LED_DATA_TYPE)
					&& (handler == DSL_LED_HD_CPU)) {
		ifxmips_led_blink_set(1);
	}
	return DSL_DEV_MEI_ERR_SUCCESS;
}

/**
	Test LED handling.
	\param   pDev           the device pointer
	\return
	- DSL_DEV_MEI_ERR_SUCCESS or
	- DSL_DEV_MEI_ERR_FAILURE
*/
DSL_DEV_MeiError_t DSL_BSP_AdslLedTest(DSL_DEV_Device_t *pDev)
{
	u32 arc_debug_data = 0x0, i;
	IFX_MEI_DMSG("ALTSEL0 %#x, ALTSEL1 %#x, OD %#x, DIR %#x\n",
		*IFXMIPS_GPIO_P0_ALTSEL0, *IFXMIPS_GPIO_P0_ALTSEL1,
		*IFXMIPS_GPIO_P0_OD, *IFXMIPS_GPIO_P0_DIR);
	IFX_MEI_DMSG("IFXMIPS_LED_AR %#x, IFXMIPS_LED_CPU0 %#x, IFXMIPS_LED_CON0 %#x, "\
					"IFXMIPS_LED_CON1 %#x\n",
		*IFXMIPS_LED_AR, *IFXMIPS_LED_CPU0, *IFXMIPS_LED_CON0, *IFXMIPS_LED_CON1);

	/* Enable ADSL LEDs by access to AUX_ARC_GPIO_CTRL Auxiliary register.
	Note that this can only be done when the ARC is idle */
	IFX_MEI_ControlModeSet(pDev, MEI_MASTER_MODE);

	_IFX_MEI_DBGLongWordRead(pDev,
						MEI_DEBUG_DEC_AUX_MASK, AUX_ARC_GPIO_CTRL, &arc_debug_data);
	IFX_MEI_DMSG("ARC_GPIO_CTRL %#x\n", arc_debug_data);
	arc_debug_data = -1;
	_IFX_MEI_DBGLongWordWrite(pDev,
						MEI_DEBUG_DEC_AUX_MASK, AUX_ARC_GPIO_CTRL, arc_debug_data);
	_IFX_MEI_DBGLongWordRead(pDev,
						MEI_DEBUG_DEC_AUX_MASK, AUX_ARC_GPIO_CTRL, &arc_debug_data);
	IFX_MEI_DMSG("ARC_GPIO_CTRL %#x\n", arc_debug_data);
	for (i = 0; i < 10; i++) {
		_IFX_MEI_DBGLongWordRead(pDev,
						MEI_DEBUG_DEC_AUX_MASK, AUX_ARC_GPIO_DATA, &arc_debug_data);
		arc_debug_data = -1;
		_IFX_MEI_DBGLongWordWrite(pDev,
						MEI_DEBUG_DEC_AUX_MASK, AUX_ARC_GPIO_DATA, arc_debug_data);
		_IFX_MEI_DBGLongWordRead(pDev,
						MEI_DEBUG_DEC_AUX_MASK, AUX_ARC_GPIO_DATA, &arc_debug_data);
		IFX_MEI_DMSG("ARC_GPIO_DATA %#x\n", arc_debug_data);
		mdelay(500);
		arc_debug_data = 0;
		_IFX_MEI_DBGLongWordWrite(pDev,
						MEI_DEBUG_DEC_AUX_MASK, AUX_ARC_GPIO_DATA, arc_debug_data);
		_IFX_MEI_DBGLongWordRead(pDev,
						MEI_DEBUG_DEC_AUX_MASK, AUX_ARC_GPIO_DATA, &arc_debug_data);
		IFX_MEI_DMSG("ARC_GPIO_DATA %#x\n", arc_debug_data);
		mdelay(500);
	}

	IFX_MEI_ControlModeSet(pDev, JTAG_MASTER_MODE);
	return DSL_DEV_MEI_ERR_SUCCESS;
}

DSL_DEV_MeiError_t DSL_BSP_AdslLedSet(DSL_DEV_Device_t *pDev, DSL_DEV_LedId_t led_number, DSL_DEV_LedMode_t mode)
{
	printk(KERN_INFO "[%s %d]: mode = %#x, led_number = %d\n", __func__, __LINE__, mode, led_number);
	switch (mode) {
	case DSL_LED_OFF:
		switch (led_number) {
		case DSL_LED_LINK_ID:
#ifdef CONFIG_LEDS_DANUBE
		bsp_led_set_blink(1, 0);
		bsp_led_set_data(1, 0);
#endif
		break;
		case DSL_LED_DATA_ID:
#ifdef CONFIG_LEDS_DANUBE
		bsp_led_set_blink(0, 0);
		bsp_led_set_data(0, 0);
#endif
		break;
		}
		break;
	case DSL_LED_FLASH:
		switch (led_number) {
		case DSL_LED_LINK_ID:
#ifdef CONFIG_LEDS_DANUBE
		bsp_led_set_blink(1, 1);  /* data */
#endif
		break;
		case DSL_LED_DATA_ID:
#ifdef CONFIG_LEDS_DANUBE
		bsp_led_set_blink(0, 1);  /* data */
#endif
		break;
	}
		break;
	case DSL_LED_ON:
		switch (led_number) {
		case DSL_LED_LINK_ID:
#ifdef CONFIG_LEDS_DANUBE
		bsp_led_set_blink(1, 0);
		bsp_led_set_data(1, 1);
#endif
		break;
		case DSL_LED_DATA_ID:
#ifdef CONFIG_LEDS_DANUBE
		bsp_led_set_blink(0, 0);
		bsp_led_set_data(0, 1);
#endif
		break;
	}
	break;
}
	return DSL_DEV_MEI_ERR_SUCCESS;
};

/**
	Compose a message.
	This function compose a message from opcode, group, address, index, size, and data

	\param      opcode        The message opcode
	\param      group         The message group number
	\param      address       The message address.
	\param      index         The message index.
	\param      size         The number of words to read/write.
	\param      data         The pointer to data.
	\param      CMVMSG        The pointer to message buffer.
*/
void makeCMV(u8 opcode, u8 group, u16 address, u16 index,
					int size, u16 *data, u16 *CMVMSG)
{
	memset (CMVMSG, 0, MEI_MSG_LENGTH * 2);
	CMVMSG[0] = (opcode << 4) + (size & 0xf);
	CMVMSG[1] = (((index == 0) ? 0 : 1) << 7) + (group & 0x7f);
	CMVMSG[2] = address;
	CMVMSG[3] = index;
	if (opcode == H2D_CMV_WRITE)
		memcpy (CMVMSG + 4, data, size * 2);
	return;
}

/**
	Send a message to ARC and read the response. This function sends a message
	to ARC, waits the response, and reads the responses. If reply needed,
	it waits for reply.
	\param   pDev        the device pointer
	\param   request     pointer to the request
	\param   reply       wait reply or not.
	\param   response    pointer to the response
	\return
	- DSL_DEV_MEI_ERR_SUCCESS or
	- DSL_DEV_MEI_ERR_FAILURE
*/
DSL_DEV_MeiError_t DSL_BSP_SendCMV(DSL_DEV_Device_t *pDev, u16 *request, int reply, u16 *response)
{
	DSL_DEV_MeiError_t meierror;

	MEI_MUTEX_LOCK(DSL_DEV_PRIVATE(pDev)->mei_cmv_mutex);
	if (!IFX_MEI_ModemReady(pDev)) {
		IFX_MEI_EMSG("Trying to send CMV while modem is not ready!\n");
		MEI_MUTEX_UNLOCK(DSL_DEV_PRIVATE(pDev)->mei_cmv_mutex);
		return DSL_DEV_MEI_ERR_FAILURE;
	}

	DSL_DEV_PRIVATE(pDev)->cmv_reply = reply;
	memset(DSL_DEV_PRIVATE(pDev)->CMV_RxMsg, 0, sizeof(DSL_DEV_PRIVATE(pDev)->CMV_RxMsg));
	DSL_DEV_PRIVATE(pDev)->arcmsgav = 0;

	meierror = IFX_MEI_MailboxWrite(pDev, request, MEI_MSG_LENGTH);

	if (meierror != DSL_DEV_MEI_ERR_SUCCESS) {
		DSL_DEV_PRIVATE(pDev)->cmv_waiting = 0;
		DSL_DEV_PRIVATE(pDev)->arcmsgav = 0;
		IFX_MEI_EMSG("MailboxWrite Fail!\n");
		IFX_MEI_EMSG("Resetting ARC...\n");
		IFX_MEI_ResetARC(pDev);
		MEI_MUTEX_UNLOCK(DSL_DEV_PRIVATE(pDev)->mei_cmv_mutex);
		return meierror;
	} else
		DSL_DEV_PRIVATE(pDev)->cmv_count++;

	if (DSL_DEV_PRIVATE(pDev)->cmv_reply == NO_REPLY) {
		MEI_MUTEX_UNLOCK(DSL_DEV_PRIVATE(pDev)->mei_cmv_mutex);
		return DSL_DEV_MEI_ERR_SUCCESS;
	}

	if (DSL_DEV_PRIVATE(pDev)->arcmsgav == 0)
		MEI_WAIT_EVENT_TIMEOUT(DSL_DEV_PRIVATE(pDev)->wait_queue_arcmsgav, (DSL_DEV_PRIVATE(pDev)->arcmsgav != 0), CMV_TIMEOUT);

	DSL_DEV_PRIVATE(pDev)->cmv_waiting = 0;
	if (DSL_DEV_PRIVATE(pDev)->arcmsgav == 0) {  /* CMV_timeout */
		DSL_DEV_PRIVATE(pDev)->arcmsgav = 0;
		IFX_MEI_EMSG("DSL_DEV_MEI_ERR_MAILBOX_TIMEOUT\n");
		MEI_MUTEX_UNLOCK(DSL_DEV_PRIVATE(pDev)->mei_cmv_mutex);
		return DSL_DEV_MEI_ERR_MAILBOX_TIMEOUT;
	} else {
		DSL_DEV_PRIVATE(pDev)->arcmsgav = 0;
		DSL_DEV_PRIVATE(pDev)->reply_count++;
		memcpy (response, DSL_DEV_PRIVATE(pDev)->CMV_RxMsg, MEI_MSG_LENGTH * 2);
		MEI_MUTEX_UNLOCK(DSL_DEV_PRIVATE(pDev)->mei_cmv_mutex);
		return DSL_DEV_MEI_ERR_SUCCESS;
	}
	MEI_MUTEX_UNLOCK(DSL_DEV_PRIVATE(pDev)->mei_cmv_mutex);
	return DSL_DEV_MEI_ERR_SUCCESS;
	}

/**
	Reset the ARC, download boot codes, and run the ARC.
	This function resets the ARC, downloads boot codes to ARC, and runs the ARC.
	\param   pDev     the device pointer
	\return
	- DSL_DEV_MEI_ERR_SUCCESS or
	- DSL_DEV_MEI_ERR_FAILURE
*/
static DSL_DEV_MeiError_t IFX_MEI_RunAdslModem(DSL_DEV_Device_t *pDev)
{
	int nSize = 0, idx = 0;

	if (mei_arc_swap_buff == NULL) {
		mei_arc_swap_buff = kmalloc(MAXSWAPSIZE * 4, GFP_KERNEL);
		if (mei_arc_swap_buff == NULL) {
			IFX_MEI_EMSG(">>> malloc fail for codeswap buff!!! <<<\n");
			return DSL_DEV_MEI_ERR_FAILURE;
		}
	}

	DSL_DEV_PRIVATE(pDev)->img_hdr = (ARC_IMG_HDR *) DSL_DEV_PRIVATE(pDev)->adsl_mem_info[0].address;
	if ((DSL_DEV_PRIVATE(pDev)->img_hdr->count) * sizeof (ARC_SWP_PAGE_HDR) > SDRAM_SEGMENT_SIZE) {
		IFX_MEI_EMSG("firmware header size is bigger than 64K segment size\n");
		return DSL_DEV_MEI_ERR_FAILURE;
	}
	/* check image size */
	for (idx = 0; idx < BAR_COUNT; idx++)
		nSize += DSL_DEV_PRIVATE(pDev)->adsl_mem_info[idx].nCopy;
	if (nSize != DSL_DEV_PRIVATE(pDev)->image_size) {
		IFX_MEI_EMSG("Firmware download is not completed. Please download firmware again!\n");
		return DSL_DEV_MEI_ERR_FAILURE;
	}
	/* TODO: check crc */
	IFX_MEI_ResetARC(pDev);
	IFX_MEI_HaltArc(pDev);
	if (ifx_mei_debug)
		printk(KERN_INFO "[%s %d] -> IFX_MEI_BarUpdate()\n", __func__, __LINE__);
	IFX_MEI_BarUpdate(pDev, DSL_DEV_PRIVATE(pDev)->nBar);

	IFX_MEI_DownloadBootCode(pDev);
/* Turn off irq */
#ifdef CONFIG_AMAZON_SE
	disable_irq(BSP_USB_OC_INT0);
	disable_irq(BSP_USB_OC_INT2);
#elif defined(CONFIG_DANUBE) || defined(CONFIG_AMAZON_S)
	disable_irq(BSP_USB_OC_INT);
#endif
	disable_irq(IFXMIPS_USB_OC_INT);
	disable_irq(pDev->nIrq[BSP_DYING_GASP]);

	//IFX_MEI_DumpRegister(pDev);
	IFX_MEI_RunArc(pDev);

	MEI_WAIT_EVENT_TIMEOUT(DSL_DEV_PRIVATE(pDev)->wait_queue_modemready, (DSL_DEV_PRIVATE(pDev)->modem_ready != 0), 1000);

	//IFX_MEI_DumpRegister(pDev);

#if 0
{
	u16 CMV_RxMsg[MEI_MSG_LENGTH];
	memset(&CMV_RxMsg, 0x00, sizeof(CMV_RxMsg));

	IFX_MEI_MailboxRead(pDev, CMV_RxMsg, MEI_MSG_LENGTH);
	if (DSL_DEV_PRIVATE(pDev)->cmv_waiting == 1) {
		IFX_MEI_DMSG("Msg waiting\n");
		DSL_DEV_PRIVATE(pDev)->arcmsgav = 1;
		DSL_DEV_PRIVATE(pDev)->cmv_waiting = 0;
		MEI_WAKEUP_EVENT(DSL_DEV_PRIVATE(pDev)->wait_queue_arcmsgav);
	} else {
		DSL_DEV_PRIVATE(pDev)->modem_ready_cnt++;
		memcpy((char *) DSL_DEV_PRIVATE(pDev)->Recent_indicator,
			(char *) CMV_RxMsg, MEI_MSG_LENGTH * 2);
		if (((CMV_RxMsg[0] & 0xff0) >> 4) == D2H_AUTONOMOUS_MODEM_READY_MSG) /* arc ready */
		{  /* check ARC ready message */
			IFX_MEI_DMSG("Modem Ready\n");
			DSL_DEV_PRIVATE(pDev)->modem_ready = 1;
			MEI_WAKEUP_EVENT(DSL_DEV_PRIVATE(pDev)->wait_queue_modemready);
		} else {
			IFX_MEI_DMSG("Modem other message\n");
	}
			}
		}
#endif

#ifdef CONFIG_AMAZON_SE
	bsp_mask_and_ack_irq(BSP_USB_OC_INT0);
	ifxmips_mask_and_ack_irq(BSP_USB_OC_INT2);
#elif defined(CONFIG_DANUBE) || defined (CONFIG_AMAZON_S)
	ifxmips_mask_and_ack_irq(BSP_USB_OC_INT);
#endif
	ifxmips_mask_and_ack_irq(IFXMIPS_USB_OC_INT);
	ifxmips_mask_and_ack_irq(pDev->nIrq[BSP_DYING_GASP]);
	
	if (DSL_DEV_PRIVATE(pDev)->modem_ready != 1) {
		IFX_MEI_EMSG("Modem failed to be ready!\n");
		return DSL_DEV_MEI_ERR_FAILURE;
	} else {
		/* Re-enable irq */
		enable_irq(pDev->nIrq[BSP_DYING_GASP]);
		IFX_MEI_DMSG("Modem is ready.\n");
		return DSL_DEV_MEI_ERR_SUCCESS;
		}
}

/**
	Get the page's data pointer
	This function caculats the data address from the firmware header.
	\param   pDe      the device pointer
	\param   Page     the page number.
	\param   data     data page or program page.
	\param   MaxSize  the maximum size to read.
	\param   Buffer   pointer to data.
	\param   Dest     pointer to the destination address.
	\return  The number of bytes to read.
 */
static int IFX_MEI_GetPage(DSL_DEV_Device_t *pDev, u32 Page, u32 data, u32 MaxSize, u32 *Buffer, u32 *Dest)
{
	u32 size;
	u32 i;
	u32 *p;
	u32 idx, offset, nBar = 0;

	if (Page > DSL_DEV_PRIVATE(pDev)->img_hdr->count)
		return -2;

	/* Get program or data size, depending on "data" flag */
	size =
		(data == GET_DATA) ?(DSL_DEV_PRIVATE(pDev)->img_hdr->page[Page].d_size) :
		(DSL_DEV_PRIVATE(pDev)->img_hdr->page[Page].p_size);
	size &= BOOT_FLAG_MASK;	/* Clear boot bit! */
	if (size > MaxSize)
		return -1;

	if (size == 0)
	return 0;

	/* Get program or data offset, depending on "data" flag */
	i = data ?(DSL_DEV_PRIVATE(pDev)->img_hdr->page[Page].d_offset) :
			(DSL_DEV_PRIVATE(pDev)->img_hdr->page[Page].p_offset);

	/* Copy data/program to buffer */
	idx = i / SDRAM_SEGMENT_SIZE;
	offset = i % SDRAM_SEGMENT_SIZE;
	p = (u32 *) ((u8 *) DSL_DEV_PRIVATE(pDev)->adsl_mem_info[idx].address + offset);

	for (i = 0; i < size; i++) {
		if (offset + i * 4 - (nBar * SDRAM_SEGMENT_SIZE) >= SDRAM_SEGMENT_SIZE) {
			idx++;
			nBar++;
			p = (u32 *)((u8 *) KSEG1ADDR ((u32)DSL_DEV_PRIVATE(pDev)->adsl_mem_info[idx].address));
			}
		Buffer[i] = *p++;
		}

	/* Pass back data/program destination address */
	*Dest = data ?(DSL_DEV_PRIVATE(pDev)->img_hdr->page[Page].d_dest) :
				(DSL_DEV_PRIVATE(pDev)->img_hdr->page[Page].p_dest);

	return size;
}

/**
	Free the memory for ARC firmware
	\param   pDev     the device pointer
	\param   type     free all memory or free the unused memory after showtime
*/
static int IFX_MEI_DFEMemoryFree(DSL_DEV_Device_t *pDev, int type)
{
	const char *free_str[4] = {"Invalid", "Free_Reload", "Free_Showtime", "Free_All"};
	int idx = 0, i;
	smmu_mem_info_t *adsl_mem_info = DSL_DEV_PRIVATE(pDev)->adsl_mem_info;

	for (idx = 0; idx < BAR_COUNT; idx++) {
		if ((type == FREE_ALL) || (adsl_mem_info[idx].type == type)) {
			if (adsl_mem_info[idx].size > 0) {
				if (idx == MAX_BAR_REGISTERS) {
					for (i = idx; i <= BAR_PAGE_CNT && mei_cache_bar[i]; i++) {
						kmem_cache_free(mei_cache, mei_cache_bar[i]);
						mei_cache_bar[i] = NULL;
					}
	} else {
					if (!mei_cache_bar[idx]) {
						IFX_MEI_DMSG("Trying to free unallocated object%d\n", idx);
	return -1;
}
					IFX_MEI_DMSG("Freeing memory BAR[%d]: %p (%s)\n", idx, adsl_mem_info[idx].org_address, free_str[adsl_mem_info[idx].type]);
					kmem_cache_free(mei_cache, mei_cache_bar[idx]);
					mei_cache_bar[idx] = NULL;
}
				adsl_mem_info[idx].org_address = 0;
				adsl_mem_info[idx].address = 0;
				adsl_mem_info[idx].size = 0;
				adsl_mem_info[idx].type = 0;
				adsl_mem_info[idx].nCopy = 0;
			}
		}
	}
	return 0;
}

/**
	Allocate the memory for ARC firmware
	\param   pDev  the device pointer
	\param   type  free all memory or free the unused memory after showtime
	\return
	- E<foo>: error
	- >= 0  : success
 */
static int IFX_MEI_DFEMemoryAlloc(DSL_DEV_Device_t *pDev, long size)
{
/*
	unsigned long mem_ptr;
	char *org_mem_ptr = NULL;
*/
	int idx = 0, i;
	long total_size = 0, cnt = 0;
	int err = 0;
	smmu_mem_info_t *adsl_mem_info = DSL_DEV_PRIVATE(pDev)->adsl_mem_info;
/*
	int allocate_size = SDRAM_SEGMENT_SIZE;
*/
	if (ifx_mei_debug)
		IFX_MEI_DMSG("image_size %ld", size);

	cnt = size / SDRAM_SEGMENT_SIZE;
	cnt += (size % SDRAM_SEGMENT_SIZE) ? 1 : 0;
	if (cnt > BAR_PAGE_CNT) {
		IFX_MEI_EMSG("\nImage size is too large!\n");
		return -EFBIG;
	}
	if (ifx_mei_debug)
		printk(KERN_INFO " -> %ld XMEM_BAR and 1 XDATA_BAR x 64KB to be allocated\n", cnt);
	if (!mei_cache)
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 26))
		mei_cache = kmem_cache_create("mei_cache", SDRAM_SEGMENT_SIZE, 32/* alignment */, 0, NULL);
#else
		mei_cache = kmem_cache_create("mei_cache", SDRAM_SEGMENT_SIZE, 32/* alignment */, 0, NULL, NULL);
#endif
	if (!mei_cache) {
		IFX_MEI_EMSG("Memory allocation failed!\n");
		return -ENOMEM;
	}

	/* Alloc Swap Pages */
	for (idx = 0; size > 0 && idx < BAR_COUNT; idx++) {
		/* skip bar15 for XDATA usage. */
		if (idx == XDATA_REGISTER)
			continue;
		if (idx == MAX_BAR_REGISTERS) { /* Deal with the last BAR 1MB */
			for (i = idx, cnt = 0; i <= BAR_PAGE_CNT && size > 0; i++, cnt++) {
				mei_cache_bar[i] = kmem_cache_alloc(mei_cache, GFP_KERNEL);
				if (!mei_cache_bar[i]) {
					IFX_MEI_EMSG("%d: kmem_cache_alloc memory fail!\n", i);
					err = -ENOMEM;
					goto allocate_error; /* Call kmem_cache_free */
				}
				size -= SDRAM_SEGMENT_SIZE;
				total_size += SDRAM_SEGMENT_SIZE;
			}
			adsl_mem_info[idx].address = mei_cache_bar[idx];
			adsl_mem_info[idx].org_address = mei_cache_bar[idx];
			adsl_mem_info[idx].size = cnt * SDRAM_SEGMENT_SIZE;
		} else {
			mei_cache_bar[idx] = kmem_cache_alloc(mei_cache, GFP_KERNEL);
			if (!mei_cache_bar[idx]) {
				IFX_MEI_EMSG("%d: kmem_cache_alloc memory fail!\n", idx);
			err = -ENOMEM;
				goto allocate_error; /* Call kmem_cache_free */
		}
			adsl_mem_info[idx].address = mei_cache_bar[idx];
			adsl_mem_info[idx].org_address = mei_cache_bar[idx];
			adsl_mem_info[idx].size = SDRAM_SEGMENT_SIZE;
			size -= SDRAM_SEGMENT_SIZE;
		total_size += SDRAM_SEGMENT_SIZE;
	}
		if (ifx_mei_debug)
			IFX_MEI_DMSG("adsl_mem_info[%d]: address %p, size %#lx\n",
							idx, adsl_mem_info[idx].address, adsl_mem_info[idx].size);
	}
	return idx;

      allocate_error:
	IFX_MEI_DFEMemoryFree(pDev, FREE_ALL);
	return err;
}

/**
	Program the BAR registers
	\param   pDev        the device pointer
	\param   nTotalBar   the number of bar to program.
*/
static int IFX_MEI_BarUpdate(DSL_DEV_Device_t *pDev, int nTotalBar)
{
	int idx = 0, temp;
	smmu_mem_info_t *adsl_mem_info =
		DSL_DEV_PRIVATE(pDev)->adsl_mem_info;

	for (idx = 0; idx < nTotalBar; idx++) {
		/* skip XDATA register */
		if (idx == XDATA_REGISTER)
			continue;
		IFX_MEI_LongWordWriteOffset(pDev, (u32) MEI_XMEM_BAR_BASE_OFFSET + idx * 4, (((uint32_t) adsl_mem_info[idx].address) & 0x0FFFFFFF));
		IFX_MEI_LongWordReadOffset(pDev, (u32) MEI_XMEM_BAR_BASE_OFFSET + idx * 4, &temp);
		if (ifx_mei_debug)
			IFX_MEI_DMSG("BAR%02d: write %08X read %08X\n", idx, (((uint32_t)adsl_mem_info[idx].address) & 0x0FFFFFFF), temp & 0x0FFFFFFF);
	}
	for (idx = nTotalBar; idx < BAR_COUNT; idx++) {
		if (idx == XDATA_REGISTER)
			continue;
		IFX_MEI_LongWordWriteOffset(pDev, (u32) MEI_XMEM_BAR_BASE_OFFSET + idx * 4, (((uint32_t)adsl_mem_info[nTotalBar - 1].address) & 0x0FFFFFFF));
		IFX_MEI_LongWordReadOffset(pDev, (u32) MEI_XMEM_BAR_BASE_OFFSET + idx * 4, &temp);
		if (ifx_mei_debug)
			IFX_MEI_DMSG("BAR%02d: write %08X read %08X\n", idx, (((uint32_t)adsl_mem_info[nTotalBar - 1].address) & 0x0FFFFFFF), temp & 0x0FFFFFFF);
		/* These are for /proc/danube_mei/meminfo purpose */
		adsl_mem_info[idx].address = adsl_mem_info[nTotalBar - 1].address;
		adsl_mem_info[idx].org_address = adsl_mem_info[nTotalBar - 1].org_address;
		adsl_mem_info[idx].size = 0; /* Prevent it from being freed */
	}
	IFX_MEI_LongWordWriteOffset(pDev, (u32) MEI_XMEM_BAR_BASE_OFFSET + XDATA_REGISTER * 4,
						(((uint32_t) adsl_mem_info [XDATA_REGISTER].address) & 0x0FFFFFFF));
	IFX_MEI_LongWordReadOffset(pDev, (u32) MEI_XMEM_BAR_BASE_OFFSET + XDATA_REGISTER * 4, &temp);
	if (ifx_mei_debug)
		IFX_MEI_DMSG("BAR%02d: write %08X read %08X\n", XDATA_REGISTER, (((uint32_t)adsl_mem_info[XDATA_REGISTER].address) & 0x0FFFFFFF), temp & 0x0FFFFFFF);
	/* update MEI_XDATA_BASE_SH */
	IFX_MEI_LongWordWriteOffset(pDev, (u32) MEI_XDATA_BASE_SH_OFFSET,
						((unsigned long)adsl_mem_info[XDATA_REGISTER].address) & 0x0FFFFFFF);
	IFX_MEI_LongWordReadOffset(pDev, (u32) MEI_XDATA_BASE_SH_OFFSET, &temp);
	if (ifx_mei_debug)
		IFX_MEI_DMSG("XDATA_REGISTER: write %08X read %08X\n", (((uint32_t)adsl_mem_info[XDATA_REGISTER].address) & 0x0FFFFFFF), temp & 0x0FFFFFFF);
	return DSL_DEV_MEI_ERR_SUCCESS;
}

/** This copies the firmware from secondary storage to 64k memory segment in SDRAM.
	\param   pDev              the device pointer
	\param   buf               data buffer pointer
	\param   size              image size
	\param   loff              offset
	\param   current_offset    current offset
	\return
	- DSL_DEV_MEI_ERR_SUCCESS or
	- DSL_DEV_MEI_ERR_FAILURE
 */
DSL_DEV_MeiError_t DSL_BSP_FWDownload(DSL_DEV_Device_t *pDev, const char *buf, unsigned long size, long *loff, long *current_offset)
{
	ARC_IMG_HDR img_hdr_tmp;
	smmu_mem_info_t *adsl_mem_info = DSL_DEV_PRIVATE(pDev)->adsl_mem_info;

	size_t nRead = 0, nCopy = 0;
	char *mem_ptr;
	ssize_t retval = -ENOMEM;
	int idx = 0;

	if (*loff == 0) {
		if (size < sizeof (img_hdr_tmp)) {
			IFX_MEI_EMSG("Firmware size is too small!\n");
			return retval;
		}
		copy_from_user((char *) &img_hdr_tmp, buf, sizeof(img_hdr_tmp));
		/* header of image_size and crc are not included. */
		DSL_DEV_PRIVATE(pDev)->image_size = le32_to_cpu(img_hdr_tmp.size) + 8;

		if (DSL_DEV_PRIVATE(pDev)->image_size > 1024 * 1024) {
			IFX_MEI_EMSG("Firmware size is too large!\n");
			return retval;
		}
		/* check if arc is halt */
		IFX_MEI_ResetARC(pDev);
		IFX_MEI_HaltArc(pDev);

		IFX_MEI_DFEMemoryFree(pDev, FREE_ALL);	/* free all */

		retval = IFX_MEI_DFEMemoryAlloc(pDev, DSL_DEV_PRIVATE(pDev)->image_size);
		if (retval < 0) {
			IFX_MEI_EMSG("Error: DFE firmware memory allocation failed!\n");
			goto error;
		}

		for (idx = 0; idx < retval; idx++) {
			/* skip XDATA register */
			if (idx == XDATA_REGISTER)
				continue;
			if (idx * SDRAM_SEGMENT_SIZE < le32_to_cpu (img_hdr_tmp.page[0].p_offset))
				adsl_mem_info[idx].type = FREE_RELOAD;
			else
				adsl_mem_info[idx].type = FREE_SHOWTIME;
			}
		DSL_DEV_PRIVATE(pDev)->nBar = retval;
		DSL_DEV_PRIVATE(pDev)->img_hdr = (ARC_IMG_HDR *) adsl_mem_info[0].address;

		mei_cache_bar[XDATA_REGISTER] = kmem_cache_alloc(mei_cache, GFP_KERNEL);
		adsl_mem_info[XDATA_REGISTER].org_address = mei_cache_bar[XDATA_REGISTER];
		adsl_mem_info[XDATA_REGISTER].address = mei_cache_bar[XDATA_REGISTER];
		adsl_mem_info[XDATA_REGISTER].size = SDRAM_SEGMENT_SIZE;

		if (!adsl_mem_info[XDATA_REGISTER].address) {
			IFX_MEI_EMSG("XDATA_REGISTER memory allocation fail!\n");
			retval = -ENOMEM;
			goto error;
		}
		adsl_mem_info[XDATA_REGISTER].type = FREE_RELOAD;
		if (ifx_mei_debug)
			printk(KERN_INFO "[%s %d] -> IFX_MEI_BarUpdate()\n", __func__, __LINE__);
		IFX_MEI_BarUpdate(pDev, DSL_DEV_PRIVATE(pDev)->nBar);
	} else if (DSL_DEV_PRIVATE(pDev)->image_size == 0) {
		IFX_MEI_EMSG("Error: Firmware size=0! \n");
		goto error;
	}
	nRead = 0;
	while (nRead < size) {
		long offset = ((long) (*loff) + nRead) % SDRAM_SEGMENT_SIZE;
		idx = (((long) (*loff)) + nRead) / SDRAM_SEGMENT_SIZE;
		mem_ptr = (char *) KSEG1ADDR ((unsigned long)(adsl_mem_info[idx].address) + offset);
		if ((size - nRead + offset) > SDRAM_SEGMENT_SIZE)
			nCopy = SDRAM_SEGMENT_SIZE - offset;
		else
			nCopy = size - nRead;
		copy_from_user (mem_ptr, buf + nRead, nCopy);
		for (offset = 0; offset < (nCopy / 4); offset++)
			((unsigned long *) mem_ptr)[offset] = le32_to_cpu(((unsigned long *)mem_ptr)[offset]);
		nRead += nCopy;
		adsl_mem_info[idx].nCopy += nCopy;
	}
	*loff += size;
	*current_offset = size;
	return DSL_DEV_MEI_ERR_SUCCESS;
      error:
	IFX_MEI_DFEMemoryFree(pDev, FREE_ALL);
	return DSL_DEV_MEI_ERR_FAILURE;
}

/**
	Register a callback event.
	\param p    pointer to callback info
	\return
	- -EINVAL   in case that the pointer or the event number is invalid
	- -1        if the event already has a callback function registered.
	- 0         success
 */
int DSL_BSP_EventCBRegister(DSL_BSP_EventCallBack_t *p)
{
	if (!p) {
		IFX_MEI_EMSG("Invalid parameter!\n");
		return -EINVAL;
	}
	if (p->event > DSL_BSP_CB_LAST || p->event < DSL_BSP_CB_FIRST) {
		IFX_MEI_EMSG("Invalid Event %d\n", p->event);
		return -EINVAL;
	}
	if (dsl_bsp_event_callback[p->event].function) {
		IFX_MEI_EMSG("Event %d already has a callback function registered!\n", p->event);
		return -1;
	} else {
		dsl_bsp_event_callback[p->event].function = p->function;
		dsl_bsp_event_callback[p->event].event   = p->event;
		dsl_bsp_event_callback[p->event].pData   = p->pData;
	}
	return 0;
}

/**
	unregister a callback event.
	\param p    pointer to callback info
	\return
	- -EINVAL   in case that the pointer or the event number is invalid
	- -1        in case that the handler is not registered
	- 0         success
 */
int DSL_BSP_EventCBUnregister(DSL_BSP_EventCallBack_t *p)
{
	if (!p) {
		IFX_MEI_EMSG("Invalid parameter!\n");
		return -EINVAL;
		}
	if (p->event > DSL_BSP_CB_LAST || p->event < DSL_BSP_CB_FIRST) {
		IFX_MEI_EMSG("Invalid Event %d\n", p->event);
		return -EINVAL;
		}
	if (dsl_bsp_event_callback[p->event].function) {
		IFX_MEI_EMSG("Unregistering Event %d...\n", p->event);
		dsl_bsp_event_callback[p->event].function = NULL;
		dsl_bsp_event_callback[p->event].pData   = NULL;
	} else {
		IFX_MEI_EMSG("Event %d is not registered!\n", p->event);
		return -1;
	}
	return 0;
}

/**
	MEI Dying Gasp interrupt handler
	\param int1
	\param void0
	\param regs    pointer to the structure of danube mips registers
 */
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 0))
static irqreturn_t IFX_MEI_Dying_Gasp_IrqHandle(int int1, void *void0)
#else
static void IFX_MEI_Dying_Gasp_IrqHandle(int int1, void *void0, struct pt_regs *regs)
#endif
{
	DSL_DEV_Device_t *pDev = (DSL_DEV_Device_t *) void0;
	DSL_BSP_CB_Type_t event;
#ifdef CONFIG_MIPS_MT_SMTC
	int cpu = cpu_data[smp_processor_id()].vpe_id;
#else
	int cpu = smp_processor_id();
#endif
	if (pDev == NULL)
		IFX_MEI_EMSG("Error: Got Interrupt but pDev is NULL!!!!\n");

	disable_irq(pDev->nIrq[BSP_DYING_GASP]);
	event = DSL_BSP_CB_DYING_GASP;

	if (dsl_bsp_event_callback[event].function)
		(*dsl_bsp_event_callback[event].function)(pDev, event, dsl_bsp_event_callback[event].pData);

	IFX_MEI_EMSG("CPU%d Dying Gasp! Shutting Down...\n", cpu);
	kill_cad_pid(SIGINT, 1);      /* Ask init to reboot us */

	return IRQ_HANDLED;
}

/**
	MEI interrupt handler
	\param int1
	\param void0
	\param regs    pointer to the structure of danube mips registers
 */
#if (LINUX_VERSION_CODE > KERNEL_VERSION(2, 6, 0))
static irqreturn_t IFX_MEI_IrqHandle(int int1, void *void0)
#else
static void IFX_MEI_IrqHandle(int int1, void *void0, struct pt_regs *regs)
#endif
{
	u32 scratch;
	DSL_DEV_Device_t *pDev = (DSL_DEV_Device_t *) void0;
	DSL_BSP_CB_Type_t event;

	if (pDev == NULL)
		IFX_MEI_EMSG("Error: Got Interrupt but pDev is NULL!!!!\n");
	/* Firmware shares ARC_TO_MEI_MSGAV in ARC_TO_MEI_INT_OFFSET amongst the following interrupt sources:
	(1) Clear EOC
	(2) Firmware Reboot
	(3) Code Swap
	The MEI driver clears this bit for these interrupts
	*/
	IFX_MEI_DebugRead(pDev, ARC_MEI_MAILBOXR, &scratch, 1);
	if (scratch & OMB_CODESWAP_MESSAGE_MSG_TYPE_MASK) {
		IFX_MEI_EMSG("Receive Code Swap Request interrupt!!!\n");
		IFX_MEI_LongWordWriteOffset(pDev, (u32) ARC_TO_MEI_INT_OFFSET, ARC_TO_MEI_MSGAV);
		return IRQ_HANDLED;
	} else if (scratch & OMB_CLEAREOC_INTERRUPT_CODE) {/* clear eoc message interrupt */
		IFX_MEI_DMSG("OMB_CLEAREOC_INTERRUPT_CODE\n");
		event = DSL_BSP_CB_CEOC_IRQ;
		IFX_MEI_LongWordWriteOffset(pDev, (u32) ARC_TO_MEI_INT_OFFSET, ARC_TO_MEI_MSGAV);
		if (dsl_bsp_event_callback[event].function)
			(*dsl_bsp_event_callback[event].function)(pDev, event, dsl_bsp_event_callback[event].pData);
	} else if (scratch & OMB_REBOOT_INTERRUPT_CODE) { /* Reboot */
		IFX_MEI_DMSG("OMB_REBOOT_INTERRUPT_CODE\n");
		event = DSL_BSP_CB_FIRMWARE_REBOOT;
		IFX_MEI_LongWordWriteOffset(pDev, (u32) ARC_TO_MEI_INT_OFFSET, ARC_TO_MEI_MSGAV);
		if (dsl_bsp_event_callback[event].function)
			(*dsl_bsp_event_callback[event].function)(pDev, event, dsl_bsp_event_callback[event].pData);
	} else { /* normal message */
		IFX_MEI_MailboxRead(pDev, DSL_DEV_PRIVATE(pDev)->CMV_RxMsg, MEI_MSG_LENGTH);
		if (DSL_DEV_PRIVATE(pDev)->cmv_waiting == 1) {
			DSL_DEV_PRIVATE(pDev)->arcmsgav = 1;
			DSL_DEV_PRIVATE(pDev)->cmv_waiting = 0;
			MEI_WAKEUP_EVENT(DSL_DEV_PRIVATE(pDev)->wait_queue_arcmsgav);
		} else {
			DSL_DEV_PRIVATE(pDev)->modem_ready_cnt++;
			memcpy((char *) DSL_DEV_PRIVATE(pDev)->Recent_indicator,
				(char *) DSL_DEV_PRIVATE(pDev)->CMV_RxMsg, MEI_MSG_LENGTH * 2);
			if (((DSL_DEV_PRIVATE(pDev)->CMV_RxMsg[0] & 0xff0) >> 4) == D2H_AUTONOMOUS_MODEM_READY_MSG) /* arc ready */
			{  /* check ARC ready message */
				IFX_MEI_DMSG("Modem Ready\n");
				DSL_DEV_PRIVATE(pDev)->modem_ready = 1;
				MEI_WAKEUP_EVENT(DSL_DEV_PRIVATE(pDev)->wait_queue_modemready);
	}
	}
		}
	return IRQ_HANDLED;
		}

/**
	Wrapper for the upper layer driver to register callback.
*/
int DSL_BSP_ATMLedCBRegister(int (*f) (void))
	{
	IFX_ADSL_LEDHandler = f;
	return 0;
	}

/**
	Wrapper for the upper layer driver to un-register callback.
*/
int DSL_BSP_ATMLedCBUnregister(int (*f) (void))
	{
	IFX_ADSL_LEDHandler = NULL;
		return 0;
	}

/**
	Wrapper for the upper layer driver execute LED handler.
*/
void DSL_BSP_ATMLedHandler(void)
	{
	if (IFX_ADSL_LEDHandler)
		IFX_ADSL_LEDHandler();
}

/**
	Create device node.
	\param num     device number. If the device number is zero the default
						number IFX_MEI_MAJOR_NUMBER is used.
	\return
	- -ENODEV   on error
	- 0         on success
 */
static int IFX_MEI_InitDevNode(int num)
{
#ifdef CONFIG_DEVFS_FS
	char buf[36];
	sprintf(buf, "%s%d", IFXMIPS_MEI_DEVNAME, num);
	ifx_mei_devfs_handle[num] =
		devfs_register(NULL, buf, DEVFS_FL_DEFAULT, dev_major, 0,
				S_IFCHR | S_IRUGO | S_IWUGO,
				&bsp_mei_operations,
				(void *) 0);
	if (ifx_mei_devfs_handle[num] == NULL) {
		IFX_MEI_EMSG("Register mei devfs error.\n");
		return -ENODEV;
		}
#else
	if (num == 0) {
		dev_major = register_chrdev(dev_major, IFXMIPS_MEI_DEVNAME, &bsp_mei_operations);
		if (dev_major < 0) {
			IFX_MEI_EMSG("register_chrdev(%d %s) failed!\n", dev_major, IFXMIPS_MEI_DEVNAME);
			return -ENODEV;
	}
	}
#endif
	return 0;
}

/**
	Delete device node.
	\param num     device number.
	\return
	- 0         on success
 */
static int IFX_MEI_CleanUpDevNode(int num)
{
#ifdef CONFIG_DEVFS_FS
	devfs_unregister(ifx_mei_devfs_handle[num]);
#else
	if (num == 0)
		unregister_chrdev(dev_major, MEI_DIRNAME);
#endif
	return 0;
}

/**
	Initialize device. In fact only one device is supported now.
	\param num     device number.
	\return
	-ENOMEM     the device pointer is invalid
	- -1        IRQ register failed
	- 0         on success
 */
static int IFX_MEI_InitDevice(int num)
{
	DSL_DEV_Device_t *pDev;
	if (num >= BSP_MAX_DEVICES)
		return -ENOMEM;
	pDev = &dsl_devices[num];
	pDev->pPriv = &IFXMIPS_MEI_Private[num];
	memset(pDev->pPriv, 0, sizeof (ifx_mei_device_private_t));
	memset(&DSL_DEV_PRIVATE(pDev)->adsl_mem_info[0], 0, sizeof (smmu_mem_info_t) * BAR_COUNT);

	if (num == 0) {
		pDev->nIrq[BSP_DFEIR]     = IFXMIPS_MEI_INT;
		pDev->nIrq[BSP_DYING_GASP] = IFXMIPS_MEI_DYING_GASP_INT;
		pDev->base_address = IFXMIPS_MEI_BASE_ADDR;
		/* Power Up DSL */
		ifxmips_pmu_enable(IFXMIPS_PMU_PWDCR_DSL);
		ifxmips_pmu_enable(IFXMIPS_PMU_PWDCR_AHB);
	}
	pDev->nInUse = 0;
	DSL_DEV_PRIVATE(pDev)->modem_ready = 0; /* IFX_MEI_InitDevice */
	DSL_DEV_PRIVATE(pDev)->arcmsgav = 0;

	/* for ARCMSGAV */
	MEI_INIT_WAKELIST("arcq", DSL_DEV_PRIVATE(pDev)->wait_queue_arcmsgav);
	/* for arc modem ready */
	MEI_INIT_WAKELIST("arcr", DSL_DEV_PRIVATE(pDev)->wait_queue_modemready);

	MEI_MUTEX_INIT(DSL_DEV_PRIVATE(pDev)->mei_cmv_mutex);

	if (request_irq(pDev->nIrq[BSP_DFEIR], IFX_MEI_IrqHandle, IRQF_DISABLED, "DFEIR", pDev) != 0) {
		IFX_MEI_EMSG("request_irq %d failed!\n", pDev->nIrq[BSP_DFEIR]);
		return -1;
	}

	if (request_irq (pDev->nIrq[BSP_DYING_GASP], IFX_MEI_Dying_Gasp_IrqHandle, IRQF_DISABLED, "DYING_GASP", pDev) != 0) {
		IFX_MEI_EMSG ("request_irq %d failed!\n", pDev->nIrq[BSP_DYING_GASP]);
		return -1;
	}
	return 0;
}

/**
	Disable interrupts and free interrupts of the device.
	\param num     device number.
	\return
	-ENOMEM     the device pointer is invalid
	- 0         on success
 */
static int IFX_MEI_ExitDevice(int num)
{
	DSL_DEV_Device_t *pDev;

	if (num >= BSP_MAX_DEVICES)
		return -ENOMEM;

	pDev = &dsl_devices[num];

	disable_irq(pDev->nIrq[BSP_DFEIR]);
	disable_irq(pDev->nIrq[BSP_DYING_GASP]);

	free_irq(pDev->nIrq[BSP_DFEIR], pDev);
	free_irq(pDev->nIrq[BSP_DYING_GASP], pDev);

	return 0;
}

static DSL_DEV_Device_t *IFX_BSP_HandleGet(int maj, int num)
{
	if (num > BSP_MAX_DEVICES)
		return NULL;
	return &dsl_devices[num];
}

DSL_DEV_Device_t *DSL_BSP_DriverHandleGet(int maj, int num)
{
	DSL_DEV_Device_t *pDev;
	if (num > BSP_MAX_DEVICES)
		return NULL;

	pDev = &dsl_devices[num];
	if (!try_module_get(pDev->owner))
		return NULL;

	pDev->nInUse++;
	return pDev;
	}

int DSL_BSP_DriverHandleDelete(DSL_DEV_Device_t *nHandle)
			{
	DSL_DEV_Device_t *pDev = (DSL_DEV_Device_t *) nHandle;
	if (pDev->nInUse)
		pDev->nInUse--;
		module_put(pDev->owner);
	return 0;
			}

static int IFX_MEI_Open(struct inode *ino, struct file *fil)
{
	int maj = MAJOR(ino->i_rdev);
	int num = MINOR(ino->i_rdev);
	DSL_DEV_Device_t *pDev = DSL_BSP_DriverHandleGet(maj, num);
	if (pDev == NULL) {
		IFX_MEI_EMSG("open(%d:%d) fail!\n", maj, num);
		return -EIO;
				}
	fil->private_data = pDev;
	return 0;
			}

static int IFX_MEI_Release(struct inode *ino, struct file *fil)
				{
	int num = MINOR(ino->i_rdev);
	if (num >= BSP_MAX_DEVICES)
		return -EIO;
	DSL_BSP_DriverHandleDelete(&dsl_devices[num]);
	return 0;
				}

/**
	The write routine is mapped to the firmware download.
*/
static ssize_t IFX_MEI_Write(struct file *filp, const char *buf, size_t size, loff_t *loff)
				{
	DSL_DEV_MeiError_t mei_error = DSL_DEV_MEI_ERR_FAILURE;
	long offset = 0;
	DSL_DEV_Device_t *pDev = (DSL_DEV_Device_t *) filp->private_data;
	if (pDev == NULL)
		return -EIO;
	mei_error = DSL_BSP_FWDownload(pDev, buf, size, (long *) loff, &offset);
	if (mei_error == DSL_DEV_MEI_ERR_FAILURE)
		return -EIO;
	return (ssize_t) offset;
				}

/**
	Wrapper for memcpy routine. In case of the copy routine should be done
	from user to kernel space the copy_from_user() is called.
*/
static void IFX_MEI_IoctlCopyFrom(int from_kernel, char *dest, char *from, int size)
				{
	if (!from_kernel)
		copy_from_user(dest, from, size);
	else
		memcpy(dest, from, size);
			}

/**
	Wrapper for memcpy() routine. In case of the copy routine should be done
	from kernel to user space the copy_to_user() is called.
*/
static void IFX_MEI_IoctlCopyTo(int from_kernel, char *dest, char *from, int size)
{
	if (!from_kernel)
		copy_to_user(dest, from, size);
	else
		memcpy(dest, from, size);
			}

/**
	Ioctl interface for the MEI driver.
	It should be called from the stacked DSL-CPE-API driver.
*/
static int IFX_MEI_Ioctls(DSL_DEV_Device_t *pDev, int from_kernel, unsigned int command, unsigned long lon)
{
	int i = 0;
	int meierr = DSL_DEV_MEI_ERR_SUCCESS;
	u32 base_address = IFXMIPS_MEI_BASE_ADDR;
	DSL_DEV_WinHost_Message_t winhost_msg, m;
	DSL_DEV_MeiDebug_t debugrdwr;
	DSL_DEV_MeiReg_t regrdwr;

	switch (command) {

	case DSL_FIO_BSP_CMV_WINHOST:
printk("DSL_FIO_BSP_CMV_WINHOST\n");
		IFX_MEI_IoctlCopyFrom(from_kernel, (char *) winhost_msg.msg.TxMessage, (char *) lon, MEI_MSG_LENGTH * 2);
		meierr = DSL_BSP_SendCMV(pDev, winhost_msg.msg.TxMessage, YES_REPLY, winhost_msg.msg.RxMessage);
		if (meierr != DSL_DEV_MEI_ERR_SUCCESS) {
			IFX_MEI_EMSG
				("WINHOST CMV fail :TxMessage:%X %X %X %X, RxMessage:%X %X %X %X %X\n",
				winhost_msg.msg.TxMessage[0], winhost_msg.msg.TxMessage[1],
				winhost_msg.msg.TxMessage[2], winhost_msg.msg.TxMessage[3],
				winhost_msg.msg.RxMessage[0], winhost_msg.msg.RxMessage[1],
				winhost_msg.msg.RxMessage[2], winhost_msg.msg.RxMessage[3],
				winhost_msg.msg.RxMessage[4]);
			meierr = DSL_DEV_MEI_ERR_FAILURE;
		} else {
			IFX_MEI_IoctlCopyTo(from_kernel, (char *) lon, (char *) winhost_msg.msg.RxMessage, MEI_MSG_LENGTH * 2);
			}
			break;

	case DSL_FIO_BSP_CMV_READ:
printk("DSL_FIO_BSP_CMV_READ\n");
		IFX_MEI_IoctlCopyFrom(from_kernel, (char *) (&regrdwr), (char *) lon, sizeof (DSL_DEV_MeiReg_t));
		IFX_MEI_LongWordRead((u32) regrdwr.iAddress, (u32 *) &(regrdwr.iData));
		IFX_MEI_IoctlCopyTo(from_kernel, (char *) lon, (char *) (&regrdwr), sizeof (DSL_DEV_MeiReg_t));
			break;

	case DSL_FIO_BSP_CMV_WRITE:
printk("DSL_FIO_BSP_CMV_WRITE\n");
		IFX_MEI_IoctlCopyFrom(from_kernel, (char *) (&regrdwr), (char *) lon, sizeof (DSL_DEV_MeiReg_t));
		IFX_MEI_LongWordWrite((u32) regrdwr.iAddress, regrdwr.iData);
		break;

	case DSL_FIO_BSP_GET_BASE_ADDRESS:
		IFX_MEI_IoctlCopyTo(from_kernel, (char *) lon, (char *) (&base_address), sizeof (base_address));
		break;

	case DSL_FIO_BSP_IS_MODEM_READY:
		i = IFX_MEI_ModemReady(pDev);
		IFX_MEI_IoctlCopyTo(from_kernel, (char *) lon, (char *) (&i), sizeof (int));
		meierr = DSL_DEV_MEI_ERR_SUCCESS;
			break;

	case DSL_FIO_BSP_RESET:
	case DSL_FIO_BSP_REBOOT:
		meierr = IFX_MEI_CpuModeSet(pDev, DSL_CPU_RESET);
		meierr = IFX_MEI_CpuModeSet(pDev, DSL_CPU_HALT);
			break;

	case DSL_FIO_BSP_HALT:
		meierr = IFX_MEI_CpuModeSet(pDev, DSL_CPU_HALT);
			break;

	case DSL_FIO_BSP_RUN:
		meierr = IFX_MEI_CpuModeSet(pDev, DSL_CPU_RUN);
		break;

	case DSL_FIO_BSP_BOOTDOWNLOAD:
		meierr = IFX_MEI_DownloadBootCode(pDev);
			break;

	case DSL_FIO_BSP_JTAG_ENABLE:
		meierr = IFX_MEI_ArcJtagEnable(pDev, 1);
		break;

	case DSL_FIO_BSP_REMOTE:
		IFX_MEI_IoctlCopyFrom(from_kernel, (char *) (&i), (char *) lon, sizeof (int));
		meierr = IFX_MEI_AdslMailboxIRQEnable(pDev, i);
			break;

	case DSL_FIO_BSP_DSL_START:
		meierr = IFX_MEI_RunAdslModem(pDev);
		if (meierr != DSL_DEV_MEI_ERR_SUCCESS) {
			IFX_MEI_EMSG("IFX_MEI_RunAdslModem() error...");
			meierr = DSL_DEV_MEI_ERR_FAILURE;
			}
					break;

	case DSL_FIO_BSP_DEBUG_READ:
	case DSL_FIO_BSP_DEBUG_WRITE:
printk("DSL_FIO_BSP_DEBUG_ %d\n", command);
		IFX_MEI_IoctlCopyFrom(from_kernel, (char *) (&debugrdwr), (char *) lon, sizeof (debugrdwr));
		if (command == DSL_FIO_BSP_DEBUG_READ)
			meierr = DSL_BSP_MemoryDebugAccess(pDev, DSL_BSP_MEMORY_READ, debugrdwr.iAddress, (u32 *) debugrdwr.buffer, debugrdwr.iCount);
		else
			meierr = DSL_BSP_MemoryDebugAccess(pDev, DSL_BSP_MEMORY_WRITE, debugrdwr.iAddress, (u32 *) debugrdwr.buffer, debugrdwr.iCount);
		IFX_MEI_IoctlCopyTo(from_kernel, (char *) lon, (char *) (&debugrdwr), sizeof (debugrdwr));
		break;
	case DSL_FIO_BSP_GET_VERSION:
		IFX_MEI_IoctlCopyTo(from_kernel, (char *) lon, (char *) (&bsp_mei_version), sizeof (DSL_DEV_Version_t));
			break;

	case DSL_FIO_BSP_GET_CHIP_INFO:
		bsp_chip_info.major = 1;
		bsp_chip_info.minor = ifxmips_get_cpu_ver();
		IFX_MEI_DMSG("DSL_FIO_BSP_GET_CHIP_INFO: version %ld.%ld\n", bsp_chip_info.major, bsp_chip_info.minor);
		IFX_MEI_IoctlCopyTo(from_kernel, (char *) lon, (char *) (&bsp_chip_info), sizeof (DSL_DEV_HwVersion_t));
		meierr = DSL_DEV_MEI_ERR_SUCCESS;
		break;

	case DSL_FIO_BSP_FREE_RESOURCE:
		makeCMV(H2D_CMV_READ, DSL_CMV_GROUP_STAT, 4, 0, 1, NULL, m.msg.TxMessage);
		if (DSL_BSP_SendCMV(pDev, m.msg.TxMessage, YES_REPLY, m.msg.RxMessage)
				!= DSL_DEV_MEI_ERR_SUCCESS) {
			meierr = DSL_DEV_MEI_ERR_FAILURE;
			return -EIO;
		}
		IFX_MEI_DMSG("RxMessage[4] = %#x\n", m.msg.RxMessage[4]);
		if (!(m.msg.RxMessage[4] & MEI_CODESWAP_COMPLETE)) {
			meierr = DSL_DEV_MEI_ERR_FAILURE;
			return -EAGAIN;
		}
		IFX_MEI_DMSG("Freeing all memories marked FREE_SHOWTIME\n");
		IFX_MEI_DFEMemoryFree(pDev, FREE_SHOWTIME);
		meierr = DSL_DEV_MEI_ERR_SUCCESS;
	break;
#ifdef CONFIG_AMAZON_SE
	case DSL_FIO_ARC_MUX_TEST:
		IFX_MEI_ARC_MUX_Test();
			break;
#endif
	default:
/* 		IFX_MEI_EMSG("Invalid IOCTL command: %d\n"); */
			break;
			}
	return meierr;
			}

int DSL_BSP_KernelIoctls(DSL_DEV_Device_t *pDev, unsigned int command, unsigned long lon)
{
	int error = 0;
	error = IFX_MEI_Ioctls(pDev, 1, command, lon);
	return error;
			}

static int IFX_MEI_UserIoctls(struct inode *ino, struct file *fil, unsigned int command, unsigned long lon)
{
	int error = 0;
	int maj = MAJOR(ino->i_rdev);
	int num = MINOR(ino->i_rdev);
	DSL_DEV_Device_t *pDev;

	pDev = IFX_BSP_HandleGet(maj, num);
	if (pDev == NULL)
		return -EIO;

	error = IFX_MEI_Ioctls(pDev, 0, command, lon);
	return error;
			}

#ifdef CONFIG_PROC_FS
/**
	Register a callback function for linux proc filesystem
*/
static int IFX_MEI_InitProcFS(int num)
{
	struct proc_dir_entry *entry;
	int i = 0;
	DSL_DEV_Device_t *pDev;
	reg_entry_t regs_temp[PROC_ITEMS] =	/* Items being debugged */
			{
		/* flag, name,     description } */
		{NULL, "arcmsgav", "arc to mei message ", 0},
		{NULL, "cmv_reply", "cmv needs reply", 0},
		{NULL, "cmv_waiting", "waiting for cmv reply from arc", 0},
		{NULL, "debug", "turn on debug messages", 0},
		{NULL, "modem_ready_cnt", "ARC to MEI indicator count", 0},
		{NULL, "cmv_count", "MEI to ARC CMVs", 0},
		{NULL, "reply_count", "ARC to MEI Reply", 0},
		{NULL, "Recent_indicator", "most recent indicator", 0},
		{NULL, "fw_version", "Firmware Version", 0},
		{NULL, "fw_date", "Firmware Date", 0},
		{NULL, "meminfo", "Memory Allocation Information", 0},
		{NULL, "version", "MEI version information", 0},
	};
	if (num >= BSP_MAX_DEVICES)
		return -EIO;
	pDev = &dsl_devices[num];
	regs_temp[0].flag = &DSL_DEV_PRIVATE(pDev)->arcmsgav;
	regs_temp[1].flag = &DSL_DEV_PRIVATE(pDev)->cmv_reply;
	regs_temp[2].flag = &DSL_DEV_PRIVATE(pDev)->cmv_waiting;
	regs_temp[3].flag = &ifx_mei_debug;
	regs_temp[4].flag = &DSL_DEV_PRIVATE(pDev)->modem_ready_cnt;
	regs_temp[5].flag = &DSL_DEV_PRIVATE(pDev)->cmv_count;
	regs_temp[6].flag = &DSL_DEV_PRIVATE(pDev)->reply_count;
	regs_temp[7].flag = (int *) &DSL_DEV_PRIVATE(pDev)->Recent_indicator;

	memcpy((char *) regs[num], (char *) regs_temp, sizeof (regs_temp));
	/* procfs */
	meidir = proc_mkdir("driver/"MEI_DIRNAME, NULL);
	if (meidir == NULL) {
		IFX_MEI_EMSG("Failed to create /proc/%s\n", MEI_DIRNAME);
		return -ENOMEM;
			}

	for (i = 0; i < NUM_OF_REG_ENTRY; i++) {
		entry = create_proc_entry(regs[num][i].name,
						S_IWUSR | S_IRUSR | S_IRGRP |
						S_IROTH, meidir);
		if (entry) {
			regs[num][i].low_ino = entry->low_ino;
			entry->proc_fops = &IFX_MEI_ProcOperations;
		} else {
			IFX_MEI_EMSG("Failed to create /proc/%s/%s\n", "driver/"MEI_DIRNAME, regs[num][i].name);
			return -ENOMEM;
		}
		}
	return 0;
	}

/**
	Reading function for linux proc filesystem
*/
static int IFX_MEI_ProcRead(struct file *file, char *buf, size_t nbytes, loff_t *ppos)
{
	unsigned long i_ino = (file->f_dentry->d_inode)->i_ino;
	char *p = buf;
	int i;
	int num;
	reg_entry_t *entry = NULL;
	DSL_DEV_Device_t *pDev = NULL;
	DSL_DEV_WinHost_Message_t m;

	for (num = 0; num < BSP_MAX_DEVICES; num++) {
	for (i = 0; i < NUM_OF_REG_ENTRY; i++) {
			if (regs[num][i].low_ino == i_ino) {
				entry = &regs[num][i];
				pDev = &dsl_devices[num];
			break;
		}
	}
	}
	if (entry == NULL)
		return -EINVAL;
	else if (strcmp(entry->name, "meminfo") == 0) {
		if (*ppos > 0)	/* Assume reading completed in previous read */
			return 0;
		p += sprintf (p, "No         Address    Size\n");
		for (i = 0; i < BAR_COUNT; i++) {
			p += sprintf (p, "BAR[%02d] Addr:0x%08X Size:%lu\n",
					i, (u32) DSL_DEV_PRIVATE(pDev)->adsl_mem_info[i].address,
					DSL_DEV_PRIVATE(pDev)->adsl_mem_info[i].size);
			/* printk( "BAR[%02d] Addr:0x%08X Size:%d\n",i,adsl_mem_info[i].address,adsl_mem_info[i].size); */
		}
		*ppos += (p - buf);
	} else if (strcmp(entry->name, "fw_version") == 0) {
		if (*ppos > 0)	/* Assume reading completed in previous read */
			return 0;
		if (DSL_DEV_PRIVATE(pDev)->modem_ready_cnt < 1)
			return -EAGAIN;
		/* major:bits 0-7 */
		/* minor:bits 8-15 */
		makeCMV (H2D_CMV_READ, DSL_CMV_GROUP_INFO, 54, 0, 1, NULL, m.msg.TxMessage);
		if (DSL_BSP_SendCMV(pDev, m.msg.TxMessage, YES_REPLY, m.msg.RxMessage) != DSL_DEV_MEI_ERR_SUCCESS)
			return -EIO;
		p += sprintf(p, "FW Version: %d.%d.", m.msg.RxMessage[4] & 0xFF, (m.msg.RxMessage[4] >> 8) & 0xFF);
		/* sub_version:bits 4-7 */
		/* int_version:bits 0-3 */
		/* spl_appl:bits 8-13 */
		/* rel_state:bits 14-15 */
		makeCMV(H2D_CMV_READ, DSL_CMV_GROUP_INFO, 54, 1, 1, NULL, m.msg.TxMessage);
		if (DSL_BSP_SendCMV(pDev, m.msg.TxMessage, YES_REPLY, m.msg.RxMessage)
				!= DSL_DEV_MEI_ERR_SUCCESS)
			return -EIO;
		p += sprintf(p, "%d.%d.%d.%d\n",
			(m.msg.RxMessage[4] >> 4) & 0xF, m.msg.RxMessage[4] & 0xF,
			(m.msg.RxMessage[4] >> 14) & 3, (m.msg.RxMessage[4] >> 8) & 0x3F);
				*ppos += (p - buf);
	} else if (strcmp(entry->name, "fw_date") == 0) {
		if (*ppos > 0)	/* Assume reading completed in previous read */
			return 0;
		if (DSL_DEV_PRIVATE(pDev)->modem_ready_cnt < 1)
			return -EAGAIN;

		makeCMV(H2D_CMV_READ, DSL_CMV_GROUP_INFO, 55, 0, 1, NULL, m.msg.TxMessage);
		if (DSL_BSP_SendCMV(pDev, m.msg.TxMessage, YES_REPLY, m.msg.RxMessage)
				!= DSL_DEV_MEI_ERR_SUCCESS)
			return -EIO;
		/* Day/Month */
		p += sprintf(p, "FW Date: %d.%d.",
					m.msg.RxMessage[4] & 0xFF, (m.msg.RxMessage[4] >> 8) & 0xFF);

		makeCMV(H2D_CMV_READ, DSL_CMV_GROUP_INFO, 55, 2, 1, NULL, m.msg.TxMessage);
		if (DSL_BSP_SendCMV(pDev, m.msg.TxMessage, YES_REPLY, m.msg.RxMessage)
				!= DSL_DEV_MEI_ERR_SUCCESS)
			return -EIO;
		/* Year */
		p += sprintf(p, "%d ", m.msg.RxMessage[4]);

		makeCMV(H2D_CMV_READ, DSL_CMV_GROUP_INFO, 55, 1, 1, NULL, m.msg.TxMessage);
		if (DSL_BSP_SendCMV(pDev, m.msg.TxMessage, YES_REPLY, m.msg.RxMessage)
				!= DSL_DEV_MEI_ERR_SUCCESS)
			return -EIO;
		/* Hour:Minute */
		p += sprintf(p, "%d:%d\n",
				(m.msg.RxMessage[4] >> 8) & 0xFF, m.msg.RxMessage[4] & 0xFF);

		*ppos += (p - buf);
	} else if (strcmp(entry->name, "version") == 0) {
		if (*ppos > 0)	/* Assume reading completed in previous read */
			return 0;
		p += sprintf (p, "IFX MEI V%ld.%ld.%ld\n",
			bsp_mei_version.major, bsp_mei_version.minor, bsp_mei_version.revision);
		p += sprintf (p, "Compiled on %s, %s for Linux kernel %s (jiffies: %lu)\n",
			__DATE__, __TIME__, UTS_RELEASE, jiffies);
		*ppos += (p - buf);
	} else if (strcmp(entry->name, "debug") == 0) {
		if (*ppos > 0) /* Assume reading completed in previous read */
			return 0;
		p += sprintf (p, "%s\n", ifx_mei_debug ? "Debug ON":"Debug Off");
		*ppos += (p - buf);
	} else if (entry->flag != (int *) DSL_DEV_PRIVATE(pDev)->Recent_indicator) {
		if (*ppos > 0)	/* Assume reading completed in previous read */
			return 0;   /*  indicates end of file */
		p += sprintf (p, "0x%08X\n\n", *(entry->flag));
		*ppos += (p - buf);
		if ((p - buf) > nbytes)	/* Assume output can be read at one time */
			return -EINVAL;
	} else {
		if ((int) (*ppos) / ((int) 7) == 16)
			return 0;   /*  indicate end of the message */
		p += sprintf (p, "0x%04X\n\n",
				*(((u16 *) (entry->flag)) + (int) (*ppos) / ((int) 7)));
		*ppos += (p - buf);
	}
	return p -  buf;
}

/**
	Writing function for linux proc filesystem
*/
static ssize_t IFX_MEI_ProcWrite(struct file *file, const char *buffer, size_t count, loff_t *ppos)
{
	unsigned long i_ino = (file->f_dentry->d_inode)->i_ino;
	reg_entry_t *current_reg = NULL;
	int i = 0, len, rlen;
	int num = 0;
	unsigned long newRegValue = 0;
	char *p, *endp = NULL;
	char str[10];
	DSL_DEV_Device_t *pDev = NULL;

	len = count < sizeof(str) ? count : sizeof(str) - 1;
	if (!access_ok(VERIFY_READ, buffer, len))
		return -EFAULT;

	for (num = 0; num < BSP_MAX_DEVICES; num++) {
	for (i = 0; i < NUM_OF_REG_ENTRY; i++) {
			if (regs[num][i].low_ino == i_ino) {
				current_reg = &regs[num][i];
				pDev = &dsl_devices[num];
			break;
		}
	}
	}
	if ((current_reg == NULL)
		|| (current_reg->flag == (int *) DSL_DEV_PRIVATE(pDev)->Recent_indicator))
		return -EINVAL;

	rlen = len - copy_from_user(str, buffer, len);
	while (rlen && str[rlen - 1] <= ' ')
		rlen--;
	str[rlen] = 0;
	for (p = str; *p && *p <= ' '; p++, rlen--);
	if (!*p)
	return 0;

	newRegValue = simple_strtoul(p, &endp, 10);
	if (!strcmp(current_reg->name, "debug")) {
		ifx_mei_debug = (int) newRegValue;
		return count;
	} else {
		*(current_reg->flag) = (int) newRegValue;
		return count + endp - buffer;
		}
	}
#endif /* CONFIG_PROC_FS */

/**
	Initialize kernel module.
*/
int __init IFX_MEI_ModuleInit(void)
{
	int i = 0;

	printk ("IFX MEI Version %ld.%02ld.%02ld\n", bsp_mei_version.major, bsp_mei_version.minor, bsp_mei_version.revision);

	for (i = 0; i < BSP_MAX_DEVICES; i++) {
		if (IFX_MEI_InitDevice (i) != 0) {
			printk ("%s: Init device fail!\n", __func__);
			return -EIO;
	}
		IFX_MEI_InitDevNode(i);
#ifdef CONFIG_PROC_FS
		IFX_MEI_InitProcFS(i);
#endif
}
	for (i = 0; i <= DSL_BSP_CB_LAST ; i++) {
		dsl_bsp_event_callback[i].function = NULL;
}

	IFX_ADSL_LEDHandler = NULL;
	IFX_MEI_set_cell_rate = NULL;

	ifx_usb_enable_afe_oc();

	return 0;
}

/**
	Shutdown kernel module.
 */
void __exit IFX_MEI_ModuleExit(void)
{
	DSL_DEV_Device_t *pDev = &dsl_devices[0];
	int i = 0;
	unsigned long flags;
	int num;
	local_irq_save(flags);
	for (num = 0; num < BSP_MAX_DEVICES; num++) {
		IFX_MEI_CleanUpDevNode (num);
#ifdef CONFIG_PROC_FS
		for (i = 0; i < NUM_OF_REG_ENTRY; i++) {
			remove_proc_entry(regs[num][i].name, meidir);
	}
#endif
	}

	remove_proc_entry ("driver/"MEI_DIRNAME, NULL);
	for (i = 0; i < BSP_MAX_DEVICES; i++) {
		for (i = 0; i < BSP_MAX_DEVICES; i++) {
			IFX_MEI_ExitDevice(i);
	}
	}
	IFX_MEI_DFEMemoryFree(pDev, FREE_ALL);   /* free all */
	kmem_cache_destroy(mei_cache);
	local_irq_restore(flags);
}

/* export function for DSL Driver */
EXPORT_SYMBOL_GPL(DSL_BSP_DriverHandleGet);
EXPORT_SYMBOL_GPL(DSL_BSP_DriverHandleDelete);

EXPORT_SYMBOL_GPL(DSL_BSP_ATMLedCBRegister);
EXPORT_SYMBOL_GPL(DSL_BSP_ATMLedCBUnregister);
EXPORT_SYMBOL_GPL(DSL_BSP_KernelIoctls);
EXPORT_SYMBOL_GPL(DSL_BSP_AdslLedInit);
EXPORT_SYMBOL_GPL(DSL_BSP_AdslLedSet);
EXPORT_SYMBOL_GPL(DSL_BSP_FWDownload);
EXPORT_SYMBOL_GPL(DSL_BSP_Showtime);

EXPORT_SYMBOL_GPL(DSL_BSP_MemoryDebugAccess);
EXPORT_SYMBOL_GPL(DSL_BSP_SendCMV);

/* provide a register/unregister function for DSL driver to
	register a event callback function */
EXPORT_SYMBOL_GPL(DSL_BSP_EventCBRegister);
EXPORT_SYMBOL_GPL(DSL_BSP_EventCBUnregister);
EXPORT_SYMBOL_GPL(DSL_BSP_ATMLedHandler);

EXPORT_SYMBOL_GPL(IFX_MEI_set_cell_rate);

MODULE_LICENSE("GPL");

module_init(IFX_MEI_ModuleInit);
module_exit(IFX_MEI_ModuleExit);


#define ADSL_BASE 0x20000
#define CRI_CCR1          CRI_BASE + 0x01*4
#define CRI_CDC0          CRI_BASE + 0x02*4
#define CRI_CDC1          CRI_BASE + 0x03*4
#define CRI_MASK0         CRI_BASE + 0x05*4
#define CRI_MASK1         CRI_BASE + 0x06*4
#define CRI_MASK2         CRI_BASE + 0x07*4
#define CRI_STATUS0       CRI_BASE + 0x08*4
#define CRI_STATUS1       CRI_BASE + 0x09*4
#define CRI_STATUS2       CRI_BASE + 0x0A*4
#define CRI_AMASK0        CRI_BASE + 0x0B*4
#define CRI_AMASK1        CRI_BASE + 0x0C*4
#define CRI_UPDCTL        CRI_BASE + 0x0D*4
#define CRI_MADST         CRI_BASE + 0x0E*4
// 0x0f is missing
#define CRI_EVENT0        CRI_BASE + 0x10*4
#define CRI_EVENT1        CRI_BASE + 0x11*4
#define CRI_EVENT2        CRI_BASE + 0x12*4

#define IRI_I_ENABLE    0x32000
#define STY_SMODE       0x3c004
#define AFE_TCR_0       0x3c0dc
#define AFE_ADDR_ADDR   0x3c0e8
#define AFE_RDATA_ADDR  0x3c0ec
#define AFE_WDATA_ADDR  0x3c0f0
#define AFE_CONFIG      0x3c0f4
#define AFE_SERIAL_CFG  0x3c0fc

#define DFE_BASE_ADDR         0xBE116000
//#define DFE_BASE_ADDR         0x9E116000

#define MEI_FR_ARCINT_C       (DFE_BASE_ADDR + 0x0000001C)
#define MEI_DBG_WADDR_C       (DFE_BASE_ADDR + 0x00000024)
#define MEI_DBG_RADDR_C       (DFE_BASE_ADDR + 0x00000028)
#define MEI_DBG_DATA_C        (DFE_BASE_ADDR + 0x0000002C)
#define MEI_DBG_DECO_C        (DFE_BASE_ADDR + 0x00000030)
#define MEI_DBG_MASTER_C      (DFE_BASE_ADDR + 0x0000003C)

/**
 * Reads the content of a register.
 *
 * @param _reg address of register to read.
 * @return contents of the register.
 *

 * Usage:<br>
 * <code>uint32_t dev_ctl = dwc_read_reg32(&dev_regs->dctl);</code>
 */
static __inline__ uint32_t dwc_read_reg32( volatile uint32_t *_reg)
{
	//return btos(readl(_reg));
	return *_reg;
	//   return readl(_reg);
};

void WriteARCmem(uint32_t addr, uint32_t data)
	{
	writel(1    ,(volatile uint32_t *)MEI_DBG_MASTER_C);
	writel(1    ,(volatile uint32_t *)MEI_DBG_DECO_C  );
	writel(addr ,(volatile uint32_t *)MEI_DBG_WADDR_C  );
	writel(data ,(volatile uint32_t *)MEI_DBG_DATA_C  );
	while( (dwc_read_reg32((volatile uint32_t *)MEI_FR_ARCINT_C) & 0x20) != 0x20 ){};
	writel(0    ,(volatile uint32_t *)MEI_DBG_MASTER_C);
	printk(KERN_INFO "WriteARCmem %08x %08x\n",addr,data);
};

uint32_t ReadARCmem(uint32_t addr)
		{
	u32 data;
	writel(1    ,(volatile uint32_t *)MEI_DBG_MASTER_C);
	writel(1    ,(volatile uint32_t *)MEI_DBG_DECO_C  );
	writel(addr ,(volatile uint32_t *)MEI_DBG_RADDR_C  );
	while( (dwc_read_reg32((volatile uint32_t *)MEI_FR_ARCINT_C) & 0x20) != 0x20 ){};
	data = dwc_read_reg32((volatile uint32_t *)MEI_DBG_DATA_C  );
	writel(0    ,(volatile uint32_t *)MEI_DBG_MASTER_C);
	printk(KERN_INFO "ReadARCmem %08x %08x\n",addr,data);
	return data;
};

void ifx_usb_enable_afe_oc(void)
{
	/* Start the clock */
	WriteARCmem(CRI_UPDCTL    ,0x00000008);
	WriteARCmem(CRI_CCR0      ,0x00000014);
	WriteARCmem(CRI_CCR1      ,0x00000500);
	WriteARCmem(AFE_CONFIG    ,0x000001c8);
	WriteARCmem(AFE_SERIAL_CFG,0x00000016); // (DANUBE_PCI_CFG_BASE+(1<<addrline))AFE serial interface clock & data latch edge
	WriteARCmem(AFE_TCR_0     ,0x00000002);
	//Take afe out of reset
	WriteARCmem(AFE_CONFIG    ,0x000000c0);
	WriteARCmem(IRI_I_ENABLE  ,0x00000101);
	WriteARCmem(STY_SMODE     ,0x00001980);

	ReadARCmem(CRI_UPDCTL    );
	ReadARCmem(CRI_CCR0      );
	ReadARCmem(CRI_CCR1      );
	ReadARCmem(AFE_CONFIG    );
	ReadARCmem(AFE_SERIAL_CFG); // (DANUBE_PCI_CFG_BASE+(1<<addrline))AFE serial interface clock & data latch edge
	ReadARCmem(AFE_TCR_0     );
	ReadARCmem(AFE_CONFIG    );
	ReadARCmem(IRI_I_ENABLE  );
	ReadARCmem(STY_SMODE     );
}
