#include <stdlib.h>
#include <stdio.h>
#include <string.h>
#include <sys/time.h>
#include <sys/types.h>
#include <sys/mman.h>
#include <sys/ioctl.h>
#include <unistd.h>
#include <fcntl.h>

#define SVIPDUMP_MEM_SIZE   0x1900000
#define SRAM_SIZE           (128 << 10)
#define DATA_OFFSET         (1 << 20)
#define DATA_SIZE           (4 << 20)

int main (int argc, char **argv)
{
    int svipfd;
    int ii;
    char out_file[32];
    unsigned char *svip_mem;
    unsigned char *svip_sram;
    unsigned char *svip_data;
    unsigned int core_start, core_stop;

    if (argc == 2)
    {
        core_stop = core_start = atoi(argv[1]);
        if (core_start < 0 || core_start > 5)
        {
            printf("Invalid core id\n");
            exit(-1);
        }
    }
    else
    {
        core_start = 0; 
        core_stop = 5;
    }

    if ((svipfd = open( "/dev/svipdump_mem", O_RDONLY )) < 0)
    {
        printf("open device failed\n");
        exit(-1);
    }

    svip_mem = (unsigned char *)mmap( (void*)0,
        SVIPDUMP_MEM_SIZE,
        PROT_READ,
        MAP_SHARED,
        svipfd,
        0 );
    if (svip_mem == (void*)-1)
        printf("mmap on svipdump_mem failed\n");

    for (ii = core_start; ii <= core_stop; ii++)
    {
        int fd_out;

        svip_sram = svip_mem + ii * SRAM_SIZE;
        svip_data = svip_mem + DATA_OFFSET + ii * DATA_SIZE;

        sprintf(out_file, "/tmp/core%d_sram.dat", ii);
        if ((fd_out = open( out_file, O_WRONLY|O_CREAT )) < 0)
        {
            printf("open output file %s failed\n", out_file);
            exit(-1);
        }
        if (write(fd_out, svip_sram, SRAM_SIZE) != SRAM_SIZE)
        {
            printf("write output file %s failed\n", out_file);
        }
        close(fd_out);

        sprintf(out_file, "/tmp/core%d_data.dat", ii);
        if ((fd_out = open( out_file, O_WRONLY|O_CREAT )) < 0)
        {
            printf("open output file %s failed\n", out_file);
            exit(-1);
        }
        if (write(fd_out, svip_data, DATA_SIZE) != DATA_SIZE)
        {
            printf("write output file %s failed\n", out_file);
        }
        close(fd_out);
    }

    close(svipfd);
}
