//=======================
//	Includes
//=======================
#include <linux/module.h>
#include <linux/device.h>
#include <linux/init.h>
#include <linux/kernel.h>   /* printk() */
#include <linux/slab.h>     /* kmalloc() */
#include <linux/fs.h>       /* everything... */
#include <linux/errno.h>    /* error codes */
#include <linux/types.h>    /* size_t */
#include <linux/cdev.h>     /* Used for struct cdev */
#include <linux/mm.h>
#include <linux/vmalloc.h>

//=======================
// 	Defines
//=======================

#define SVIPDUMP_MAJOR      240
#define SVIPDUMP_DEVICE     "svipdump_mem"

#define SVIP_DDR_SIZE		0x01900000
#define SVIP_DDR_OFFSET		0x05f00000

static int svipdump_mem_major = SVIPDUMP_MAJOR;
static struct class *svipdump_mem_class;

static int open_svipdump_mem(struct inode* inode, struct file* filp);
static int close_svipdump_mem(struct inode* inode, struct file* filp);
static int mmap_svipdump_mem(struct file * filp, struct vm_area_struct * vma);

static struct file_operations svipdump_mem_fops = {
    .open       = open_svipdump_mem,
    .mmap       = mmap_svipdump_mem,
    .release    = close_svipdump_mem,
};


/*
 * mmap file ops
 */

//===========================================================================
static int 
mmap_svipdump_mem(
	struct file * filp, 
	struct vm_area_struct * vma
)
//===========================================================================
{
	unsigned long physical = SVIP_DDR_OFFSET >> PAGE_SHIFT;
	unsigned long vsize = vma->vm_end - vma->vm_start;

	//vma->vm_page_prot = pgprot_noncached (vma->vm_page_prot) ;

    if (vsize > SVIP_DDR_SIZE) vsize = SVIP_DDR_SIZE;
    
	if (remap_pfn_range (vma,
                         vma->vm_start,
                         physical,
                         vsize,
                         vma->vm_page_prot)) {
        return -EAGAIN ;
    }
	return 0;
}

/* ioread32 ioremap_nocache memset memcmp */
static int
open_svipdump_mem(
    struct inode* inode,
    struct file* filp
)
{
    filp->f_op = &svipdump_mem_fops;

    return 0;
}

static int
close_svipdump_mem(
    struct inode* inode,
    struct file* filp
)
{
    return 0;
}

/*
 * Module Init
 */
static int svipdump_mem_init(void)
{
    int result;

    result = register_chrdev(svipdump_mem_major, SVIPDUMP_DEVICE, &svipdump_mem_fops);

    if (result < 0)
    {
        printk("register dev failed, SVIP memory dump major %d!\n", svipdump_mem_major);
        return result;
    }
    else
        printk("registered %s device\n", SVIPDUMP_DEVICE);

    svipdump_mem_class = class_create(THIS_MODULE, SVIPDUMP_DEVICE);
    device_create(svipdump_mem_class, NULL, MKDEV(svipdump_mem_major, 0), NULL, SVIPDUMP_DEVICE);

    return 0;
}


static void svipdump_mem_cleanup(void)
{
   unregister_chrdev( svipdump_mem_major, SVIPDUMP_DEVICE );
}


module_init(svipdump_mem_init);
module_exit(svipdump_mem_cleanup);

MODULE_DESCRIPTION("SVIP fw memory dump driver - www.grandstream.com");
MODULE_AUTHOR("Grandstream Networks, System group");
