/************************************************************************
 *
 * Copyright (c) 2005
 * Infineon Technologies AG
 * St. Martin Strasse 53; 81669 Muenchen; Germany
 *
 * This program is free software; you can redistribute it and/or
 * modify it under the terms of the GNU General Public License
 * as published by the Free Software Foundation; either version
 * 2 of the License, or (at your option) any later version.
 *
 ************************************************************************/

/* Group definitions for Doxygen */
/** \defgroup API API-Functions */
/** \defgroup Internal Internally used functions */

#ifndef __KERNEL__
#define __KERNEL__
#endif

#include <linux/kernel.h>
#include <linux/slab.h>
#include <linux/errno.h>
#include <linux/types.h>
#include <linux/interrupt.h>
#include <linux/uaccess.h>
#include <linux/in.h>
#include <linux/netdevice.h>
#include <linux/inetdevice.h>
#include <linux/etherdevice.h>
#include <linux/ip.h>
#include <linux/tcp.h>
#include <linux/skbuff.h>
#include <linux/mm.h>
#include <linux/platform_device.h>
#include <linux/ethtool.h>
#include <linux/init.h>
#include <linux/delay.h>
#include <asm/checksum.h>
#include <asm/ifxmips/ifxmips.h>
#include <asm/ifx/ifx_dma_core.h>
#include <asm/ifx/ifx_dma.h>
#include <linux/udp_redirect.h>

#define IFXMIPS_DMA_CH_ON     IFX_DMA_CH_ON
#define IFXMIPS_DMA_CH_OFF    IFX_DMA_CH_OFF

#ifdef CONFIG_DEBUG_MINI_BOOT
#define IKOS_MINI_BOOT
#endif

/* debugging */
#undef INCAIP2_SW_DUMP
//#define INCAIP2_SW_DUMP

#define INCAIP2_SW_EMSG(fmt,args...) printk("%s: " fmt, __FUNCTION__ , ##args)

#define INCAIP2_SW_CHIP_NO 1
#define INCAIP2_SW_CHIP_ID 0
#define INCAIP2_SW_DEVICE_NO 1

#ifdef INCAIP2_SW_DEBUG_MSG
#define INCAIP2_SW_DMSG(fmt,args...) printk("%s: " fmt, __FUNCTION__ , ##args)
#else
#define INCAIP2_SW_DMSG(fmt,args...)
#endif

#define SVIP_NET_DEV_ETH0_IDX       0

#if LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24)
#define SVIP_IP_HDR(ethhdr) ip_hdr(ethhdr)
#else
#define SVIP_IP_HDR(ethhdr) (ethhdr)->nh.iph
#endif /* LINUX_VERSION_CODE >= KERNEL_VERSION(2,6,24) */


/***************************************** Module Parameters *************************************/
static char *mode = "bridge";
module_param(mode, charp, 0000);
MODULE_PARM_DESC(mode, "<description>");

#ifdef HAVE_TX_TIMEOUT
static int timeout = 10*HZ;
module_param(timeout, int, 0);
MODULE_PARM_DESC(timeout, "Transmission watchdog timeout in seconds>");
#endif

int ifx_probed=0;
int ifx_chip_init=0;

u32 ifx_eth_dma_rx=0;
u32 ifx_eth_dma_alloc=0;
u32 ifx_eth_dma_rx_err=0;
u32 ifx_eth_dma_alloc_drop=0;
u32 ifx_eth_dma_free=0;
u32 ifx_eth_dma_tx=0;
u32 ifx_eth_dma_tx_full=0;
u32 ifx_eth_dma_tx_drop=0;
u32 ifx_eth_dma_free_err=0;

/* phy link status */
extern int old_link_status[2];
extern int old_phy_speed[2];
extern int old_phy_duplex[2];

static u32 *paddr_eth0;
static u32 *paddr_eth0_0;

#ifdef IKOS_MINI_BOOT
#ifdef CONFIG_INCAIP2
extern s32 incaip2_sw_to_mbx(struct sk_buff* skb);
#endif
#ifdef CONFIG_SVIP
extern s32 svip_sw_to_mbx(struct sk_buff* skb);
#endif
#endif
extern void enable_ch_tx_complete_irq(struct dma_channel_info *pCh);
extern void disable_ch_tx_complete_irq(struct dma_channel_info *pCh);

extern int open_chan_2(struct dma_channel_info* pCh);
extern int close_chan_2(struct dma_channel_info* pCh);

struct ifxmips_mii_priv {
	struct net_device_stats stats;
	struct dma_device_info *dma_device;
	struct sk_buff *skb;
};

static struct net_device *ifxmips_mii0_dev;
static unsigned char mac_addr[MAX_ADDR_LEN];

/**
 * Print hexdump of data.
 * This function prints a hexdump of the given data. If INCAIP2_SW_DUMP is not
 * defined, this function will be do nothing.
 *
 * \param   len      Length of data buffer
 * \param   *data    Pointer to data
 * \ingroup Internal
 */
static inline void ifx_eth_dump_skb(u32 len, char * data)
{
#ifdef INCAIP2_SW_DUMP
	int i;

	for(i=0;i<len;i++)
    {
		printk("%2.2x ",(u8)(data[i]));
		if (i % 16 == 15)
			printk("\n");
	}
	printk("\n");
#endif
}


static unsigned char my_ethaddr[MAX_ADDR_LEN];
/**
 * Initialize MAC address.
 * This function copies the ethernet address from kernel command line.
 *
 * \param   line     Pointer to parameter
 * \return  0        OK
 * \ingroup Internal
 */
static int __init ifx_eth_ethaddr_setup(char *line)
{
	char *ep;
	int i;

	memset(my_ethaddr, 0, MAX_ADDR_LEN);
	/* there should really be routines to do this stuff */
	for (i = 0; i < 6; i++)
	{
		my_ethaddr[i] = line ? simple_strtoul(line, &ep, 16) : 0;
		if (line)
			line = (*ep) ? ep+1 : ep;
	}
	INCAIP2_SW_DMSG("mac address %2x-%2x-%2x-%2x-%2x-%2x \n"
		,my_ethaddr[0]
		,my_ethaddr[1]
		,my_ethaddr[2]
		,my_ethaddr[3]
		,my_ethaddr[4]
		,my_ethaddr[5]);
	return 0;
}
__setup("ethaddr=", ifx_eth_ethaddr_setup);


/**
 * Open RX DMA channels.
 * This function opens all DMA rx channels.
 *
 * \param   dma_dev     pointer to DMA device information
 * \ingroup Internal
 */
static void ifx_eth_open_rx_dma(struct dma_device_info *dma_dev)
{
   int i;

   for(i=0; i<dma_dev->num_rx_chan; i++)
   {
      dma_dev->rx_chan[i]->open(dma_dev->rx_chan[i]);
   }
}


/**
 * Open TX DMA channels.
 * This function opens all DMA tx channels.
 *
 * \param   dev      pointer to net device structure that comprises
 *                   DMA device information pointed to by it's priv field.
 * \ingroup Internal
 */
static void ifx_eth_open_tx_dma(struct dma_device_info *dma_dev)
{
   int i;

   for (i=0; i<dma_dev->num_tx_chan; i++)
   {
      dma_dev->tx_chan[i]->open(dma_dev->tx_chan[i]);
   }
}

#ifdef CONFIG_NET_HW_FLOWCONTROL
/**
 * Enable receiving DMA.
 * This function enables the receiving DMA channel.
 *
 * \param   dev      pointer to net device structure that comprises
 *                   DMA device information pointed to by it's priv field.
 * \ingroup Internal
 */
void ifx_eth_xon(struct net_device *dev)
{
   struct switch_priv *sw_dev = (struct switch_priv *)dev->priv;
   struct dma_device_info* dma_dev=(struct dma_device_info *)(sw_dev->dma_device);
   unsigned long flag;

   local_irq_save(flag);

   INCAIP2_SW_DMSG("wakeup\n");
   ifx_eth_open_rx_dma(dma_dev);

   local_irq_restore(flag);
}
#endif /* CONFIG_NET_HW_FLOWCONTROL */


/**
 * Open network device.
 * This functions opens the network device and starts the interface queue.
 *
 * \param   dev  Device structure for Ethernet device
 * \return  0    OK, device opened
 * \return  -1   Error, registering DMA device
 * \ingroup API
 */
int ifxmips_mii_open(struct net_device *dev)
{
	struct ifxmips_mii_priv *priv = (struct ifxmips_mii_priv *)dev->priv;
	struct dma_device_info *dma_dev = priv->dma_device;

   ifx_eth_open_rx_dma(dma_dev);
   ifx_eth_open_tx_dma(dma_dev);

	netif_start_queue(dev);
	return 0;
}


/**
 * Close network device.
 * This functions closes the network device, which will also stop the interface
 * queue.
 *
 * \param   dev  Device structure for Ethernet device
 * \return  0    OK, device closed (cannot fail)
 * \ingroup API
 */
int ifxmips_mii_release(struct net_device *dev)
{
	struct ifxmips_mii_priv *priv = (struct ifxmips_mii_priv *)dev->priv;
	struct dma_device_info *dma_dev = priv->dma_device;
	int i;

	for (i = 0; i < dma_dev->max_rx_chan_num; i++)
		dma_dev->rx_chan[i]->close(dma_dev->rx_chan[i]);
	netif_stop_queue(dev);
	return 0;
}


/**
 * Read data from DMA device.
 * This function reads data from the DMA device. The function is called by
 * the switch/DMA pseudo interrupt handler dma_intr_handler on occurence of
 * a DMA receive interrupt.
 *
 * \param   dev      Pointer to network device structure
 * \param   dma_dev  Pointer to dma device structure
 * \return  OK       In case of successful data reception from dma
 *          -EIO     Incorrect opt pointer provided by device
 * \ingroup Internal
 */
int ifxmips_mii_hw_receive(struct net_device *dev, struct dma_device_info *dma_dev)
{
	struct ifxmips_mii_priv *priv = (struct ifxmips_mii_priv *)dev->priv;
	unsigned char *buf = NULL;
	struct sk_buff *skb = NULL;
	int len = 0;

	len = dma_device_read(dma_dev, &buf, (void **)&skb);

	if (len >= ETHERNET_PACKET_DMA_BUFFER_SIZE) {
		printk(KERN_INFO "ifxmips_mii0: packet too large %d\n", len);
		goto ifxmips_mii_hw_receive_err_exit;
	}

	if (skb == NULL) {
		printk(KERN_INFO "ifxmips_mii0: cannot restore pointer\n");
		goto ifxmips_mii_hw_receive_err_exit;
	}

	if (len > (skb->end - skb->tail)) {
		printk(KERN_INFO "ifxmips_mii0: BUG, len:%d end:%p tail:%p\n",
			len, skb->end, skb->tail);
		goto ifxmips_mii_hw_receive_err_exit;
	}

	skb_put(skb, len);
	skb->dev = dev;
	skb->protocol = eth_type_trans(skb, dev);
	netif_rx(skb);

	priv->stats.rx_packets++;
	priv->stats.rx_bytes += len;
	return 0;

ifxmips_mii_hw_receive_err_exit:
	if (len == 0) {
		if (skb)
			dev_kfree_skb_any(skb);
		priv->stats.rx_errors++;
		priv->stats.rx_dropped++;
		return -EIO;
	} else {
		return len;
	}
}


/**
 * Write data to Ethernet switch.
 * This function writes the data comprised in skb structure via DMA to the
 * Ethernet Switch. It is installed as the switch driver's hard_start_xmit
 * method.
 *
 * \param   skb  Pointer to socket buffer structure that contains the data
 *               to be sent
 * \param   dev  Pointer to network device structure which is used for
 *               data transmission
 * \return  1    Transmission error
 * \return  0    OK, successful data transmission
 * \ingroup API
 */
static int ifxmips_mii_hw_tx(char *buf, int len, struct net_device *dev)
{
	int ret = 0;
	struct ifxmips_mii_priv *priv = dev->priv;
	struct dma_device_info *dma_dev = priv->dma_device;
	ret = dma_device_write(dma_dev, buf, len, priv->skb);
	return ret;
}

/* Select the DMA channel numbers, refer dma_setup_init function */
static int ifx_select_tx_chan (struct sk_buff *skb, struct net_device *dev)
{
    struct iphdr *iph;
    struct in_device *in_dev;
    struct in_ifaddr *ifa1, *ifa2;

    iph = SVIP_IP_HDR(skb);
    if (iph->protocol != IPPROTO_UDP)
    {
       /* all non-UDP packets go through channel 0 */
       return 0;
    }

    if (!paddr_eth0)
    {
       /* find pointers to IP addresses of eth0 */
       if ((in_dev=in_dev_get(dev)) != NULL)
       {
          ifa1 = in_dev->ifa_list;
          if (ifa1 != NULL)
             ifa2 = ifa1->ifa_next;
          if (ifa1 != NULL && ifa2 != NULL)
          {
             if (ifa1->ifa_address != 0)
             {
                paddr_eth0 = &ifa1->ifa_address;
             }
             if (ifa2->ifa_address != 0)
             {
                paddr_eth0_0 = &ifa2->ifa_address;
             }
          }
          in_dev_put(in_dev);
       }
    }
    else
    {
       if (iph->saddr == *paddr_eth0_0)
       {
          return 1;
       }
       else
          return 0;
    }
    return 0;
}

static int ifxmips_mii_tx(struct sk_buff *skb, struct net_device *dev)
{
	int len;
	char *data;
	struct ifxmips_mii_priv *priv = dev->priv;
	struct dma_device_info *dma_dev = priv->dma_device;
	/* set frame trailer to zero */ 
    len = skb->len;
    if( skb->len < ETH_ZLEN)
    {
        if (skb_padto(skb, ETH_ZLEN))
            return 0;
        len = ETH_ZLEN;
    }

	len = skb->len < ETH_ZLEN ? ETH_ZLEN : skb->len;
	data = skb->data;
	priv->skb = skb;
	dev->trans_start = jiffies;
   /* select the TX channel */
   dma_dev->current_tx_chan = ifx_select_tx_chan (skb, dev);

	wmb();

#ifdef CONFIG_LTQ_UDP_REDIRECT
   /* hook tx packets here */
   if ( udp_do_tx_redirect_fn )
   {
      if (udp_do_tx_redirect_fn(skb))
      {
         return 0;
      }
   }
#endif

	if (ifxmips_mii_hw_tx(data, len, dev) != len) {
		dev_kfree_skb_any(skb);
		priv->stats.tx_errors++;
		priv->stats.tx_dropped++;
	} else {
		priv->stats.tx_packets++;
		priv->stats.tx_bytes += len;
	}

	return 0;
}


/**
 * Transmission timeout callback.
 * This functions is called when a trasmission timeout occurs. It will wake up
 * the interface queue again.
 *
 * \param   dev Device structure for Ethernet device
 * \ingroup API
 */
void ifxmips_mii_tx_timeout(struct net_device *dev)
{
   struct ifxmips_mii_priv *priv = (struct ifxmips_mii_priv *)dev->priv;
   struct dma_device_info *dma_dev = priv->dma_device;
   int i;

   if (printk_ratelimit())
      printk(KERN_INFO "ifxmips_mii0: tx timeout\n");
   priv->stats.tx_errors++;
   for (i=0; i<dma_dev->num_tx_chan; i++)
   {
      close_chan_2(dma_dev->tx_chan[i]);
      open_chan_2(dma_dev->tx_chan[i]);
   }
   netif_wake_queue(dev);
}


/**
 * Get device statistics.
 * This functions returns the device statistics, stored in the device structure.
 *
 * \param   dev   Device structure for Ethernet device
 * \return  stats Pointer to statistics structure
 * \ingroup API
 */
static struct net_device_stats *ifxmips_get_stats(struct net_device *dev)
{
	return (struct net_device_stats *)dev->priv;
}


/**
 * Pseudo Interrupt handler for DMA.
 * This function processes DMA interrupts notified to the switch device driver.
 * The function is installed at the DMA core as interrupt handler for the
 * switch dma device.
 * It handles the following DMA interrupts:
 * passes received data to the upper layer in case of rx interrupt,
 * In case of a dma receive interrupt the received data is passed to the upper layer.
 * In case of a transmit buffer full interrupt the transmit queue is stopped.
 * In case of a transmission complete interrupt the transmit queue is restarted.
 *
 * \param   dma_dev pointer to dma device structure
 * \param   status  type of interrupt being notified (RCV_INT: dma receive
 *                  interrupt, TX_BUF_FULL_INT: transmit buffer full interrupt,
 *                  TRANSMIT_CPT_INT: transmission complete interrupt)
 * \return  OK      In case of successful data reception from dma
 * \ingroup Internal
 */
int dma_intr_handler(struct dma_device_info *dma_dev, int status)
{
	int i;

	switch (status)
   {
	case RCV_INT:
		ifxmips_mii_hw_receive(ifxmips_mii0_dev, dma_dev);
		break;

	case TX_BUF_FULL_INT:
      printk(KERN_INFO "ifxmips_mii0: tx buffer full\n");
      netif_stop_queue(ifxmips_mii0_dev);
      if (printk_ratelimit())
         printk(KERN_INFO "ifxmips_mii0: tx buffer full\n");
      for (i = 0; i < dma_dev->max_tx_chan_num; i++)
      {
			if ((dma_dev->tx_chan[i])->control == IFXMIPS_DMA_CH_ON)
         {
            enable_ch_tx_complete_irq(dma_dev->tx_chan[i]);
         }
		}
		break;

	case TRANSMIT_CPT_INT:
      if (netif_queue_stopped(ifxmips_mii0_dev))
      {
         if (printk_ratelimit())
            printk(KERN_INFO "ifxmips_mii0: tx complete\n");
         for (i = 0; i < dma_dev->max_tx_chan_num; i++)
         {
		      if ((dma_dev->tx_chan[i])->control == IFXMIPS_DMA_CH_ON)
            {
               disable_ch_tx_complete_irq(dma_dev->tx_chan[i]);
            }
         }
		   netif_wake_queue(ifxmips_mii0_dev);
      }
		break;
	}

	return 0;
}


/**
 * Allocates buffer sufficient for Ethernet Frame.
 * This function is installed as DMA callback function to be called on DMA
 * receive interrupt.
 *
 * \param   len          Unused
 * \param   *byte_offset Pointer to byte offset
 * \param   **opt        pointer to skb structure
 * \return  NULL         In case of buffer allocation fails
 *          buffer       Pointer to allocated memory
 * \ingroup Internal
 */
unsigned char *ifxmips_etop_dma_buffer_alloc(int len, int *byte_offset, void **opt)
{
	unsigned char *buffer = NULL;
	struct sk_buff *skb = NULL;

	skb = dev_alloc_skb(ETHERNET_PACKET_DMA_BUFFER_SIZE);
	if (skb == NULL)
		return NULL;

	buffer = (unsigned char *)(skb->data);
	skb_reserve(skb, 2);
	*(int *)opt = (int)skb;
	*byte_offset = 2;

	return buffer;
}


/**
 * Free DMA buffer.
 * This function frees a buffer, which can be either a data buffer or an
 * skb structure.
 *
 * \param   *dataptr Pointer to data buffer
 * \param   *opt     Pointer to skb structure
 * \return  0        OK
 * \ingroup Internal
 */
void ifxmips_etop_dma_buffer_free(unsigned char *dataptr, void *opt)
{
	struct sk_buff *skb = NULL;

	if (opt == NULL) {
		kfree(dataptr);
	} else {
		skb = (struct sk_buff *)opt;
		dev_kfree_skb_any(skb);
	}
}

u32 ifxmii_ethtool_op_get_settings(struct net_device *dev, struct ethtool_cmd * ecmd)
{   
    int port = 0;

    memset(ecmd, 0, sizeof(struct ethtool_cmd));

    ecmd->supported =
        (SUPPORTED_10baseT_Half | SUPPORTED_10baseT_Full |
         SUPPORTED_100baseT_Half | SUPPORTED_100baseT_Full |
         SUPPORTED_1000baseT_Half | SUPPORTED_1000baseT_Full |
         SUPPORTED_Autoneg | SUPPORTED_TP );

    ecmd->port = PORT_TP;

    switch ( old_phy_speed[port] )
    {
        case 0:
           ecmd->speed = SPEED_10;
            break;
        case 1:
            ecmd->speed = SPEED_100;
            break;
        case 2:
            ecmd->speed = SPEED_1000;
            break;
        default:
            ecmd->speed = 0;
            break;
    }

    if ( old_phy_duplex[port] == 1 )
        ecmd->duplex = DUPLEX_FULL;
    else
        ecmd->duplex = DUPLEX_HALF;

    ecmd->autoneg = AUTONEG_ENABLE;

    return 0;
}

u32 ifxmii_ethtool_op_get_link(struct net_device *dev)
{   
    return  old_link_status[0]? 1 : 0;
}

static const struct ethtool_ops ifxmii_ethtool_ops = {
    .get_settings   = ifxmii_ethtool_op_get_settings,
    .get_link       = ifxmii_ethtool_op_get_link,
};

/**
 * Initialize switch driver.
 * This functions initializes the switch driver structures and registers the
 * Ethernet device.
 *
 * \param   dev    Device structure for Ethernet device
 * \return  0      OK
 * \return  ENOMEM No memory for structures available
 * \return  -1     Error during DMA init or Ethernet address configuration.
 * \ingroup API
 */
static int ifxmips_mii_dev_init(struct net_device *dev)
{
	int i;
	struct ifxmips_mii_priv *priv;

	ether_setup(dev);
	printk(KERN_INFO "ifxmips_mii0: %s is up\n", dev->name);

	dev->ethtool_ops = &ifxmii_ethtool_ops;
	dev->open = ifxmips_mii_open;
	dev->stop = ifxmips_mii_release;
	dev->hard_start_xmit = ifxmips_mii_tx;
	dev->get_stats = ifxmips_get_stats;
	dev->tx_timeout = ifxmips_mii_tx_timeout;
	dev->watchdog_timeo = 10 * HZ;
	memset(dev->priv, 0, sizeof(struct ifxmips_mii_priv));
	priv = dev->priv;
    priv->dma_device = dma_device_reserve("SW");
	if (!priv->dma_device) {
		BUG();
		return -ENODEV;
	}
	priv->dma_device->buffer_alloc = ifxmips_etop_dma_buffer_alloc;
	priv->dma_device->buffer_free = ifxmips_etop_dma_buffer_free;
	priv->dma_device->intr_handler = dma_intr_handler;
	priv->dma_device->num_tx_chan = 2;
	priv->dma_device->num_rx_chan = 3;

	for (i = 0; i < priv->dma_device->max_rx_chan_num; i++)
   {
		priv->dma_device->rx_chan[i]->packet_size = ETHERNET_PACKET_DMA_BUFFER_SIZE;
	}

	for (i = 0; i < priv->dma_device->max_tx_chan_num; i++)
   {
      /*priv->dma_device->tx_chan[i]->tx_weight=DEFAULT_SW_CHANNEL_WEIGHT;*/
     /* TX channel priority: 1 - 3 */
     if (i != 0)
     {
        priv->dma_device->tx_chan[i]->tx_weight=DEFAULT_SW_CHANNEL_WEIGHT;
     }
     else
     {
        priv->dma_device->tx_chan[i]->tx_weight=2;
     }
      priv->dma_device->tx_chan[i]->packet_size = ETHERNET_PACKET_DMA_BUFFER_SIZE;
   }

	dma_device_register(priv->dma_device);

	printk(KERN_INFO "ifxmips_mii0: using mac=");
	for (i = 0; i < 6; i++) {
		dev->dev_addr[i] = mac_addr[i];
		printk("%02X%c", dev->dev_addr[i], (i == 5) ? ('\n') : (':'));
	}

	return 0;
}


static void ifxmips_mii_chip_init(int mode)
{
#if 0
	ifxmips_pmu_enable(IFXMIPS_PMU_PWDCR_DMA);
	ifxmips_pmu_enable(IFXMIPS_PMU_PWDCR_PPE);

	if (mode == REV_MII_MODE)
		ifxmips_w32_mask(PPE32_MII_MASK, PPE32_MII_REVERSE, IFXMIPS_PPE32_CFG);
	else if (mode == MII_MODE)
		ifxmips_w32_mask(PPE32_MII_MASK, PPE32_MII_NORMAL, IFXMIPS_PPE32_CFG);
	ifxmips_w32(PPE32_PLEN_UNDER | PPE32_PLEN_OVER, IFXMIPS_PPE32_IG_PLEN_CTRL);
	ifxmips_w32(PPE32_CGEN, IFXMIPS_PPE32_ENET_MAC_CFG);
	wmb();
#endif
}


static int ifxmips_mii_probe(struct platform_device *dev)
{
	int result = 0;
	unsigned char *mac = (unsigned char *)dev->dev.platform_data;
	ifxmips_mii0_dev = alloc_etherdev(sizeof(struct ifxmips_mii_priv));
	ifxmips_mii0_dev->init = ifxmips_mii_dev_init;
	memcpy(mac_addr, mac, 6);
	strcpy(ifxmips_mii0_dev->name, "eth%d");
	ifxmips_mii_chip_init(REV_MII_MODE);
	result = register_netdev(ifxmips_mii0_dev);
	if (result) {
		printk(KERN_INFO "ifxmips_mii0: error %i registering device \"%s\"\n", result, ifxmips_mii0_dev->name);
		goto out;
	}
	printk(KERN_INFO "ifxmips_mii0: driver loaded!\n");

out:
	return result;
}


static int ifxmips_mii_remove(struct platform_device *dev)
{
	struct ifxmips_mii_priv *priv = (struct ifxmips_mii_priv *)ifxmips_mii0_dev->priv;

	printk(KERN_INFO "ifxmips_mii0: ifxmips_mii0 cleanup\n");

	dma_device_unregister(priv->dma_device);
	dma_device_release(priv->dma_device);
	kfree(priv->dma_device);
	kfree(ifxmips_mii0_dev->priv);
	unregister_netdev(ifxmips_mii0_dev);
	return 0;
}


static struct platform_driver ifxmips_mii_driver = {
	.probe = ifxmips_mii_probe,
	.remove = ifxmips_mii_remove,
	.driver = {
		.name = "ifxmips_mii0",
		.owner = THIS_MODULE,
	},
};


/**
 * Initialize switch driver as module.
 * This functions initializes the switch driver structures and registers the
 * Ethernet device for module usage.
 *
 * \return  0      OK
 * \return  ENODEV An error occured during initialization
 * \ingroup API
 */
int __init ifxmips_mii_init(void)
{
	int ret = platform_driver_register(&ifxmips_mii_driver);
	if (ret)
		printk(KERN_INFO "ifxmips_mii0: Error registering platfom driver!\n");
	return ret;
}


/**
 * Remove driver module.
 * This functions removes the driver and unregisters all devices.
 *
 * \ingroup API
 */
static void __exit ifxmips_mii_cleanup(void)
{
	platform_driver_unregister(&ifxmips_mii_driver);
}


module_init(ifxmips_mii_init);
module_exit(ifxmips_mii_cleanup);

MODULE_LICENSE("GPL");
MODULE_AUTHOR("IFX");
