#include <linux/module.h>
#include <linux/kernel.h>
#include <linux/types.h>
#include <linux/fcntl.h>
#include <linux/socket.h>
#include <linux/sockios.h>
#include <linux/skbuff.h>
#include <linux/in.h>
#include <linux/errno.h>
#include <linux/timer.h>
#include <linux/mm.h>
#include <linux/inet.h>
#include <linux/if_vlan.h>
#include <linux/if_pppox.h>
#include <linux/netdevice.h>
#include <linux/netfilter_ipv4.h>
#include <linux/fdtable.h>
#include <asm/system.h>
#include <asm/uaccess.h>
#include <asm/ioctls.h>
#include <net/snmp.h>
#include <net/ip.h>
#include <net/ipv6.h>
#include <net/protocol.h>
#include <net/sock.h>
#include <net/udp.h>
#include <net/udplite.h>
#include <net/icmp.h>
#include <net/route.h>
#include <net/inet_common.h>
#include <net/checksum.h>

#define GSSPLICE_DIRECT_SEND     0
#define GSSPLICE_DIRECT_RECEIVE  0

#include <linux/gssplice/gssplice.h>

#define GSSPLICE_MAX_BYTES  512

#if 0
#ifdef __KERNEL__
extern int udpredirect_memcpy_fromiovecend(unsigned char *kdata, struct iovec *iov,
                int offset, int len);
extern int udpredirect_csum_partial_copy_fromiovecend(unsigned char *kdata,
                      struct iovec *iov,
                      int offset,
                      unsigned int len, int *csump);
#endif
#endif

typedef long (*ingress_fn)(void* pSpliceCh, int nLen, void* pData);
typedef long (*egress_fn)(void* pSpliceCh, int nLen, void* pData, void *pSpliceOutCh, void *pSpliceOutArg, int nCounter);
typedef long (*socket_ingress_fn)(void* pSpliceCh, int nLen, void* pData, void *addr);

static ingress_fn gssplice_vinetic_ingress = NULL;
static egress_fn gssplice_vinetic_egress = NULL;
socket_ingress_fn gssplice_socket_ingress = NULL;

struct udpfakehdr
{
    struct udphdr uh;
    u32 saddr;
    u32 daddr;
    struct iovec *iov;
    u32 wcheck;
};

unsigned int direct_recieve_hash_missed = 0;

#if GSSPLICE_DIRECT_RECEIVE

#define MAX_DIRECT_RECEIVE          32  // need be 2^
#define DIRECT_RCV_INDEX(port)      ((port >> 2) & (MAX_DIRECT_RECEIVE - 1)) /* SXIE: application increases port number each by 4 */

/*direct recv address structure*/
struct direct_rcv_entry
{
    u32 port;
    u32 sIP;
    u32 sport;
    void* vin;
    struct direct_rcv_entry *next;
    struct direct_rcv_entry **link_back;
};

/*global direct structure*/
struct direct_rcv_entry *dr_table[MAX_DIRECT_RECEIVE];      /* Forward address table */
struct direct_rcv_entry dr_pool[MAX_DIRECT_RECEIVE];

int direct_receive_init(void)
{
    int ii;

    for (ii = 0; ii < MAX_DIRECT_RECEIVE; ii++)
    {
        dr_table[ii] = NULL;

        dr_pool[ii].port = 0;
        dr_pool[ii].sport = 0;
        dr_pool[ii].sIP = 0;
        dr_pool[ii].vin = NULL;
        dr_pool[ii].next = NULL;
        dr_pool[ii].link_back = NULL;
    }
    return 0;
}

void* dr_locate_entry( int port, int sport, int sIP )
{
    register int index = DIRECT_RCV_INDEX(port);
    struct direct_rcv_entry* pointer;

    //printk("dr locate entry: port=%d, sport=%d, sIP=%x\n", port, ntohs(sport), sIP);
    if (dr_table[index] == NULL)
    {
        return NULL;
    }
    pointer = dr_table[index];
    while (pointer != NULL) {
        if (pointer->port == port)
        {
            //printk("found entry: port=%d, sport=%d, sIP=%x vin=%p\n", port, ntohs(sport), sIP, pointer->vin);
            if (pointer->sIP == sIP && pointer->sport == sport)
                return pointer->vin;
        }
        pointer = pointer->next;
        direct_recieve_hash_missed++;
    }

    return NULL;
}

int dr_save_entry( int port, int sport, int sIP, void* vin )
{
    int ii = 0;
    register int index;
    struct direct_rcv_entry* pointer;
    struct direct_rcv_entry* pointer_rev = NULL;
    unsigned long flags;
    int ret = 0;

    local_irq_save(flags);

    index = DIRECT_RCV_INDEX(port);
    pointer = dr_table[index];

    //printk("dr save entry: port=%d, sport=%d, sIP=%x, vin=%p\n", port, ntohs(sport), sIP, vin);
    while (pointer != NULL) {
        if (pointer->port == port && pointer->sIP == sIP && pointer->sport == sport)
            break;
        pointer_rev = pointer;
        pointer = pointer->next;
    }
    if (pointer == NULL)
    {
        /* allocate entry item in dr pool */
        while (dr_pool[ii].port > 0) {
            ii ++;
            if (ii == MAX_DIRECT_RECEIVE)
            {
                /* no free entry, return -1 */
                ret = -1;
                goto out;
            }
        } 
        pointer = &dr_pool[ii];

        /* insert entry into dr table */
        pointer->next = dr_table[index];
        pointer->link_back = &dr_table[index];
        dr_table[index] = pointer;
    }

    /* update entry fields */
    pointer->port = port;
    pointer->sport = sport;
    pointer->sIP = sIP;
    pointer->vin = vin;

out:
    local_irq_restore(flags);
    return ret;

}

int dr_del_entry( void *vin )
{
    int ii = 0;
    struct direct_rcv_entry* pointer;
    struct direct_rcv_entry* pointer_rev = NULL;
    unsigned long flags;
    int ret = 0;

    local_irq_save(flags);
    /* allocate entry item in dr pool */
    while (dr_pool[ii].vin != vin) {
        ii ++;
        if (ii == MAX_DIRECT_RECEIVE)
        {
            /* no free entry, return -1 */
            ret = -1;
            goto out;
        }
    } 

    pointer = *dr_pool[ii].link_back;
    while (pointer != NULL) {
        if (pointer->vin == vin)
            break;
        pointer_rev = pointer;
        pointer = pointer->next;
    }
    if (pointer == NULL)
    {
        ret = -1;
        goto out;
    }

    /* unhook from dr table */
    if (pointer_rev == NULL)
        *dr_pool[ii].link_back = NULL;
    else
    {
        pointer_rev->next = pointer->next;
    }

    //printk("dr del entry: port=%d, sport=%d, sIP=%x, vin=%p\n", pointer->port, ntohs(pointer->sport), pointer->sIP, pointer->vin);
    pointer->port = 0;
    pointer->sport = 0;
    pointer->sIP = 0;
    pointer->vin = NULL;
    pointer->next = NULL;
  
out:
    local_irq_restore(flags);
    return ret;

}
#endif

int gssplice_directreceive(struct sk_buff *skb, unsigned int vlan_tagged, unsigned int pppoe)
{
#if GSSPLICE_DIRECT_RECEIVE
    unsigned int vlan_header_size = vlan_tagged? VLAN_HLEN : 0;
    unsigned int pppoe_header_size = pppoe? 8 : 0; //(sizeof(struct pppoe_hdr) + 2)
    struct iphdr *iph = (struct iphdr *)(skb->data + vlan_header_size + pppoe_header_size);
    struct udphdr *uh = (struct udphdr *)((char *)iph + sizeof(struct iphdr));
    int ret;
    void *vin_out = NULL;
    unsigned short port, sport;
    int sIP;
    unsigned char protocol = iph->protocol;

    /* When the interface is in promisc. mode, drop all the crap
     * that it receives, do not try to analyse it.
     */
    if (skb->pkt_type == PACKET_OTHERHOST)
        return -1;

    /* check packet type, assume driver already checked to be sure IP packet */
    if (protocol != 0x11) // not UDP
    {
        return -1;
    }
    sIP = iph->saddr;
    port = ntohs(uh->dest);
    sport = uh->source; /* sport use net order */

    /* check entry in dr table */
    vin_out = dr_locate_entry(port, sport, sIP);

    if (vin_out == NULL)
        return -1;

    /* direct splice in */
    {
        unsigned char *rtp_data = skb->data + sizeof(struct iphdr) + sizeof(struct udphdr) + vlan_header_size + pppoe_header_size;
        int bytes_available = skb->len - sizeof(struct udphdr) - sizeof(struct iphdr) - vlan_header_size - pppoe_header_size;
        int bytes_udp = ntohs(uh->len) - sizeof(struct udphdr);

        if (bytes_available > bytes_udp)    /* remove ethernet padding bytes if there are */
            bytes_available = bytes_udp;

        /* write the rtp packet to vinetic */
        if (gssplice_vinetic_ingress)
            ret = gssplice_vinetic_ingress(vin_out, bytes_available, rtp_data);
        else
            ret = -1;
    }
    if (ret == 0) /* splice success */
        dev_kfree_skb_any(skb);

    return ret;
#else
    return -1;
#endif
}

int gssplice_directreceive_disable( void *vin)
{
#if GSSPLICE_DIRECT_RECEIVE
    dr_del_entry(vin);
#endif
    return 0;
}

/*
 * Lightweight file lookup - no refcnt increment if fd table isn't shared. 
 * You can use this only if it is guranteed that the current task already 
 * holds a refcnt to that file. That check has to be done at fget() only
 * and a flag is returned to be passed to the corresponding fput_light().
 * There must not be a cloning between an fget_light/fput_light pair.
 */
struct file *gssplice_fget_light(unsigned int fd)
{
    struct file *file = NULL;
    struct files_struct *files = current->files;

    //printk("fd=%d, file*=%x\n", fd, files->fd[fd]);
    //if (likely((atomic_read(&files->count) == 1))) 
    {
        file = fcheck_files(files, fd);
	    //if (fd < files->max_fds)
		//    file = files->fd[fd];
    } 
    /*
    else 
    {
        file = NULL;
    }
    */
    if (file == NULL)
    {
        printk("gssplice: map fd to file failed, fd=%d\n", fd);
    }

    return file;
}

static struct socket *sock_from_fd(int sock_fd)
{
    struct file *file;

    if ((file = gssplice_fget_light(sock_fd)) == NULL)
        return NULL;

    return file->private_data;  /* set in sock_map_fd */
}

static void *tapi_from_fd(int vin_fd)
{
    struct file *file;

    if ((file = gssplice_fget_light(vin_fd)) == NULL)
        return NULL;

    //printk("tapi from fd %d : %x\n", vin_fd, file->private_data);
    return file->private_data;  /* set in tapi_open */
}

#if 0
static int gssplice_update_ipudp_header(struct sk_buff *skb)
{
    struct udphdr *uh;
    struct iphdr *iph;
    struct rtable *rt = skb_rtable(skb);

    if ( !skb )
    {
        return -1;
    }

    /*Bugs fix: change the source IP and port which app user like*/
    iph = ip_hdr(skb);
    ip_select_ident(iph, &rt->u.dst, NULL);

    uh = udp_hdr(skb);
    uh->check = 0;

    /* computing udp checksum */
    skb->ip_summed = CHECKSUM_UNNECESSARY; //CHECKSUM_HW;
    skb->csum = offsetof(struct udphdr, check);
    uh->check = ~csum_tcpudp_magic(iph->saddr, iph->daddr, ntohs(uh->len), IPPROTO_UDP, 0);

    if (uh->check == 0)
    {
        //ASSERT("err udp checkum!\n");
        uh->check = -1;
    }

    /* 
     * Computing ip checkum
     */
    ip_send_check(iph);

    /* 
     * TODO: process ip option 
     * ip_options_build(skb, opt, daddr, rt, 0);
     */

    return 0;
}
#endif

#if GSSPLICE_DIRECT_SEND
int gssplice_udp_directsend(struct sock *sk, void *pData, unsigned int len, struct sockaddr_in * usin)
{
    struct sk_buff* skb = sk->splice_saved_skb;
    struct iphdr *iph = ip_hdr(skb);
    struct udphdr *uh = (struct udphdr *)((char *)iph + sizeof(struct iphdr));
    struct ethhdr *eh = (struct ethhdr *)skb->data;
    struct vlan_ethhdr *veh = (struct vlan_ethhdr *)skb->data;
    struct pppoe_hdr *ph;
    signed char *data;
    u32 wcheck = 0;
    unsigned int packet_len;
    unsigned int update_pppoe_hdr = 0;

    if (skb_shared(skb))
    {
        /* skb not sent out yet */
        /*
         * skb_copy seems cause issue based on autodial test result, ie. when network 
         * is too busy to sending out packets, too many skb will be allocated here, 
         * results in out of system memory.
         */
#if 0   
        printk("dup skb for direct send, sk=%p, ts=%ld\n", sk, jiffies*1000/HZ);
        skb = skb_copy(skb, GFP_ATOMIC);
#else
        /*
         * SXIE: if skb not sent out, then the network is too busy, fall back to normal send is not helpful but
         * waste more CPU resource. So just return len to silently drop it.
         * return -1; 
         */
        return len;   
#endif
        
    }
    else
    {
        /* increase skb reference count */
        skb = skb_get(skb);
    }

    if (unlikely(skb_is_nonlinear(skb)))
    {
        skb_linearize(skb);
        printk("linearize saved skb\n");
        //re-pointer iph and uh
        iph = ip_hdr(skb);
        uh = (struct udphdr *)((char *)iph + sizeof(struct iphdr));

        eh = (struct ethhdr *)skb->data;
        veh = (struct vlan_ethhdr *)skb->data;
    }

    /* check rtp size */
    packet_len = ntohs(uh->len) - sizeof(struct udphdr);
    if (unlikely(packet_len > skb->len) || unlikely(iph->protocol != 0x11)) // not UDP
    {
        /* Will this happen? */
        printk("saved skb corrupted, skb pkt len=%d, nh=%p, data=%p\n", packet_len, iph, skb->data);
        dev_kfree_skb_any(skb);
        skb = sk->splice_saved_skb;
        sk->splice_saved_skb = NULL;
        dev_kfree_skb_any(skb);
        /* return -2 for falling back to non splice send */
        return -2;
    }

    if (unlikely(len != packet_len))
    {
        /*
         * This used to be something like free the skb and return -2 to
         * tell vinetic driver to fall back to normal send.
         * Now we have better handling on this packet buffer size adaption.
         */
        if ((eh->h_proto == __constant_htons(ETH_P_PPP_SES)) ||
            (eh->h_proto == __constant_htons(ETH_P_8021Q) && veh->h_vlan_encapsulated_proto == __constant_htons(ETH_P_PPP_SES)))
        {
            update_pppoe_hdr = 1;
        }

        if (len < packet_len)
        {
            /* trim skb data size */
            printk("trim skb to %d from %d\n", skb->len - packet_len + len, skb->len);
            skb_trim(skb, skb->len - packet_len + len);
        }
        else
        {
            /* expand skb data buffer if needed */
            if(skb_tailroom(skb) < (len - packet_len))
            {
                skb_pad(skb, len - packet_len);
            }

            printk("inc skb by %d\n", len - packet_len);
            /* increase skb data size */
            skb_put(skb, len - packet_len);
        }
        ip_hdr(skb)->tot_len = htons(ntohs(ip_hdr(skb)->tot_len) + len - packet_len);
        //re-pointer iph and uh
        iph = ip_hdr(skb);
        uh = (struct udphdr *)((char *)iph + sizeof(struct iphdr));
        //need recalc the ip header checksum
        iph->check = 0; 
        iph->check = ip_fast_csum((unsigned char *)iph, iph->ihl);

	    if (update_pppoe_hdr == 1)
        {
            /* update pppoe payload length */
            eh = (struct ethhdr *)skb->data;
            veh = (struct vlan_ethhdr *)skb->data;

            if (eh->h_proto == __constant_htons(ETH_P_PPP_SES))
            {
                ph = (struct pppoe_hdr *)((char *)eh + sizeof(struct ethhdr));
                ph->length = htons(ntohs(ph->length) - packet_len + len);
            }
            else if (eh->h_proto == __constant_htons(ETH_P_8021Q) && veh->h_vlan_encapsulated_proto == __constant_htons(ETH_P_PPP_SES))
            {
                ph = (struct pppoe_hdr *)((char *)veh + sizeof(struct vlan_ethhdr));
                ph->length = htons(ntohs(ph->length) - packet_len + len);
            }
            else
            {
                /* critical error if this is hit */
                printk("Error! update pppoe header failed\n");
            }
        }
    }

    /* copy data into skb */
    data = (char *)uh + sizeof(struct udphdr);
    memcpy(data, pData, len);
    
    /* re-compute the udp checksum */
    uh->check = 0;
    uh->len = htons(len + sizeof(struct udphdr));
    wcheck = csum_partial((char *)data, len, wcheck);
    wcheck = csum_partial((char *)uh, sizeof(struct udphdr), wcheck);
    uh->check = csum_tcpudp_magic(iph->saddr, iph->daddr, ntohs(uh->len), IPPROTO_UDP, wcheck);
    if (uh->check == 0)
        uh->check = -1;
    /* call dev driver to send packet */
    skb->dev->netdev_ops->ndo_start_xmit(skb, skb->dev);

    return len;
}
#endif

#if 0
int gssplice_udp_send_iov(struct sock *sk, struct iovec *iov, int len, struct sockaddr_in * usin)
{
    struct inet_sock *inet = inet_sk(sk);
    struct udp_sock *up = udp_sk(sk);
    int ulen = len;
    struct ipcm_cookie ipc;
    struct rtable *rt = NULL;
    int free = 0;
    int connected = 0;
    __be32 daddr, faddr, saddr;
    __be16 dport;
    u8  tos;
    int err, is_udplite = IS_UDPLITE(sk);
    int (*getfrag)(void *, char *, int, int, int, struct sk_buff *);

    if (len > 0xFFFF)
        return -EMSGSIZE;

    ipc.opt = NULL;
    ipc.shtx.flags = 0;

    if (up->pending) {
        /*
         * There are pending frames.
         * The socket lock must be held while it's corked.
         */
        lock_sock(sk);
        if (likely(up->pending)) {
            if (unlikely(up->pending != AF_INET)) {
                release_sock(sk);
                return -EINVAL;
            }
            goto do_append_data;
        }
        release_sock(sk);
    }
    ulen += sizeof(struct udphdr);

    /*
     *  Get and verify the address.
     */
    if (usin) {
        if (usin->sin_family != AF_INET) {
            if (usin->sin_family != AF_UNSPEC)
                return -EAFNOSUPPORT;
        }

        daddr = usin->sin_addr.s_addr;
        dport = usin->sin_port;
        if (dport == 0)
            return -EINVAL;
    } else {
        if (sk->sk_state != TCP_ESTABLISHED)
            return -EDESTADDRREQ;
        daddr = inet->daddr;
        dport = inet->dport;
        /* Open fast path for connected socket.
           Route will not be used, if at least one option is set.
         */
        connected = 1;
    }
    ipc.addr = inet->saddr;

    ipc.oif = sk->sk_bound_dev_if;
    err = sock_tx_timestamp(NULL, sk, &ipc.shtx);
    if (err)
        return err;

    if (!ipc.opt)
        ipc.opt = inet->opt;

    saddr = ipc.addr;
    ipc.addr = faddr = daddr;

    if (ipc.opt && ipc.opt->srr) {
        if (!daddr)
            return -EINVAL;
        faddr = ipc.opt->faddr;
        connected = 0;
    }
    tos = RT_TOS(inet->tos);
    if (sock_flag(sk, SOCK_LOCALROUTE) ||
        (ipc.opt && ipc.opt->is_strictroute)) {
        tos |= RTO_ONLINK;
        connected = 0;
    }

    if (ipv4_is_multicast(daddr)) {
        if (!ipc.oif)
            ipc.oif = inet->mc_index;
        if (!saddr)
            saddr = inet->mc_addr;
        connected = 0;
    }

    if (connected)
        rt = (struct rtable *)sk_dst_check(sk, 0);

    if (rt == NULL) {
        struct flowi fl = { .oif = ipc.oif,
                    .mark = sk->sk_mark,
                    .nl_u = { .ip4_u =
                          { .daddr = faddr,
                        .saddr = saddr,
                        .tos = tos } },
                    .proto = sk->sk_protocol,
                    .flags = inet_sk_flowi_flags(sk),
                    .uli_u = { .ports =
                           { .sport = inet->sport,
                         .dport = dport } } };
        struct net *net = sock_net(sk);

        security_sk_classify_flow(sk, &fl);
        err = ip_route_output_flow(net, &rt, &fl, sk, 1);
        if (err) {
            if (err == -ENETUNREACH)
                IP_INC_STATS_BH(net, IPSTATS_MIB_OUTNOROUTES);
            goto out;
        }

        err = -EACCES;
        if ((rt->rt_flags & RTCF_BROADCAST) &&
            !sock_flag(sk, SOCK_BROADCAST))
            goto out;
        if (connected)
            sk_dst_set(sk, dst_clone(&rt->u.dst));
    }

//back_from_confirm:
    saddr = rt->rt_src;
    if (!ipc.addr)
        daddr = ipc.addr = rt->rt_dst;

    lock_sock(sk);
    if (unlikely(up->pending)) {
        /* The socket is already corked while preparing it. */
        /* ... which is an evident application bug. --ANK */
        release_sock(sk);

        LIMIT_NETDEBUG(KERN_DEBUG "udp cork app bug 2\n");
        err = -EINVAL;
        goto out;
    }
    /*
     *  Now cork the socket to pend data.
     */
    inet->cork.fl.fl4_dst = daddr;
    inet->cork.fl.fl_ip_dport = dport;
    inet->cork.fl.fl4_src = saddr;
    inet->cork.fl.fl_ip_sport = inet->sport;
    up->pending = AF_INET;

do_append_data:
    up->len += ulen;
    getfrag  =  is_udplite ?  udplite_getfrag : ip_generic_getfrag;

    {
        unsigned int saved_allocation = 0;

        if (in_interrupt())
        {
            saved_allocation = sk->sk_allocation;
            sk->sk_allocation = GFP_ATOMIC;
        }
            
        sk->splice_record_skb = 1;  //ask ip_build_xmit to save the skb

        err = ip_append_data(sk, getfrag, iov, ulen,
            sizeof(struct udphdr), &ipc, &rt, 0);
        if (err)
            udp_flush_pending_frames(sk);
        /*
        else if (!corkreq)
            err = udp_push_pending_frames(sk);
            */
        else if (unlikely(skb_queue_empty(&sk->sk_write_queue)))
            up->pending = 0;
        release_sock(sk);
        if (in_interrupt())
            sk->sk_allocation = saved_allocation;
    }

out:
    ip_rt_put(rt);
    if (free)
        kfree(ipc.opt);
    if (!err)
        return len;
    /*
     * ENOBUFS = no kernel mem, SOCK_NOSPACE = no sndbuf space.  Reporting
     * ENOBUFS might not be good (it's not tunable per se), but otherwise
     * we don't have a good statistic (IpOutDiscards but it can be too many
     * things).  We could add another new stat but at least for now that
     * seems like overkill.
     */
    if (err == -ENOBUFS || test_bit(SOCK_NOSPACE, &sk->sk_socket->flags)) {
        UDP_INC_STATS_USER(sock_net(sk),
                UDP_MIB_SNDBUFERRORS, is_udplite);
    }
    return err;
}

long gssplice_socket_SpliceIn(struct socket* sock_out, int nLen, void* pData, struct sockaddr_in* usin)
{
    struct iovec gssplice_iovec;
    struct sock *sk;
    long ret = 0;

    if (sock_out == NULL || sock_out->file == NULL || sock_out->sk == NULL || sock_out->file->private_data != sock_out) 
    {
        return -10;
    }

    sk = sock_out->sk;

#if GSSPLICE_DIRECT_SEND
    if (sk->splice_saved_skb)  //direct send is possible
    {
        ret = gssplice_udp_directsend(sk, pData, (unsigned int)nLen, usin);
        if (ret < 0)
            printk("direct send failed\n");
    }
#endif
    //SXIE: we have skb free issue in IRQ context sending packets out without directsend
    if (ret <= 0 && !in_interrupt())   // == 0, no direct send, < 0, direct send failed
    {
        gssplice_iovec.iov_base = pData;
        gssplice_iovec.iov_len = nLen;

        //printk("socket splice in: socket=%p, sk=%p,  usin.daddr=%x len=%d\n", sock_out, sk, usin->sin_addr.s_addr, nLen);

        //printk("directsend failed, fall back\n");
        ret = gssplice_udp_send_iov(sk, &gssplice_iovec, nLen, usin);
        if (ret < 0)
        {
            printk("ds fallback still failed, ret=%ld\n", ret);
        }
    }

    return ret;
}

long do_gssplice_v2v(void* vin_in, void* vin_out, int splice_mode)
{
    unsigned char gssplice_buffer[GSSPLICE_MAX_BYTES];
    int bytes_available = GSSPLICE_MAX_BYTES;
    long ret = 0;
    void *splice_out_ch = NULL;
    int splice_counter = 0;

    switch (splice_mode)
    {
        case GSMODECONTINOUS:
            splice_out_ch = vin_out;
            splice_counter = -1;
            break;
        case GSMODEAUTO:
            splice_out_ch = vin_out;
            splice_counter = GSMOD_AUTO_COUNTER;
            break;
        case GSMODEOFF:
        case GSMODESINGLE:
        default:
            splice_out_ch = NULL;
            splice_counter = 0;
            break;
    };

    //printk("v2v\n");
    if (gssplice_vinetic_egress)
        ret = gssplice_vinetic_egress(vin_in, bytes_available, gssplice_buffer, splice_out_ch, NULL, splice_counter);

    if (ret <= 0)
    {
        /* no data available */
        goto out;
    }
    else
        bytes_available = (int) ret;

    if (gssplice_vinetic_ingress)
        ret = gssplice_vinetic_ingress(vin_out, bytes_available, gssplice_buffer);
    else
        ret = -1;
out:
    return ret;
}

long do_gssplice_v2s(void* vin_in, struct socket* sock_out, struct sockaddr_in * usin, int splice_mode)
{
    int bytes_available = 0;
    unsigned char gssplice_buffer[GSSPLICE_MAX_BYTES];
    long ret = 0;
    void *splice_out_ch = NULL;
    int splice_counter = 0;

    switch (splice_mode)
    {
        case GSMODECONTINOUS:
            splice_out_ch = sock_out;
            splice_counter = -1;
            break;
        case GSMODEAUTO:
            splice_out_ch = sock_out;
            splice_counter = GSMOD_AUTO_COUNTER;
            break;
        case GSMODEOFF:
        case GSMODESINGLE:
        default:
            splice_out_ch = NULL;
            splice_counter = 0;
            break;
    };

    /* set input as buffer size */
    bytes_available = GSSPLICE_MAX_BYTES; 

    //printk("v2s %p, %d\n", splice_out_ch, jiffies*1000/HZ);
    //printk("v2s, vin=%p, splice out ch=%p\n", vin_in, splice_out_ch);
    //printk("v2s-%s, lcl port=%d, rmt port=%d\n",  splice_mode == GSMODEOFF? "off": "on", sock_out->sk->num, ntohs(usin->sin_port));
    if (gssplice_vinetic_egress)
        ret = gssplice_vinetic_egress(vin_in, bytes_available, gssplice_buffer, splice_out_ch, usin, splice_counter);
    else
    {
        /* egress not hooked, no data available */
        printk("egress not hooked\n");
        goto out;
    }
    if (ret < 0)
        printk("egress failed\n");

    /* send iov thru out socket */
    if (ret > 0 && splice_mode != GSMODEOFF )
    {
        ret = gssplice_socket_SpliceIn(sock_out, ret, gssplice_buffer, usin);
    }
    if (ret < 0)
        printk("splicein failed\n");

#if GSSPLICE_DIRECT_SEND
    if (splice_out_ch == NULL)
    {
        struct sk_buff *skb = sock_out->sk->splice_saved_skb;
        if (skb)
        {
            sock_out->sk->splice_saved_skb = NULL;
            dev_kfree_skb_any(skb);
        }
#if GSSPLICE_DIRECT_RECEIVE
        /* cancel direct receive */
        if (sock_out->sk->splice_out_ch != NULL)
        {
            gssplice_directreceive_disable(sock_out->sk->splice_out_ch);
            sock_out->sk->splice_out_ch = NULL;
        }
#endif
    }
#endif

    return ret;

out:
    return 0;
}

long gssplice_skb_write_vin(struct sk_buff *skb, void* vin_out)
{
    int bytes_available = 0;
    int err;
    unsigned char gssplice_buffer[GSSPLICE_MAX_BYTES];
    struct iovec gssplice_iovec = { gssplice_buffer, GSSPLICE_MAX_BYTES };
    long ret = 0;

    if (!skb)
    {
        /* no data available */
        goto out;
    }
    bytes_available = skb->len - sizeof(struct udphdr);

    if (unlikely(bytes_available > GSSPLICE_MAX_BYTES))
    {
        /* skip splice if size too large */
        ret = -1;
        goto out;
    }
    if (unlikely(skb_is_nonlinear(skb)))
    {
        printk("skb to splice in is non linear\n");
        /* skip checksum? */
        err = skb_copy_datagram_iovec(skb, sizeof(struct udphdr), &gssplice_iovec, bytes_available);

        if (err)
        {
            ret = -1;
            goto out;
        }

        /* write the rtp packet to vinetic */
        if (gssplice_vinetic_ingress)
            ret = gssplice_vinetic_ingress(vin_out, bytes_available, gssplice_buffer);
        else
            ret = -1;
    }
    else
    {
        unsigned char *rtp_data = skb->data + sizeof(struct udphdr);
        /* write the rtp packet to vinetic */
        if (gssplice_vinetic_ingress)
            ret = gssplice_vinetic_ingress(vin_out, bytes_available, rtp_data);
        else
            ret = -1;
    }

out:
    return ret;
}

long gssplice_internal_s2v(struct sock* sk_in, void* vin_out)
{
    struct sk_buff *skb;//, *skb_real;
    int err;
    long ret = 0;

    /* call udp_recvmsg to read out packet */
    skb = skb_recv_datagram(sk_in, 0, 1, &err);
    if (!skb)
    {
        /* no data available */
        goto out;
    }
    ret = gssplice_skb_write_vin(skb, vin_out);
#if GSSPLICE_DIRECT_RECEIVE
    if (ret == 0) //success
    {
        dr_save_entry(ntohs(udp_hdr(skb)->dest), udp_hdr(skb)->source, ip_hdr(skb)->saddr, vin_out);
    }
#endif

    if (skb)
        skb_free_datagram(sk_in, skb);
out:
    return ret;

}

long do_gssplice_s2v(struct socket* sock_in, void* vin_out, int splice_mode)
{
    struct sk_buff *skb; //, *skb_real;
    int bytes_available = 0;
    int err;
    unsigned char gssplice_buffer[GSSPLICE_MAX_BYTES];
    struct iovec gssplice_iovec = { gssplice_buffer, GSSPLICE_MAX_BYTES };
    long ret = 0;
    void *splice_out_ch = NULL;
    int splice_counter = 0;

    switch (splice_mode)
    {
        case GSMODECONTINOUS:
            splice_out_ch = vin_out;
            splice_counter = -1;
            break;
        case GSMODEAUTO:
            splice_out_ch = vin_out;
            splice_counter = GSMOD_AUTO_COUNTER;
            break;
        case GSMODEOFF:
        case GSMODESINGLE:
        default:
            splice_out_ch = NULL;
            splice_counter = 0;
#if GSSPLICE_DIRECT_RECEIVE
            dr_del_entry(vin_out);
#endif
            break;
    };

    sock_in->sk->splice_out_ch = splice_out_ch;
    sock_in->sk->splice_out_arg = NULL;
    sock_in->sk->splice_counter = splice_counter;

    //printk("s2v %p, %d\n", splice_out_ch, jiffies*1000/HZ);
    //printk("s2v-%s, local port=%d\n", splice_mode == GSMODEOFF? "off": "on", sock_in->sk->num);
    /* call udp_recvmsg to read out packet */
    skb = skb_recv_datagram(sock_in->sk, 0, 1, &err);

    if (skb) 
    {
        /* data packet available */
        bytes_available = skb->len - sizeof(struct udphdr);

        if (unlikely(bytes_available > GSSPLICE_MAX_BYTES))
        {
            /* skip splice if size too large */
            ret = -1;
            goto out_free;
        }
        /* skip checksum? */
        err = skb_copy_datagram_iovec(skb, sizeof(struct udphdr), &gssplice_iovec, bytes_available);

        if (err)
        {
            ret = -1;
            goto out_free;
        }


        /* write the rtp packet to vinetic */
        if (gssplice_vinetic_ingress)
            ret = gssplice_vinetic_ingress(vin_out, bytes_available, gssplice_buffer);
        else
            ret = -1;

#if GSSPLICE_DIRECT_RECEIVE
        if (ret == 0) //success
        {
            dr_save_entry(ntohs(udp_hdr(skb)->dest), udp_hdr(skb)->source, ip_hdr(skb)->saddr, vin_out);
        }
#endif

        skb_free_datagram(sock_in->sk, skb);
        // SXIE: loop in case we have queued a lot of packets already
        while ((skb = skb_recv_datagram(sock_in->sk, 0, 1, &err)) != NULL)
            skb_free_datagram(sock_in->sk, skb);
    } 

out_free:

    if (skb)
        skb_free_datagram(sock_in->sk, skb);

    return ret;
}

long gssplice_internal_s2s(struct sock *sk_in, struct sock *sk_out, struct sockaddr_in* usin)
{
    struct inet_sock *inet = inet_sk(sk_out);
    struct ip_options *opt = inet->opt;
    struct rtable   *rt;
    struct udphdr   *uh;
    struct iphdr    *iph;
    struct sk_buff  *skb; //, *skb_real;
    int bytes_available = 0;
    long ret = 0;
    int err;

    /* call udp_recvmsg to read out packet */
    skb = skb_recv_datagram(sk_in, 0, 1, &err);
    if (!skb)
    {
        /* no data available */
        goto out;
    }
    bytes_available = skb->len - sizeof(struct udphdr);

    if (unlikely(bytes_available > GSSPLICE_MAX_BYTES))
    {
        /* skip splice if size too large */
        ret = -1;
        goto out_free;
    }

    /* relay skb to the out socket */
    iph = ip_hdr(skb);
    uh = udp_hdr(skb);

    uh->source  = inet->sport;
    uh->dest    = usin->sin_port;

    /* Make sure we can route this packet. */
    rt = (struct rtable *)__sk_dst_check(sk_out, 0);
    if (rt == NULL) {
        __be32 daddr;

        /* Use correct destination address if we have options. */
        daddr = inet->daddr;
        if(opt && opt->srr)
            daddr = opt->faddr;

        {
            struct flowi fl = { .oif = sk_out->sk_bound_dev_if,
                        .mark = sk_out->sk_mark,
                        .nl_u = { .ip4_u =
                              { .daddr = daddr,
                            .saddr = inet->saddr,
                            .tos = RT_CONN_FLAGS(sk_out) } },
                        .proto = sk_out->sk_protocol,
                        .flags = inet_sk_flowi_flags(sk_out),
                        .uli_u = { .ports =
                               { .sport = inet->sport,
                             .dport = inet->dport } } };

            /* If this fails, retransmit mechanism of transport layer will
             * keep trying until route appears or the connection times
             * itself out.
             */
            security_sk_classify_flow(sk_out, &fl);
            if (ip_route_output_flow(sock_net(sk_out), &rt, &fl, sk_out, 0))
                goto no_route;
        }
        sk_setup_caps(sk_out, &rt->u.dst);
    }
    skb_dst_set(skb, dst_clone(&rt->u.dst));

    if (opt && opt->is_strictroute && rt->rt_dst != rt->rt_gateway)
        goto no_route;

    gssplice_update_ipudp_header(skb);

    *((__be16 *)iph) = htons((4 << 12) | (5 << 8) | (inet->tos & 0xff));
    if (ip_dont_fragment(sk_out, &rt->u.dst))
        iph->frag_off = htons(IP_DF);
    else
        iph->frag_off = 0;
    //iph->ttl      = ip_select_ttl(inet, &rt->u.dst);
    iph->protocol = sk_out->sk_protocol;
    iph->saddr    = rt->rt_src;
    iph->daddr    = rt->rt_dst;
    /* Transport layer set skb->h.foo itself. */

    if (opt && opt->optlen) {
        iph->ihl += opt->optlen >> 2;
        ip_options_build(skb, opt, inet->daddr, rt, 0);
    }

    ip_select_ident_more(iph, &rt->u.dst, sk_out,
                 (skb_shinfo(skb)->gso_segs ?: 1) - 1);

    skb->priority = sk_out->sk_priority;
    skb->mark = sk_out->sk_mark;

    /* Send it out. */
    ip_local_out(skb);

    goto out;

    /* failed */
no_route:
    IP_INC_STATS(sock_net(sk_out), IPSTATS_MIB_OUTNOROUTES);
out_free:
    if (skb)
        skb_free_datagram(sk_in, skb);
out:
    return ret;
}

long do_gssplice_s2s(struct socket* sock_in, struct socket* sock_out, struct sockaddr_in * usin, int splice_mode)
{
    struct sock *sk = sock_out->sk;
    void *splice_out_ch = NULL;
    int splice_counter = 0;

    switch (splice_mode)
    {
        case GSMODECONTINOUS:
            splice_out_ch = sock_out;
            splice_counter = -1;
            break;
        case GSMODEAUTO:
            splice_out_ch = sock_out;
            splice_counter = GSMOD_AUTO_COUNTER;
            break;
        case GSMODEOFF:
        case GSMODESINGLE:
        default:
            splice_out_ch = NULL;
            splice_counter = 0;
            break;
    };

    sock_in->sk->splice_out_ch = splice_out_ch;
    //sock_in->sk->splice_out_arg = usin;
    sock_in->sk->splice_counter = splice_counter;

    return gssplice_internal_s2s(sock_in->sk, sk, usin);
}
#endif

#define MAX_SOCK_ADDR	128
asmlinkage long sys_gssplice(int fd_in, int fd_out, unsigned int flags, struct sockaddr *addr)
{
    int splice_dir = flags & GSSPLICDIR_BITS;
    //int splice_mode = flags & GSSPLICEMODE_BITS;
    //struct sockaddr_in *usin = NULL;
    int error;
    struct socket *sock_in = NULL/*, *sock_out = NULL*/;
    void *vin_in = NULL/*, *vin_out = NULL*/;

    //printk("gssplice system call entered, 0x%x,0x%x,0x%x,0x%p\n", fd_in, fd_out, flags, addr);

    /* check fd_in type */
    error = -EBADF;
    if (splice_dir == GSVINETICVINETIC || splice_dir == GSVINETICSOCKET)
    {
        /* fd_in is vinetic device */
        if ((vin_in = tapi_from_fd(fd_in)) == NULL)
        {
            printk("fail to get tapi fd\n");
            goto err_exit;
        }
    }
    else
    {
        /* fd_in is socket fd */
        if ((sock_in = sock_from_fd(fd_in)) == NULL)
        {
            goto err_exit;
        }
    }

#if 0
    /* check fd_out type */
    if (splice_dir == GSVINETICVINETIC || splice_dir == GSSOCKETVINETIC)
    {
        /* fd_out is vinetic device */
        if ((vin_out = tapi_from_fd(fd_out)) == NULL)
        {
            printk("fail to get tapi fd\n");
            goto err_exit;
        }
    }
    else
    {
        /* fd_out is socket fd */
        if ((sock_out = sock_from_fd(fd_out)) == NULL)
        {
            printk("fail to get sock fd\n");
            goto err_exit;
        }
#if 0
        usin = (struct sockaddr_in *)sock_out->sk->splice_in_addr;

        if(addr)
        {
            error = move_addr_to_kernel(addr, sizeof(struct sockaddr_in), (struct sockaddr *)usin);
            if (error < 0)
            {
                printk("fail to copy sockaddr\n");
                goto err_exit;
            }
        }
        else if (splice_mode != GSMODEOFF)
        {
            printk("splice to socket with NULL sockaddr\n");
            goto err_exit;
        }
#endif
    }
#endif

#if 0
    switch (splice_dir)
    {
        case GSVINETICVINETIC:
            error = (int)do_gssplice_v2v(vin_in, vin_out, splice_mode);
            break;
        case GSVINETICSOCKET:
            //printk("sock_out, fd=%d\n", fd_out);
            error = (int)do_gssplice_v2s(vin_in, sock_out, usin, splice_mode);
            if (error < 0)
                printk("v2s fail, error=%d\n", error);
            break;
        case GSSOCKETVINETIC:
            //printk("sock_in, fd=%d\n", fd_in);
            error = (int)do_gssplice_s2v(sock_in, vin_out, splice_mode);
            break;
        case GSSOCKETSOCKET:
            /* socket relay */
            error = (int)do_gssplice_s2s(sock_in, sock_out, usin, splice_mode);
        default:
            error = -1;
    }
#else
    error = 0;
#endif

err_exit:
    if (error < 0)
        printk("gssplice fail\n");
    return error;
}

int gssplice_register_hooks(void *fn_in, void *fn_out)
{
    printk("gssplice_register_hooks, vinetic in=0x%p, out=0x%p\n", fn_in, fn_out);
#if GSSPLICE_DIRECT_RECEIVE
    direct_receive_init();
#endif

    gssplice_vinetic_ingress = (ingress_fn)fn_in;
    gssplice_vinetic_egress = (egress_fn)fn_out;
    //gssplice_socket_ingress = (socket_ingress_fn)gssplice_socket_SpliceIn;
    return 0;
}

EXPORT_SYMBOL(gssplice_register_hooks);
EXPORT_SYMBOL(gssplice_socket_ingress);

