/******************************************************************************

                               Copyright (c) 2009
                            Lantiq Deutschland GmbH
                     Am Campeon 3; 81726 Munich, Germany

  THE DELIVERY OF THIS SOFTWARE AS WELL AS THE HEREBY GRANTED NON-EXCLUSIVE,
  WORLDWIDE LICENSE TO USE, COPY, MODIFY, DISTRIBUTE AND SUBLICENSE THIS
  SOFTWARE IS FREE OF CHARGE.

  THE LICENSED SOFTWARE IS PROVIDED "AS IS" AND INFINEON EXPRESSLY DISCLAIMS
  ALL REPRESENTATIONS AND WARRANTIES, WHETHER EXPRESS OR IMPLIED, INCLUDING
  WITHOUT LIMITATION, WARRANTIES OR REPRESENTATIONS OF WORKMANSHIP,
  MERCHANTABILITY, FITNESS FOR A PARTICULAR PURPOSE, DURABILITY, THAT THE
  OPERATING OF THE LICENSED SOFTWARE WILL BE ERROR FREE OR FREE OF ANY THIRD
  PARTY CLAIMS, INCLUDING WITHOUT LIMITATION CLAIMS OF THIRD PARTY INTELLECTUAL
  PROPERTY INFRINGEMENT.

  EXCEPT FOR ANY LIABILITY DUE TO WILFUL ACTS OR GROSS NEGLIGENCE AND EXCEPT
  FOR ANY PERSONAL INJURY INFINEON SHALL IN NO EVENT BE LIABLE FOR ANY CLAIM
  OR DAMAGES OF ANY KIND, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE,
  ARISING FROM, OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER
  DEALINGS IN THE SOFTWARE.

*******************************************************************************/
#ifndef _LTQ_SVIP_NAT_PT_IO_H_
#define _LTQ_SVIP_NAT_PT_IO_H_

#include <linux/autoconf.h>

#ifdef CONFIG_LTQ_SVIP_NAT_PT
#include <asm/ioctl.h>
#include <linux/ifx_types.h>
#ifndef __KERNEL__
#include <sys/socket.h>
#else
#include <linux/socket.h>
#endif

/* Define this to enable socket TOS setting for the target destination */
#define SUPPORT_SVIP_NAT_PT_DSCP

#define SVIP_NAT_PT_DEVICE_NAME           "svip_nat_pt"
#define PATH_SVIP_NAT_PT_DEVICE_NAME      "/dev/"SVIP_NAT_PT_DEVICE_NAME

#define MAJOR_NUM_SVIP_NAT_PT 10
#define MINOR_NUM_SVIP_NAT_PT 121
#define SVIP_PROBE_SOURCE_PORT 65534

#define SVIP_UDP_FROM         (CONFIG_LTQ_SVIP_NAT_UDP_PORT_BASE)

/** @defgroup SVIP_NAT_PT_API SVIP NAT Protocol Translation ioctl interface.
   Interface to add an IP protocol transformation rule into the SVIP NAT table.
   Transformation is performed on outgoing IPv4 traffic to IPv6 and
   incoming IPv6 to IPv4 traffic.
	The interface can be used for IPv4 to IPv4 NAT translation, as well.
  @{  */

/** Used to add a new rule to the SVIP NAT PT table. If a rule already
    exists for the target UDP port, that rule shall be overwritten.

   \param SVIP_NAT_PT_IO_Rule_t* The parameter points to a \ref SVIP_NAT_PT_IO_Rule_t structure.
*/
#define FIO_SVIP_NAT_PT_RULE_ADD    _IOW(MAJOR_NUM_SVIP_NAT_PT, 1, SVIP_NAT_PT_IO_Rule_t)

/** Used to remove a SVIP NAT_PT rule from the SVIP NAT PT table. No check is
    performed whether the rule already exists or not. The remove operation is
    performed as long as the target UDP port is within the defined port range.

   \param SVIP_NAT_PT_IO_Rule_t* The parameter points to a \ref SVIP_NAT_PT_IO_Rule_t structure.
*/
#define FIO_SVIP_NAT_PT_RULE_REMOVE _IOW(MAJOR_NUM_SVIP_NAT_PT, 2, SVIP_NAT_PT_IO_Rule_t)

/** Used to list all rules in the SVIP Custom NAT table.

   \param <none>
*/
#define FIO_SVIP_NAT_PT_RULE_LIST   _IO(MAJOR_NUM_SVIP_NAT_PT, 3)
#define FIO_SVIP_NAT_PT_RULE_PREPARE _IOW(MAJOR_NUM_SVIP_NAT_PT, 4, SVIP_NAT_PORT_MAP_Rule_t)
#define FIO_SVIP_NAT_PT_RULE_GET _IOWR(MAJOR_NUM_SVIP_NAT_PT, 5, SVIP_NAT_PT_IO_Rule_t)
#define FIO_SVIP_NAT_PT_HOOK_SET _IOWR(MAJOR_NUM_SVIP_NAT_PT, 6, SVIP_NAT_PT_HOOK_t)
#define FIO_SVIP_NAT_PT_STATS_GET _IOWR(MAJOR_NUM_SVIP_NAT_PT, 7, SVIP_NAT_PKT_STATS_t)

#ifndef ETH_ALEN
#define ETH_ALEN   6 /* Octets in one ethernet address */
#endif

/** Type to identify IPv4 and IPv6 addresses */
typedef enum
{
   SVIP_IPV4_ADDR_TYPE = AF_INET,
   SVIP_IPV6_ADDR_TYPE = AF_INET6
} SVIP_IP_ADDR_TYPE_t;

typedef IFX_uint32_t SVIP_IPV4_ADDR_t;

/** IPv6 address type, style taken over from Linux */
typedef struct SVIP_IPV6_ADDR
{
   union
   {
      IFX_uint8_t addr8[16];
      IFX_uint16_t addr16[8];
      IFX_uint32_t addr32[4];
   } ipv6_u;
#define ipv6_addr8 ipv6_u.addr8
#define ipv6_addr16 ipv6_u.addr16
#define ipv6_addr32 ipv6_u.addr32
} SVIP_IPV6_ADDR_t;

/** union of IPv4 and IPv6 addresses */
typedef union
{
   SVIP_IPV4_ADDR_t v4;
   SVIP_IPV6_ADDR_t v6;
} SVIP_IP_ADDR_UNION_t;

/** UDP error correction method. */
typedef enum
{
   /** RED (redundancy) will be used during the session. */
   IFX_SVIP_T38_RED = 0,
   /** FEC (forward error correction) will be used during the session. */
   IFX_SVIP_T38_FEC = 1
}IFX_SVIP_T38_EC_MODE_t;

/*! T.38 indicator types */
enum t30_indicator_types_e
{
    T38_IND_NO_SIGNAL = 0,
    T38_IND_CNG,
    T38_IND_CED,
    T38_IND_V21_PREAMBLE,
    T38_IND_V27TER_2400_TRAINING,
    T38_IND_V27TER_4800_TRAINING,
    T38_IND_V29_7200_TRAINING,
    T38_IND_V29_9600_TRAINING,
    T38_IND_V17_7200_SHORT_TRAINING,
    T38_IND_V17_7200_LONG_TRAINING,
    T38_IND_V17_9600_SHORT_TRAINING,
    T38_IND_V17_9600_LONG_TRAINING,
    T38_IND_V17_12000_SHORT_TRAINING,
    T38_IND_V17_12000_LONG_TRAINING,
    T38_IND_V17_14400_SHORT_TRAINING,
    T38_IND_V17_14400_LONG_TRAINING,
    T38_IND_V8_ANSAM,
    T38_IND_V8_SIGNAL,
    T38_IND_V34_CNTL_CHANNEL_1200,
    T38_IND_V34_PRI_CHANNEL,
    T38_IND_V34_CC_RETRAIN,
    T38_IND_V33_12000_TRAINING,
    T38_IND_V33_14400_TRAINING
};

/*! T.38 message types */
enum t38_message_types_e
{
    T38_TYPE_OF_MSG_T30_INDICATOR = 0,
    T38_TYPE_OF_MSG_T30_DATA
};

/** UDP port in network-byte order */
typedef IFX_uint16_t SVIP_UDP_PORT_t;

/** SVIP NAT rule which allows protocol tranformation between IPv4 and IPv6 */
typedef struct SVIP_NAT_PT_IO_Rule
{
   /** Remote peer MAC address */
   IFX_uint8_t remMAC[ETH_ALEN]; /* <- does not seem compulsory */
   /** Remote peer IP address type */
   SVIP_IP_ADDR_TYPE_t remTypeIP;
   /** Remote peer IP address */
   SVIP_IP_ADDR_UNION_t remIP;
   /** Remote peer UDP port number */
   SVIP_UDP_PORT_t remUDP;
   /** Local peer (target SVIP) MAC address */
   IFX_uint8_t locMAC[ETH_ALEN];  /* <- does not seem compulsory */
   /** Local peer (target SVIP) IP address type */
   SVIP_IP_ADDR_TYPE_t locTypeIP;
   /** Local peer IP address */
   SVIP_IP_ADDR_UNION_t locIP;
   /** Local peer (target SVIP) UDP port number */
   SVIP_UDP_PORT_t locUDP;
   /** Differentiated services; this field is defined in RFC 2474. */
   IFX_uint8_t nDSCP;
   /** Set the NAT rule index for local rtp port nat table in kernel */
   IFX_uint8_t nNatRuleIndex;
   IFX_uint8_t nMasterChip;
   IFX_uint8_t u8_Reserved[7];
} SVIP_NAT_PT_IO_Rule_t;

typedef struct SVIP_NAT_PT_HOOK
{
   /** Set the NAT rule index for local rtp port nat table in kernel */
   IFX_uint8_t nNatRuleIndex;
   /** Flag: TRUE if we need to hook the T38 stream and increase the seq number. */
   IFX_uint8_t hookT38;
   /** How many no-signal packets should be sent. */
   IFX_uint8_t sendNoSignal;
   /** UDP error correction method. */
   IFX_uint8_t udpErrCorrection;
   /** UDP maximum datagram size; the maximum size of a
      UDPTL packet that is accepted by the remote
      device. */
   IFX_uint16_t udpDatagramSize;
   /* Flag to indicate whether the far end no-signal is detected. */
   IFX_uint8_t noSignalFarEnd;
   IFX_uint8_t u8_Reserved;
} SVIP_NAT_PT_HOOK_t;

typedef struct SVIP_NAT_PORT_MAP_Rule
{
   SVIP_UDP_PORT_t locUDP;
   IFX_uint8_t nNatRuleIndex;
   IFX_uint8_t u8_Reserved;
   IFX_uint32_t u32_Reserved;
} SVIP_NAT_PORT_MAP_Rule_t;

typedef struct SVIP_NAT_PKT_STATS
{
   /** Local peer (target SVIP) UDP port number */
   SVIP_UDP_PORT_t locUDP;
   IFX_uint32_t preceived;
   IFX_uint32_t oreceived;
   IFX_uint32_t psent;
   IFX_uint32_t osent;
} SVIP_NAT_PKT_STATS_t;

/** @} */
#endif

#endif
