/*
 * Web page tool
 */
//#define X86TEST 1


#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include "provision.h"

#define HT502_LINE          0x01000001   /* V1.0B and above */
#define HT503_LINE          0x00000000   /* All version */
#define HT504_LINE          0x01020001   /* V1.2B and above */
#define HT508_LINE          0x01020001   /* V1.2B and above */
#define HT516_LINE          0x00000000   /* All version */
#define HT524_LINE          0x00000000   /* All version */
#define HT532_LINE          0x00000000   /* All version */

#ifdef X86TEST
#define nvram_set(a,b) printf("Set %s = [%s]\n", a, b)
#else
extern int nvram_set(const char *name, const char *value);
#endif

#if 0
unsigned short fact_para_checksum(HT_FACTORYPARA *factPara);

HT_FACTORYPARA factPara;

unsigned short fact_para_checksum(HT_FACTORYPARA *factPara)
{
    HT_FACTORYPARA tmpData;
    unsigned char *p = (unsigned char *)&tmpData;
    unsigned short sum = 0;
    unsigned short checksum = 0;
    unsigned length = 0;
    unsigned short sum2 = 0;
    int i;

    memcpy(&tmpData, factPara, sizeof(HT_FACTORYPARA));
    tmpData.checksum = 0;
    length = (tmpData.length[1] << 16) | tmpData.length[0]; // le
    for (i = 0; i < length; i+=2)
    {
        sum += (p[i] << 8 | p[i+1]);    // fact config use be for checksum calc
    }
    checksum = 0xffff - sum + 0x1;

    return (checksum >> 8 | checksum << 8); // be
}

int extract_hwrev_from_V1(char *PartNumber, int *hwver_major, int *hwver_minor, int *hwver_rev)
{
    *hwver_major = PartNumber[8] - '0';
    *hwver_minor = PartNumber[9] - '0';
    *hwver_rev = PartNumber[10] - 'A';
}
#endif

int main (int argc, char **argv)
{

#ifdef X86TEST
	if (1)
#else
	if (strcmp(argv[0], "detect_vinetic") == 0)
#endif
	{
		FILE *f;
        unsigned short prod_id = 0;
        unsigned long rev_id = 0;
        int vinetic_v22 = 1;

#ifndef X86TEST
		if (argc != 1) 
        {
            printf("invalid cmd format!\n");
			return -1;
        }
#endif

#if 0
#ifdef X86TEST
		if ((f = fopen(argv[1], "rb+")) == NULL) 
#else
		if ((f = fopen("/proc/provision/dev_info/dev_data", "rb+")) == NULL) 
#endif
        {
            printf("Fact config data not found!\n");
			return -1;
        }

        /* read out factory data from /proc */
        if (fread(&factPara, 1, sizeof(factPara), f) <= 1)
        {
            printf("Error getting factory config data!\n");
            fclose(f);
            return -1;
        }
        /* verify checksum */
        if (factPara.checksum != fact_para_checksum(&factPara))
        {
            printf("Invalid factory config data!\n");
            fclose(f);
            return -1;
        }
        else
        {
            if (IS_V1_FACT(&factPara))
            {
                int hwver_major, hwver_minor, hwver_rev;

#ifdef X86TEST
                printf("V1 fact data format detected, PN=%s\n", (char *)&factPara.u.v1.PartNumber);
#endif
                if (compare_pn_str((char *)&factPara.u.v1.PartNumber, HW_HT502_PNSTR) == 0)
                {
                    prod_id = HW_HT502_ID;
                }
                else if (compare_pn_str((char *)&factPara.u.v1.PartNumber, HW_HT503_PNSTR) == 0)
                {
                    prod_id = HW_HT503_ID;
                }
                else if (compare_pn_str((char *)&factPara.u.v1.PartNumber, HW_HT504_PNSTR) == 0)
                {
                    prod_id = HW_HT504_ID;
                }
                else if (compare_pn_str((char *)&factPara.u.v1.PartNumber, HW_HT508_PNSTR) == 0)
                {
                    prod_id = HW_HT508_ID;
                }
                else if (compare_pn_str((char *)&factPara.u.v1.PartNumber, HW_HT516_PNSTR) == 0)
                {
                    prod_id = HW_HT516_ID;
                }
                else if (compare_pn_str((char *)&factPara.u.v1.PartNumber, HW_HT524_PNSTR) == 0)
                {
                    prod_id = HW_HT524_ID;
                }
                else if (compare_pn_str((char *)&factPara.u.v1.PartNumber, HW_HT532_PNSTR) == 0)
                {
                    prod_id = HW_HT532_ID;
                }

                extract_hwrev_from_V1((char *)&factPara.u.v1.PartNumber, &hwver_major, &hwver_minor, &hwver_rev);
                rev_id = (hwver_major & 0xff) << 24 | (hwver_minor & 0xff) << 16 | (hwver_rev & 0xff); 
            }
            else
            {
#ifdef X86TEST
                printf("V0 fact data format detected\n");
#endif
                prod_id = factPara.hw_model;
                rev_id = ((unsigned long)factPara.RevID[0][0]) << 16 | ((unsigned long)factPara.RevID[0][1]);
            }
#ifdef X86TEST
            printf("prod_id=0x%x, rev_id=0x%08x\n", prod_id, rev_id);
#endif
        }
#ifdef X86TEST
        {
            long macH = ((factPara.mac[0] & 0xff) << 16) | (factPara.mac[0] & 0xff00) |(factPara.mac[1] & 0xff);
            long macL = ((factPara.mac[1] & 0xff00) << 8) | ((factPara.mac[2] & 0xff) << 8) |((factPara.mac[2] & 0xff00) >> 8);
            //printf("mac=0x%04x%04x%04x\n", factPara.mac[0], factPara.mac[1], factPara.mac[2]);
            printf("mac=%06x%06x\n", macH,macL);
        }
#endif
	
        /* get vinetic version according to the prod_id and rev_id */
        switch (prod_id) {
        case HW_HT502_ID:    //ht502
            if (rev_id >= HT502_LINE)
                vinetic_v22 = 1;
            break;
        case HW_HT503_ID:    //ht503
            if (rev_id >= HT503_LINE)
                vinetic_v22 = 1;
            break;
        case HW_HT504_ID:    //ht504
            if (rev_id >= HT504_LINE)
                vinetic_v22 = 1;
            break;
        case HW_HT508_ID:    //ht508
            if (rev_id >= HT508_LINE)
                vinetic_v22 = 1;
            break;
        case HW_HT516_ID:    //ht516
            if (rev_id >= HT516_LINE)
                vinetic_v22 = 1;
            break;
        case HW_HT524_ID:    //ht524
            if (rev_id >= HT524_LINE)
                vinetic_v22 = 1;
            break;
        case HW_HT532_ID:    //ht532
            if (rev_id >= HT532_LINE)
                vinetic_v22 = 1;
            break;
        default:
            break;
        }
		fclose(f);
#endif
        if (vinetic_v22)
            nvram_set(":vinetic_ver", "v22");
        else
            nvram_set(":vinetic_ver", "v21");

	}
	else
	{
		printf("Applet unknown\n");
	}
	
	return 0;
}


