#include <stdio.h>
#include <string.h>
#include <errno.h>
#include <fcntl.h>
#include <stdlib.h>
#include <sys/ioctl.h>
#include <sys/select.h>
#include <signal.h>
#include <unistd.h>

#define GXW_KP_DEVICE                "/dev/gxw42xx_keypad"

#define EVENT_LINK                  0x4000
#define EVENT_LINK_ON               (EVENT_LINK | 0x01)
#define EVENT_LINK_OFF              (EVENT_LINK | 0x02)
#define EVENT_NOTIFY                0x2000
#define EVENT_NOTIFY_RESET1          (EVENT_NOTIFY   | 0x01)

typedef struct 
{
    unsigned long    event_id;
    unsigned long    timestamp;        /* in ms */
} GXW_INPUT_EVENT, *GXW_INPUT_EVENT_P;

typedef struct 
{
    unsigned long    param;                /* for input/output */
    unsigned long    reserved;
} GXW_KP_IOCTL_PARAM, *GXW_KP_IOCTL_PARAM_P;

/* IOCTL command */
typedef enum
{
    GXW_KP_IOCTL_START = 0,

    C55_IOCTL_END
} GXW_KP_IOCTL_CMD;

int main()
{
    int fd_kp;
    int rtn;
    GXW_KP_IOCTL_PARAM param;
    GXW_INPUT_EVENT input;
    fd_set rfds;
    struct timeval tv;
    int rc;
    int running = 1;
    ssize_t size;
    char buf[100];

    fd_kp = open( GXW_KP_DEVICE, O_RDWR );
    
    while ( running )
    {
        FD_ZERO( &rfds );
        tv.tv_sec = 3;
        tv.tv_usec = 0;
        FD_SET( fd_kp, &rfds );

        rc = select( fd_kp + 1, &rfds, NULL, NULL, NULL  );
        if ( rc <= 0 )
        {
            printf( "select failed: %d\n", rc );
            //return -1;
        }

        if ( FD_ISSET( fd_kp, &rfds ) )
        {

            rtn = read( fd_kp, &input, sizeof(input) );
            if ( rtn < -1 )
            {
                printf( "read event failed: %d\n", rtn );
                return -1;
            }

            if ( input.event_id  == EVENT_LINK_ON || input.event_id == EVENT_LINK_OFF )
            {
                printf( "##link event received\n" );

                signal( SIGCHLD, SIG_IGN ); // to get rid of the zombie child process

                if ( fork() == 0 )
                {
                    if (system( input.event_id  == EVENT_LINK_ON? "/etc/init.d/network start" : "/etc/init.d/network stop") == -1)
                    {
                        printf("network_change script call failed\n");
                    }
                    return;
                }
            }
            else if ( input.event_id  == EVENT_NOTIFY_RESET1 )
            {
                printf( "##Reset event received\n" );

                signal( SIGCHLD, SIG_IGN ); // to get rid of the zombie child process

                if ( fork() == 0 )
                {
                    if (system( "/etc/init.d/reset_script 1" ) == -1)
                    {
                        printf("reset script call failed\n");
                    }
                    return;
                }
            }
            else
                printf( "##unknown event: %lx\n", input.event_id );

        }
    }

    return 0;
}
