#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <signal.h>
#include "nvram.h"
#include "msglib.h"
#include <errno.h>
#include "pvalue_validation/pvalue_validation_lib.h"

#define MAX_RECEIVER                        4
#define MAX_CLIENT                          (MAX_RECEIVER + 1)

#define MONITOR_MSGQ_KEY 0x2983383   /* Need be globally unique */
/************/
#define DEFAULT_SCRIPT "daemonize /etc/init.d/rcM /"
#define NO_OF_SCRIPTS_PER_BLOCK 10
#define RUN_SCRIPT 1
#define DONOT_RUN_SCRIPT -1
/************/

int     server_qid;
int     monitor_qid;
admin_msg server_msg;

int     server_qid_cc;
int     monitor_qid_cc;
admin_msg server_msg_cc;
int     ch_user_qid[MAX_RECEIVER];
int debug_print; /*Bug 14696 debug print*/

struct _receiver
{
    long user_id;
    char group;
};

/*struct for scripts*/
struct _scripts
{
    int index[NO_OF_SCRIPTS_PER_BLOCK];
    unsigned char run[NO_OF_SCRIPTS_PER_BLOCK];
    unsigned char enabled[NO_OF_SCRIPTS_PER_BLOCK];
    unsigned char level;
    struct _scripts *next;
};
typedef struct _scripts scripts;
scripts script_array = { .index = { 0 }, .run = { 0 }, .enabled = { 0 }, .level = 0, .next = NULL};

/*functions to manage script struct*/
static void run_scripts( scripts *head, char* script_to_run );
static void clear_scripts( scripts *head );
static void free_scripts( scripts *head );
static int add_script( int i, scripts *head );

static void signalHandler( int cause, siginfo_t* info, void* ptr );
static void signalHandler_cc( int cause, siginfo_t* info, void* ptr );

int msg_server()
{
    int    srvr_qid;
    key_t  msgkey;
    admin_msg srvr_msg;
    event_msg client_msg;
    int ii;
    int ret;
    struct _receiver rcvr_arr[MAX_RECEIVER];

    memset( &srvr_msg, 0, sizeof( admin_msg ) );
    memset( &client_msg, 0, sizeof( event_msg ) );

    for ( ii = 0; ii < MAX_RECEIVER; ii ++ )
    {
        rcvr_arr[ii].user_id = -1;
        rcvr_arr[ii].group = 0;
    }

    /* Generate our IPC key value */
    msgkey = SERVER_MSGQ_KEY;

    /* Open/create the queue */
    if ( ( srvr_qid = open_queue( msgkey ) ) == -1 )
    {
        perror( "open_queue" );
        exit( 1 );
    }

    for ( ;; )
    {
        if ( ( ret = read_message( srvr_qid, ADMMSG_TYPE_ALL, ( struct msgbuf * )&srvr_msg, sizeof( srvr_msg ) ) ) != -1 )
        {
            //printf("admin msg received, type=%ld\n", srvr_msg.mtype);
            if ( srvr_msg.mtype == ADMMSG_MSG_SERVER_EXIT )
            {
                break;
            }

            switch ( srvr_msg.mtype )
            {

                case ADMMSG_MSG_REGISTER_USER:

                    //printf("received registration request from user id: %i\n", srvr_msg.user_id);
                    for ( ii = 0; ii < MAX_RECEIVER; ii ++ )
                    {
                        if ( rcvr_arr[ii].user_id == -1 || rcvr_arr[ii].user_id == srvr_msg.user_id )
                        {
                            rcvr_arr[ii].user_id = srvr_msg.user_id;
                            rcvr_arr[ii].group = srvr_msg.group;
                            printf( "client [0x%lx] groups(0x%x) registered on %d\n", srvr_msg.user_id, srvr_msg.group, ii );
                            break;
                        }
                    }

                    break;

                case ADMMSG_MSG_DELETE_USER:

                    for ( ii = 0; ii < MAX_RECEIVER; ii ++ )
                    {
                        if ( rcvr_arr[ii].user_id == srvr_msg.user_id )
                        {
                            printf( "client [0x%lx] groups(0x%x) deleted on %d\n", srvr_msg.user_id, srvr_msg.group, ii );
                            rcvr_arr[ii].user_id = -1;
                            rcvr_arr[ii].group = 0;
                            break;
                        }
                    }

                    break;

                case ADMMSG_MSG_DISPATCH_SETTING_CHG_EVT:
                case ADMMSG_MSG_DONE_SCRIPTS_EVT:

                    if ( srvr_msg.mtype == ADMMSG_MSG_DISPATCH_SETTING_CHG_EVT )
                    {
                        client_msg.mtype = EVTMSG_TYPE_SETTING_CHANGE_NOTIFY;
                    }
                    else if ( srvr_msg.mtype == ADMMSG_MSG_DONE_SCRIPTS_EVT )
                    {
                        client_msg.mtype = EVTMSG_TYPE_DONE_SCRIPTS_EVT;
                    }

                    strncpy( client_msg.mbody, srvr_msg.mbody, ADMIN_EVENT_MSG_LENGTH );

                    srvr_msg.group |= MSG_GROUP_ID_BROADCAST; //always send to broadcast

                    for ( ii = 0; ii < MAX_RECEIVER; ii ++ )
                    {
//printf( "%s:%d %lu %x==%x\n", __FILE__, __LINE__, rcvr_arr[ii].user_id, rcvr_arr[ii].groups, srvr_msg.groups );
                        if ( 0 != ( rcvr_arr[ii].group & srvr_msg.group ) )
                        {
//printf( "%s:%d\n", __FILE__, __LINE__ );
                            if ( send_message( rcvr_arr[ii].user_id, ( struct msgbuf * )&client_msg, sizeof( client_msg ) ) == -1 )
                            {
                                printf( "%s:%d event dispatch error on %d\n", __FILE__, __LINE__, ii );
                            }
                        }
                    }

                    break;

                default:
                    break;
            }
        }
        else
        {
            perror( "read_admin_message" );
        }

    }

    remove_queue( srvr_qid );

    return 0;
}

int msg_monitor()
{
    key_t  msgkey;
    event_msg client_msg;
    struct sigaction sa;

    debug_print = 0; /*Bug 14696 disable debug print by default*/ 

    /* Generate our IPC key value */
    msgkey = SERVER_MSGQ_KEY;

    /* Open/create the queue */
    if ( ( server_qid = open_queue( msgkey ) ) == -1 )
    {
        perror( "open_queue" );
        exit( 1 );
    }

    msgkey = MONITOR_MSGQ_KEY;

    if ( ( monitor_qid = open_queue( msgkey ) ) == -1 )
    {
        perror( "open_queue" );
        exit( 1 );
    }

    /* Set signal handler for removing client msg queue */
    sa.sa_sigaction = signalHandler;
    sigemptyset( &sa.sa_mask );
    sa.sa_flags = SA_SIGINFO;
    /*For removing client msg queue*/
    sigaction( SIGTERM, &sa, 0 );
    sigaction( SIGINT, &sa, 0 );
    /*For enabling disabling debug prints*/
    sigaction( SIGUSR1, &sa, 0 );

    server_msg.mtype = ADMMSG_MSG_REGISTER_USER;
    server_msg.user_id = monitor_qid;

    if ( send_message( server_qid, &server_msg, sizeof( server_msg ) ) != 0 )
    {
        perror( "send_admin_message" );
        exit( 1 );
    }

    for ( ;; )
    {
        if ( ( read_message( monitor_qid, EVTMSG_TYPE_ALL, &client_msg, sizeof( client_msg ) ) ) != -1 )
        {
            if ( client_msg.mtype == EVTMSG_TYPE_SETTING_CHANGE_NOTIFY )
            {
                if ( debug_print )
                {
                    if ( strlen( client_msg.mbody ) )
                    {
                        printf( "Event notify received, pvalue %s changed\n", client_msg.mbody );
                    }
                    else
                    {
                        printf( "Event notify received, setting changed\n" );
                    }
                }
            }
        }
    }

    return 0;
}

static void signalHandler( int cause, siginfo_t* info, void* ptr )
{
    /*Bug 14696 add handling to enable/disable debug print*/ 
    if ( cause == SIGUSR1 )
    {
        if ( !debug_print )
        {
            debug_print = 1;
        }
        else
        {
            debug_print = 0;
        }
        return;
    }
    server_msg.mtype = ADMMSG_MSG_DELETE_USER;
    server_msg.user_id = monitor_qid;

    if ( send_message( server_qid, &server_msg, sizeof( server_msg ) ) != 0 )
    {
        perror( "send_admin_message" );
    }

    remove_queue( monitor_qid );

    printf( "receiver exit on kill\n" );
    exit( 1 );
}

int msg_monitor_cc( char* groups, char* script )
{
    key_t  msgkey;
    event_msg client_msg;
    struct sigaction sa;
    int i, reboot_flag = 0, script_flag = 0, script_ready = 0;
    char user_defined_script[256] = { 0 };
    int run_script = 0;
    int errsv = 0;
    debug_print = 0; /*Bug 14696 disable debug print by default*/ 

    /* Generate our IPC key value */
    msgkey = SERVER_MSGQ_KEY;

    /* Open/create the queue */
    if ( ( server_qid_cc = open_queue( msgkey ) ) == -1 )
    {
        perror( "open_queue" );
        exit( 1 );
    }

    msgkey = 0;

    if ( ( monitor_qid_cc = open_queue( msgkey ) ) == -1 )
    {
        perror( "open_queue" );
        exit( 1 );
    }

    /* Set signal handler for removing client msg queue */
    sa.sa_sigaction = signalHandler_cc;
    sigemptyset( &sa.sa_mask );
    sa.sa_flags = SA_SIGINFO;
    /*For removing client msg queue*/
    sigaction( SIGTERM, &sa, 0 );
    sigaction( SIGINT, &sa, 0 );
    /*For enabling disabling debug prints*/
    sigaction( SIGUSR1, &sa, 0 );

    server_msg_cc.mtype = ADMMSG_MSG_REGISTER_USER;
    server_msg_cc.user_id = monitor_qid_cc;
    //extract groups and set
    server_msg_cc.group = MSG_GROUP_ID_BROADCAST;  //default to getting notifications from all groups

    get_group( groups, &( server_msg_cc.group ) );
    if (send_message_retry( server_qid_cc, &server_msg_cc, sizeof(server_msg_cc), -1, &errsv ) != 0)
    {
        perror( "send_admin_message" );
        exit( 1 );
    }

    //clear scripts array
    clear_scripts( &script_array );

    //use user defined script instead of default
    if ( script )
    {
        strncpy( user_defined_script, script, 255 );
        run_script = 1;
    }

    for ( ;; )
    {
        //run scripts if ready
        if ( script_flag == RUN_SCRIPT && script_ready )
        {
            script_flag = DONOT_RUN_SCRIPT;
            script_ready = 0;

            if ( run_script )
            {
                run_scripts( &script_array, user_defined_script );
            }
            else
            {
                run_scripts( &script_array, NULL );
            }
            /* Clear array for next runtime. */
            clear_scripts( &script_array );
        }

        if ( ( read_message( monitor_qid_cc, EVTMSG_TYPE_ALL, &client_msg, sizeof( client_msg ) ) ) != -1 )
        {
            if ( client_msg.mtype == EVTMSG_TYPE_SETTING_CHANGE_NOTIFY )
            {
                if ( strlen( client_msg.mbody ) )
                {
                    if ( debug_print )
                    {
                        printf("Event notify received, pvalue %s changed\n", client_msg.mbody);
                    }

                    //if commit then execute scripts
                    if ( !reboot_flag && !strcmp( "nvram_commit", client_msg.mbody ) )
                    {
                        script_ready = 1;

                        if ( script_flag != RUN_SCRIPT && script_flag != DONOT_RUN_SCRIPT )
                        {
                            script_flag = RUN_SCRIPT;
                        }
                    }
                    else if ( !strcmp( "reboot_req", client_msg.mbody ) )
                    {
                        reboot_flag = 1;
                    }
                    else
                    {
                        i = 0;
                        char scripts[512] = { 0 };

                        //int pvalue_scripts( client_msg.mbody, scripts, scripts_size );
                        if ( !pvalue_scripts( client_msg.mbody, scripts, 512 ) )
                        {
                            char *pSave;
                            char* tmp = strtok_r( scripts, ",", &pSave );

                            while ( tmp )
                            {
                                errno = 0;
                                i = strtol( tmp, NULL, 10 );

                                if ( !errno && i )
                                {
                                    add_script( i, &script_array );
                                }

                                tmp = strtok_r( NULL, ",", &pSave );
                            }
                        }
                    }
                }
                else
                {
                    if ( debug_print )
                    {
                        printf( "Event notify received, setting changed\n" );
                    }
                }
            }
            else if ( client_msg.mtype == EVTMSG_TYPE_DONE_SCRIPTS_EVT )
            {
                //raise flag to run scripts
                script_flag = RUN_SCRIPT;

                if ( strlen( client_msg.mbody ) )
                {
                    if ( debug_print )
                    {
                        printf( "Event scripts done received, %s done\n", client_msg.mbody );
                    }
                }
            }
        }
    }

    return 0;
}

static void signalHandler_cc( int cause, siginfo_t* info, void* ptr )
{
    /*Bug 14696 add handling to enable/disable debug print*/ 
    if ( cause == SIGUSR1 )
    {
        if ( !debug_print )
        {
            debug_print = 1;
        }
        else
        {
            debug_print = 0;
        }
        return;
    }
    server_msg_cc.mtype = ADMMSG_MSG_DELETE_USER;
    server_msg_cc.user_id = monitor_qid_cc;

    if ( send_message( server_qid_cc, &server_msg_cc, sizeof( server_msg_cc ) ) != 0 )
    {
        perror( "send_admin_message" );
    }

    remove_queue( monitor_qid_cc );

    //free script array blocks
    if ( script_array.next )
    {
        free_scripts( script_array.next );
    }

    printf( "receiver exit on kill\n" );
    exit( 1 );
}

/*Free script blocks*/
void free_scripts( scripts *head )
{
    scripts *curr = head;
    scripts *prev = NULL;

    if ( !curr )
    {
        return;
    }

    while ( curr )
    {
        prev = curr;
        curr = curr->next;
        free( prev );
    }
}

/* Initialize script blocks*/
void clear_scripts( scripts *head )
{
    int i = 0;
    scripts *curr = head;

    if ( !curr )
    {
        return;
    }

    while ( curr )
    {
        for ( i = 0 ; i < NO_OF_SCRIPTS_PER_BLOCK; i++ )
        {
            curr->index[i] = 0;
            curr->run[i] = 0;
            curr->enabled[i] = 0;
        }
        curr->level = 0;
        curr = curr->next;
    }
}

/*Collect all runnable scripts and send to script_to_run.
  Set runnable flags to false*/
void run_scripts( scripts *head, char *script_to_run )
{
    int i = 0;
    scripts *curr = head;
    char script_cmd[10] = { 0 };
    char *cmd = NULL;
    char *final_cmd = NULL;
    unsigned int cmdsize = 64;

    if ( !curr )
    {
        return;
    }

    if ( script_to_run && strlen( script_to_run ) > cmdsize )
    {
        cmdsize = strlen( script_to_run ) + 64;
    }

    final_cmd = ( char * )malloc( sizeof( char ) * cmdsize );

    if ( final_cmd == NULL )
    {
        fprintf( stderr, "Insufficient memory\n" );
        return;
    }

    memset( final_cmd, 0, cmdsize );

    if ( script_to_run && strlen( script_to_run ) )
    {
        sprintf( final_cmd, script_to_run );
    }
    else
    {
        sprintf( final_cmd, DEFAULT_SCRIPT );
    }

    cmd = final_cmd + strlen( final_cmd );

    while ( curr )
    {
        for ( i = 0 ; i < curr->level; i++ )
        {
            if ( curr->enabled[i] && curr->run[i] )
            {
                snprintf( script_cmd, 10, " %d", curr->index[i] );

                if ( strlen( final_cmd ) + strlen( script_cmd ) + 1 >= cmdsize )
                {
                    char *tmp_cmd = final_cmd;
                    cmdsize = ( strlen( final_cmd ) + strlen( script_cmd ) + 1 ) * 2;
                    final_cmd = ( char * )realloc( tmp_cmd, sizeof( char ) * cmdsize );

                    if ( final_cmd == NULL )
                    {
                        fprintf( stderr, "Insufficient memory\n" );
                        free( tmp_cmd );
                        return;
                    }

                    cmd = final_cmd + strlen( final_cmd );
                }

                strcat( cmd, script_cmd );
                cmd += strlen( script_cmd );
                memset( script_cmd, 0, 10 );
                curr->run[i] = 0 ;  /*set run flag to false*/
            }
        }

        curr = curr->next;
    }

    system( final_cmd );
    free( final_cmd );
}

/*Add a script to script blocks.
 *If already added then enable runnable flag*/
int add_script( int script_index, scripts *head )
{
    int i = 0;
    scripts *curr = head;
    scripts *prev = NULL;

    if ( !curr )
    {
        return -1;
    }

    while ( curr )
    {
        for ( i = 0 ; i < curr->level; i++ )
        {
            if ( curr->index[i] == script_index ) /*if script is already added*/
            {
                if ( curr->enabled[i] )
                {
                    curr->run[i] = 1;  /*set run flag to true*/
                    return 0;
                }
                else
                {
                    return 1;
                }
            }
        }

        if ( curr->level < NO_OF_SCRIPTS_PER_BLOCK )  /*add to present block if it is not full*/
        {
            curr->index[curr->level] = script_index;
            curr->run[curr->level] = 1;              /*set run flag to true*/
            curr->enabled[curr->level] = 1;          /*set enabled flag to true*/
            curr->level++;
            return 0;
        }

        prev = curr;
        curr = curr->next;
    }

    curr = ( scripts * )malloc( sizeof( scripts ) ); /*add to a new block*/

    if ( !curr )
    {
        return -1;
    }

    prev->next = curr;
    curr->next = NULL;
    curr->index[0] = script_index;
    curr->run[0] = 1;                           /*set run flag to true*/
    curr->enabled[0] = 1;                       /*set enabled flag to true*/
    curr->level = 1;

    for ( i = 1 ; i < NO_OF_SCRIPTS_PER_BLOCK; i++ )
    {
        curr->index[i] = 0;
        curr->run[i] = 0;
        curr->enabled[i] = 0;
    }

    return 0;
}

