/*
 * Web page tool
 */

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <sys/reboot.h>
#include <sys/syslog.h>
#include <dirent.h>
#include <stdbool.h>
#include <sys/dir.h>
#include <unistd.h>
#include <errno.h>
#include "nvram.h"
#include "msg_notify.h"
#include "nvram_linux.h"
#include "msg_server.h"
#include "msglib.h"
#include "tftp_url_process.h"
#include "pvalue_validation/pvalue_validation_lib.h"

#define GPIO_CHECK_INTERVAL             200         // ms
#define ONLINE_BLINK_INTERVAL_COUNT     5           // 1s
#define PROVISION_BLINK_INTERVAL_COUNT  2           // 400ms
#define FACTORY_RESET_INTERVAL_COUNT    25          // 5s
#define PROC_PATH_TO_MAC    "/proc/gsboard/dev_info/dev_mac"


#define FIELD_START         "<%"
#define FIELD_END           "%>"
#define FORM_FIELD_START    "FORM_P"
#define FORM_FIELD_END      "="
#define CONF_FIELD_END      "="
#define CONF_FIELD_END_CH   '='

#define IDBUF_SIZE          64

typedef enum _Config_Change_Attr
{
    CCA_CHANGED  = 0x01,
    CCA_REBOOT   = 0x02,
} Config_Change_Attr;

struct lstr
{
    char id[MAX_KEYLEN];
    char value[MAX_VALLEN];
};
typedef struct lstr lstr;

static lstr last_field;
static char buf[LINE_BUF];
static char id[MAX_KEYLEN], target_value_compare[MAX_VALLEN], output[MAX_VALLEN];
static char tmpbuf[MAX_VALLEN];
static char *idbuf = NULL ;    /* pointer point to the stored ids */
static unsigned long idbuf_size = IDBUF_SIZE;
static unsigned long idbuf_len = 0;
static char tmpbuf2[MAX_VALLEN];

#ifndef NVRAM_STORE_KEYNAME
extern char* get_fieldname_by_id( char* idstr )
{
#if 0
    unsigned short id = ( unsigned short )atoi( idstr );
    char tmp_idstr[100];

    sprintf( tmp_idstr, "%d", id ); // should be safe here since the id is unsigned short

    if ( strcmp( tmp_idstr, idstr ) == 0 )
    {
        return strdup( tmp_idstr );
    }

    return NULL;
#else
    // non number style field name used, so disable the check
    return strdup( idstr );
#endif

}
#endif

/*
*Takes a program, working directory and arguments
*and daemonizes it
*/
static int daemonize( char **args )
{
    pid_t pid = -1;
    pid_t sid = -1;
    int fd;

    pid = fork();

    if ( pid < 0 )
    {
        exit( EXIT_FAILURE );
    }

    if ( pid > 0 )
    {
        exit( EXIT_SUCCESS );
    }

    umask( 0 );

    sid = setsid( );

    if ( sid < 0 )
    {
        exit( EXIT_FAILURE );
    }

    if ( ( chdir( args[1] ) ) < 0 )
    {
        exit( EXIT_FAILURE );
    }

    fd = open( "/dev/null", O_RDWR );

    close( STDIN_FILENO ) ;
    close( STDOUT_FILENO ) ;
    close( STDERR_FILENO ) ;

    dup2( fd, STDIN_FILENO );
    dup2( fd, STDOUT_FILENO );
    dup2( fd, STDERR_FILENO );
    if ( fd > STDERR_FILENO ) {
        close( fd );
    }

    execvp( args[0], args + 1 );
    return errno;
}

static char* get_cert( char *buf, unsigned int bufSize, char*tmp_buf, unsigned int tmpBufSize, FILE *f, int *error )
{
    memset( tmp_buf, 0, tmpBufSize );

    if ( strstr( buf, "-----END" ) == NULL ) //multiple line cert/key
    {
        do
        {
            strcat( tmp_buf, buf );
        }
        while ( !feof( f )
                && ( ( fgets( buf, bufSize - 1, f ) ) != NULL )
                && ( strstr( buf, "-----END" ) == NULL )
                && ( strlen( tmp_buf ) + strlen( buf ) <= tmpBufSize - 1 ) );
        if ( ( strstr( buf, "-----END" ) != NULL ) //custom certificate
            && ( strlen( tmp_buf ) + strlen( buf ) <= tmpBufSize - 1 ) )
        {
            long currfp = ftell( f );
            strcat( tmp_buf, buf );
            if (( ( fgets( buf, bufSize - 1, f ) ) != NULL )
                && ( strstr( buf, "-----BEGIN" ) != NULL )
                && ( strstr( buf, "=" ) == NULL ) /* next cert may be another P value*/
                && ( strlen( tmp_buf ) + strlen( buf ) <= tmpBufSize - 1 ))
            {
                do
                {
                    strcat( tmp_buf, buf );
                } while (!feof( f )
                         && ( ( fgets( buf, bufSize - 1, f )) != NULL )
                         && ( strstr( buf, "-----END" ) == NULL )
                         && ( strlen( tmp_buf ) + strlen( buf ) <= tmpBufSize - 1 ));
            }
            else
            {
                fseek( f, currfp, SEEK_SET );
                buf[0] = 0;
            }
        }
    }
    if ( strlen( tmp_buf ) + strlen( buf ) <= tmpBufSize - 1 )  //check tmp_buf size
    {
        strcat( tmp_buf, buf );
    }
    else
    {
        fprintf( stderr, "Certificate is too long\n" );
        *error = EFBIG;
        return NULL;
    }
    if ( strstr( tmp_buf, "-----END" ) == NULL )
    {
        return NULL;
    }

    return tmp_buf;
}

static char* skip_space_and_comments( char *buf, unsigned int bufSize )
{
    char *p = buf;

    while ( ( ( *p == ' ' ) || ( *p == '\t' ) ) && ( (p - buf) < bufSize ) )
    {
        ++p;
    }

    //* skip empty lines and comments */
    if ( ( (p - buf) >= bufSize ) 
        || ( *p == '\0' ) 
         /* do not skip bash script header */
        || ( ( *p == '#' ) && ( ( ( p - buf + 1 ) == bufSize ) || ( *(p + 1 ) != '!' ) ) )
        || ( *p == '\n' ) )
    {
        return NULL;
    }

    return p;
}

static char* get_pvalue_name( char *buf, unsigned int bufSize, char *name_buf, unsigned int nameBufSize )
{
    char *p = buf;
    char *pName = name_buf;

    while ( ( *p != '\0' ) && ( *p != CONF_FIELD_END_CH ) && ( ( p - buf ) < bufSize ) && ( ( pName - name_buf ) < nameBufSize ) )
    {
        *pName = *p;
        ++pName;
        ++p;
    }

    if ( ( ( pName - name_buf ) >= nameBufSize ) || ( ( p - buf ) >= bufSize ) || ( *p != CONF_FIELD_END_CH ) )
    {
        return NULL;
    }
    *pName = 0;
    return name_buf;
}

static inline char *field_replace( char *str )
{
    char *value = NULL;

    id[0] = target_value_compare[0] = output[0] = 0;
    sscanf( str, "%s %s %s", id, target_value_compare, output );

    if ( strcmp( id, last_field.id ) != 0 )
    {
        char *name = NULL;
        last_field.id[0] = last_field.value[0] = 0;
        // nvram get
        name = get_fieldname_by_id( id );

        if ( name )
        {
            value  = nvram_get_safe( name, tmpbuf, MAX_VALLEN );
            free( name );
        }

        if ( value )
        {
            strcpy( last_field.value, value );
            strcpy( last_field.id, id );
        }
    }

    value = last_field.value;

    if ( target_value_compare[0] == 0 )
    {
        strncpy( output, value, MAX_VALLEN );
    }
    else if ( strlen( target_value_compare ) != strlen( value ) || strncmp( value, target_value_compare, strlen( target_value_compare ) ) != 0 )
    {
        output[0] = 0;
    }

    output[MAX_VALLEN - 1] = 0;
    return output;
}

/*
*
*/
void get_random_str( char *str )
{
    int val, strcase, i;

    srandom( time( NULL ) );

    for ( i = 0 ; i < 8 ; i++ )
    {
        val = random() % 26;
        strcase = random() % 2;

        if ( strcase )
        {
            str[i] = val + 65;
        }
        else
        {
            str[i] = val + 97;
        }
    }
}

/*
* Added to handle invalid user input
*/
static char *output_line_conf( char *line, int allow_heredoc, int skipped )
{
    char *tok[3];
    char *l, *p = NULL, *p2, *res;
    char pre_str[] = "$( cat<<'11111111'\n", post_str[] = "11111111\n)";
    char rand_str[8];
    int pos = 0, i, value_pos;
    char *tmp;

    get_random_str( rand_str );

    for ( i = 0 ; i < 8 ; i++ )
    {
        pre_str[i + 9] = rand_str[i];
        post_str[i] = rand_str[i];
    }

    tok[0] = tok[1] = tok[2] = NULL;
    l = line;
    tmp = line;

    if ( !skipped )
    {
        /* Bug 11749
         * Remove commented lines.
         * A new line is a valid character as it indicates the end of the line
         * since we have used fgets to get the line in the first place.
         */
        while ( tmp != NULL && *tmp != '\n' && *tmp < 0x21 )
        {
            tmp++;
        }

        if ( *tmp == '#' )
        {
            return NULL;
        }
    }

    tok[pos++] = l;

    while ( l != NULL )
    {
        if ( ( p = strstr( l, FIELD_START ) ) == NULL )
        {
            break;
        }

        p2 = strstr( p, FIELD_END );

        if ( p2 == NULL )
        {
            break;
        }

        *p = 0;
        *p2 = 0;
        value_pos = pos;
        tok[pos++] = field_replace( p + strlen( FIELD_START ) );

        l = p2;
        l += strlen( FIELD_END );
        tok[pos++] = l;

        for ( i = 0 ; i < pos - 1 ; i ++ )
        {
            if ( i == value_pos && tok[i] && strlen( tok[i] ) && allow_heredoc )
            {
                fprintf( stdout, "%s%s\n%s", pre_str, tok[i], post_str );
            }
            else
            {
                fprintf( stdout, "%s", tok[i] ? tok[i] : "" );
            }
        }

        tok[0] = tok[pos - 1];
        pos = 1;
    }

    res = p;

    for ( i = 0 ; i < pos ; i ++ )
    {
        fprintf( stdout, "%s", tok[i] ? tok[i] : "" );
    }

    fflush( stdout );

    return res;
}

static void initialize_idbuf()
{
    if ( idbuf == NULL )
    {
        idbuf = (char *)malloc( IDBUF_SIZE );

        if ( idbuf == NULL )
        {
            return;
        }

        idbuf_len = 0;
        idbuf_size = IDBUF_SIZE;
    }

    memset( idbuf, 0, idbuf_len );
}

static void free_idbuf()
{
    if ( idbuf )
    {
        free( idbuf );
        idbuf = NULL;
        idbuf_size = IDBUF_SIZE;
        idbuf_len = 0;
    }
}

static int compare_ids( char *id )
{
    unsigned long buflen = 0;
    char *tmp_ptr = idbuf;

    if ( idbuf == NULL )
    {
        initialize_idbuf();
    }

    while ( buflen < idbuf_len )
    {
        size_t str_len = strlen( tmp_ptr ) + 1;

        if ( *tmp_ptr == 0 )
        {
            break;
        }

        if ( strcmp( tmp_ptr, id ) ) //if different, compare with next string
        {
            tmp_ptr += str_len;
            buflen += str_len;
            continue;
        }

        return 0;
    }

    if ( strlen( id ) + idbuf_len + 1 > idbuf_size )
    {
        char *temp = NULL;

        idbuf_size *= 2; //double the buffer size
        temp = (char *)realloc( idbuf, idbuf_size );

        if ( temp != NULL )     //realloc was successful
        {
            idbuf = temp;
        }
        else                    //there was an error
        {
            printf( "Error allocating memory!\n" );
            return 1;
        }

        memset( idbuf + idbuf_len, 0, idbuf_size - idbuf_len );
    }

    //insert entry into idbuf
    strcpy( idbuf + idbuf_len, id );
    idbuf_len += strlen( id ) + 1;
    return 1;
}

static int compare_nvram_value( char *str, int reboot_flag )
{
    char *old_value = NULL;
    char *id, *new_value;
    int len;

    if ( ( id = str ) == NULL )
    {
        return -1;
    }

    if ( str[0] == '#' ) // skip if started with #
    {
        return -1;
    }

    new_value = strstr( id, CONF_FIELD_END );

    if ( new_value == NULL ) // no "=", format not right
    {
        return -1;
    }

    *new_value = 0;
    new_value += strlen( CONF_FIELD_END );

    /* remove the '\n' if needed */
    len = strlen( new_value );

    if ( len && new_value[len - 1] == '\n' )
    {
        new_value[len - 1] = 0;
    }

    if ( strlen( id ) && compare_ids( id ) )
    {
        if ( reboot_flag == 0 || check_pvalue_reboot_flag( id ) == 1 )
        {
            old_value = nvram_l2get_safe( id, tmpbuf, MAX_VALLEN ); /*l2get will get all values whether locked or not*/

            if ( old_value && *old_value != 0 )
            {
                if ( *new_value != 0 && strcmp( old_value, new_value ) != 0 )
                {
                    return 1;
                }
                else if ( *new_value == 0 )
                {
                    return 1;
                }
            }
            else if ( *new_value != 0 )
            {
                return 1;
            }
        }
    }

    return 0;
}

static void print_nvram_value( char *str, int reboot_flag )
{
    char *old_value = NULL;
    char *id, *new_value;

    if ( ( id = str ) == NULL )
    {
        return;
    }

    if ( str[0] == '#' ) // skip if started with #
    {
        return;
    }

    new_value = strstr( id, CONF_FIELD_END );

    if ( new_value == NULL ) // no "=", format not right
    {
        return;
    }

    *new_value = 0;
    new_value += strlen( CONF_FIELD_END );

    if ( strlen( id ) && compare_ids( id ) )
    {
        if ( reboot_flag == 0 || check_pvalue_reboot_flag( id ) == 1 )
        {
            old_value = nvram_l2get_safe( id, tmpbuf, MAX_VALLEN ); /*l2get will get all values whether locked or not*/

            if ( old_value )
            {
                fprintf( stdout, "%s=%s\n", id, old_value );
            }
            else
            {
                fprintf( stdout, "%s=\n", id );
            }
        }
    }

    return;
}

static int print_unique( char *str )
{
    char *id, *new_value;

    if ( ( id = str ) == NULL )
    {
        return -1;
    }

    if ( str[0] == '#' ) // skip if started with #
    {
        return -1;
    }

    new_value = strstr( id, CONF_FIELD_END );

    if ( new_value == NULL ) // no "=", format not right
    {
        return -1;
    }

    *new_value = 0;
    new_value += strlen( CONF_FIELD_END );

    if ( strlen( id ) )
    {
        if ( compare_ids( id ) )
        {
            fprintf( stdout, "%s=%s", id, new_value );
            return 0;
        }
        else
        {
            return 1;
        }
    }

    return -1;
}

static char *output_line( char *line )
{
    char *tok[3];
    char *l = NULL, *p = NULL, *p2 = NULL, *res = NULL;
    int pos = 0, i;

    tok[0] = tok[1] = tok[2] = NULL;
    l = line;
    tok[pos++] = l;

    while ( l != NULL )
    {
        if ( ( p = strstr( l, FIELD_START ) ) == NULL )
        {
            break;
        }

        p2 = strstr( p, FIELD_END );

        if ( p2 == NULL )
        {
            break;
        }

        *p = 0;
        *p2 = 0;
        tok[pos++] = field_replace( p + strlen( FIELD_START ) );

        l = p2;
        l += strlen( FIELD_END );
        tok[pos++] = l;

        for ( i = 0; i < pos - 1; i ++ )
        {
            fprintf( stdout, "%s", tok[i] ? tok[i] : "" );
        }

        tok[0] = tok[pos - 1];
        pos = 1;
    }

    res = p;

    for ( i = 0; i < pos; i ++ )
    {
        fprintf( stdout, "%s", tok[i] ? tok[i] : "" );
    }

    fflush( stdout );

    return res;
}

static int check_and_set_reboot( const char *name )
{
    if ( check_pvalue_reboot_flag( name ) == 1 )
    {
        char *str = NULL;
        str = nvram_get( ":reboot_req" );

        if ( !str || !atoi( str ) )
        {
            nvram_set( ":reboot_req", "1" );
        }
        return 1;
    }
    return 0;
}

static  inline int html_special_chars(char *input, char *output, int outputLen)
{
    int iPos = 0, oPos = 0;

    for (; ;)
    {
        switch (input[iPos])
        {
        case '<':
            {
                if ( outputLen - oPos >= 4 )
                {
                    memcpy(output + oPos, "&lt;", 4);
                    oPos += 4;
                }
                else
                {
                    goto abnormal_exit;
                }
            }
            break;
        case '>':
            {
                if ( outputLen - oPos >= 4 )
                {
                    memcpy(output + oPos, "&gt;", 4);
                    oPos += 4;
                }
                else
                {
                    goto abnormal_exit;
                }
            }
            break;
        case '\'':
            {
                if ( outputLen - oPos >= 6 )
                {
                    memcpy(output + oPos, "&apos;", 6);
                    oPos += 6;
                }
                else
                {
                    goto abnormal_exit;
                }
            }
            break;
        case '\"':
            {
                if ( outputLen - oPos >= 6 )
                {
                    memcpy(output + oPos, "&quot;", 6);
                    oPos += 6;
                }
                else
                {
                    goto abnormal_exit;
                }
            }
            break;
        case '&':
            {
                if ( outputLen - oPos >= 5 )
                {
                    memcpy(output + oPos, "&amp;", 5);
                    oPos += 5;
                }
                else
                {
                    goto abnormal_exit;
                }
            }
            break;
        case '\0':
            {
                if ( outputLen > oPos ) output[oPos] = 0;
                return oPos;
            }
            break;
        default:
            {
                if ( outputLen > oPos )
                {
                    output[oPos] = input[iPos];
                    oPos += 1;
                }
                else
                {
                    goto abnormal_exit;
                }
            }
            break;
        }
        ++iPos;
    }

abnormal_exit:

    if ( outputLen > oPos ) output[oPos] = 0;
    return oPos;
}

static inline int form_field_save(char *id, char *str, int htmlencode)
{
    static char new_value[MAX_VALLEN];
    char *old_value = NULL;
    int field_changed = 0;
    int nvram_set_success = 0;

    {
        char *name = NULL;
        // nvram set
        name = get_fieldname_by_id( id );

        if ( name )
        {
            old_value = nvram_get_safe( name, tmpbuf, MAX_VALLEN );

            if ( old_value )
            {
                old_value = strdup( old_value );
            }

            if ( strlen( str ) > 0 )
            {
                if ( htmlencode )
                {
                    html_special_chars(str, new_value, MAX_VALLEN);
                }
                else
                {
                    strncpy(new_value, str, MAX_VALLEN - 1);
                    new_value[MAX_VALLEN - 1] = 0;
                }
                /* value changed? */
                if ( old_value == NULL || (strlen(old_value) != strlen( new_value ) ) || strcmp( old_value, new_value ) != 0)   // value changed
                {
                    field_changed = 1;
                }

                if ( field_changed && nvram_set_flag( name, new_value, WEB_CHANGE_FLAG ) == 0 )
                {
                    nvram_set_success = 1;
                }
            }
            else if ( strcmp( name, "192" ) == 0 || strcmp( name, "237" ) == 0 ) // SXIE: special handling for firmware/cfg server uri
            {
                new_value[0] = 0;

                /* value changed? */
                if ( old_value == NULL || strlen( old_value ) != 0 ) // value changed
                {
                    field_changed = 1;
                }

                if ( field_changed && nvram_set_flag( name, new_value, WEB_CHANGE_FLAG ) == 0 )
                {
                    nvram_set_success = 1;
                }
            }
            else
            {
                /* value changed? */
                if (old_value != NULL)  /* key pair is previously defined */
                {
                    field_changed = 1;
                }
                if (nvram_unset_flag(name, WEB_CHANGE_FLAG) == 0)
                {
                    nvram_set_success = 1;
                }
            }
            /*
            if (nvram_set_success && field_changed)
                printf("%s value changed from %s to %s\n", name, old_value, strlen(str) > 0 ? new_value: "null" );
            */
            if ( nvram_set_success && field_changed )
            {
                check_and_set_reboot( name );
            }

            free( name );

            if ( old_value )
            {
                free( old_value );
                old_value = NULL;
            }
        }
    }

    if ( !nvram_set_success ) /* if set failed, then field is not changed */
    {
        field_changed = 0;
    }

    return field_changed;
}

static int check_restricted_values( char *val )
{
    int i =  0;
    const char* restricted_list[] =
    {
        "session_user",
        "session_id",
        "timestamp",
        "present_page"
    };
    int no_of_items = 4;

    for ( i = 0 ; i < no_of_items ; i++ )
    {
        if ( strncmp( restricted_list[i], val, strlen( restricted_list[i] ) ) == 0 ||
            strncmp( restricted_list[i], val+1, strlen( restricted_list[i] ) ) == 0 )
        {
            return 1;
        }
    }

    return 0;

}

static int check_and_save_field(char *line, int check_privilege, int htmlencode, int is_admin )
{
    char *tok[3];
    char *l, *p, *p2;
    int pos = 0;
    int field_changed = 0;

    tok[0] = tok[1] = tok[2] = NULL;
    l = line;

    while ( l != NULL )
    {
        tok[pos++] = l;

        if ( ( p = strstr( l, FORM_FIELD_START ) ) == NULL )
        {
            break;
        }

        p2 = strstr( p, FORM_FIELD_END );

        if ( p2 == NULL )
        {
            break;
        }

        *p = 0;
        *p2 = 0;
        tok[pos++] = p + strlen( FORM_FIELD_START );

        l = p2;
        l += strlen( FORM_FIELD_END );
        tok[pos++] = l;

        //check privilege level here
        //tok[1]
        if ( check_restricted_values( tok[1] ) )
        {
            char str[8];
            char *att_str = ( char * )nvram_get( "hacked_pvalues" );
            int attempts = 0;

            if ( att_str )
            {
                attempts = atoi( att_str ) + 1;
            }
            else
            {
                attempts = 1;
            }

            sprintf( str, "%d", attempts );
            nvram_set( ":hacked_pvalues", str );
            break;
        }
        if ( check_privilege && nvram_check_privilege( tok[1] ) && !is_admin )
        {
            char str[8];
            char *att_str = ( char * )nvram_get( "hacked_pvalues" );
            int attempts = 0;

            if ( att_str )
            {
                attempts = atoi( att_str ) + 1;
            }
            else
            {
                attempts = 1;
            }

            sprintf( str, "%d", attempts );
            nvram_set( ":hacked_pvalues", str );
            break;
        }
        if (form_field_save(tok[1], tok[2], htmlencode) == 1)
        {
            field_changed = 1;
        }
        break;
    }

    //res = p;
    //return res;
    return field_changed;
}

static int
load_setting(char *str, int set_default_only, int acl_verified)
{
    char *old_value = NULL;
    char *id, *new_value;

    if ( ( id = str ) == NULL )
    {
        return 0;
    }

    if ( set_default_only && str[0] == '#' ) // for nv_default, skip if started with #
    {
        return 0;
    }

    new_value = strstr( id, CONF_FIELD_END );

    if ( new_value == NULL ) // no "=", format not right
    {
        return 0;
    }

    *new_value = 0;
    new_value += strlen( CONF_FIELD_END );

    if ( !set_default_only ) //Avoid duplicate id in config file to cause loop reboot
    {
        if ( !compare_ids( id ) )
        {
            return 0;
        }
        /*
        char * tmpstr = malloc ( strlen( id ) + 3 );
        if ( tmpstr == NULL )
        {
            printf("Error allocating memory!\n");
        }
        sprintf( tmpstr, "_%s_", id );     //to distinguish P67 from P167

        if ( strstr( idbuf, tmpstr ) != NULL )  //seek idbuf to find id
        {
            printf("Duplicated id in config file! Skipp this entry\n");
            return 0;
        }
        else
        {
            //printf("This entry hasn't been checked\n");
            //insert entry into idbuf
            if ( strlen( id ) + strlen( idbuf ) + 3 > idbuf_size )  //if idbuf is not big enough
            {
                idbuf_size *= 2;
                char *temp = realloc( idbuf, idbuf_size );
                if ( temp != NULL )     //realloc was successful
                {
                    idbuf = temp;
                }
                else                    //there was an error
                {
                    printf("Error allocating memory!\n");
                }
            }
            strcat( idbuf, "_" );
            strcat( idbuf, id );
            strcat( idbuf, "_ " );
        }
        if ( tmpstr )
        {
            free( tmpstr );
        }
        */
    }

    if ( strlen( id ) )
    {
        if ( acl_verified )
        {
            old_value = nvram_l2get_safe( id, tmpbuf, MAX_VALLEN );
        }
        else
        {
            old_value = nvram_get_safe( id, tmpbuf, MAX_VALLEN );
        }
    }
    else
    {
        return 0;
    }

    if ( set_default_only && old_value != NULL )
    {
        return 0;
    }
    if ( new_value )
    {
        /* remove the '\n' if needed */
        int len = strlen( new_value );

        if ( len && new_value[len - 1] == '\n' )
        {
            new_value[len - 1] = 0;
        }
    }

    if ( old_value == NULL || // key pair not defined
            strcmp( old_value, new_value ) != 0 ) // value changed
    {
        int ret;

        if ( acl_verified )
        {
            if ( ( ret = nvram_l2set( id, new_value ) ) >= 0 )
            {
                if ( check_and_set_reboot( id ) )
                {
                    return CCA_REBOOT;
                }
                else
                {
                    return CCA_CHANGED;
                }
            }
            else
            {
                return 0;
            }
        }
        else
        {
            if ( ( ret = nvram_set_provision( id, new_value ) ) >= 0 )
            {
                if ( check_and_set_reboot( id ) )
                {
                    return CCA_REBOOT;
                }
                else
                {
                    return CCA_CHANGED;
                }
            }
            else
            {
                return 0;
            }
        }
    }

    return 0;
}

static int
nvlock( char *str, int lock_level )
{
    char *id;

    if ( ( id = str ) == NULL )
    {
        return 0;
    }

    if ( str[0] == '#' ) // skip if started with #
    {
        return 0;
    }

    if ( strlen( str ) == 0 ) // skip
    {
        return 0;
    }

    if ( lock_level == 1 )
    {
        nvram_l1lock( id );
    }
    else if ( lock_level == 2 )
    {
        nvram_l2lock( id );
    }
    else
    {
        printf( "Unknown lock level\n" );
    }

    return 0;
}

static char *
presave_setting( char *str_in, char * source_id )
{
    char *id, *target_value;
    char str[LINE_BUF];

    strcpy( str, str_in );

    if ( ( id = str ) == NULL )
    {
        return NULL;
    }

    target_value = strstr( id, CONF_FIELD_END );

    if ( target_value == NULL )
    {
        return NULL;
    }

    *target_value = 0;
    target_value += strlen( CONF_FIELD_END );

    if ( ( strlen( id ) != strlen( source_id ) ) || ( strcmp( id, source_id ) != 0 ) )
    {
        return NULL;
    }

    {
        if ( target_value )
        {
            /* remove the '\n' if needed */
            int len = strlen( target_value );

            if ( len && target_value[len - 1] == '\n' )
            {
                target_value[len - 1] = 0;
            }

            return strdup( target_value );
        }
    }

    return NULL;
}

static int
compare_setting( char *str_in, char * source_id, char *source_value )
{
    char *id, *target_value;
    char str[LINE_BUF];

    strcpy( str, str_in );

    if ( ( id = str ) == NULL )
    {
        return -1;
    }

    target_value = strstr( id, CONF_FIELD_END );

    if ( target_value == NULL || source_value == NULL )
    {
        return -1;
    }

    *target_value = 0;
    target_value += strlen( CONF_FIELD_END );

    if ( ( strlen( id ) != strlen( source_id ) ) || ( strcmp( id, source_id ) != 0 ) )
    {
        //printf("compare id [%s] [%s]\n", id, source_id);
        return -1;
    }

    {
        if ( target_value )
        {
            /* remove the '\n' if needed */
            int len = strlen( target_value );

            if ( len && target_value[len - 1] == '\n' )
            {
                target_value[len - 1] = 0;
            }

            //printf("compare value [%s] [%s]\n", target_value, source_value);
            if ( strcmp( target_value, source_value ) == 0 )
            {
                return 0;
            }
        }
    }

    return -1;
}

static int
is_cert( char *name, char *buf )
{
    char *begin = NULL;
    int type = check_datatype( name );

    if ( type == CERT )
    {
        if ( buf )
        {
            begin = strstr( buf + strlen(name), "-----BEGIN" );
        }
        if ( buf && !begin )
        {
            return 0;
        }
        return 1;
    }

    return 0;
}

static void
nvram_usage( void )
{
    fprintf( stderr, "usage: nvram [get name] [set name=value] [unset name] [show] [space]\n" );
    exit( 0 );
}

static void get_mac( char* mac, unsigned int length )
{
    size_t size = 0;
    char* p = NULL;
    FILE *fp = fopen( PROC_PATH_TO_MAC, "r" );

    if ( fp == NULL )
    {
        return;
    }

    getline( &p, &size, fp );

    if ( p != NULL )
    {
        unsigned int octet[6];
        int n = sscanf( p, "%x:%x:%x:%x:%x:%x", &octet[0], &octet[1],
                        &octet[2], &octet[3], &octet[4], &octet[5] );

        if ( n == 6 )
            snprintf( mac, length, "%02X%02X%02X%02X%02X%02X",
                      octet[0], octet[1], octet[2], octet[3], octet[4],
                      octet[5] );

        free( p );
    }

    fclose( fp );
}

/* hack for some PMON default nvram values which have '\r' appended */
static void
puts_trim_cr( char *str )
{
    int len = strlen( str );

    if ( len && ( str[len - 1] == '\r' ) )
    {
        len--;
    }

    printf( "%.*s\n", len, str );
}

static void
puts_trim_cr_with_p( char *str )
{
    int len = strlen( str );

    if ( len && ( str[len - 1] == '\r' ) )
    {
        len--;
    }

    printf( "P%.*s\n", len, str );
}

static void print_xml_config_file_header()
{
    char mac[18];

    memset( mac, 0, 18 );
    get_mac( mac, 18 );
    printf ( "%s\n%s\n%s\n%s%s%s\n%s\n", "<?xml version=\"1.0\" encoding=\"UTF-8\"?>",
            "<!-- GXW42XX XML Provisioning Configuration -->",
            "<gs_provision version=\"1\">",
            "<mac>", mac, "</mac>",
            "\t<config version=\"1\">" );

}

static void print_xml_config_file_trailer()
{
    printf( "\t</config>\n</gs_provision>\n" );
}

static void puts_trim_cr_xml( char* name, char* value, char* filler )
{
    if ( value )
    {
        printf( "%s<P%s>%s</P%s>\n", filler, name, value, name );
    }
    else
    {
        printf( "%s<P%s></P%s>\n", filler, name, name );
    }
}

bool match_applet( const char* applet, char** input, int applet_size, int input_size )
{
    int applet_len;
    if ( *input == NULL )
    {
        return false;
    }

    if ( input_size != applet_size )
    {
        return false;
    }

    applet_len = strlen( applet );
    if ( strlen(*input) != applet_len )
    {
        return false;
    }

    if ( strncmp(*input, applet, applet_len) != 0 )
    {
        return false;
    }

    return true;
}

void parseRequestString( char* request, char* json_value, char* sid, char* remoteIP )
{
    unsigned int pname_pvalue_length = 0;
    bool success = true;
    bool validSession;
    char * pch;
    char *pSave;

    if( request == NULL || request[0] == '\0' )
    {
        return;
    }

    int section_count = 1;
    int i;

    for( i = 0; request[i] != '\0'; i++ )
    {
        if ( request[i] == ':' )
        {
            section_count++;
        }
    }
    char* tokens[section_count];
    i = 0;
    pch = strtok_r ( request, ":", &pSave );

    char tmpbuf3[MAX_VALLEN];
    memset( tmpbuf3, 0, MAX_VALLEN * sizeof( char ) );

    //char tmpbuf4[MAX_VALLEN];

    const char* phoneSID = nvram_get_safe( "session_id", tmpbuf, MAX_VALLEN );
    const char* phoneIP = nvram_get_safe( "logged_in_addr", tmpbuf2, MAX_VALLEN );
    const char* timestamp = nvram_get_safe( "timestamp", tmpbuf3, MAX_VALLEN );
    long int timestampConvert = strtol( tmpbuf3, NULL, 10 );
    long int endSession = timestampConvert + 600;

    time_t now;
    time( &now );

    if ( phoneSID == NULL || phoneSID[0] == '\0' || sid == NULL
            || strcmp( phoneSID, sid ) != 0 || remoteIP == NULL
            || remoteIP[0] == '\0' || strcmp( phoneIP, remoteIP ) != 0
            || timestampConvert == 0 || now > endSession )
    {
        validSession = false;
    }
    else
    {
        validSession = true;
    }

    while ( pch != NULL )
    {
        //char value[STR_LEN];
        //getValue( pch, value );
        char* pname = NULL;
        if ( pch[0] == 'P' )
        {
            pname = pch + 1;
        }
        else
        {
            pname = pch;
        }
        char* real_pname = get_fieldname_by_id( pname );

        if ( pch != NULL && pch[0] != '\0' )
        {
            pname_pvalue_length += strlen( pch );
        }

        pname_pvalue_length += 6;  /*Four ", One :, One \0 */
        if ( pname_pvalue_length > MAX_VALLEN )
        {
            pname_pvalue_length = MAX_VALLEN;
        }

        long int attemptConvert = strtol( real_pname, NULL, 10 );
        //if ( attemptConvert != 0 && check_for_password( attemptConvert ) == 0 )
        if ( nvram_check_password( real_pname ) || nvram_match_datatype( real_pname, "tr069" ) )
        {
            tokens[i] = (char*)malloc( pname_pvalue_length* sizeof(char) );
            snprintf(tokens[i], pname_pvalue_length, "\"%s\":\"\"", pch );
            i++;
            pch = strtok_r ( NULL, ":", &pSave );
            continue;
            //password requested, skip
        }

        if ( validSession == false && attemptConvert != 0 && attemptConvert != 68 &&
             attemptConvert != 1395 && attemptConvert != 1561 && attemptConvert != 1683 &&
             attemptConvert != 28116 )
        {
            tokens[i] = (char*)malloc( pname_pvalue_length* sizeof( char ) );
            snprintf( tokens[i], pname_pvalue_length, "\"%s\":\"\"", pch );
            i++;
            pch = strtok_r ( NULL, ":", &pSave );
            continue;
            //requested a numbered pvalue that isnt 1395/1561/1683/68/28116 when no valid session exists
        }

        const char* pvalue = nvram_get_safe( real_pname, tmpbuf, MAX_VALLEN );
        int cnt_i = 0, cnt_j = 0;
        char escaped_pvalue[MAX_VALLEN] = { 0 };

        if ( pvalue != NULL && pvalue[0] != '\0')
        {
            while ( '\0' != pvalue[cnt_i] )
            {
                if ( 0x22 == pvalue[cnt_i] || 0x5C == pvalue[cnt_i] )   /*0x22 = *, 0x5C = \*/
                {
                    escaped_pvalue[cnt_j++] = '\\';
                }
                escaped_pvalue[cnt_j] = pvalue[cnt_i];
                cnt_j++;
                cnt_i++;
            }
            pname_pvalue_length += strlen( escaped_pvalue );
        }
        if ( pname_pvalue_length > MAX_VALLEN )
        {
            pname_pvalue_length = MAX_VALLEN;
        }

        tokens[i] = (char*)malloc( pname_pvalue_length * sizeof( char ) );
        if( pvalue != NULL && pvalue[0] != '\0')
        {
            snprintf( tokens[i], pname_pvalue_length, "\"%s\":\"%s\"", pch, escaped_pvalue );
        }
        else
        {
            snprintf( tokens[i], pname_pvalue_length, "\"%s\":\"\"", pch );
        }
        i++;
        pch = strtok_r ( NULL, ":", &pSave );
    }
    if ( success == true )
    {
        char* response_header = "{\"response\":\"success\", \"body\":{";
        strcpy( json_value, response_header );
        i = 0;
        for ( i = 0; i < section_count && tokens[i] != NULL; i++ )
        {
            strcat( json_value, tokens[i] );
            strcat( json_value, "," );
        }
        json_value[strlen( json_value  )- 1] = '}';
        strcat( json_value, "}" );
        //printf(" this is json format %s", json_value);
    }
    else
    {
        char* response_header = "{\"response\":\"error\", \"body\":{ \"message\" : \"wrong request\"}";
        strcpy( json_value, response_header );
    }

    output_line( json_value );
    for( i = 0; i < section_count; i++ )
    {
        if ( tokens[i] != NULL )
        {
            free( tokens[i] );
        }
    }
}

int main( int argc, char **argv )
{
    char* pname = argv[0];

    if ( strchr ( pname, '/' ) != NULL )
    {
        do
        {
            /* move behind next / */
            pname = strchr ( pname, '/' ) + 1;
        }
        while ( pname != NULL && strlen( pname ) > 0 && strchr ( pname, '/' ) != NULL );
    }
    if ( pname == NULL ) return -1;

    if ( strcmp( pname, "nvcfg_get" ) == 0 )
    {
        char* name;
        char* buf = nvram_alloc_ram_size_buffer();
        int value, i;
        int pvalue_whitelist [] = {8201, 8202, 8205, 8209, 8220, 8221};

        if ( buf == NULL )
        {
            return -1;
        }

        nvram_getall( buf, 0 );
        for ( name = buf; *name; name += strlen( name ) + 1 )
        {
            if ( isdigit( name[0] ) && strncmp( name, "279=", 4 ) != 0 &&
                    strncmp( name, "280=", 4 ) != 0 && strncmp( name, "110=", 4 ) != 0 &&
                    strncmp( name, "111=", 4 ) != 0 && strncmp( name, "112=", 4 ) != 0 &&
                    strncmp( name, "281=", 4 ) != 0 )
            {
                value = atoi( name );
                if ( value > 8229 || value < 8200 )
                {
                    puts_trim_cr_with_p( name );
                }
                else if ( value >= 8200 && value <= 8229 )
                {
                    for ( i = 0; i < sizeof( pvalue_whitelist ) / sizeof( int ); i++ )
                    {
                        if ( value == pvalue_whitelist[i] )
                        {
                            puts_trim_cr_with_p( name );
                        }
                    }
                }
            }
        }

        free( buf );
        return 0;
    }
    else if ( strncmp( pname, "nvparse", 8 ) == 0 || strncmp( pname, "nvdefault", 10 ) == 0)
    {
        /* command nvparse or nvdefault */
        FILE *f;
        int is_nvparse = 0;

        if ( argc != 2 )
        {
            return -1;
        }

        if ( strcmp( pname, "nvparse") == 0)
        {
            is_nvparse = 1;
            fprintf( stderr, "nvparse %s\n", argv[1] );
        }
        else
        {
            fprintf( stderr, "Loading nvdefault from %s\n", argv[1] );
        }

        if ( ( f = fopen( argv[1], "r" ) ) == NULL )
        {
            return -1;
        }

        while ( !feof( f ) && ( fgets( buf, LINE_BUF - 1, f ) ) != NULL )
        {
            /* SXIE: load_setting will remove the tail '\n' */
            if ( is_nvparse )
            {
                output_line( buf );
            }
            else
            {
                char * str_start_ptr = NULL;
                char tmp_pvalue_name[MAX_KEYLEN] = {0};
                char *pName = NULL;

                str_start_ptr = skip_space_and_comments( buf, LINE_BUF );
                if ( NULL == str_start_ptr )
                {
                    continue;
                }

                pName = get_pvalue_name( str_start_ptr, LINE_BUF - (str_start_ptr - buf), tmp_pvalue_name, MAX_KEYLEN );
                if ( NULL == pName )
                {
                    continue;
                }

                /* nvdefault */
                /* bug 120410, add support to load default for certificates such as P8220, P8472 */
                if ( is_cert( pName, buf ) )
                {
                    char tmp_buf[LINE_BUF];
                    int error = 0;
                    if ( get_cert( buf, LINE_BUF, tmp_buf, LINE_BUF, f, &error ) != NULL )
                    {
                        load_setting( tmp_buf, 1, 0 );
                    }
                    continue;
                }
                else
                {
                    load_setting( str_start_ptr, 1, 0 );
                }
            }
        }
        fclose( f );
    }
    else if ( strncmp( pname, "nvparse_conf", 13 ) == 0 || strncmp( pname, "genvpn_conf", 11 ) == 0 )
    {
        FILE *f;
        char *p;
        int heredoc = 1, vpn = 0, skipped = 0;

        if ( argc != 2 )
        {
            return -1;
        }

        if ( strncmp( pname, "genvpn_conf", 11 ) == 0 )
        {
            heredoc = 0;
            vpn = 1;
            skipped = 1;
        }

        if ( ( f = fopen( argv[1], "r" ) ) == NULL )
        {
            return -1;
        }

        while ( !feof( f ) && ( fgets( buf, LINE_BUF - 1, f ) ) != NULL )
        {
            if ( vpn )
            {
                p = skip_space_and_comments( buf, LINE_BUF );
            }
            else
            {
                p = buf;
            }
            if ( p )
            {
                output_line_conf( p, heredoc, skipped );
            }
        }

        fclose( f );
    }
    else if ( strncmp( pname, "nvlockl1", 9 ) == 0 || strncmp( pname, "nvlockl2", 9 ) == 0 )
    {
        FILE *f;

        if ( argc != 2 )
        {
            return -1;
        }

        if ( ( f = fopen( argv[1], "r" ) ) == NULL )
        {
            return -1;
        }

        while ( !feof( f ) && ( fgets( buf, LINE_BUF - 1, f ) ) != NULL )
        {
            /* remove the '\n' if there is one */
            int len = strlen( buf );

            if ( len && buf[len - 1] == '\n' )
            {
                buf[len - 1] = 0;
            }

            if ( strncmp( pname, "nvlockl1", 9 ) == 0 )
            {
                nvlock( buf, 1 );
            }
            else
            {
                nvlock( buf, 2 );
            }
        }

        fclose( f );
    }
    else if (strncmp( pname, "nvcfg_print_unique", 19 ) == 0 )
    {
        FILE *f;
        int ret = 0;

        if ( argc != 2 )
        {
            return -1;
        }

        if ( ( f = fopen( argv[1], "r" ) ) == NULL )
        {
            return -1;
        }

        initialize_idbuf();

        while ( !feof( f ) && ( fgets( buf, LINE_BUF - 1, f ) ) != NULL )
        {
            char * str_start_ptr = NULL;
            char tmp_pvalue_name[MAX_KEYLEN] = {0};
            char *pName = NULL;

            str_start_ptr = skip_space_and_comments( buf, LINE_BUF );
            if ( NULL == str_start_ptr )
            {
                continue;
            }

            pName = get_pvalue_name( str_start_ptr, LINE_BUF - (str_start_ptr - buf), tmp_pvalue_name, MAX_KEYLEN );
            if ( NULL == pName )
            {
                continue;
            }

            if ( is_cert( pName, buf ) )
            {
                char tmp_buf[LINE_BUF];
                int error = 0;
                if ( get_cert( buf, LINE_BUF, tmp_buf, LINE_BUF, f, &error ) != NULL )
                {
                    if ( print_unique( tmp_buf ) == 1 )
                    {
                        ret = 1;
                    }
                }
                continue;
            }
            if ( print_unique( buf ) == 1 )
            {
                ret = 1;
            }
        }

        free_idbuf();
        fclose( f );
        return ret;
    }
    else if ( strncmp( pname, "nvcfg_print_present", 20 ) == 0 )
    {
        FILE *f;
        int reboot_flag = 0;

        if ( argc != 2 && argc != 3 )
        {
            return -1;
        }

        if ( ( f = fopen( argv[1], "r" ) ) == NULL )
        {
            return -1;
        }

        if ( argc == 3 && !strncmp( "reboot_only", argv[2], 12 ) )
        {
            reboot_flag = 1;
        }

        initialize_idbuf();

        while ( !feof( f ) && ( fgets( buf, LINE_BUF - 1, f ) ) != NULL )
        {
            char * str_start_ptr = NULL;
            char tmp_pvalue_name[MAX_KEYLEN] = {0};
            char *pName = NULL;

            str_start_ptr = skip_space_and_comments( buf, LINE_BUF );
            if ( NULL == str_start_ptr )
            {
                continue;
            }

            pName = get_pvalue_name( str_start_ptr, LINE_BUF - (str_start_ptr - buf), tmp_pvalue_name, MAX_KEYLEN );
            if ( NULL == pName )
            {
                continue;
            }
            if ( is_cert( pName, buf ) )
            {
                char tmp_buf[LINE_BUF];
                int error = 0;
                if ( get_cert( buf, LINE_BUF, tmp_buf, LINE_BUF, f, &error ) != NULL )
                {
                    print_nvram_value( tmp_buf, reboot_flag );
                }
                continue;
            }
            print_nvram_value( buf, reboot_flag );
        }

        free_idbuf();
        fclose( f );
    }
    else if ( strncmp( pname, "nvcfg_compare", 14 ) == 0 )
    {
        FILE *f;
        int reboot_flag = 0;

        if ( argc != 2 && argc != 3 )
        {
            return -1;
        }

        if ( ( f = fopen( argv[1], "r" ) ) == NULL )
        {
            return -1;
        }

        if ( argc == 3 && !strncmp( "reboot_only", argv[2], 12 ) )
        {
            reboot_flag = 1;
        }

        initialize_idbuf();

        while ( !feof( f ) && ( fgets( buf, LINE_BUF - 1, f ) ) != NULL )
        {
            char * str_start_ptr = NULL;
            char tmp_pvalue_name[MAX_KEYLEN] = {0};
            char *pName = NULL;

            str_start_ptr = skip_space_and_comments( buf, LINE_BUF );
            if ( NULL == str_start_ptr )
            {
                continue;
            }

            pName = get_pvalue_name( str_start_ptr, LINE_BUF - (str_start_ptr - buf), tmp_pvalue_name, MAX_KEYLEN );
            if ( NULL == pName )
            {
                continue;
            }
            if ( is_cert( pName, buf ) )
            {
                char tmp_buf[LINE_BUF];
                int error = 0;
                if ( get_cert( buf, LINE_BUF, tmp_buf, LINE_BUF, f, &error ) != NULL )
                {
                    if ( compare_nvram_value( tmp_buf, reboot_flag ) > 0 )
                    {
                        free_idbuf();
                        fclose( f );
                        fprintf( stdout, "YES" );
                        return 0;
                    }
                }
                continue;
            }
            if ( compare_nvram_value( buf, reboot_flag ) > 0 )
            {
                free_idbuf();
                fclose( f );
                fprintf( stdout, "YES" );
                return 0;
            }
        }

        free_idbuf();
        fclose( f );
    }
    else if ( strncmp( pname, "nvload", 7 ) == 0 )
    {
        FILE *f;
        char *p1 = NULL;
        char *p9998 = NULL;
        char *p9999 = NULL;
        char *config_p9998 = NULL;
        int line_count = 0;
        int config_changed = 0;
        int p1matched = 1;
        int needaclcheck = 0;
        int aclverified = 0;

        if ( argc != 2 && argc != 3 )
        {
            return -1;
        }

        if ( argc == 3 )
        {
            // need verify P1 for config authenticate
            p1 = argv[2];
        }

        p9998 = nvram_l2get_safe( "9998", tmpbuf, MAX_VALLEN );

        if ( p9998 )
        {
            p9998 = strdup( p9998 );
        }

        p9999 = nvram_l2get_safe( "9999", tmpbuf, MAX_VALLEN );

        if ( p9999 )
        {
            p9999 = strdup( p9999 );
        }

        if ( p9999 != NULL && p9998 != NULL )
        {
            if ( p9999[0] == '1' && strlen( p9998 ) > 0 )
            {
                needaclcheck = 1;
            }
        }

        if ( ( f = fopen( argv[1], "r" ) ) == NULL )
        {
            return -1;
        }

        /* pre-scan process */
        if ( p1 != NULL && strlen( p1 ) > 0 )
        {
            p1matched = 0;
        }

        //if ( p1matched == 0 || needaclcheck )
        {
            int scan_p1 = 1;
            int scan_acl = 1;

            while ( !feof( f ) && ( fgets( buf, LINE_BUF - 1, f ) ) != NULL )
            {
                if ( config_p9998 == NULL )
                {
                    config_p9998 = presave_setting( buf, "9998" );
                }

                /* SXIE: compare_setting will remove the tail '\n' */
                // check P1 value
                if ( scan_p1 && compare_setting( buf, "1", p1 ) == 0 ) //SXIE: per the provision scripts, p1 actually is p2
                {
                    p1matched = 1;
                    scan_p1 = 0;
                    //if (!needaclcheck)
                    //    break;
                }

                if ( needaclcheck && scan_acl )
                {
                    if ( compare_setting( buf, "9997", p9998 ) == 0 )
                    {
                        //printf("acl check passed\n");
                        aclverified = 1;
                        scan_acl = 0;
                    }
                }
            }

            fseek( f, 0L, SEEK_SET ); /* rewind the file position */
        }

        if ( p1matched == 1 )
        {
            char * tmp_p9999 = NULL;

            initialize_idbuf();

            while ( !feof( f ) && ( fgets( buf, LINE_BUF - 1, f ) ) != NULL )
            {
                char * str_start_ptr = NULL;

                /* SXIE: load_setting will remove the tail '\n' */
                line_count++;

                str_start_ptr = skip_space_and_comments( buf, LINE_BUF );
                if ( NULL == str_start_ptr )
                {
                    continue;
                }

                if ( tmp_p9999 != NULL )
                {
                    free( tmp_p9999 );
                    tmp_p9999 = NULL;
                }
                tmp_p9999 = presave_setting( str_start_ptr, "9999" );

                /* set P9999 to non 0 must have a good p9998 */
                if ( tmp_p9999 == NULL || compare_setting( str_start_ptr, "9999", "0" ) == 0 ||  // not setting 9999 or set 9999 to 0
                   ( config_p9998 && strlen( config_p9998 ) > 0 ) )  // or set 9999 and has a valid 9998
                {
                    char tmp_pvalue_name[MAX_KEYLEN] = {0};
                    char *pName = NULL;

                    pName = get_pvalue_name( str_start_ptr, LINE_BUF - (str_start_ptr - buf), tmp_pvalue_name, MAX_KEYLEN );
                    if ( NULL == pName )
                    {
                        continue;
                    }

                    if ( is_cert( pName, buf ) )
                    {
                        char tmp_buf[LINE_BUF];
                        int error = 0;
                        if ( get_cert( buf, LINE_BUF, tmp_buf, LINE_BUF, f, &error ) != NULL )
                        {
                            config_changed |= load_setting( tmp_buf, 0, aclverified );
                        }
                        else
                        {
                            if ( error )
                            {
                                break;
                            }
                        }
                        continue;
                    }

                    config_changed |= load_setting( str_start_ptr, 0, aclverified );
                }
            }
            if ( tmp_p9999 != NULL )
            {
                free( tmp_p9999 );
                tmp_p9999 = NULL;
            }

            free_idbuf();
        }

        if ( config_changed )
        {
            nvram_commit();
            if ( config_changed & CCA_REBOOT )
            {
                nvram_set(":config_changed", "2");
            }
            else
            {
                nvram_set(":config_changed", "1");
            }
        }

        fclose( f );

        if ( config_p9998 )
        {
            free( config_p9998 );
        }

        if ( p9998 )
        {
            free( p9998 );
        }

        if ( p9999 )
        {
            free( p9999 );
        }
    }
    else if ( strncmp( pname, "nvconfig", 9 ) == 0 )
    {
        char **ep;
        extern char **environ;
        int web_setting_changed = 0;
        int check_privilege = 1;
        int htmlencode = 0;
        char szuser[MAX_VALLEN] = {0};
        int is_admin = 0;

        /* use safe API, or we may be unable to save page */
        nvram_get_safe("session_user", szuser, MAX_VALLEN - 1);
        if ( strncmp( szuser, "admin", 6 ) == 0 )
        {
            is_admin = 1;
        }
        for ( ep = environ; *ep; ep++ )
        {
            strncpy( buf, *ep, LINE_BUF - 1 );
            buf[LINE_BUF - 1] = 0;
            if ( check_and_save_field(buf, check_privilege, htmlencode, is_admin ) != 0 )
            {
                web_setting_changed = 1;
            }
        }

        if ( web_setting_changed )
        {
            if ( argc != 2 || strncmp( argv[1], "nocommit", 9 ) )
            {
                nvram_commit();
            }

            return 0;
        }

        return 1;
    }
    else if ( strncmp( pname, "nvram", 6 ) == 0 )
    {
        char *name, *value = NULL;

        /* Skip program name */
        --argc;
        ++argv;

        if ( !*argv )
        {
            nvram_usage();
        }

        if ( match_applet( "get", argv, 2, argc ) )
        {
            if ( *++argv )
            {
                if ( ( value = nvram_get_safe( *argv, tmpbuf, MAX_VALLEN ) ) )
                {
                    puts_trim_cr( value );
                }
            }
        }
        else if ( match_applet( "getunsafe", argv, 2, argc ) )
        {
            if ( *++argv )
            {
                if ( ( value = ( char * )nvram_get( *argv ) ) )
                {
                    puts_trim_cr( value );
                }
            }
        }
        else if ( match_applet( "l2get", argv, 2, argc ) )
        {
            if ( *++argv )
            {
                if ( ( value = nvram_l2get_safe( *argv, tmpbuf, MAX_VALLEN ) ) )
                {
                    puts_trim_cr( value );
                }
            }
        }
        else if ( match_applet( "set", argv, 2, argc ) )
        {
            if ( *++argv )
            {
                strncpy( value = tmpbuf, *argv, sizeof(buf) );
                name = strsep( &value, CONF_FIELD_END );
                return nvram_set( name, value );
            }
        }
        else if ( match_applet( "setwithflag", argv, 2, argc ) ||
                match_applet( "setwithflag", argv, 3, argc ) )
        {
            if ( *++argv )
            {
                unsigned int flag = 0;
                strncpy( value = tmpbuf, *argv, MAX_VALLEN );
                if ( *++argv )
                {
                    flag = atoi( *argv );
                }
                name = strsep( &value, CONF_FIELD_END );
                nvram_set_flag( name, value, flag );
            }
        }
        else if ( match_applet( "unvalidatedset", argv, 2, argc ) )
        {
            if ( *++argv )
            {
                strncpy( value = tmpbuf, *argv, MAX_VALLEN );
                name = strsep( &value, CONF_FIELD_END );
                nvram_set_unvalidated( name, value );
            }
        }
        else if ( match_applet( "setgroups", argv, 3, argc ) )
        {
            if ( argc >= 3 )
            {
                nvram_set_notification_group_bitmask( argv[1], argv[2] );
            }
        }
        else if ( match_applet( "cleargroups", argv, 3, argc ) )
        {
            if ( argc >= 3 )
            {
                nvram_clear_notification_group_bitmask( argv[1], argv[2] );
            }
        }
        else if ( match_applet( "getgroups", argv, 2, argc ) )
        {
            if ( *++argv )
            {
                unsigned int groups;

                get_pvalue_notification_group_mask( *argv, &groups );
                printf( "0x%x\n", groups );
            }
        }
        else if ( match_applet( "addgroup", argv, 3, argc ) )
        {
            if ( argc >= 3 )
            {
                nvram_add_notification_group( argv[1], argv[2] );
            }
        }
        else if ( match_applet( "removegroup", argv, 3, argc ) )
        {
            if ( argc >= 3 )
            {
                nvram_remove_notification_group( argv[1], argv[2] );
            }
        }
        else if ( match_applet( "hasgroup", argv, 3, argc ) )
        {
            if ( argc >= 3 )
            {
                if ( nvram_has_notification_group_str( argv[1], argv[2] ) )
                {
                    printf( "true\n" );
                }
                else
                {
                    printf( "false\n" );
                }
            }
        }
        else if ( match_applet( "setnotify", argv, 2, argc ) )
        {
            if ( *++argv )
            {
                fprintf( stderr, "Warning, setnotify will be obsoleted in later days, " \
                    "addgroup and setgroups are recommended commands\n" );
                nvram_set_notification( *argv );
            }
        }
        else if ( match_applet( "l1set", argv, 2, argc ) )
        {
            if ( *++argv )
            {
                strncpy( value = tmpbuf, *argv, MAX_VALLEN );
                name = strsep( &value, CONF_FIELD_END );
                nvram_l1set( name, value );
            }
        }
        else if ( match_applet( "l2set", argv, 2, argc ) )
        {
            if ( *++argv )
            {
                strncpy( value = tmpbuf, *argv, MAX_VALLEN );
                name = strsep( &value, CONF_FIELD_END );
                nvram_l2set( name, value );
            }
        }
        else if ( match_applet("l1lock", argv, 2, argc ) )
        {
            if ( *++argv )
            {
                nvram_l1lock( *argv );
            }
        }
        else if ( match_applet( "l2lock", argv, 2, argc ) )
        {
            if ( *++argv )
            {
                nvram_l2lock( *argv );
            }
        }
        else if (match_applet("unset", argv, 2, argc))
        {
            if ( *++argv )
            {
                nvram_unset( *argv );
            }
        }
        else if ( match_applet( "unsetnotify", argv, 2, argc ) )
        {
            if ( *++argv )
            {
                fprintf( stderr, "Warning, unsetnotify will be obsoleted in later days, " \
                    "removegroup and cleargroups are recommended commands\n" );
                nvram_unset_notification( *argv );
            }
        }
        else if ( match_applet( "checknotify", argv, 2, argc ) )
        {
            if ( *++argv )
            {
                fprintf( stderr, "Warning, checknotify will be obsoleted in later days, " \
                    "hasgroup and getgroups are recommended commands\n" );
                printf( "%d\n", nvram_check_notification(*argv ) );
            }
        }
        else if ( match_applet( "default", argv, 2, argc ) )
        {
            if ( *++argv )
            {
                char buffer[LINE_BUF] = { 0 };
                int ret_val = 0;
                ret_val = nvram_get_default( *argv, buffer, LINE_BUF );

                if ( !ret_val )
                {
                    if ( strlen( buffer ) )
                    {
                        printf( "%s\n", buffer );
                    }
                }
                else
                {
                    printf( "Error\n" );
                }
            }
        }
        else if ( match_applet( "alias", argv, 2, argc ) )
        {
            if ( *++argv )
            {
                char buffer[LINE_BUF] = { 0 };
                int ret_val = 0;
                ret_val = nvram_get_alias( *argv, buffer, LINE_BUF );

                if ( !ret_val )
                {
                    if ( strlen( buffer ) )
                    {
                        printf( "%s\n", buffer );
                    }
                }
                else
                {
                    printf( "Error\n" );
                }
            }
        }
        else if ( match_applet( "lock", argv, 2, argc ) )
        {
            if ( *++argv )
            {
                int ret_val = 0;
                ret_val = nvram_check_lock( *argv );

                if ( ret_val )
                {
                    printf( "Locked\n" );
                }
                else
                {
                    printf( "Unlocked\n" );
                }
            }
        }
        else if ( match_applet( "help", argv, 2, argc ) )
        {
            if ( *++argv )
            {
                char buffer[LINE_BUF] = { 0 };
                int ret_val = 0;
                ret_val = nvram_get_help( *argv, buffer, LINE_BUF );

                if ( !ret_val )
                {
                    if ( strlen( buffer ) )
                    {
                        printf( "%s\n", buffer );
                    }
                }
                else
                {
                    printf( "Error\n" );
                }
            }
        }
        else if ( match_applet( "scripts", argv, 2, argc ) )
        {
            if ( *++argv )
            {
                char buffer[LINE_BUF] = { 0 };
                int ret_val = 0;
                ret_val = nvram_get_scripts( *argv, buffer, LINE_BUF );

                if ( !ret_val )
                {
                    if ( strlen( buffer ) )
                    {
                        printf( "%s\n", buffer );
                    }
                }
                else
                {
                    printf( "Error\n" );
                }
            }
        }
        else if ( match_applet( "attributes", argv, 2, argc ) )
        {
            if ( *++argv )
            {
                char buffer[LINE_BUF] = { 0 };
                int ret_val = 0;
                ret_val = nvram_get_limits( *argv, buffer, LINE_BUF );

                if ( !ret_val )
                {
                    if ( strlen( buffer ) )
                    {
                        printf( "%s\n", buffer );
                    }
                }
                else
                {
                    printf( "Error\n" );
                }
            }
        }
        else if ( match_applet( "commit", argv, 1, argc ) )
        {
            nvram_commit();
        }
        else if ( match_applet( "commit_sync", argv, 1, argc ) )
        {
            nvram_commit_sync();
        }
        else if ( match_applet( "show", argv, 1, argc ) || match_applet( "getall", argv, 1, argc ) )
        {
            char *nvram_buf = nvram_alloc_ram_size_buffer();

            if ( nvram_buf )
            {
                nvram_getall( nvram_buf, 0 );

                for ( name = nvram_buf; *name; name += strlen( name ) + 1 )
                {
                    puts_trim_cr( name );
                }

                free( nvram_buf );
            }
            else
            {
                printf( "error in allocate memory\n" );
            }

            //size = sizeof(struct nvram_header) + (int) name - (int) buf;
            //fprintf(stderr, "size: %d bytes (%d left)\n", size, NVRAM_SPACE - size);
        }
        else if ( match_applet( "download", argv, 1, argc ) // nvram download
            || match_applet( "download", argv, 2, argc ) // nvram download xml
            || match_applet( "export", argv, 1, argc ) ) // nvram export
        {
            char *nvram_buf = nvram_alloc_ram_size_buffer();
            char *equals = NULL;
            int xml_download = 0;
            int download_all = 0;   // bug94446/94447. indicate if passwords are included
            if ( match_applet( "export", argv, 1, argc ) )
            {
                download_all = 1;   // bug94446/94447. including passwords
            }

            if ( *++argv )
            {
                if ( !strcmp( *argv, "xml" ) )
                {
                    xml_download = 1;
                }
            }
            else if ( download_all )
            {
                xml_download = 1;   // bug94446/94447. download to xml file
            }

            if ( nvram_buf )
            {
                if ( xml_download )
                {
                    print_xml_config_file_header();
                }

                nvram_getall( nvram_buf, 0 );

                for ( name = nvram_buf; *name; name += strlen( name ) + 1 )
                {
                    if ( isdigit( name[0] ) )
                    {
                        equals = strchr( name, CONF_FIELD_END_CH );

                        if ( equals == NULL )
                        {
                            continue;
                        }

                        *equals = 0;
                        if ( ( download_all || ( !nvram_check_password( name ) ) ) // bug94446. download including passwords
                            )
                        {
                            if ( xml_download )
                            {
                                char new_value[MAX_VALLEN];
                                value = equals + sizeof( char );
                                html_special_chars(value, new_value, MAX_VALLEN - 1);
                                puts_trim_cr_xml( name, new_value, "\t\t" );
                                *equals = CONF_FIELD_END_CH;
                                equals = NULL;
                                continue;
                            }

                            *equals = CONF_FIELD_END_CH;
                            puts_trim_cr_with_p( name );
                        }
                        else
                        {
                            *equals = CONF_FIELD_END_CH;
                        }

                        equals = NULL;
                    }
                }

                free( nvram_buf );

                if ( xml_download )
                {
                    print_xml_config_file_trailer();
                }
            }
            else
            {
                printf( "error in allocate memory\n" );
            }
        }
        else if ( match_applet( "space", argv, 1, argc ) )
        {
            int space_used = nvram_space_used();
            printf( "NVRAM space used: %d%%\n", space_used );
        }
        else if ( match_applet( "eraseall", argv, 1, argc ) )
        {
            nvram_erase_all();
        }
        else if ( match_applet( "reset", argv, 1, argc ) )
        {
            if ( *++argv )
            {
                char *e;
                long int n = strtol( *argv, &e, 10 );

                if ( *e == '\0' )
                {
                    if ( n > 0 )
                    {
                        char *nvram_buf = nvram_alloc_ram_size_buffer();

                        if ( nvram_buf )
                        {
                            nvram_getall( nvram_buf, 0 );
                            nvram_reset_level( nvram_buf, n - 1, 0 );
                            free( nvram_buf );
                        }
                        else
                        {
                            printf( "error in allocate memory\n" );
                        }
                    }
                }
            }
        }
        else if ( match_applet( "checkdatatype", argv, 2, argc ) )
        {
            if ( *++argv )
            {
                printf( "%d\n", nvram_check_datatype( *argv ) );
            }
        }
        else if ( match_applet( "reset_list", argv, 3, argc ) )
        {
            char *mode;

            if ( argc != 3 )
            {
                return -1;
            }

            mode = *++argv;

            if ( !strncmp( mode, "1", 1 ) )
            {
                return nvram_erase_list( *++argv );
            }
            else if ( !strncmp( mode, "2", 1 ) )
            {
                return nvram_erase_not_list( *++argv );
            }
        }
        else
        {
            return -1;
        }
    }
    else if ( strncmp( pname, "msg_server", 11 ) == 0 )
    {
        msg_server();
    }
    else if ( strncmp( pname, "msg_monitor", 12 ) == 0 )
    {
        msg_monitor();
    }
    else if ( strncmp( pname, "msg_monitor_cc", 15 ) == 0 )
    {
        if ( argc >= 3 )
        {
            msg_monitor_cc( argv[1], argv[2] );
        }
        else if ( argc == 2 )
        {
            msg_monitor_cc( argv[1], NULL );
        }
        else
        {
            msg_monitor_cc( NULL, NULL );
        }
    }
    else if (strncmp( pname, "scripts_signal", 15 ) == 0)
    {
        if ( argc >= 3 )
        {
            scripts_signal( argv[1], argv[2] );
        }
        else if ( argc == 2 )
        {
            scripts_signal( argv[1], NULL );
        }
        else
        {
            scripts_signal( NULL, NULL );
        }
    }
    else if ( strncmp( pname, "tftp_url_process", 17 ) == 0 )
    {
        if ( argc != 2 )
        {
            return -1;
        }

        return tftp_url_process( argv[1] );
    }
    else if ( strncmp( pname, "http_url_process", 17 ) == 0 )
    {
        if ( argc == 3 && ( strncmp( argv[1], "-s", 2 ) == 0 ) )
        {
            return http_url_process( argv[2], "-s" );
        }

        if ( argc != 2 )
        {
            return -1;
        }

        return http_url_process( argv[1], NULL );
    }
    else if ( strncmp( pname, "daemonize", 10 ) == 0 )
    {
        if ( argc == 2 && ( !strncmp( argv[1], "-h", 3 ) || !strncmp( argv[1], "--help", 7 ) ) )
        {
            printf( "Usage: daemonize <name of program> <working dir> [arguments....]\n" );
            return 0;
        }

        if ( argc < 3 )
        {
            return -1;
        }

        return daemonize( argv + 1 );
    }
    else if ( strncmp( pname, "send_telemetry_message", 23 ) == 0 )
    {
        if ( argc < 3 )
        {
            return -1;
        }
        else
        {
            send_telemetry_message( argc - 2, argv + 2, atoi( argv[1] ) );
        }
    }
    else if ( strcmp( pname, "nvparser" ) == 0 )
    {
        if( argc <=1 )
        {
            printf("Format error\n");
            return 0;
        }
        char json_value[MAX_STR_LEN];
        memset( json_value, 0, MAX_STR_LEN * sizeof( char ) );
        parseRequestString( argv[1], json_value, argv[2], argv[3] );
    }
    else
    {
        printf( "Applet unknown %s\n", argv[0] );
    }

    return 0;
}

/*checks if the given string has hexadecimal characters only */
int is_hex( char *input_str )
{
	int i = 0 ;
	/*if string is NULL then it is not numeric*/
	if( input_str == NULL )
	    return -1 ;
	while( input_str[i] != '\0' )
	{
		/* if value is between 0 and 9 ( ascii ) continue */
		if( ( ( input_str[i] >= '0' ) &&
		      ( input_str[i] <= '9' ) ) ||
			( ( input_str[i] >= 'a' ) &&
			  ( input_str[i] <= 'f' ) ) ||
			( ( input_str[i] >= 'A' ) &&
			  ( input_str[i] <= 'F' ) )
		  )
		{
			i++ ;
			continue ;
		}
		return -1 ; /*return not numeric */
	}
	return 0 ; /*return numeric */
}

