#ifndef __NVRAM_LINUX_HEAD__
#define __NVRAM_LINUX_HEAD__

/*
 * NVRAM variable manipulation (Linux user mode half)
 *
 * Copyright 2004, Broadcom Corporation
 * All Rights Reserved.
 *
 * THIS SOFTWARE IS OFFERED "AS IS", AND BROADCOM GRANTS NO WARRANTIES OF ANY
 * KIND, EXPRESS OR IMPLIED, BY STATUTE, COMMUNICATION OR OTHERWISE. BROADCOM
 * SPECIFICALLY DISCLAIMS ANY IMPLIED WARRANTIES OF MERCHANTABILITY, FITNESS
 * FOR A SPECIFIC PURPOSE OR NONINFRINGEMENT CONCERNING THIS SOFTWARE.
 *
 * $Id: nvram_linux.c,v 1.15 2013/12/13 18:05:02 bmathew Exp $
 */

int nvram_init( void *unused );
char * nvram_get_safe( const char *name, char * buffer, const unsigned int length );
char * nvram_get( const char *name );
char * nvram_l2get_safe( const char *name, char * buffer, unsigned int length );
char * nvram_l2get( const char *name );
int nvram_getall( char *buf, int count );
int nvram_load_safe( const char *name, void * data_buf, const unsigned int buf_size );
int nvram_set( const char *name, const char *value );
int nvram_set_unvalidated( const char *name, const char *value );
int nvram_set_flag( const char *name, const char *value, unsigned int flag );
int nvram_unset( const char *name );
int nvram_unset_flag( const char *name, unsigned int flag );
int nvram_settemp( const char *name, const char *value );
int nvram_unsettemp( const char *name );
int nvram_l1lock( const char *name );
int nvram_l1set( const char *name, const char *value );
int nvram_l2lock( const char *name );
int nvram_l2set( const char *name, const char *value );
int nvram_commit( void );
int nvram_commit_sync( void );
int nvram_space_used( void );
int nvram_get_ram_size( void );
char * nvram_alloc_ram_size_buffer( void );
int nvram_ram_used( void );
int nvram_erase_all( void );
int file2nvram( char *filename, char *varname );
int nvram2file( char *varname, char *filename );
int file_to_buf( char *path, char *buf, int len );

/*
*Erase a list of pvalues leaving the rest untouched.
*/
int nvram_erase_list( const char *filename );

/*
*Erase pvalues not on the list leaving the list untouched.
*/
int nvram_erase_not_list( const char *filename );

/*
 * reset nvram values at a certain privilege level or
 * at and below a certain privilege level
 */
void nvram_reset_level( char *buf, int privilege_level, int mode );

/*
Add pvalue change notification groups
*/
int nvram_set_notification_group_bitmask( const char* name, const char* group_mask );

/*
Remove pvalue change notification groups
*/
int nvram_clear_notification_group_bitmask( const char* name, const char* group_mask );

/*
Set pvalue change privilege flag
*/
int nvram_set_privilege( const char* name, int val );

/*
Check pvalue check privilege flag
*/
int nvram_check_privilege( const char* name );

/*
Check if pvalue is a password
*/
int nvram_check_password( const char* name );

/*
Check if pvalue matches datatype
*/
int nvram_match_datatype( const char* name, const char* type );

/*
Get the datatype of pvalue
*/
int nvram_check_datatype( const char* name );

/*
Get proper name of pvalue
*/
int nvram_get_name( const char* name, char *buffer, unsigned int size );

/*
Get default of pvalue
*/
int nvram_get_default( const char* name, char *buffer, unsigned int size );

/*
Get alias of pvalue
*/
int nvram_get_alias( const char* name, char *buffer, unsigned int size );

/*
Get help for pvalue
*/
int nvram_get_help( const char* name, char *buffer, unsigned int size );

/*
Get scripts of pvalue
*/
int nvram_get_scripts( const char* name, char *buffer, unsigned int size );

/*
Get limits of pvalue
*/
int nvram_get_limits( const char* name, char *buffer, unsigned int size );

/*
Add groups to pvalue notification
*/
int nvram_add_notification_group( const char* name, const char* group );

/*
Remove groups from pvalue notification
*/
int nvram_remove_notification_group( const char* name, const char* group );

/*
Determine if a pvalue has a notificatiion group
*/
int nvram_has_notification_group_str( const char* name, const char * group );

int nvram_set_notification( const char* name );
int nvram_unset_notification( const char* name );
int nvram_check_notification( const char* name );

#endif
