/*
Takes in a string and parses out path, server address and port number
*/

#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include "tftp_url_process.h"

#define INVALID_ADDRESS -1
#define INVALID_PORT -2
#define INVALID_PATH -3
#define NO_ARGUMENTS 1
#define BUFFER_LENGTH 256
#define PORT_BUFFER_LENGTH 6
#define MAX_PORT_NUMBER 65535
#define TERMINAL_CHAR '\0'
#define SUCCESS 0

#if 0
main( int argc, char **argv )
{
    if ( argc > 1 )
    {
        //printf("%s\n", argv[1]);
        tftp_url_process( argv[1] );
    }
}
#endif

/*prints out the output string in the format 'http://server_address:port/path' or
'https://server_address:port/path'
return 0 if successfull
returns INVALID_ADDRESS if invalid address
returns INVALID_PORT if invalid port
*/
int http_url_process( char *input_str, char *type )
{
    char server_addr[BUFFER_LENGTH], port[PORT_BUFFER_LENGTH],
    path[BUFFER_LENGTH];
    int return_value;
    int j = 0;

    /* get the server address */
    return_value = get_server_address( input_str, server_addr );

    /* checking if the server address is valid */
    if ( return_value == INVALID_ADDRESS )
    {
        return return_value;
    }

    /* get the port if it exists and is valid */
    return_value = get_port( input_str, port, return_value );

    if ( return_value == INVALID_PORT )
    {
        return return_value;
    }

    /* get the path*/
    return_value = get_path( input_str, path, return_value );

    /*check if path is valid*/
    if ( return_value == INVALID_PATH )
    {
        return return_value;
    }

    /* skip the first "/" as according to test team some old tftp server like Dlink one can not handle the leading "/" */
    if ( path[0] != TERMINAL_CHAR && path[0] == '/' )
    {
#if 0
        /* remove leading / */
        j++;
#else
        /* remove leading / if only filename is there */
        int ii = j = 1;

        while ( path[ii] != TERMINAL_CHAR )
        {
            if ( path[ii] == '/' )
            {
                j = 0;
                break;
            }

            ii++;
        }

#endif
    }

    if ( !type )
    {
        printf( "http://" );
    }
    else
    {
        printf( "https://" );
    }

    /*printing server address */
    j = 0;

    while ( server_addr[j] != TERMINAL_CHAR )
    {
        printf( "%c", server_addr[j] );
        j++;
    }

    if ( port[0] != TERMINAL_CHAR )
    {
        printf( ":" );
    }

    /*printing port */
    j = 0;

    while ( port[j] != TERMINAL_CHAR )
    {
        printf( "%c", port[j] );
        j++;
    }

    /*printing path and filename */
    j = 0;

    while ( path[j] != TERMINAL_CHAR )
    {
        printf( "%c", path[j] );
        j++;
    }

    return SUCCESS;
}

/*prints out the output string in the format 'path server_address port'
return 0 if successfull
returns INVALID_ADDRESS if invalid address
returns INVALID_PORT if invalid port
*/
int tftp_url_process( char *input_str )
{
    char server_addr[BUFFER_LENGTH], port[PORT_BUFFER_LENGTH],
         path[BUFFER_LENGTH];
    int return_value;
    int j = 0;

    /* get the server address */
    return_value = get_server_address( input_str, server_addr );

    /* checking if the server address is valid */
    if ( return_value == INVALID_ADDRESS )
    {
        return return_value;
    }

    /* get the port if it exists and is valid */
    return_value = get_port( input_str, port, return_value );

    if ( return_value == INVALID_PORT )
    {
        return return_value;
    }

    /* get the path*/
    return_value = get_path( input_str, path, return_value );

    /*check if path is valid*/
    if ( return_value == INVALID_PATH )
    {
        return return_value;
    }

    /* skip the first "/" as according to test team some old tftp server like Dlink one can not handle the leading "/" */
    if ( path[0] != TERMINAL_CHAR && path[0] == '/' )
    {
#if 0
        /* remove leading / */
        j++;
#else
        /* remove leading / if only filename is there */
        int ii = j = 1;

        while ( path[ii] != TERMINAL_CHAR )
        {
            if ( path[ii] == '/' )
            {
                j = 0;
                break;
            }

            ii++;
        }

#endif
    }

    /*printing path and filename */
    while ( path[j] != TERMINAL_CHAR )
    {
        printf( "%c", path[j] );
        j++;
    }

    printf( " " );

    /*printing server address */
    j = 0;

    while ( server_addr[j] != TERMINAL_CHAR )
    {
        printf( "%c", server_addr[j] );
        j++;
    }

    printf( " " );

    /*if no port given exit*/
    if ( port[0] == TERMINAL_CHAR )
    {
        return SUCCESS;
    }

    /*printing port */
    j = 0;

    while ( port[j] != TERMINAL_CHAR )
    {
        printf( "%c", port[j] );
        j++;
    }

    return SUCCESS;
}

/*
extracts server address into server_addr
returns INVALID_ADDRESS if invalid address detected
returns position of end of server_address in input_str
*/
int get_server_address( char *input_str, char *server_addr )
{
    int cnt = 0, j = 0;
    int slash_flg = 0;

    /*while input_str does not end and count is less than buffer length*/
    while ( ( input_str[j] != TERMINAL_CHAR ) &&
            ( cnt < ( BUFFER_LENGTH - 1 ) )
          )
    {
        /*if input_str has ended break loop*/
        if ( input_str[j] == TERMINAL_CHAR )
        {
            break;
        }

        /*if there are leading slashes ignore them and continue*/
        if ( ( input_str[j] == '/' ) && !slash_flg )
        {
            j++;
            continue;
        }
        /*if a colon is found server address has ended break loop*/
        else if ( input_str[j] == ':' )
        {
            break;
        }
        /*if a slash is found server address has ended break loop*/
        else if ( ( input_str[j] == '/' ) && slash_flg )
        {
            break;
        }

        slash_flg++;
        server_addr[cnt] = input_str[j];
        cnt++;
        j++;
    }

    /*set terminating null character */
    server_addr[cnt] = TERMINAL_CHAR;

    /*if the address is a invalid address*/
    if ( check_address( server_addr ) == INVALID_ADDRESS )
    {
        return INVALID_ADDRESS;    /*return invalid */
    }

    return j; /*return present position */
}

/*extracts port number if it exists into port
returns INVALID_PORT if port number invalid
returns index value of the end of port number in input_str
*/
int get_port( char *input_str, char *port, int position )
{
    int cnt = 0;
    int colon_flg = position; /*store position */
    int colon_position = position;

    /*while input_str does not end and count is less than the port buffer
                                                                    length*/
    while ( ( input_str[position] != TERMINAL_CHAR ) &&
            ( cnt < ( PORT_BUFFER_LENGTH - 1 ) )
          )
    {
        /*if input_str has ended break loop*/
        if ( input_str[position] == TERMINAL_CHAR )
        {
            break;
        }

        /*if colon has not been found ignore characters and continue*/
        if ( ( input_str[position] != ':' ) &&
                ( colon_flg != -1 )
           )
        {
            position++;
            continue;
        }
        /* if colon has been found set flag and proceed*/
        else if ( ( input_str[position] == ':' ) &&
                  ( colon_flg != -1 )
                )
        {
            colon_flg = -1;
            position++;
        }

        /*if non numeric character has been found port has ended break loop*/
        if ( ( input_str[position] < 48 ) ||
                ( input_str[position] > 57 )
           )
        {
            break;
        }

        port[cnt] = input_str[position];
        cnt++;
        position++;
    }

    /*set terminating null character */
    port[cnt] = TERMINAL_CHAR;

    /*if colon was found port exists*/
    if ( colon_flg == -1 )
    {
        /* port number cannot be greater than MAX_PORT_NUMBER*/
        if ( atoi( port ) > MAX_PORT_NUMBER )
        {
            return INVALID_PORT;    /*return invalid port */
        }

        /*port cannot be empty with a colon*/
        if ( position == ( colon_position + 1 ) )
        {
            return INVALID_PORT;
        }

        return position; /*return present position */
    }

    return colon_flg;    /*else return old position */
}

/*
extracts path from input_str into path
*/
int get_path( char *input_str, char *path, int position )
{
    int cnt = 0;
    int slash_flg = 0;

    /*while input_str does not end and count is less than the buffer length*/
    while ( ( input_str[position] != TERMINAL_CHAR ) &&
            ( cnt < ( BUFFER_LENGTH - 1 ) )
          )
    {
        /*if input_str has ended break loop*/
        if ( input_str[position] == TERMINAL_CHAR )
        {
            break;
        }

        /*if there are leading slashes ignore them and continue*/
        if ( ( input_str[position] != '/' ) && !slash_flg )
        {
            position++;
            continue;
        }

        /*if there are more than one slash in succession ignore them and
                                                               continue*/
        if ( ( input_str[position] == '/' ) &&
                ( path[cnt - 1] == '/' )
           )
        {
            position++;
            continue;
        }

        slash_flg++;
        path[cnt] = input_str[position];
        cnt++;
        position++;
    }

    /*set terminating null character */
    path[cnt] = TERMINAL_CHAR;

    if ( path[0] == TERMINAL_CHAR )
    {
        return INVALID_PATH;
    }

    return SUCCESS;
}

/*
Checks if the address is valid
*/
int check_address( char *address_str )
{
    char temp_addr[BUFFER_LENGTH], *addr1, *addr2, *addr3, *addr4;
    int i = 0;
    char *pSave;

    if ( address_str[0] == TERMINAL_CHAR )
    {
        return INVALID_ADDRESS;    /*return invalid address*/
    }

    /*copy to temporary buffer */
    strncpy( temp_addr, address_str, BUFFER_LENGTH - 1 );
    temp_addr[BUFFER_LENGTH - 1] = '\0';
    /* take out four tokens */
    addr1 = strtok_r( temp_addr, ".", &pSave );
    addr2 = strtok_r( NULL, ".", &pSave );
    addr3 = strtok_r( NULL, ".", &pSave );
    addr4 = strtok_r( NULL, ".", &pSave );

    /* if all four tokens exists and they are all numeric*/
    /*must be an IP address*/
    if ( addr1 && addr2 && addr3 && addr4 &&
            !is_numeric( addr1 ) && !is_numeric( addr2 ) &&
            !is_numeric( addr3 ) && !is_numeric( addr4 )
       )
    {
        /*if all values are between 0 and 255 except the first one */
        if ( ( atoi( addr1 ) < 255 ) && ( atoi( addr1 ) > -1 ) &&
                ( atoi( addr2 ) <= 255 ) && ( atoi( addr2 ) > -1 ) &&
                ( atoi( addr3 ) <= 255 ) && ( atoi( addr3 ) > -1 ) &&
                ( atoi( addr4 ) <= 255 ) && ( atoi( addr4 ) > 0 )
           )
        {
            return 0;    /*return OK*/
        }

        return INVALID_ADDRESS; /*else return invalid address*/
    }

    /*checking non numeric address for illegal characters*/
    while ( address_str[i] != TERMINAL_CHAR && i < ( BUFFER_LENGTH - 1 ) )
    {
        if ( ( ( address_str[i] >= '0' ) && ( address_str[i]  <= '9' ) ) ||
                ( ( address_str[i] >= 'A' ) && ( address_str[i]  <= 'Z' ) ) ||
                ( ( address_str[i] >= 'a' ) && ( address_str[i]  <= 'z' ) ) ||
                ( address_str[i] == '-' ) ||
                ( address_str[i] == '.' )
           )
        {
            i++;
            continue;
        }

        return INVALID_ADDRESS; /*return invalid address*/
    }

    return 0; /*return OK */
}

/*checks if the given string has numeric characters only */
int is_numeric( char *input_str )
{
    int i = 0;

    /*if string is NULL then it is not numeric*/
    if ( input_str == NULL )
    {
        return -1;
    }

    while ( input_str[i] != TERMINAL_CHAR )
    {
        /* if value is between 0 and 9 ( ascii ) continue */
        if ( ( input_str[i] >= '0' ) &&
                ( input_str[i] <= '9' )
           )
        {
            i++;
            continue;
        }

        return -1; /*return not numeric */
    }

    return 0; /*return numeric */
}

int check_server_address( char *input_str )
{
    char server_addr[256], port[6];
    int return_value;

    /* get the server address */
    return_value = get_server_address( input_str, server_addr );

    /* checking if the server address is valid */
    if ( return_value < 0 )
    {
        return return_value;
    }

    /* get the port if it exists and is valid */
    return_value = get_port( input_str, port, return_value );
    return return_value;
}

/*
checks if a url is valid(assumed http or https)
*/
int check_url( char *input_str )
{
    int cnt = 0, colon_flg = 0;

    while ( input_str[cnt] != TERMINAL_CHAR &&
            cnt < ( BUFFER_LENGTH - 1 ) )
    {
        if ( input_str[cnt] == ':' && !colon_flg )
        {
            colon_flg = cnt;

            if ( !cnt && colon_flg )
            {
                return INVALID_ADDRESS;
            }

            break;
        }

        if ( ( ( input_str[cnt] >= '0' ) && ( input_str[cnt]  <= '9' ) ) ||
                ( ( input_str[cnt] >= 'A' ) && ( input_str[cnt]  <= 'Z' ) ) ||
                ( ( input_str[cnt] >= 'a' ) && ( input_str[cnt]  <= 'z' ) ) ||
                ( input_str[cnt] == '-' ) ||
                ( input_str[cnt] == '.' ) ||
                ( input_str[cnt] == '+' )
           )
        {
            cnt++;
            continue;
        }

        return INVALID_ADDRESS;
    }

    if ( !colon_flg )
    {
        return INVALID_ADDRESS;
    }

    return check_server_address( input_str + colon_flg + 1 );
}
