#include <stdio.h>
#include <stdlib.h>
#include <errno.h>
#include <limits.h>
#include <sys/types.h>
#include <sys/socket.h>
#include <sys/un.h>
#include <unistd.h>
#include "pvalue_validation_lib.h"
#include "datatypes.h"
#include "../nvtool/nvram.h"

int validate_pvalue( const char *name, const char *value )
{
    int sock;
    int ret_val = NOTFOUND;
    struct sockaddr_un server;
    char *data;
    char buf[5] = { 0 };

    sock = socket( AF_UNIX, SOCK_STREAM, 0 );

    if ( sock < 0 )
    {
        perror( "Opening validation socket" );
        return SOCKETERROR;
    }

    server.sun_family = AF_UNIX;
    strcpy( server.sun_path, "/tmp/.pvalue_validation" );

    if ( connect( sock, ( struct sockaddr * ) &server, sizeof( struct sockaddr_un ) ) < 0 )
    {
        close( sock );
        //perror( "Connecting to validation socket" );
        return SOCKETERROR;
    }

    if ( !name )
    {
        data = (char *)malloc( sizeof( char ) * 10 );
        strcpy( data, "VALIDATE " );
    }
    else if ( !value )
    {
        data = (char *)malloc( sizeof( char ) * ( strlen( name ) + 11 ) );
        strcpy( data, "VALIDATE " );
        strcat( data, name );
        strcat( data, " " );
    }
    else
    {
        data = (char *)malloc( sizeof( char ) * ( strlen( name ) + strlen( value ) + 11 ) );
        strcpy( data, "VALIDATE " );
        strcat( data, name );
        strcat( data, " " );
        strcat( data, value );
    }

    if ( write( sock, data, strlen( data ) * sizeof( char ) ) < 0 )
    {
        perror( "Writing on validation socket" );
    }

    free( data );

    if ( read( sock, buf, 5 ) < 0 )
    {
        perror( "Reading validation message" );
    }

    close( sock );
    errno = 0;
    ret_val = strtol( buf, ( char ** ) NULL, 10 );

    if ( ( ( errno == ERANGE ) && ( ( ret_val == LONG_MIN ) || ( ret_val == LONG_MAX ) ) ) ||
            ( ( errno != 0 ) && ( ret_val == 0 ) ) )
    {
        perror( "Invalid return value" );
        return SERVERERROR;
    }

    return ret_val;
}

int query_server( char *query_str )
{
    int sock;
    int ret_val = NOTFOUND;
    struct sockaddr_un server;
    char buf[5] = { 0 };

    if ( !query_str )
    {
        return ret_val;
    }

    if ( strlen( query_str ) > 100 )
    {
        return ret_val;
    }

    sock = socket( AF_UNIX, SOCK_STREAM, 0 );

    if ( sock < 0 )
    {
        perror( "Opening validation socket" );
        return SOCKETERROR;
    }

    server.sun_family = AF_UNIX;
    strcpy( server.sun_path, "/tmp/.pvalue_validation" );

    if ( connect( sock, ( struct sockaddr * ) &server, sizeof( struct sockaddr_un ) ) < 0 )
    {
        close( sock );
        //perror( "Connecting to validation socket" );
        return SOCKETERROR;
    }

    //printf( "%s:%d %s\n", __FILE__, __LINE__, data );
    if ( write( sock, query_str, strlen( query_str ) * sizeof( char ) ) < 0 )
    {
        perror( "Writing on validation socket" );
    }

    if ( read( sock, buf, 5 ) < 0 )
    {
        perror( "Reading validation message" );
    }

    close( sock );
    errno = 0;
    ret_val = strtol( buf, ( char ** ) NULL, 10 );

    if ( ( ( errno == ERANGE ) && ( ( ret_val == LONG_MIN ) || ( ret_val == LONG_MAX ) ) ) ||
            ( ( errno != 0 ) && ( ret_val == 0 ) ) )
    {
        perror( "Invalid return value" );
        return SERVERERROR;
    }

    return ret_val;
}

int query_server2( const char *query_str, char *ret_str, unsigned int size )
{
    int sock;
    struct sockaddr_un server;
    char buf[BUFSIZE] = { 0 };

    sock = socket( AF_UNIX, SOCK_STREAM, 0 );

    if ( sock < 0 )
    {
        //perror( "Opening validation socket" );
        return SOCKETERROR;
    }

    server.sun_family = AF_UNIX;
    strcpy( server.sun_path, "/tmp/.pvalue_validation" );

    if ( connect( sock, ( struct sockaddr * ) &server, sizeof( struct sockaddr_un ) ) < 0 )
    {
        close( sock );
        //perror( "Connecting to validation socket" );
        return SOCKETERROR;
    }

    if ( write( sock, query_str, strlen( query_str ) * sizeof( char ) ) < 0 )
    {
        perror( "Writing on validation socket" );
    }

    if ( read( sock, buf, BUFSIZE ) < 0 )
    {
        perror( "Reading validation message" );
    }

    close( sock );

    if ( size >= BUFSIZE )
    {
        strncpy( ret_str, buf, BUFSIZE - 1 );
        ret_str[BUFSIZE - 1] = '\0';
    }
    else
    {
        strncpy( ret_str, buf, size );
        ret_str[size - 1] = '\0';
    }

    return 0;
}

int str_to_group_mask( const char* groups_str, unsigned int* groups )
{
    long long val;
    char *endptr = NULL;
    int retval = 0;

    if ( ( NULL != groups_str ) && ( NULL != groups ) )
    {
        val = strtoll( groups_str, &endptr, 10 );

        if ( val < 0 || val > UINT_MAX )
        {
            retval = -1;
        }
        else if ( endptr == groups_str )
        {
            retval = -1;
        }
        else
        {
            *groups = val;
        }
    }
    else
    {
        retval = -1;
    }

    return retval;
}

int pvalue_name( const char *name, char *ret_str, unsigned int size )
{
    char data[100];
    int len = 0;

    if ( !name )
    {
        return NOTFOUND;
    }

    len = strlen( name );

    if ( !len || len > 93 )
    {
        return NOTFOUND;
    }

    snprintf( data, 100, "PROPERNAME %s", name );

    return query_server2( data, ret_str, size );
}

int pvalue_default( const char *name, char *ret_str, unsigned int size )
{
    char data[100];
    int len = 0;
    int ret_val;

    if ( !name )
    {
        return NOTFOUND;
    }

    len = strlen( name );

    if ( !len || len > 91 )
    {
        return NOTFOUND;
    }

    snprintf( data, 100, "DEFAULT %s", name );

    ret_val =  query_server2( data, ret_str, size );

    if ( ret_val < 0 || !strcmp( "NODEFAULT", ret_str ) || !strcmp( "NOTFOUND", ret_str ) )
    {
        return NOTFOUND;
    }

    return 0;
}

int pvalue_base( const char *name )
{
    char data[128];
    int len = 0;
    int ret_val = 0;
    char ret_str[128];

    if ( !name )
    {
        return NOTFOUND;
    }

    len = strlen( name );

    if ( !len || len > 91 )
    {
        return NOTFOUND;
    }

    snprintf( data, 100, "BASE %s", name );

    ret_val = query_server2( data, ret_str, 128 );

    if ( ret_val < 0 )
    {
        return NOTFOUND;
    }

    return ( int )strtol( ret_str, NULL, 10 );
}

int pvalue_alias( const char *name, char *ret_str, unsigned int size )
{
    char data[100];
    int len = 0;

    if ( !name )
    {
        return NOTFOUND;
    }

    len = strlen( name );

    if ( !len || len > 93 )
    {
        return NOTFOUND;
    }

    snprintf( data, 100, "ALIAS %s", name );

    return query_server2( data, ret_str, size );
}

int pvalue_originfile( const char *name, char *ret_str, unsigned int size )
{
    char data[100];
    int len = 0;

    if ( !name )
    {
        return NOTFOUND;
    }

    len = strlen( name );

    if ( !len || len > 93 )
    {
        return NOTFOUND;
    }

    snprintf( data, 100, "ORIGINFILE %s", name );

    return query_server2( data, ret_str, size );
}


int get_pvalue_notification_groups( const char *name, char *ret_str, unsigned int size )
{
    char data[100];
    int len = 0;

    if ( !name )
    {
        return NOTFOUND;
    }

    len = strlen( name );

    if ( !len || len > 93 )
    {
        return NOTFOUND;
    }

    snprintf( data, 100, "GROUPS %s", name );

    return query_server2( data, ret_str, size );
}

int get_pvalue_notification_group_mask( const char *name, unsigned int *mask )
{
    char buf[128];
    int ret_val;

    ret_val = get_pvalue_notification_groups( name, buf, sizeof( buf ) );

    if ( ret_val == 0 )
    {
        ret_val = str_to_group_mask( buf, mask );
    }

    return ret_val;
}

int pvalue_has_notification_group( const char *name, unsigned int group )
{
    int ret;
    unsigned int group_mask;

    if ( group > 31 )
    {
        return 0;
    }

    ret = get_pvalue_notification_group_mask( name, &group_mask );

    if ( ret != 0 )
    {
        return 0;
    }

    if ( 0 != ( group_mask & ( 1 << group ) ) )
    {
        return 1;
    }

    return 0;
}

int pvalue_help( const char *name, char *ret_str, unsigned int size )
{
    char data[100];
    int len = 0;

    if ( !name )
    {
        return NOTFOUND;
    }

    len = strlen( name );

    if ( !len || len > 93 )
    {
        return NOTFOUND;
    }

    snprintf( data, 100, "HELP %s", name );

    return query_server2( data, ret_str, size );
}

int pvalue_desc( const char *name, char *ret_str, unsigned int size )
{
    char data[100];
    int len = 0;

    if ( !name )
    {
        return NOTFOUND;
    }

    len = strlen( name );

    if ( !len || len > 94 )
    {
        return NOTFOUND;
    }

    snprintf( data, 100, "DESC %s", name );

    return query_server2( data, ret_str, size );
}

int pvalue_scripts( const char *name, char *ret_str, unsigned int size )
{
    char data[100];
    int len = 0;

    if ( !name )
    {
        return NOTFOUND;
    }

    len = strlen( name );

    if ( !len || len > 93 )
    {
        return NOTFOUND;
    }

    snprintf( data, 100, "SCRIPTS %s", name );

    return query_server2( data, ret_str, size );
}

int set_pvalue_privilege_flag( const char *name, int value )
{
    char data[100];
    int len = 0;

    if ( !name )
    {
        return NOTFOUND;
    }

    len = strlen( name );

    if ( !len || len > 78 )
    {
        return NOTFOUND;
    }

    snprintf( data, 100, "SETPRIVILEGEFLAG %s %d", name, value );

    return query_server( data );
}

int check_pvalue_privilege_flag( const char *name )
{
    char data[100];
    int len = 0;

    if ( !name )
    {
        return NOTFOUND;
    }

    len = strlen( name );

    if ( !len || len > 85 )
    {
        return NOTFOUND;
    }

    snprintf( data, 100, "PRIVILEGEFLAG %s", name );

    return query_server( data );
}

int check_pvalue_lock_flag( const char *name )
{
    char data[100];
    int len = 0;

    if ( !name )
    {
        return NOTFOUND;
    }

    len = strlen( name );

    if ( !len || len > 91 )
    {
        return NOTFOUND;
    }

    snprintf( data, 100, "LOCKFLAG %s", name );

    return query_server( data );
}

int add_or_remove_pvalue_notification_groups_by_mask( const char *name, unsigned int groups, int add )
{
    char data[100];
    int len = 0;

    if ( !name )
    {
        return NOTFOUND;
    }

    len = strlen( name );

    if ( !len || len > 82 )
    {
        return NOTFOUND;
    }

    snprintf( data, 100, "ADDORREMGROUPS %s %u %d", name, groups, add );

    return query_server( data );
}

int add_or_remove_pvalue_notification_group_by_num( const char *name, unsigned int group, int add )
{
    unsigned int group_mask;

    if ( group < MAX_MSG_GROUP_INTEGER_ID )
    {
        group_mask = 1 << group;
        return add_or_remove_pvalue_notification_groups_by_mask( name, group_mask, add );
    }

    printf( "Error: %s invalid input, group must be less than %d, got %d\n", __FUNCTION__, MAX_MSG_GROUP_INTEGER_ID, group );
    return -1;
}

int add_pvalue_notification_group( const char *name, unsigned int group )
{
    return add_or_remove_pvalue_notification_group_by_num( name, group, 1 );
}

int remove_pvalue_notification_group( const char *name, unsigned int group )
{
    return add_or_remove_pvalue_notification_group_by_num( name, group, 0 );
}

int set_pvalue_notification_group_bitmask( const char *name, unsigned int group_mask )
{
    return add_or_remove_pvalue_notification_groups_by_mask( name, group_mask, 1 );
}

int clear_pvalue_notification_group_bitmask( const char *name, unsigned int group_mask )
{
    return add_or_remove_pvalue_notification_groups_by_mask( name, group_mask, 0 );
}

int check_pvalue_reboot_flag( const char *name )
{
    char data[100];
    int len = 0;

    if ( !name )
    {
        return NOTFOUND;
    }

    len = strlen( name );

    if ( !len || len > 88 )
    {
        return NOTFOUND;
    }

    snprintf( data, 100, "REBOOTFLAG %s", name );

    return query_server( data );
}

int check_pvalue_script_flag( const char *name )
{
    char data[100];
    int len = 0;

    if ( !name )
    {
        return NOTFOUND;
    }

    len = strlen( name );

    if ( !len || len > 89 )
    {
        return NOTFOUND;
    }

    snprintf( data, 100, "SCRIPTFLAG %s", name );

    return query_server( data );
}

int check_datatype( const char *name )
{
    char data[100];
    int len = 0;

    if ( !name )
    {
        return NOTFOUND;
    }

    len = strlen( name );

    if ( !len || len > 90 )
    {
        return NOTFOUND;
    }

    snprintf( data, 100, "CATEGORY %s", name );

    return query_server( data );
}

struct category_struct
{
    enum category cat;
    const char *name;
};

const struct category_struct category_names[] =
{
    { NUMBER, "number" },
    { UTF8_STRING, "utf8string" },
    { STRING, "string" },
    { PASSWORD, "password" },
    { ADDRESS, "address" },
    { IPADDRESS, "ipaddress" },
    { SERVERADDRESS, "serveraddress" },
    { NETMASK, "netmask" },
    { URL, "url" },
    { TR069, "tr069" },
    { CERT, "cert" },
    { BLANK, "blank" },
    { DEVICEPASSWORD, "device_password" },
    {}
};

const char* get_str_datatype( enum category cat )
{
    const struct category_struct *cs;

    for ( cs = category_names; cs->name; cs++ )
    {
        if ( cat == cs->cat )
        {
            return cs->name;
        }
    }

    return NULL;
}

enum category get_cat_datatype( char *str )
{
    const struct category_struct *cs;

    if ( str == NULL || str[0] == '\0' )
    {
        return CATNOTFOUND;
    }

    for ( cs = category_names; cs->name; cs++ )
    {
        if ( strcmp( str, cs->name ) == 0 )
        {
            return cs->cat;
        }
    }

    return CATNOTFOUND;
}

