#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <sys/resource.h>
#include <sys/time.h>
#include "pvalue_validation_lib.h"

#define COUNT 1000
void calc_time_diff( struct rusage *p1, struct rusage *p2,
                     long int *u_sec, long int *u_usec,
                     long int *s_sec, long int *s_usec )
{
    long int udiff = p2->ru_utime.tv_usec - p1->ru_utime.tv_usec;
    long int diff = p2->ru_utime.tv_sec - p1->ru_utime.tv_sec;
    printf( "%ld %ld %ld %ld\n", p2->ru_utime.tv_sec, p2->ru_utime.tv_usec, p1->ru_utime.tv_sec, p1->ru_utime.tv_usec );
    printf( "%ld %ld %ld %ld\n", p2->ru_stime.tv_sec, p2->ru_stime.tv_usec, p1->ru_stime.tv_sec, p1->ru_stime.tv_usec );

    if ( udiff < 0 )
    {
        p2->ru_utime.tv_usec += 1000000;
        p2->ru_utime.tv_sec -= 1;
        udiff = p2->ru_utime.tv_usec - p1->ru_utime.tv_usec;
        diff = p2->ru_utime.tv_sec - p1->ru_utime.tv_sec;
    }

    *u_sec = diff;
    *u_usec = udiff;

    udiff = p2->ru_stime.tv_usec - p1->ru_stime.tv_usec;
    diff = p2->ru_stime.tv_sec - p1->ru_stime.tv_sec;

    if ( udiff < 0 )
    {
        p2->ru_stime.tv_usec += 1000000;
        p2->ru_stime.tv_sec -= 1;
        udiff = p2->ru_stime.tv_usec - p1->ru_stime.tv_usec;
        diff = p2->ru_stime.tv_sec - p1->ru_stime.tv_sec;
    }

    *s_sec = diff;
    *s_usec = udiff;
}

int main( int argc, char **argv )
{
    char ret_str[2048]  = { 0 };
    int cmd, ret_val = 0;
    int who = RUSAGE_SELF;
    struct rusage before_usage;
    struct rusage after_usage;
    struct timeval t1, t2;
    long int u_sec, u_usec, s_sec, s_usec;
    int i = 0, count = COUNT;
    int zero_count = 0, one_count = 0, none_count = 0, d_count = 0;

    if ( argc < 2 )
    {
        printf( "usage:%s <default|validate> [name] [value]\n", argv[0] );
        return 1;
    }

    if ( !strncmp( argv[1], "default", 7 ) )
    {
        cmd = 0;
    }
    else if ( !strncmp( argv[1], "validate", 8 ) )
    {
        cmd = 1;
    }
    else
    {
        printf( "usage:%s <default|validate> [name] [value]\n", argv[0] );
        return 1;
    }

    if ( cmd == 0 )
    {
        if ( argc == 2 )
        {
            getrusage( who, &before_usage );
            pvalue_default( NULL, ret_str, 2048 );
            getrusage( who, &after_usage );
        }
        else
        {
            getrusage( who, &before_usage );
            pvalue_default( argv[2], ret_str, 2048 );
            getrusage( who, &after_usage );
        }

        if ( ret_str[0] != '\0' )
        {
            printf( "%s\n", ret_str );
        }

        calc_time_diff( &before_usage, &after_usage, &u_sec, &u_usec, &s_sec, &s_usec );
        printf( "%ld.%ld user %ld.%ld system\n", u_sec, u_usec, s_sec, s_usec );
        return 0;
    }

    if ( argc == 2 )
    {
        for ( i = 0 ; i < COUNT ; i++ )
        {
            ret_val = validate_pvalue( NULL, NULL );
        }
    }
    else if ( argc == 3 )
    {
        gettimeofday( &t1, NULL );

        for ( i = 0 ; i < COUNT ; i++ )
        {
            ret_val = validate_pvalue( argv[2], NULL );

            switch ( ret_val )
            {
                case 0:
                    zero_count++;
                    break;

                case 1:
                    one_count++;
                    break;

                case -1:
                    none_count++;
                    break;

                default:
                    d_count++;
                    break;
            }
        }

        gettimeofday( &t2, NULL );
    }
    else
    {
        if ( argc > 4 )
        {
            count = atoi( argv[4] );
        }

        gettimeofday( &t1, NULL );

        for ( i = 0 ; i < count ; i++ )
        {
            ret_val = validate_pvalue( argv[2], argv[3] );

            //printf("%d  ",i);
            switch ( ret_val )
            {
                case 0:
                    zero_count++;
                    break;

                case 1:
                    one_count++;
                    break;

                case -1:
                    none_count++;
                    break;

                default:
                    d_count++;
                    break;
            }

            //printf( "%d %d %d %d\n", zero_count, one_count, none_count, d_count );
        }

        gettimeofday( &t2, NULL );
    }

    if ( t2.tv_usec < t1.tv_usec )
    {
        printf( "Client time taken for %d queries %lu.%06lu seconds\n", count, t2.tv_sec - t1.tv_sec - 1, 1000000 + t2.tv_usec - t1.tv_usec );
    }
    else
    {
        printf( "Client time taken for %d queries %lu.%06lu seconds\n", count, t2.tv_sec - t1.tv_sec, t2.tv_usec - t1.tv_usec );
    }

    printf( "%s %s %d %d %d %d\n", argv[2], argv[3], zero_count, one_count, none_count, d_count );
    //calc_time_diff( &before_usage, &after_usage, &u_sec, &u_usec, &s_sec, &s_usec );
    //printf( "%ld.%ld user %ld.%ld system\n", u_sec, u_usec, s_sec, s_usec );
    return 0;
}
