#ifndef VALIDATE_H
#define VALIDATE_H

#define INVALID_ADDRESS -1
#define INVALID_PORT -2
#define INVALID_PATH -3
#define NO_ARGUMENTS 1
#define BUFFER_LENGTH 256
#define PORT_BUFFER_LENGTH 6
#define MAX_PORT_NUMBER 65535
#define TERMINAL_CHAR '\0'
#define SUCCESS 0

#ifdef __cplusplus
extern "C" {
#endif

/*
*check if string contains numeric values
*/
int is_numeric( const char *value );

/*
Checks if the address is valid
*/
int check_address( char *address_str );

/*
Checks if the ip address is valid
*/
int check_ip_address( char *address_str );

/*
extracts server address into server_addr
returns INVALID_ADDRESS if invalid address detected
returns position of end of server_address in input_str
*/
int get_server_address( char *input_str, char *server_addr );

/*extracts port number if it exists into port
returns INVALID_PORT if port number invalid
returns index value of the end of port number in input_str
*/
int get_port( char *input_str, char *port, int position );

/*
extracts path from input_str into path
*/
int get_path( char *input_str, char *path, int position );

int check_server_address( char *input_str );

/*
Checks if the netmask is valid
*/
int check_netmask( char *address_str );

/*
checks if a url is valid(assumed http or https)
*/
int check_url( char *input_str );

/*
checks if string is utf8
*/
int validate_utf8( const unsigned char *input_str, int debug );

int strlen_utf8( const unsigned char *input_str, int debug );

long get_code_point_utf8( const unsigned char *input_str, unsigned char **ret_ptr, int debug );

int validate_certificate( char* value, char is_unpkcs8 );
#ifdef __cplusplus
}
#endif

#endif
