#include <stdio.h>
#include <ctype.h>
#include <stdlib.h>
#include <string.h>
#include <errno.h>
#include <limits.h>
#include "validate_functions.h"
#include "datatypes.h"
#include "xml_pvalues.h"
#include "pvalue_validation_lib.h"
#include "../nvtool/nvram.h"

#define MALLOC(x) malloc(x)
#define FREE(x) free(x)
#define STRDUP(x) strdup(x)

long long int Lmin, Lmax;

void ( *free_type[13] )( union type * ) =
{
    &free_number_type,
    &free_utf8_string_type,
    &free_string_type,
    &free_password_type,
    &free_address_type,
    &free_ipaddress_type,
    &free_serveraddress_type,
    &free_netmask_type,
    &free_url_type,
    &free_tr069_type,
    &free_cert_type,
    &free_blank_type,
    &free_device_password_type,
};

void ( *print_type[13] )( union type * ) =
{
    &print_number_type,
    &print_utf8_string_type,
    &print_string_type,
    &print_password_type,
    &print_address_type,
    &print_ipaddress_type,
    &print_serveraddress_type,
    &print_netmask_type,
    &print_url_type,
    &print_tr069_type,
    &print_cert_type,
    &print_blank_type,
    &print_device_password_type,
};

int ( *sprint_type[13] )( union type *, char*, unsigned int ) =
{
    &sprint_number_type,
    &sprint_utf8_string_type,
    &sprint_string_type,
    &sprint_password_type,
    &sprint_address_type,
    &sprint_ipaddress_type,
    &sprint_serveraddress_type,
    &sprint_netmask_type,
    &sprint_url_type,
    &sprint_tr069_type,
    &sprint_cert_type,
    &sprint_blank_type,
    &sprint_device_password_type,
};

int ( *validate_type[13] )( union type *, char * ) =
{
    &validate_number_type,
    &validate_utf8_string_type,
    &validate_string_type,
    &validate_password_type,
    &validate_address_type,
    &validate_ipaddress_type,
    &validate_serveraddress_type,
    &validate_netmask_type,
    &validate_url_type,
    &validate_tr069_type,
    &validate_cert_type,
    &validate_blank_type,
    &validate_device_password_type,
};

int llist_add( llist **head, long long int val )
{
    llist *temp = *head;

    if ( !head )
    {
        return 1;
    }

    if ( !temp )
    {
        *head = (llist*)MALLOC( sizeof( llist ) );

        if ( *head == NULL )
        {
            fprintf( stderr, "Not enough memory\n" );
            return -1;
        }

        ( *head )->val = val;
        ( *head )->next = NULL;
        return 0;
    }

    while ( temp->next != NULL )
    {
        temp = temp->next;
    }

    temp->next = (llist*)MALLOC( sizeof( llist ) );

    if ( temp->next == NULL )
    {
        fprintf( stderr, "Not enough memory\n" );
        return -1;
    }

    temp->next->val = val;
    temp->next->next = NULL;
    return 0;
}

void llist_free( llist **head )
{
    llist *temp = *head, *temp1;

    if ( !head || !( *head ) )
    {
        return;
    }

    while ( temp->next != NULL )
    {
        temp1 = temp->next;
        FREE( temp );
        temp = temp1;
    }

    FREE( temp );
    *head = NULL;
}

int llist_find( llist **head, long long int val )
{
    llist *temp = *head;

    if ( !head || !( *head ) )
    {
        return 1;
    }

    while ( temp->next != NULL )
    {
        if ( val == temp->val )
        {
            return 0;
        }

        temp = temp->next;
    }

    if ( val == temp->val )
    {
        return 0;
    }

    return -1;
}

int llist_multiple_check( llist **head, long long int value )
{
    llist *temp = *head;

    if ( !head || !( *head ) )
    {
        return 1;
    }

    while ( temp->next != NULL )
    {
        if ( temp->val != 0 && value != 0 && ( value / temp->val ) > 0 && ( value % temp->val ) == 0 )
        {
            return 0;
        }

        temp = temp->next;
    }

    if ( temp->val != 0 && value != 0 && ( value / temp->val ) > 0 && ( value % temp->val ) == 0 )
    {
        return 0;
    }

    return -1;
}

void llist_print( llist **head )
{
    llist *temp = *head;

    if ( !head || !( *head ) )
    {
        printf( "\n" );
        return;
    }

    while ( temp->next != NULL )
    {
        printf( "%lld ", temp->val );
        temp = temp->next;
    }

    printf( "%lld\n", temp->val );
}

int llist_sprint( llist **head, char *ret_str, unsigned int size )
{
    unsigned int len = 0, tmp_len = 0;
    char *tmp_str = ret_str;
    llist *temp = *head;

    if ( !head || !( *head ) )
    {
        return 0;
    }

    while ( temp->next != NULL )
    {
        tmp_len = snprintf( tmp_str, size - len, "%lld,", temp->val );
        tmp_str += tmp_len;
        len += tmp_len;
        temp = temp->next;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    len += snprintf( tmp_str, size - len, "%lld", temp->val );
    return len;
}

int cllist_add( cllist **head, unsigned char val )
{
    cllist *temp = *head;

    if ( !head )
    {
        return 1;
    }

    if ( !temp )
    {
        *head = (cllist *)MALLOC( sizeof( cllist ) );

        if ( *head == NULL )
        {
            fprintf( stderr, "Not enough memory\n" );
            return -1;
        }

        ( *head )->val = val;
        ( *head )->next = NULL;
        return 0;
    }

    while ( temp->next != NULL )
    {
        temp = temp->next;
    }

    temp->next = (cllist *)MALLOC( sizeof( cllist ) );

    if ( temp->next == NULL )
    {
        fprintf( stderr, "Not enough memory\n" );
        return -1;
    }

    temp->next->val = val;
    temp->next->next = NULL;
    return 0;
}

void cllist_free( cllist **head )
{
    cllist *temp = *head, *temp1;

    if ( !head || !( *head ) )
    {
        return;
    }

    while ( temp->next != NULL )
    {
        temp1 = temp->next;
        FREE( temp );
        temp = temp1;
    }

    FREE( temp );
    *head = NULL;
}

int cllist_find( cllist **head, unsigned char val )
{
    cllist *temp = *head;

    if ( !head || !( *head ) )
    {
        return 1;
    }

    while ( temp->next != NULL )
    {
        if ( val == temp->val )
        {
            return 0;
        }

        temp = temp->next;
    }

    if ( val == temp->val )
    {
        return 0;
    }

    return -1;
}

void cllist_print( cllist **head )
{
    cllist *temp = *head;

    if ( !head || !( *head ) )
    {
        printf( "\n" );
        return;
    }

    while ( temp->next != NULL )
    {
        printf( "|%c |", temp->val );
        temp = temp->next;
    }

    printf( "|%c|\n", temp->val );
}

int cllist_sprint( cllist **head, char *ret_str, unsigned int size )
{
    unsigned int len = 0, tmp_len = 0;
    char *tmp_str = ret_str;
    cllist *temp = *head;

    if ( !head || !( *head ) )
    {
        return 0;
    }

    while ( temp->next != NULL )
    {
        tmp_len = snprintf( tmp_str, size - len, "|%c|", temp->val );
        tmp_str += tmp_len;
        len += tmp_len;
        temp = temp->next;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    len += snprintf( tmp_str, size - len, "|%c| ", temp->val );
    return len;
}

int rlist_add( rlist **head, long long int min, long long int max )
{
    rlist *temp = *head;

    if ( !head )
    {
        return 1;
    }

    if ( !temp )
    {
        *head = (rlist *)MALLOC( sizeof( rlist ) );

        if ( *head == NULL )
        {
            fprintf( stderr, "Not enough memory\n" );
            return -1;
        }

        ( *head )->min = min;
        ( *head )->max = max;
        ( *head )->next = NULL;
        return 0;
    }

    while ( temp->next != NULL )
    {
        temp = temp->next;
    }

    temp->next = (rlist *)MALLOC( sizeof( rlist ) );

    if ( temp->next == NULL )
    {
        fprintf( stderr, "Not enough memory\n" );
        return -1;
    }

    temp->next->min = min;
    temp->next->max = max;
    temp->next->next = NULL;
    return 0;
}

void rlist_free( rlist **head )
{
    rlist *temp = *head, *temp1;

    if ( !head || !( *head ) )
    {
        return;
    }

    while ( temp->next != NULL )
    {
        temp1 = temp->next;
        FREE( temp );
        temp = temp1;
    }

    FREE( temp );
    *head = NULL;
}

int rlist_validate( rlist **head, long long int val )
{
    rlist *temp = *head;

    if ( !head || !( *head ) )
    {
        return 1;
    }

    while ( temp->next != NULL )
    {
        if ( val >= temp->min && val <= temp->max )
        {
            return 0;
        }

        temp = temp->next;
    }

    if ( val >= temp->min && val <= temp->max )
    {
        return 0;
    }

    return -1;
}

void rlist_print( rlist **head )
{
    rlist *temp = *head;

    if ( !head || !( *head ) )
    {
        printf( "\n" );
        return;
    }

    while ( temp->next != NULL )
    {
        printf( "%lld %lld ", temp->min, temp->max );
        temp = temp->next;
    }

    printf( "%lld %lld\n", temp->min, temp->max );
}


int rlist_sprint( rlist **head, char *ret_str, unsigned int size )
{
    unsigned int len = 0, tmp_len = 0;
    char *tmp_str = ret_str;
    rlist *temp = *head;

    if ( !head || !( *head ) )
    {
        return 0;
    }

    while ( temp->next != NULL )
    {
        tmp_len = snprintf( tmp_str, size - len, "%lld-%lld,", temp->min, temp->max );
        tmp_str += tmp_len;
        len += tmp_len;
        temp = temp->next;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    len += snprintf( tmp_str, size - len, "%lld-%lld", temp->min, temp->max );
    return len;
}

int crlist_add( crlist **head, unsigned char min, unsigned char max )
{
    crlist *temp = *head;

    if ( !head )
    {
        return 1;
    }

    if ( !temp )
    {
        *head = (crlist *)MALLOC( sizeof( crlist ) );

        if ( *head == NULL )
        {
            fprintf( stderr, "Not enough memory\n" );
            return -1;
        }

        ( *head )->min = min;
        ( *head )->max = max;
        ( *head )->next = NULL;
        return 0;
    }

    while ( temp->next != NULL )
    {
        temp = temp->next;
    }

    temp->next = (crlist *)MALLOC( sizeof( crlist ) );

    if ( temp->next == NULL )
    {
        fprintf( stderr, "Not enough memory\n" );
        return -1;
    }

    temp->next->min = min;
    temp->next->max = max;
    temp->next->next = NULL;
    return 0;
}

void crlist_free( crlist **head )
{
    crlist *temp = *head, *temp1;

    if ( !head || !( *head ) )
    {
        return;
    }

    while ( temp->next != NULL )
    {
        temp1 = temp->next;
        FREE( temp );
        temp = temp1;
    }

    FREE( temp );
    *head = NULL;
}

int crlist_validate( crlist **head, unsigned char val )
{
    crlist *temp = *head;

    if ( !head || !( *head ) )
    {
        return 1;
    }

    while ( temp->next != NULL )
    {
        if ( val >= temp->min && val <= temp->max )
        {
            return 0;
        }

        temp = temp->next;
    }

    if ( val >= temp->min && val <= temp->max )
    {
        return 0;
    }

    return -1;
}

void crlist_print( crlist **head )
{
    crlist *temp = *head;

    if ( !head || !( *head ) )
    {
        printf( "\n" );
        return;
    }

    while ( temp->next != NULL )
    {
        printf( "%c %c ", temp->min, temp->max );
        temp = temp->next;
    }

    printf( "%c/0x%x %c/0x%x\n", temp->min, temp->min, temp->max, temp->max );
}

int crlist_sprint( crlist **head, char* ret_str, unsigned int size )
{
    unsigned int len = 0, tmp_len = 0;
    char* tmp_str = ret_str;
    crlist *temp = *head;

    if ( !head || !( *head ) )
    {
        return 0;
    }

    while ( temp->next != NULL )
    {
        tmp_len = snprintf( tmp_str, size - len, "%c-%c,", temp->min, temp->max );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len > ( size - 1 ) )
        {
            return len;
        }

        temp = temp->next;
    }

    len += snprintf( tmp_str, size - len, "%c/0x%x-%c/0x%x", temp->min, temp->min, temp->max, temp->max );
    return len;
}

int str_list_add( str_list **head, char *str )
{
    str_list *temp = *head;

    if ( !head )
    {
        return 1;
    }

    if ( !temp )
    {
        *head = (str_list *)MALLOC( sizeof( str_list ) );

        if ( *head == NULL )
        {
            fprintf( stderr, "Not enough memory\n" );
            return -1;
        }

        ( *head )->str = STRDUP( str );

        if ( ( *head )->str == NULL )
        {
            fprintf( stderr, "Not enough memory\n" );
            return -1;
        }

        ( *head )->next = NULL;
        return 0;
    }

    while ( temp->next != NULL )
    {
        temp = temp->next;
    }

    temp->next = (str_list *)MALLOC( sizeof( str_list ) );

    if ( temp->next == NULL )
    {
        fprintf( stderr, "Not enough memory\n" );
        return -1;
    }

    temp->next->str = STRDUP( str );

    if ( temp->next->str == NULL )
    {
        fprintf( stderr, "Not enough memory\n" );
        return -1;
    }

    temp->next->next = NULL;
    return 0;
}

void str_list_free( str_list **head )
{
    str_list *temp = *head, *temp1;

    if ( !head || !( *head ) )
    {
        return;
    }

    while ( temp->next != NULL )
    {
        temp1 = temp->next;
        FREE( temp->str );
        FREE( temp );
        temp = temp1;
    }

    FREE( temp->str );
    FREE( temp );
    *head = NULL;
}

int str_list_find( str_list **head, char *str )
{
    str_list *temp = *head;

    if ( !head || !( *head ) )
    {
        return 1;
    }

    while ( temp->next != NULL )
    {
        if ( !strcmp( str, temp->str ) && ( strlen( str ) == strlen( temp->str ) ) )
        {
            return 0;
        }

        temp = temp->next;
    }

    if ( !strcmp( str, temp->str ) )
    {
        return 0;
    }

    return -1;
}

void str_list_print( str_list **head )
{
    str_list *temp = *head;

    if ( !head || !( *head ) )
    {
        printf( "\n" );
        return;
    }

    while ( temp->next != NULL )
    {
        printf( "%s ", temp->str );
        temp = temp->next;
    }

    printf( "%s\n", temp->str );
}

int str_list_sprint( str_list **head, char* ret_str, unsigned int size )
{
    unsigned int len = 0, tmp_len = 0;
    char* tmp_str = ret_str;
    str_list *temp = *head;

    if ( !head || !( *head ) )
    {
        return 0;
    }

    while ( temp->next != NULL )
    {
        tmp_len = snprintf( tmp_str, size - len, "%s,", temp->str );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        temp = temp->next;
    }

    len += snprintf( tmp_str, size - len, "%s ", temp->str );
    return len;
}

void print_number_type( union type *t )
{
    printf( "Base %d\nMin ", ( t->number ).base );
    llist_print( &( ( t->number ).min ) );
    printf( "Max " );
    llist_print( &( ( t->number ).max ) );
    printf( "Not Min " );
    llist_print( &( ( t->number ).not_min ) );
    printf( "Not Max " );
    llist_print( &( ( t->number ).not_max ) );
    printf( "Range " );
    rlist_print( &( ( t->number ).range ) );
    printf( "Not Range " );
    rlist_print( &( ( t->number ).not_range ) );
}

void print_utf8_string_type( union type *t )
{
    llist_print( &( ( t->utf8_str ).min ) );
    printf( "Max " );
    llist_print( &( ( t->utf8_str ).max ) );
    printf( "Not Min " );
    llist_print( &( ( t->utf8_str ).not_min ) );
    printf( "Not Max " );
    llist_print( &( ( t->utf8_str ).not_max ) );
    printf( "Range " );
    rlist_print( &( ( t->utf8_str ).range ) );
    printf( "Not Range " );
    rlist_print( &( ( t->utf8_str ).not_range ) );
}

void print_string_type( union type *t )
{
    printf( "Min " );
    cllist_print( &( t->str ).min );
    printf( "Max " );
    cllist_print( &( t->str ).max );
    printf( "Not Min " );
    cllist_print( &( t->str ).not_min );
    printf( "Not Max " );
    cllist_print( &( t->str ).not_max );
    printf( "Range " );
    crlist_print( &( t->str ).range );
    printf( "Not Range " );
    crlist_print( &( t->str ).not_range );
    printf( "Min Length %d\n", ( t->str ).minlength );
    printf( "Max Length %d\n", ( t->str ).maxlength );
    printf( "ASCII-only=%s\n", ( ( t->str ).is_ascii ? "true" : "false" ) );
}

void print_password_type( union type *t )
{
    print_string_type( t );
}

void print_cert_type( union type *t )
{
    printf( "Min Length %d\n", ( t->cert ).minlength );
    printf( "Max Length %d\n", ( t->cert ).maxlength );
    printf( "ASCII-only=%s\n", ( ( t->cert ).is_ascii ? "true" : "false" ) );
    printf( "UN-PKCS8-only=%s\n", ( ( t->cert ).is_unpkcs8 ? "true" : "false" ) );
}


void print_address_type( union type *t )
{
    if ( ( t->address ).allowed_address )
    {
        printf( "Allowed address " );
        str_list_print( &( t->address ).allowed_address );
    }

    if ( ( t->address ).not_allowed_address )
    {
        printf( "Not Allowed address " );
        str_list_print( &( t->address ).not_allowed_address );
    }
}

void print_ipaddress_type( union type *t )
{
    if ( ( t->ipaddress ).allowed_address )
    {
        printf( "Allowed address " );
        str_list_print( &( t->ipaddress ).allowed_address );
    }

    if ( ( t->ipaddress ).not_allowed_address )
    {
        printf( "Not Allowed address " );
        str_list_print( &( t->ipaddress ).not_allowed_address );
    }
}

void print_serveraddress_type( union type *t )
{
    if ( ( t->serveraddress ).allowed_address )
    {
        printf( "Allowed address " );
        str_list_print( &( t->serveraddress ).allowed_address );
    }

    if ( ( t->serveraddress ).not_allowed_address )
    {
        printf( "Not Allowed address " );
        str_list_print( &( t->serveraddress ).not_allowed_address );
    }
}

void print_netmask_type( union type *t )
{
    if ( ( t->netmask ).allowed_mask )
    {
        printf( "Allowed mask " );
        str_list_print( &( t->netmask ).allowed_mask );
    }

    if ( ( t->netmask ).not_allowed_mask )
    {
        printf( "Not Allowed mask " );
        str_list_print( &( t->netmask ).not_allowed_mask );
    }
}

void print_url_type( union type *t )
{
    if ( ( t->url ).allowed_url )
    {
        printf( "Allowed url " );
        str_list_print( &( t->url ).allowed_url );
    }

    if ( ( t->url ).not_allowed_url )
    {
        printf( "Not Allowed url " );
        str_list_print( &( t->url ).not_allowed_url );
    }
}

void print_tr069_type( union type *t )
{
    printf( "TR069\n" );
}

void print_blank_type( union type *t )
{
    printf( "Blank\n" );
}

void print_device_password_type( union type *t )
{
    printf( "Device Password\n" );
}
int sprint_number_type( union type *t, char* ret_str, unsigned int size )
{
    unsigned int len = 0, tmp_len = 0;
    char* tmp_str = ret_str;

    tmp_len = snprintf( tmp_str, size - len, " Base=%d ", ( t->number ).base );
    tmp_str += tmp_len;
    len += tmp_len;

    if ( len >= ( size - 1 ) )
    {
        return len;
    }

    if ( ( t->number ).min )
    {
        tmp_len = snprintf( tmp_str, size - len, " Min=" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = llist_sprint( &( ( t->number ).min ), tmp_str, size - len );
        len += tmp_len;
        tmp_str += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->number ).max )
    {
        tmp_len = snprintf( tmp_str, size - len, " Max=" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = llist_sprint( &( ( t->number ).max ), tmp_str, size - len );
        tmp_str += tmp_len;
        len += tmp_len;
        tmp_len = 0;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->number ).not_min )
    {
        tmp_len = snprintf( tmp_str, size - len, " Not Min=" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = llist_sprint( &( ( t->number ).not_min ), tmp_str, size - len );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->number ).not_max )
    {
        tmp_len = snprintf( tmp_str, size - len, " Not Max=" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = llist_sprint( &( ( t->number ).not_max ), tmp_str, size - len );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->number ).range )
    {
        tmp_len = snprintf( tmp_str, size - len, " Range=[" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = rlist_sprint( &( ( t->number ).range ), tmp_str, size - len );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = snprintf( tmp_str, size - len, "] " );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->number ).not_range )
    {
        tmp_len = snprintf( tmp_str, size - len, " Not Range=[" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = rlist_sprint( &( ( t->number ).not_range ), tmp_str, size - len );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = snprintf( tmp_str, size - len, "] " );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    return len;
}

int sprint_utf8_string_type( union type *t, char* ret_str, unsigned int size )
{
    unsigned int len = 0, tmp_len = 0;
    char *tmp_str = ret_str;

    if( ( t->utf8_str ).min )
    {
        tmp_len = snprintf( tmp_str, size - len, " Min=" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = llist_sprint( &( ( t->utf8_str ).min ), tmp_str, size - len );
        len += tmp_len;
        tmp_str += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
	}

    if( ( t->utf8_str ).max )
    {
        tmp_len = snprintf( tmp_str, size - len, " Max=" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = llist_sprint( &( ( t->utf8_str ).max ), tmp_str, size - len );
        tmp_str += tmp_len;
        len += tmp_len;
        tmp_len = 0;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->utf8_str ).not_min )
    {
        tmp_len = snprintf( tmp_str, size - len, " Not Min=" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = llist_sprint( &( ( t->utf8_str ).not_min ), tmp_str, size - len );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->utf8_str ).not_max )
    {
        tmp_len = snprintf( tmp_str, size - len, " Not Max=" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = llist_sprint( &( ( t->utf8_str ).not_max ), tmp_str, size - len );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->utf8_str ).range )
    {
        tmp_len = snprintf( tmp_str, size - len, " Range=[" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = rlist_sprint( &( ( t->utf8_str ).range ), tmp_str, size - len );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = snprintf( tmp_str, size - len, "] " );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->utf8_str ).not_range )
    {
        tmp_len = snprintf( tmp_str, size - len, " Not Range=[" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = rlist_sprint( &( ( t->utf8_str ).not_range ), tmp_str, size - len );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = snprintf( tmp_str, size - len, "] " );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    return len;
}

int sprint_cert_type( union type *t, char* ret_str, unsigned int size )
{
    unsigned int len = 0, tmp_len = 0;
    char* tmp_str = ret_str;

    if ( ( t->cert ).minlength > -1 )
    {
        tmp_len = snprintf( tmp_str, size - len, " Min Length=%d ", ( t->cert ).minlength );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->cert ).maxlength > -1 )
    {
        tmp_len = snprintf( tmp_str, size - len, " Max Length=%d ", ( t->cert ).maxlength );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->cert ).maxlength > -1 )
    {
        tmp_len = snprintf( tmp_str, size - len, "ASCII-only=%s ", ( ( t->cert ).is_ascii ? "true" : "false" ) );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
        tmp_len = snprintf( tmp_str, size - len, "UN-PKCS8-only=%s ", ( ( t->cert ).is_unpkcs8 ? "true" : "false" ) );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    return len;
}

int sprint_string_type( union type *t, char* ret_str, unsigned int size )
{
    unsigned int len = 0, tmp_len = 0;
    char* tmp_str = ret_str;

    if ( ( t->str ).min )
    {
        tmp_len = snprintf( tmp_str, size - len, " Min=" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = cllist_sprint( &( t->str ).min, tmp_str, size - len );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->str ).max )
    {
        tmp_len = snprintf( tmp_str, size - len, " Max=" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = cllist_sprint( &( t->str ).max, tmp_str, size - len );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->str ).not_min )
    {
        tmp_len = snprintf( tmp_str, size - len, " Not Min=" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = cllist_sprint( &( t->str ).not_min, tmp_str, size - len );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->str ).not_max )
    {
        tmp_len = snprintf( tmp_str, size - len, " Not Max=" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = cllist_sprint( &( t->str ).not_max, tmp_str, size - len );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->str ).range )
    {
        tmp_len = snprintf( tmp_str, size - len, " Range=[" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = crlist_sprint( &( t->str ).range, tmp_str, size - len );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = snprintf( tmp_str, size - len, "] " );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->str ).not_range )
    {
        tmp_len = snprintf( tmp_str, size - len, " Not Range=[" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = crlist_sprint( &( t->str ).not_range, tmp_str, size - len );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = snprintf( tmp_str, size - len, "] " );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->str ).minlength > -1 )
    {
        tmp_len = snprintf( tmp_str, size - len, " Min Length=%d ", ( t->str ).minlength );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->str ).maxlength > -1 )
    {
        tmp_len = snprintf( tmp_str, size - len, " Max Length=%d ", ( t->str ).maxlength );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->str ).maxlength > -1 )
    {
        len += snprintf( tmp_str, size - len, "ASCII-only=%s ", ( ( t->str ).is_ascii ? "true" : "false" ) );
    }

    return len;
}

int sprint_password_type( union type *t, char* ret_str, unsigned int size )
{
    return sprint_string_type( t, ret_str, size );
}

int sprint_address_type( union type *t, char* ret_str, unsigned int size )
{
    unsigned int len = 0, tmp_len = 0;
    char* tmp_str = ret_str;

    if ( ( t->address ).maxlength > -1 )
    {
        tmp_len = snprintf( tmp_str, size - len, " Max Length=%d ", ( t->address ).maxlength );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->address ).allowed_address )
    {
        tmp_len = snprintf( tmp_str, size - len, " Allowed address=" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = str_list_sprint( &( t->address ).allowed_address, tmp_str, size - len );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->address ).not_allowed_address )
    {
        tmp_len = snprintf( tmp_str, size - len, " Not Allowed address=" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        len += str_list_sprint( &( t->address ).not_allowed_address, tmp_str, size - len );
    }

    return len;
}

int sprint_ipaddress_type( union type *t, char* ret_str, unsigned int size )
{
    unsigned int len = 0, tmp_len = 0;
    char* tmp_str = ret_str;

    if ( ( t->ipaddress ).allowed_address )
    {
        tmp_len = snprintf( tmp_str, size - len, " Allowed address=" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = str_list_sprint( &( t->ipaddress ).allowed_address, tmp_str, size - len );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->ipaddress ).not_allowed_address )
    {
        tmp_len = snprintf( tmp_str, size - len, " Not Allowed address=" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        len += str_list_sprint( &( t->ipaddress ).not_allowed_address, tmp_str, size - len );
    }

    return len;
}

int sprint_serveraddress_type( union type *t, char* ret_str, unsigned int size )
{
    unsigned int len = 0, tmp_len = 0;
    char* tmp_str = ret_str;

    if ( ( t->serveraddress ).maxlength > -1 )
    {
        tmp_len = snprintf( tmp_str, size - len, " Max Length=%d ", ( t->serveraddress ).maxlength );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->serveraddress ).allowed_address )
    {
        tmp_len = snprintf( tmp_str, size - len, " Allowed address=" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = str_list_sprint( &( t->serveraddress ).allowed_address, tmp_str, size - len );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->serveraddress ).not_allowed_address )
    {
        tmp_len = snprintf( tmp_str, size - len, " Not Allowed address=" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        len += str_list_sprint( &( t->serveraddress ).not_allowed_address, tmp_str, size - len );
    }

    return len;
}

int sprint_netmask_type( union type *t, char* ret_str, unsigned int size )
{
    unsigned int len = 0, tmp_len = 0;
    char* tmp_str = ret_str;

    if ( ( t->netmask ).allowed_mask )
    {
        tmp_len = snprintf( tmp_str, size - len, " Allowed mask=" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = str_list_sprint( &( t->netmask ).allowed_mask, tmp_str, size - len );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->netmask ).not_allowed_mask )
    {
        tmp_len = snprintf( tmp_str, size - len, " Not Allowed mask=" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        len += str_list_sprint( &( t->netmask ).not_allowed_mask, tmp_str, size - len );
    }

    return len;
}

int sprint_url_type( union type *t, char* ret_str, unsigned int size )
{
    unsigned int len = 0, tmp_len = 0;
    char* tmp_str = ret_str;

    if ( ( t->url ).maxlength > -1 )
    {
        tmp_len = snprintf( tmp_str, size - len, " Max Length=%d ", ( t->url ).maxlength );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->url ).allowed_url )
    {
        tmp_len = snprintf( tmp_str, size - len, " Allowed url=" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        tmp_len = str_list_sprint( &( t->url ).allowed_url, tmp_str, size - len );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }
    }

    if ( ( t->url ).not_allowed_url )
    {
        tmp_len = snprintf( tmp_str, size - len, " Not Allowed url=" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return len;
        }

        len += str_list_sprint( &( t->url ).not_allowed_url, tmp_str, size - len );
    }

    return len;
}

int sprint_tr069_type( union type *t, char* ret_str, unsigned int size )
{
    return sprintf( ret_str, "TR069" );
}

int sprint_blank_type( union type *t, char* ret_str, unsigned int size )
{
    return sprintf( ret_str, "Blank" );
}

int sprint_device_password_type( union type *t, char* ret_str, unsigned int size )
{
    return sprintf( ret_str, "Device Password" );
}

void free_number_type( union type *t )
{
    if ( !t )
    {
        return;
    }

    llist_free( &( t->number ).min );
    llist_free( &( t->number ).max );
    llist_free( &( t->number ).not_min );
    llist_free( &( t->number ).not_max );
    rlist_free( &( t->number ).range );
    rlist_free( &( t->number ).not_range );
    llist_free( &( t->number ).mult );
    llist_free( &( t->number ).not_mult );
}

void free_utf8_string_type( union type *t )
{
    if ( !t )
    {
        return;
    }

    llist_free( &( t->utf8_str ).min );
    llist_free( &( t->utf8_str ).max );
    llist_free( &( t->utf8_str ).not_min );
    llist_free( &( t->utf8_str ).not_max );
    rlist_free( &( t->utf8_str ).range );
    rlist_free( &( t->utf8_str ).not_range );
}

void free_string_type( union type *t )
{
    if ( !t )
    {
        return;
    }

    cllist_free( &( t->str ).min );
    cllist_free( &( t->str ).max );
    cllist_free( &( t->str ).not_min );
    cllist_free( &( t->str ).not_max );
    crlist_free( &( t->str ).range );
    crlist_free( &( t->str ).not_range );
}

void free_password_type( union type *t )
{
    if ( !t )
    {
        return;
    }

    free_string_type( t );
}

void free_cert_type( union type *t )
{
    if ( !t )
    {
        return;
    }
}

void free_address_type( union type *t )
{
    if ( !t )
    {
        return;
    }

    str_list_free( &( t->address ).allowed_address );
    str_list_free( &( t->address ).not_allowed_address );
}

void free_ipaddress_type( union type *t )
{
    if ( !t )
    {
        return;
    }

    str_list_free( &( t->ipaddress ).allowed_address );
    str_list_free( &( t->ipaddress ).not_allowed_address );
}

void free_serveraddress_type( union type *t )
{
    if ( !t )
    {
        return;
    }

    str_list_free( &( t->serveraddress ).allowed_address );
    str_list_free( &( t->serveraddress ).not_allowed_address );
}

void free_netmask_type( union type *t )
{
    if ( !t )
    {
        return;
    }

    str_list_free( &( t->netmask ).allowed_mask );
    str_list_free( &( t->netmask ).not_allowed_mask );
}

void free_url_type( union type *t )
{
    if ( !t )
    {
        return;
    }

    str_list_free( &( t->url ).allowed_url );
    str_list_free( &( t->url ).not_allowed_url );
}

void free_tr069_type( union type *t )
{
}

void free_blank_type( union type *t )
{
}

void free_device_password_type( union type *t )
{
}
int validate_number_type( union type *t, char *value )
{
    int i = 0 ;
    char *endptr = NULL ;
    long long int val, tmp_val ;

    /*ensure pointers are not NULL*/
    if ( ( NULL == t ) || ( NULL == value ) )
    {
        return 1;
    }

    /*check for case where value is empty string*/
    if ( '\0' == value[0] )
    {
        return 1;
    }

    errno = 0;
    val = strtoll( value, &endptr, ( t->number ).base );

    /*check for errors*/
    if ( ( errno == ERANGE && ( val == Lmax || val == Lmin ) ) ||
            ( errno != 0 && val == 0 ) )
    {
        return 1;
    }

    /*check if there are trailing characters not successfully parsed by strtoll*/
    if ( '\0' != *endptr )
    {
        return 1;
    }

    /*bug in uclibc - strtol accepts '`'( backtick ) as legal substituting it with 9*/
    while ( value[i] )
    {
        if ( value[i] ==  '`' ) //0x60
        {
            return 1;
        }

        i++;
    }

    if ( *endptr || !value[0] )
    {
        return 1;
    }
    tmp_val = val;
    endptr = value;

    if ( tmp_val == 0 )
    {
        i = 1;
    }
    else
    {
        if ( value[0] == '-' /* || value[0] == '+' */ ) /* bug 120590, dont allow '+' in number category */
        {
            endptr = &value[1];
        }
        tmp_val = strtoll( endptr, NULL, ( t->number ).base );

        i = 0 ;
        while ( tmp_val > 0 ) /*check number of digits*/
        {
            tmp_val /= ( t->number ).base;
            i++;
        }
    }
    if ( (unsigned int)i != strlen( endptr ) ) //Compare number of digits
    {
        return 1;
    }
    if( ( ( t->number ).min ) && ( val < ( ( t->number ).min )->val ) )
    {
        return 1;
    }
    if( ( ( t->number ).max ) && ( val > ( ( t->number ).max )->val ) )
    {
        return 1;
    }
    if( ( ( t->number ).not_min ) && ( val <= ( ( t->number ).not_min )->val ) )
    {
        return 1;
    }
    if( ( ( t->number ).not_max ) && ( val >= ( ( t->number ).not_max )->val ) )
    {
        return 1;
    }
    if ( rlist_validate( &( ( t->number ).range ), val ) < 0 )
    {
        return 1;
    }

    if ( !rlist_validate( &( ( t->number ).not_range ), val ) )
    {
        return 1;
    }

    if ( ( ( t->number ).mult ) && llist_multiple_check( &( ( t->number ).mult ), val ) )
    {
        /*0 is a special case*/
        if ( val == 0 )
        {
            return 0;
        }

        return 1;
    }

    if ( ( ( t->number ).not_mult ) && !llist_multiple_check( &( ( t->number ).not_mult ), val ) )
    {
        return 1;
    }

    return 0;
}

int validate_utf8_string_type( union type *t, char *value )
{
    unsigned char *str_ptr = NULL, *ret_ptr = NULL;

    if ( !value[0] )
    {
        return 1;
    }
    str_ptr = (unsigned char *)value;
    while ( str_ptr )
    {
        int val = get_code_point_utf8( str_ptr, &ret_ptr, 0 );
        if ( val < 0 )
        {
            return 1;
        }
        str_ptr = ret_ptr ;
        if ( ( ( t->utf8_str ).min ) && ( val < ( ( t->utf8_str ).min )->val ) )
        {
            return 1;
        }
        if ( ( ( t->utf8_str ).max ) && ( val > ( ( t->utf8_str ).max )->val ) )
        {
            return 1;
        }
        if ( ( ( t->utf8_str ).not_min ) && ( val <= ( ( t->utf8_str ).not_min )->val ) )
        {
            return 1;
        }
        if ( ( ( t->utf8_str ).not_max ) && ( val >= ( ( t->utf8_str ).not_max )->val ) )
        {
            return 1;
        }
        if ( rlist_validate( &( ( t->utf8_str ).range ), val ) < 0 )	
        {
            return 1;
        }
        if ( !rlist_validate( &( ( t->utf8_str ).not_range ), val ) )
        {
            return 1;
        }
    }
    return 0;
}

int utf8_cert_validate_range( union type *t, char *input_str )
{
    int i = 0;
    unsigned char val;

    if ( !input_str[0] )
    {
        return 0;
    }

    /*ensure string is valid UTF-8*/
    if ( 0 != validate_utf8( ( unsigned char * )input_str, 0 ) )
    {
        printf( "not valid utf8\n" );
        return 1;
    }

    while ( input_str[i] )
    {
        /*byte value is 0xxx xxxx - ascii*/
        if ( !( input_str[i] & 0x80 ) )
        {
            val = input_str[i];

            /*disallow control characters*/
            if ( ( val < 0x20 ) && ( val != '\n' ) )
            {
                printf( "got control character 0x%x\n", val );
                return 1;
            }
        }
        else if ( ( t->cert ).is_ascii )
        {
            printf( "got non-ascii character for ascii-only value\n" );
            return 1; //fail here if is_ascii flag set (non-ascii character found)
        }

        i++ ; //note ascii characters cannot occur within UTF-8 characters so this is safe
    }

    return 0;
}

int utf8_str_validate_range( union type *t, char *input_str )
{
    int i = 0;
    unsigned char val;

    if ( !input_str[0] )
    {
        return 0;
    }

    /*ensure string is valid UTF-8*/
    if ( 0 != validate_utf8( ( unsigned char * )input_str, 0 ) )
    {
        //printf( "not valid utf8\n" );
        return 1;
    }

    while ( input_str[i] )
    {
        /*byte value is 0xxx xxxx - ascii*/
        if ( !( input_str[i] & 0x80 ) )
        {
            val = input_str[i];

            /*disallow control characters*/
            if ( val < 0x20 )
            {
                return 1;
            }

            if ( crlist_validate( &( ( t->str ).range ), val ) < 0 )
            {
                return 1;
            }

            if ( !crlist_validate( &( ( t->str ).not_range ), val ) )
            {
                return 1;
            }

        }
        else if ( ( t->str ).is_ascii )
        {
            //printf( "got non-ascii character for ascii-only value\n" );
            return 1; //fail here if is_ascii flag set (non-ascii character found)
        }

        i++ ; //note ascii characters cannot occur within UTF-8 characters so this is safe
    }

    return 0;
}

int validate_cert_type( union type *t, char *value )
{
    //int i = 0;

    if ( ( t->cert ).minlength > -1 && strlen( value ) < (unsigned int)( t->cert ).minlength )
    {
        //printf( "error stringlength %u too short\n", strlen( value ) );
        return 1;
    }

    if ( ( t->cert ).maxlength > -1 && strlen( value ) > (unsigned int)( t->cert ).maxlength )
    {
        //printf( "error stringlength %u too long\n", strlen( value ) );
        return 1;
    }

    if ( utf8_cert_validate_range( t, value ) )
    {
        //printf( "range validation fail\n" );
        return 1;
    }

    if ( validate_certificate( value, ( t->cert ).is_unpkcs8 ) )
    {
        return 1;
    }

    return 0;
}

int validate_string_type( union type *t, char *value )
{
    int i = 0;

    if ( ( t->str ).minlength > -1 && strlen( value ) < (unsigned int)( t->str ).minlength )
    {
        return 1;
    }

    if ( ( t->str ).maxlength > -1 && strlen( value ) > (unsigned int)( t->str ).maxlength )
    {
        return 1;
    }

    if ( utf8_str_validate_range( t, value ) )
    {
        return 1;
    }

    while ( i < strlen( value ) )
    {
        if ( ( ( t->str ).min ) && ( value[i] < ( ( t->str ).min )->val ) )
        {
            return 1;
        }

        if ( ( ( t->str ).max ) && ( value[i] > ( ( t->str ).max )->val ) )
        {
            return 1;
        }

        if ( ( ( t->str ).not_min ) && ( value[i] <= ( ( t->str ).not_min )->val ) )
        {
            return 1;
        }

        if ( ( ( t->str ).not_max ) && ( value[i] >= ( ( t->str ).not_max )->val ) )
        {
            return 1;
        }

        if ( crlist_validate( &( ( t->str ).range ), value[i] ) < 0 )
        {
            return 1;
        }

        if ( !crlist_validate( &( ( t->str ).not_range ), value[i] ) )
        {
            return 1;
        }

        i++;
    }

    return 0;
}

int validate_password_type( union type *t, char *value )
{
    return validate_string_type( t, value );
}

int validate_address_type( union type *t, char *value )
{
    if ( ( ( t->address ).maxlength > -1 ) && strlen( value ) > (unsigned int)( t->address ).maxlength )
    {
        return 1;
    }

    int ret = str_list_find( &( ( t->address ).allowed_address ), value );
    if ( 0 == ret )
    {
        return 0;
    }
    else if ( -1 == ret )
    {
        return 1;
    }
    // if 1 == ret that means the list is empty

    if ( !str_list_find( &( ( t->address ).not_allowed_address ), value ) )
    {
        return 1;
    }

    if ( check_address( value ) )
    {
        return 1;
    }

    return 0;
}

int validate_ipaddress_type( union type *t, char *value )
{
    int ret = str_list_find( &( ( t->ipaddress ).allowed_address ), value );
    if ( 0 == ret )
    {
        return 0;
    }
    else if ( -1 == ret )
    {
        return 1;
    }
    // if 1 == ret that means the list is empty

    if ( !str_list_find( &( ( t->ipaddress ).not_allowed_address ), value ) )
    {
        return 1;
    }

    if ( check_ip_address( value ) < 0 )
    {
        return 1;
    }

    return 0;
}

int validate_serveraddress_type( union type *t, char *value )
{
    if ( ( ( t->serveraddress ).maxlength > -1 ) && strlen( value ) > (unsigned int)( t->serveraddress ).maxlength )
    {
        return 1;
    }

    int ret = str_list_find( &( ( t->serveraddress ).allowed_address ), value );
    if ( 0 == ret )
    {
        return 0;
    }
    else if ( -1 == ret )
    {
        return 1;
    }
    // if 1 == ret that means the list is empty

    if ( !str_list_find( &( ( t->serveraddress ).not_allowed_address ), value ) )
    {
        return 1;
    }

    if ( check_server_address( value ) < 0 )
    {
        return 1;
    }

    return 0;
}

int validate_netmask_type( union type *t, char *value )
{
    int ret = str_list_find( &( ( t->netmask ).allowed_mask ), value );
    if ( 0 == ret )
    {
        return 0;
    }
    else if ( -1 == ret )
    {
        return 1;
    }
    // if 1 == ret that means the list is empty

    if ( !str_list_find( &( ( t->netmask ).not_allowed_mask ), value ) )
    {
        return 1;
    }

    if ( check_netmask( value ) < 0 )
    {
        return 1;
    }

    return 0;
}

int validate_url_type( union type *t, char *value )
{
    if ( ( ( t->url ).maxlength > -1 ) && strlen( value ) > (unsigned int)( t->url ).maxlength )
    {
        return 1;
    }

    int ret = str_list_find( &( ( t->url ).allowed_url ), value ); 
    if ( 0 == ret )
    {
        return 0;
    }
    else if ( -1 == ret )
    {
        return 1;
    }
    // if 1 == ret that means the list is empty

    if ( !str_list_find( &( ( t->url ).not_allowed_url ), value ) )
    {
        return 1;
    }

    if ( check_url( value ) < 0 )
    {
        return 1;
    }

    return 0;
}

int validate_tr069_type( union type *t, char *value )
{
    return 0;
}

int validate_blank_type( union type *t, char *value )
{
    return 0;
}

int validate_device_password_type( union type *t, char *value )
{
    int i = 0;
    unsigned int large_count = 0;
    unsigned int small_count = 0;
    unsigned int num_count = 0;
    unsigned int special_count = 0;
    unsigned int required = 0;

    if ( ( t->device_passwd ).minlength > -1 && strlen( value ) < (unsigned int)( t->device_passwd ).minlength )
    {
        return 1;
    }

    if ( ( t->device_passwd ).maxlength > -1 && strlen( value ) > (unsigned int)( t->device_passwd ).maxlength )
    {
        return 1;
    }

    while ( i < strlen( value ) )
    {
        if ( 0x21 > value[i] || 0x7E < value[i] )  /*ascii only*/
        {
            return 1;
        }
        if ( 0x20 < value[i] && 0x30 > value[i] ) /*special*/
        {
            if ( 1 == ( t->device_passwd ).special )
            {
                special_count++;
            }
        }
        else if ( 0x2F < value[i] && 0x3A > value[i] )  /*0-9*/
        {
            if ( 1 == ( t->device_passwd ).num )
            {
                num_count++;
            }
        }
        else if ( 0x39 < value[i] && 0x41 > value[i] ) /*special*/
        {
            if ( 1 == ( t->device_passwd ).special )
            {
                special_count++;
            }
        }
        else if ( 0x40 < value[i] && 0x5B > value[i] )    /*A-Z*/
        {
            if ( 1 == ( t->device_passwd ).large )
            {
                large_count++;
            }
        }
        else if ( 0x5A < value[i] && 0x61 > value[i] ) /*special*/
        {
            if ( 1 == ( t->device_passwd ).special )
            {
                special_count++;
            }
        }
        else if ( 0x60 < value[i] && 0x7B > value[i] )  /*a-z*/
        {
            if ( 1 == ( t->device_passwd ).small )
            {
                small_count++;
            }
        }
        else if ( 0x7A < value[i] && 0x7F > value[i] ) /*special*/
        {
            if ( 1 == ( t->device_passwd ).special )
            {
                special_count++;
            }
        }
        i++;
    }
    if ( 0 < large_count )
    {
        required++;
    }
    if ( 0 < small_count )
    {
        required++;
    }
    if ( 0 < num_count )
    {
        required++;
    }
    if ( 0 < special_count )
    {
        required++;
    }
    if ( ( t->device_passwd ).required > required )
    {
        return 1;
    }
    return 0;
}

pvalue_xml* rotate( pvalue_xml* node, int next )
{
    pvalue_xml *ret_ptr = node->next[!next];
    node->next[!next] = ret_ptr->next[next];
    ret_ptr->next[next] = node;
    node->balance = 0;
    ret_ptr->balance = 0;
    return ret_ptr;
}

pvalue_xml* double_rotate( pvalue_xml* node, int next )
{
    pvalue_xml *ret_ptr = node->next[!next]->next[next];
    node->next[!next]->next[next] = ret_ptr->next[!next];
    ret_ptr->next[!next] = node->next[!next];
    node->next[!next] = ret_ptr->next[next];
    ret_ptr->next[next] = node;

    if ( !ret_ptr->balance )
    {
        node->balance = 0;
        ret_ptr->next[!next]->balance = 0;
    }
    else if ( ret_ptr->balance > 0 )
    {
        ( node->balance > 0 ) ? ( node->balance = -1, ret_ptr->next[!next]->balance = 0 ) : ( node->balance = 0, ret_ptr->next[!next]->balance = -1 );
    }
    else
    {
        ( node->balance > 0 ) ? ( node->balance = 0, ret_ptr->next[!next]->balance = 1 ) : ( node->balance = 1, ret_ptr->next[!next]->balance = 0 );
    }

    ret_ptr->balance = 0;
    return ret_ptr;
}

/*
 * free node
 */
void free_pvalue_xml( pvalue_xml *node )
{
    if ( !node )
    {
        return;
    }

    if ( node->name )
    {
        FREE( node->name );
    }

    if ( node->defval )
    {
        FREE( node->defval );
    }

    if ( node->alias )
    {
        FREE( node->alias );
    }

    if ( node->help )
    {
        FREE( node->help );
    }

    llist_free( &( node->scripts ) );
    free_type[node->cat]( &( node->t ) );
    FREE( node );
}

int pvalue_xml_tree_add( pvalue_xml **head, pvalue_xml *pvalue ) 
{
    int next, next_stack[MAX_TREE_HEIGHT], top = 0;
    pvalue_xml *temp = *head, *temp1, *node_stack[MAX_TREE_HEIGHT];
    if ( !head )
    {
        return -1;
    }

    temp1 = (pvalue_xml *)MALLOC( sizeof( pvalue_xml ) );

    if ( temp1 == NULL )
    {
        fprintf( stderr, "Not enough memory\n" );
        return -1;
    }

    temp1->hash = pvalue->hash;
    temp1->name = pvalue->name;
    temp1->defval = pvalue->defval;
    temp1->alias = pvalue->alias;
    temp1->help = pvalue->help;
    temp1->mandatory = pvalue->mandatory;
    temp1->cat = pvalue->cat;
    temp1->t = pvalue->t;
    temp1->balance = 0;
    temp1->notification_groups = pvalue->notification_groups;
    temp1->privilege_flag = pvalue->privilege_flag;
    temp1->reboot_flag = pvalue->reboot_flag;
    temp1->scripts = pvalue->scripts;
    temp1->lock_flag = pvalue->lock_flag;
    temp1->next[0] = NULL;
    temp1->next[1] = NULL;

    if ( !temp )
    {
        *head = temp1;
        return 0;
    }

    if ( temp->hash == temp1->hash )
    {
        free_pvalue_xml( temp1 );
        return 1;
    }

    next = ( temp->hash < temp1->hash );

    while ( temp->next[next] != NULL )
    {
        next_stack[top] = next;

        if ( top >= MAX_TREE_HEIGHT )
        {
            fprintf( stderr, "Not enough memory for tree\n" );
            free_pvalue_xml( temp1 );
            return -1;
        }

        node_stack[top++] = temp;
        temp = temp->next[next];

        if ( temp->hash == temp1->hash )
        {
            free_pvalue_xml( temp1 );
            return 1;
        }

        next = ( temp->hash < temp1->hash );
    }

    temp->next[next] = temp1;
    temp->balance += next ? 1 : -1;

    if ( !temp->balance )
    {
        return 0;
    }

    while ( top > 0 )
    {
        node_stack[top - 1]->balance += next_stack[top - 1] ? 1 : -1;

        if ( !node_stack[top - 1]->balance )
        {
            return 0;
        }
        else if ( ( node_stack[top - 1]->balance <= 1 ) && ( node_stack[top - 1]->balance >= -1 ) )
        {
            temp = node_stack[top - 1];
            top--;
        }
        else
        {
            if ( ( ( node_stack[top - 1]->balance <  0 ) && ( temp->balance <  0 ) ) ||
                    ( ( node_stack[top - 1]->balance >  0 ) && ( temp->balance >  0 ) )
               )
            {
                if ( !( top - 1 ) )
                {
                    node_stack[top - 1] = rotate( node_stack[top - 1], temp->balance < 0 ? 1 : 0 );
                }
                else
                {
                    node_stack[top - 2]->next[next_stack[top - 2]] = rotate( node_stack[top - 1], temp->balance < 0 ? 1 : 0 );
                }
            }
            else
            {
                if ( !( top - 1 ) )
                {
                    node_stack[top - 1] = double_rotate( node_stack[top - 1], temp->balance < 0 ? 0 : 1 );
                }
                else
                {
                    node_stack[top - 2]->next[next_stack[top - 2]] = double_rotate( node_stack[top - 1], temp->balance < 0 ? 0 : 1 );
                }
            }

            if ( !( top - 1 ) )
            {
                *head = node_stack[top - 1];
            }

            return 0;
        }
    }

    return 0;
}

/*
* print node
*/
char* print_node( pvalue_xml *node, char* ret_str, unsigned int size )
{
    unsigned int len = 0, tmp_len = 0;
    char *tmp_str = ret_str;

    if ( !node )
    {
        return NULL;
    }

    if ( node->alias )
    {
        tmp_len = snprintf( tmp_str, size - len, "Alias=%s ", node->alias );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return ret_str;
        }
    }

    if ( node->help )
    {
        tmp_len = snprintf( tmp_str, size - len, "Help=%s ", node->help );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return ret_str;
        }
    }

    if ( node->defval )
    {
        tmp_len = snprintf( tmp_str, size - len, "Default=%s ", node->defval );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return ret_str;
        }
    }

    tmp_len = cat_to_str( node->cat, tmp_str );
    tmp_str += tmp_len;
    len += tmp_len;

    if ( len >= ( size - 1 ) )
    {
        return ret_str;
    }

    tmp_len = snprintf( tmp_str, size - len, "M=%d ", node->mandatory );
    tmp_str += tmp_len;
    len += tmp_len;

    if ( len >= ( size - 1 ) )
    {
        return ret_str;
    }

    tmp_len = snprintf( tmp_str, size - len, "R=%d ", node->reboot_flag );
    tmp_str += tmp_len;
    len += tmp_len;

    if ( len >= ( size - 1 ) )
    {
        return ret_str;
    }

    tmp_len = snprintf( tmp_str, size - len, "P=%d ", node->privilege_flag );
    tmp_str += tmp_len;
    len += tmp_len;

    if ( len >= ( size - 1 ) )
    {
        return ret_str;
    }

    tmp_len = snprintf( tmp_str, size - len, "L=%d ", node->lock_flag );
    tmp_str += tmp_len;
    len += tmp_len;

    if ( len >= ( size - 1 ) )
    {
        return ret_str;
    }

    if ( node->scripts )
    {
        tmp_len = snprintf( tmp_str, size - len, "Ss=" );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return ret_str;
        }

        tmp_len = llist_sprint( &( node->scripts ), tmp_str, size - len );
        tmp_str += tmp_len;
        len += tmp_len;

        if ( len >= ( size - 1 ) )
        {
            return ret_str;
        }
    }

    sprint_type[node->cat]( &( node->t ), tmp_str, size - len );

    return ret_str;
}

/*
* print tree
*/
void pvalue_xml_tree_print( pvalue_xml **head )
{
    pvalue_xml *temp = *head;

    if ( !head || !( *head ) )
    {
        return;
    }

    printf( "Hash %llu balance %d ", temp->hash, temp->balance );

    if ( temp->next[0] )
    {
        printf( "next0 %llu ", temp->next[0]->hash );
    }

    if ( temp->next[1] )
    {
        printf( "next1 %llu ", temp->next[1]->hash );
    }

    printf( "\n" );
    printf( "Name = %s Hash = %llu Default value = %s Alias value = %s",
            temp->name, temp->hash, temp->defval, temp->alias);
    printf( " Mandatory = %d Category = %s Help = %s\n", temp->mandatory,
                get_str_datatype(temp->cat), temp->help );
    print_type[temp->cat]( &( temp->t ) );

    pvalue_xml_tree_print( &( temp->next[0] ) );
    pvalue_xml_tree_print( &( temp->next[1] ) );
}

/*
* free tree
*/
void pvalue_xml_tree_free( pvalue_xml **head )
{
    pvalue_xml *temp = *head, *temp1;

    if ( !head || !( *head ) )
    {
        return;
    }

    while ( temp )
    {
        if( temp->next[0] )
        {
            temp1 = temp->next[0];
            temp->next[0] = temp1->next[1];
            temp1->next[1] = temp;
        }
        else
        {
            temp1 = temp->next[1];
            free_pvalue_xml( temp );
        }
        temp = temp1 ;
    } 
}

/*
* Find pvalue by hash and return pointer
*/
pvalue_xml * pvalue_xml_tree_find( pvalue_xml **head, unsigned long long int hash )
{
    unsigned long long int next;
#ifdef DESKTOP_DEBUG
    int lookup_count = 0;
#endif

    pvalue_xml *temp = *head;

    if ( !head || !( *head ) )
    {
#ifdef DESKTOP_DEBUG
        printf( "No of lookups is %d\n", lookup_count );
#endif
        return 0;
    }

    while ( temp )
    {
        next = temp->hash < hash;

        if ( temp->hash != hash )
        {
            temp = temp->next[next];
#ifdef DESKTOP_DEBUG
            lookup_count++;
#endif
            continue;
        }

#ifdef DESKTOP_DEBUG
        printf( "No of lookups is %d\n", lookup_count );
#endif
        return temp;
    }

#ifdef DESKTOP_DEBUG
    printf( "No of lookups is %d\n", lookup_count );
#endif
    return NULL;
}

/*
* validate value for pvalue
*/
int pvalue_validate( pvalue_xml * pvalue, char * value )
{
    if ( !value || !strlen( value ) )
    {
        return pvalue->mandatory;
    }

    return validate_type[pvalue->cat]( &( pvalue->t ), value );
}

/*
* validate value for hash
*/
int pvalue_xml_tree_validate( pvalue_xml **head, unsigned long long int hash, char *value )
{
    pvalue_xml *temp;

    if ( NULL != ( temp = ( ( pvalue_xml * )pvalue_xml_tree_find( head, hash ) ) ) )
    {
        if ( !value || !strlen( value ) )
        {
            return temp->mandatory;
        }

        return validate_type[temp->cat]( &( temp->t ), value );
    }

    return -1;
}

/*
* find default value of hash
*/
char* pvalue_xml_tree_default( pvalue_xml **head, unsigned long long int hash )
{
    pvalue_xml *temp;

    if ( NULL != ( temp = ( ( pvalue_xml * )pvalue_xml_tree_find( head, hash ) ) ) )
    {
        return temp->defval;
    }

    return NULL;
}

/*
* find pvalue number
*/
int pvalue_xml_tree_propername( pvalue_xml **head, unsigned long long int hash, char * buf, int length )
{
    pvalue_xml *temp;

    if ( NULL != ( temp = ( ( pvalue_xml * )pvalue_xml_tree_find( head, hash ) ) ) )
    {
        if ( temp->name && strlen( temp->name ) > 0 )
        {
            return snprintf( buf, length, "%s", temp->name );
        }
    }

    return 0;
}

/*
* find alias value of hash
*/
char* pvalue_xml_tree_alias( pvalue_xml **head, unsigned long long int hash )
{
    pvalue_xml *temp;
    if ( NULL != ( temp = ( ( pvalue_xml * )pvalue_xml_tree_find( head, hash ) ) ) )
    {
        return temp->alias;
    }

    return 0;
}

/*
* find group of hash
*/
unsigned int pvalue_xml_tree_groups( pvalue_xml **head, unsigned long long int hash )
{
    pvalue_xml *temp;

    if ( NULL != ( temp = ( ( pvalue_xml * )pvalue_xml_tree_find( head, hash ) ) ) )
    {
        return temp->notification_groups;
    }

    return 0;
}

/*
* get pvalue help
*/
char* pvalue_xml_tree_help( pvalue_xml **head, unsigned long long int hash )
{
    pvalue_xml *temp;

    if ( NULL != ( temp = ( ( pvalue_xml * )pvalue_xml_tree_find( head, hash ) ) ) )
    {
        return temp->help;
    }

    return 0;
}

/*
* get base of number pvalue
*/
int pvalue_xml_tree_get_base( pvalue_xml **head, unsigned long long int hash )
{
    pvalue_xml *temp;

    if ( NULL != ( temp = ( ( pvalue_xml * )pvalue_xml_tree_find( head, hash ) ) ) )
    {
        if ( NUMBER == temp->cat )
        {
            return ( temp->t ).number.base;
        }
    }

    return 0;
}

/*
* find scripts value of hash
*/
char* pvalue_xml_tree_scripts( pvalue_xml **head, unsigned long long int hash, char* ret_str, size_t size )
{
    pvalue_xml *temp;

    if ( NULL != ( temp = ( ( pvalue_xml * )pvalue_xml_tree_find( head, hash ) ) ) )
    {
        if ( temp->scripts )
        {
            llist_sprint( &( temp->scripts ), ret_str, size );
            return ret_str;
        }
        else
        {
            return 0;
        }
    }

    return 0;
}

/*
* return description of hash
*/
char* pvalue_xml_tree_description( pvalue_xml **head, unsigned long long int hash, char* ret_str, unsigned int size )
{
    pvalue_xml *temp;

    if ( NULL != ( temp = ( ( pvalue_xml * )pvalue_xml_tree_find( head, hash ) ) ) )
    {
        return print_node( temp, ret_str, size );
    }

    return NULL;
}

/*
* set change flag value for hash
* if hash does not exist add to tree
*/
int pvalue_set_privilege_flag( pvalue_xml **head, unsigned long long int hash, int value )
{
    pvalue_xml *temp;
    pvalue_xml pvalue;

    if ( NULL != ( temp = ( ( pvalue_xml * )pvalue_xml_tree_find( head, hash ) ) ) )
    {
        temp->privilege_flag = value;
    }
    else
    {
        memset( &pvalue, 0, sizeof( pvalue_xml ) );
        pvalue.cat = BLANK;
        pvalue.hash = hash;
        pvalue.privilege_flag = value;

        if ( 0 > pvalue_xml_tree_add( head, &pvalue ) )
        {
            return -1;
        }
    }

    return 0;
}

/*
* check change flag value for hash
*/
int pvalue_check_privilege_flag( pvalue_xml **head, unsigned long long int hash )
{
    pvalue_xml *temp;

    if ( NULL != ( temp = ( ( pvalue_xml * )pvalue_xml_tree_find( head, hash ) ) ) )
    {
        return temp->privilege_flag;
    }

    return 0;
}

/*
* set change flag value for hash
* if hash does not exist add to tree
*/
int pvalue_set_notification_groups( pvalue_xml **head, unsigned long long int hash, unsigned int groups )
{
    pvalue_xml *temp;
    pvalue_xml pvalue;

    groups |= MSG_GROUP_ID_BROADCAST;  //broadcast bit must always be set for pvalues

    if ( NULL != ( temp = ( ( pvalue_xml * )pvalue_xml_tree_find( head, hash ) ) ) )
    {
        temp->notification_groups = groups;
    }
    else
    {
        memset( &pvalue, 0, sizeof( pvalue_xml ) );
        pvalue.cat = BLANK;
        pvalue.hash = hash;
        pvalue.notification_groups = groups;
        if ( 0 > pvalue_xml_tree_add( head, &pvalue ) )
        {
            return -1;
        }
    }

    return 0;
}

/*
* check reboot flag value for hash
*/
int pvalue_check_reboot_flag( pvalue_xml **head, unsigned long long int hash )
{
    pvalue_xml *temp;

    if ( NULL != ( temp = ( ( pvalue_xml * )pvalue_xml_tree_find( head, hash ) ) ) )
    {
        return temp->reboot_flag;
    }

    return 0;
}

/*
* check category flag value for hash
*/
int pvalue_check_category( pvalue_xml **head, unsigned long long int hash )
{
    pvalue_xml *temp;

    if ( NULL != ( temp = ( ( pvalue_xml * )pvalue_xml_tree_find( head, hash ) ) ) )
    {
        return temp->cat;
    }

    return -1;
}

/*
* check lock flag value for hash
*/
int pvalue_check_lock_flag( pvalue_xml **head, unsigned long long int hash )
{
    pvalue_xml *temp;

    if ( NULL != ( temp = ( ( pvalue_xml * )pvalue_xml_tree_find( head, hash ) ) ) )
    {
        return temp->lock_flag;
    }

    return 0;
}

/*
* compute a hash from string
*/
unsigned long long int hash( char *str )
{
    unsigned long long int hash = 0;
    while ( *str )
    {
        hash = 31 * hash + *str++;
    }
    return hash;
}

/*
* long long limits not available in limits.h
*/
void calculate_int_limits()
{
#ifdef LLONG_MAX
    Lmax = LLONG_MAX;
    Lmin = LLONG_MIN;
#elif defined LONG_LONG_MAX
    Lmax = LONG_LONG_MAX;
    Lmin = LONG_LONG_MIN;
#else
    int i;
    int size = sizeof( long long int ) * 8 - 1;

    Lmin = 1;

    for ( i = 0; i < size; i++ )
    {
        Lmin *= 2;
    }

    Lmax = Lmin - 1;
#endif
#ifdef DESKTOP_DEBUG
    printf( " %lld Long long max\n%lld Long long min\n", Lmax, Lmin );
#endif
}

/*
 * converts enum to string. enum definition in datatypes.h
 */
int cat_to_str( enum category c, char *str )
{
    const char *pCat = get_str_datatype(c);
    if ( pCat )
    {
        return sprintf( str, "Category=%s ", pCat );
    }

    return 0;
}
