/*
* Contains various structures and their related function definitions to populate
* those structures
*/
#ifndef XML_PVALUES_H
#define XML_PVALUES_H

#define MAX_TREE_HEIGHT 50


typedef struct _llist
{
    long long int val;
    struct _llist *next;
} llist;

typedef struct _rlist
{
    long long int min;
    long long int max;
    struct _rlist *next;
} rlist;

typedef struct _number
{
    int base;
    llist *min;
    llist *max;
    llist *not_min;
    llist *not_max;
    rlist *range;
    rlist *not_range;
    llist *mult;
    llist *not_mult;
} number_type;

typedef struct _utf8_string 
{
	llist *min;
	llist *max;
	llist *not_min;
	llist *not_max;
	rlist *range;
	rlist *not_range;
}utf8_string_type;

typedef struct _cllist
{
    unsigned char val;
    struct _cllist *next;
} cllist;

typedef struct _crlist
{
    unsigned char min;
    unsigned char max;
    struct _crlist *next;
} crlist;

typedef struct _string
{
	cllist *min;
	cllist *max;
	cllist *not_min;
	cllist *not_max;
    crlist *range;
    crlist *not_range;
    int minlength;
    int maxlength;
    unsigned char is_ascii;  //only allow ascii characters
} string_type;

typedef struct _cert
{
    int minlength;
    int maxlength;
    unsigned char is_ascii;  //only allow ascii characters
    unsigned char is_unpkcs8; /* only allow un-pkcs8 data or not */
} cert_type;

typedef struct _str_list
{
    char *str;
    struct _str_list *next;
} str_list;

typedef struct _address
{
    int maxlength;
    str_list *allowed_address;
    str_list *not_allowed_address;
} address_type;

typedef struct _ipaddress
{
    str_list *allowed_address;
    str_list *not_allowed_address;
} ipaddress_type;

typedef struct _serveraddress
{
    int maxlength;
    str_list *allowed_address;
    str_list *not_allowed_address;
} serveraddress_type;

typedef struct _netmask
{
    str_list *allowed_mask;
    str_list *not_allowed_mask;
} netmask_type;

typedef struct _url
{
    int maxlength;
    str_list *allowed_url;
    str_list *not_allowed_url;
} url_type;

typedef struct _device_password
{
    int small;
    int large;
    int num;
    int special;
    int required;
    int minlength;
    int maxlength;
    unsigned char is_ascii;  //only allow ascii characters
} device_password_type;
union type
{
    number_type number;
    utf8_string_type utf8_str;
    string_type str;
    /*password is type str*/
    address_type address;
    ipaddress_type ipaddress;
    serveraddress_type serveraddress;
    netmask_type netmask;
    url_type url;
    cert_type cert;
    device_password_type device_passwd;
    /*
     * no tr069
     * or blank
    */
};

typedef struct _pvalue_xml
{
    unsigned long long int hash;
    char *name;
    char *defval;
    char *alias;
    char *help;
    int mandatory;
    enum category cat;
    union type t;
    int balance;
    unsigned int notification_groups;
    int privilege_flag;
    int reboot_flag;
    int lock_flag;
    llist *scripts;
    struct _pvalue_xml *next[2];
} pvalue_xml;

int llist_add( llist **, long long int );
void llist_free( llist ** );
int llist_find( llist **, long long int );
int llist_multiple_check( llist **, long long int );
void llist_print( llist ** );
int llist_sprint( llist **, char *, unsigned int );

int cllist_add( cllist **, unsigned char );
void cllist_free( cllist ** );
int cllist_find( cllist **, unsigned char );
void cllist_print( cllist ** );
int cllist_sprint( cllist **, char *, unsigned int );

int rlist_add( rlist **, long long int, long long int );
void rlist_free( rlist ** );
int rlist_validate( rlist **, long long int );
void rlist_print( rlist ** );
int rlist_sprint( rlist **, char *, unsigned int );

int crlist_add( crlist **, unsigned char, unsigned char );
void crlist_free( crlist ** );
int crlist_validate( crlist **, unsigned char );
void crlist_print( crlist ** );
int crlist_sprint( crlist **, char *, unsigned int );

int utf8_str_validate_range( union type *t, char *input_str );

int str_list_add( str_list **, char * );
void str_list_free( str_list ** );
int str_list_find( str_list **, char * );
void str_list_print( str_list ** );
int str_list_sprint( str_list **, char *, unsigned int );

void print_number_type( union type * );
int sprint_number_type( union type *, char *, unsigned int );
void free_number_type( union type * );
int validate_number_type( union type *, char * );

void print_utf8_string_type( union type * );
int sprint_utf8_string_type( union type *, char *, unsigned int );
void free_utf8_string_type( union type * );
int validate_utf8_string_type( union type *, char * );

void print_cert_type( union type * );
int sprint_cert_type( union type *, char *, unsigned int );
void free_cert_type( union type * );
int validate_cert_type( union type *, char * );

void print_string_type( union type * );
int sprint_string_type( union type *, char *, unsigned int );
void free_string_type( union type * );
int validate_string_type( union type *, char * );

void print_password_type( union type * );
int sprint_password_type( union type *, char *, unsigned int );
void free_password_type( union type * );
int validate_password_type( union type *, char * );

void print_address_type( union type * );
int sprint_address_type( union type *, char *, unsigned int );
void free_address_type( union type * );
int validate_address_type( union type *, char * );

void print_ipaddress_type( union type * );
int sprint_ipaddress_type( union type *, char *, unsigned int );
void free_ipaddress_type( union type * );
int validate_ipaddress_type( union type *, char * );

void print_serveraddress_type( union type * );
int sprint_serveraddress_type( union type *, char *, unsigned int );
void free_serveraddress_type( union type * );
int validate_serveraddress_type( union type *, char * );

void print_netmask_type( union type * );
int sprint_netmask_type( union type *, char *, unsigned int );
void free_netmask_type( union type * );
int validate_netmask_type( union type *, char * );

void print_url_type( union type * );
int sprint_url_type( union type *, char *, unsigned int );
void free_url_type( union type * );
int validate_url_type( union type *, char * );

void print_tr069_type( union type * );
int sprint_tr069_type( union type *, char *, unsigned int );
void free_tr069_type( union type * );
int validate_tr069_type( union type *, char * );

void print_blank_type( union type * );
int sprint_blank_type( union type *, char *, unsigned int );
void free_blank_type( union type * );
int validate_blank_type( union type *, char * );

void print_device_password_type( union type * );
int sprint_device_password_type( union type *, char *, unsigned int );
void free_device_password_type( union type * );
int validate_device_password_type( union type *, char * );
/*
* Tree functions
*/
pvalue_xml* rotate( pvalue_xml*, int ) ;
pvalue_xml* double_rotate( pvalue_xml*, int ) ;
void free_pvalue_xml( pvalue_xml* );
int pvalue_xml_tree_add( pvalue_xml **, pvalue_xml * );
void pvalue_xml_tree_print( pvalue_xml ** );
void pvalue_xml_tree_free( pvalue_xml ** );
char* print_node( pvalue_xml *, char*, unsigned int );

pvalue_xml * pvalue_xml_tree_find( pvalue_xml **, unsigned long long int );
int pvalue_xml_tree_validate( pvalue_xml **, unsigned long long int, char * );
char* pvalue_xml_tree_default( pvalue_xml **, unsigned long long int );
char* pvalue_xml_tree_alias( pvalue_xml **, unsigned long long int );
char* pvalue_xml_tree_scripts( pvalue_xml **, unsigned long long int, char *, size_t );
char* pvalue_xml_tree_description( pvalue_xml **, unsigned long long int, char *, unsigned int );
int pvalue_set_notify_flag( pvalue_xml **, unsigned long long int, int );
int pvalue_check_notify_flag( pvalue_xml **, unsigned long long int );
int pvalue_set_privilege_flag( pvalue_xml **, unsigned long long int, int );
int pvalue_check_privilege_flag( pvalue_xml **, unsigned long long int );
int pvalue_check_reboot_flag( pvalue_xml **, unsigned long long int );
int pvalue_check_category( pvalue_xml **, unsigned long long int );
int pvalue_check_lock_flag( pvalue_xml **, unsigned long long int );
void pvalue_limits_print( pvalue_xml* );
int pvalue_validate( pvalue_xml *, char * );
char* pvalue_xml_tree_help( pvalue_xml **, unsigned long long int );
int pvalue_xml_tree_get_base( pvalue_xml **, unsigned long long int );
int pvalue_xml_tree_propername( pvalue_xml **, unsigned long long int, char *, int );
int pvalue_set_notification_groups( pvalue_xml **, unsigned long long int, unsigned int );
unsigned int pvalue_xml_tree_groups( pvalue_xml **, unsigned long long int );

/*
* hash function to convert pvalue name into a unique int
*/
unsigned long long int hash( char * );

/*
 * Calculates upper and lower limits for long long int.
 * if they are not available in limits.h
 */
void calculate_int_limits();

/*
 * converts enum to string. enum definition in datatypes.h
 */
int cat_to_str( enum category, char * );

#endif
