/*=======================*/
/*  Includes             */
/*=======================*/
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <ctype.h>
#include <unistd.h>
#include <fcntl.h>
#include <sys/ioctl.h>
#include <mtd/mtd-user.h>

#include "configs/gxw4200.h"
#include "environment.h"

#define FOR_TEST 0

static char env_mtd [32];
void process_console(void);
static int find_env_part(void);

void erase_partition(mtd_info_t *mtd_info, int fd) 
{
    erase_info_t ei;
    ei.length = mtd_info->erasesize;

    for(ei.start = 0; ei.start < mtd_info->size; ei.start += mtd_info->erasesize) 
    {
        ioctl(fd, MEMUNLOCK, &ei);
        ioctl(fd, MEMERASE, &ei);
    }
}

#if FOR_TEST
void erasemtd()
{
    int fd;
    mtd_info_t mtd_info;

    fd = open(env_mtd, O_RDWR);
    if (fd == -1){
        printf("Error: cannot open %s \n", env_mtd);
    }

    ioctl(fd, MEMGETINFO, &mtd_info);

    printf("begin to erase partition\n");
    erase_partition(&mtd_info, fd);
    close(fd);
}

void readmtd()
{
    int fd, read_bytes, i, rt;
    char env_data[CONFIG_ENV_SIZE];

    fd = open(env_mtd, O_RDWR);
    if (fd == -1)
    {
        printf("Error: cannot open %s \n", env_mtd);
    }
    
    read_bytes = read(fd,env_data,ENV_SIZE);
    for (i=0; i<8192; i++)
    {
        if(env_data[i]=='\0')
            printf("0");
        printf("%c", env_data[i]);
    }

    close(fd);
    
}

void writemtd()
{
    int fd, read_bytes, i, rt;
    //char env_data[ENV_SIZE];
    env_t * env_ptr;

    fd = open(env_mtd, O_RDWR);
    if (fd == -1)
    {
        printf("Error: cannot open %s \n", env_mtd);
    }
    
    env_ptr = (env_t*) malloc (sizeof (env_t)); 
    memset(env_ptr, 0, sizeof(env_t));
    sprintf(env_ptr->data, "%s%c%s%c%s%c", "console=yes", '\0', "bootcmd=nand read 0x81000000 0x2e4000 0x19c000; bootm 0x81000000", '\0', 
                "bootargs=root=/dev/mtdblock2 panic=1", '\0');
    
    env_ptr->crc = crc32(0, env_ptr->data, ENV_SIZE);
 
    rt = write(fd, env_ptr, sizeof(env_t));
    printf("write %d data to flash\n", rt);
    
    free(env_ptr);
    close(fd);

}
#endif

/* return mtd number of ubootenv partition */
static int find_env_part()
{
    char buf[256];
    char dev_num[16];
    char size[32];
    char erase_size[32];
    char name[32];
    int i;

    FILE* fp;
    
    fp = fopen( "/proc/mtd", "r" );
    if ( fp != NULL )
    {
        while ( fscanf(fp, "%s %s %s %s", dev_num, size, erase_size, name)!= EOF )
        {
            //printf("name is %s\n", name);
            if(strcmp(name, "\"ubootenv\"") == 0)
            {
                i = atoi(dev_num+3);
                //printf("find ubootenv partition, number is %d\n", i);
                sprintf(env_mtd, "%s%d", "/dev/mtd", i);
                return i;
            }
        }
    } 
    return -1;
}

int  main (int argc, char* argv[])
{
    if (find_env_part() < 0)
    {   
        printf("Cannot find uboot env mtd partition, exit\n");
        return -1;
    }
#if FOR_TEST
    if(strcmp (argv[1],"erase")==0 )
    {
        erasemtd();
    }
    else if (strcmp (argv[1], "write") == 0)
    {
        writemtd();
    }
    else if (strcmp (argv[1], "read")==0)
    {
        readmtd(); 
    }
    else if(strcmp (argv[1], "console")==0)
    {
        //process_console(argc, argv);
        process_console();
    }
    else if(strcmp (argv[1], "part")==0)
    {
        printf("part: %d\n", find_env_part());
    }
    else 
    {
        printf("donnot have this command\n");
    }
#else
    process_console();
#endif

    return 1;
}

void process_console()
{
    int fd, read_bytes, i, rt;
    char* env_data;
    char * pch, * env, * remain;
    char * nxt=NULL;
    char buf[256];
    int console_on = 1;
    mtd_info_t mtd_info;
    static env_t * env_ptr;

    printf("Enable kernel serial console (yes or no)? ");
    fflush( stdout ); 
    do
    {
        if(fgets( buf, sizeof( buf ) - 1, stdin ) != NULL)
        {
            if(strncmp(buf, "yes", 3)==0)
            {
                console_on = 1;    
                break;
            }
            else if(strncmp(buf, "no", 2)==0)
            { 
                console_on = 0;
                break;
            }
            else
				printf("Enable kernel serial console (yes or no)? ");
        } 
    }     
    while(1);
    
    env_ptr = (env_t *)malloc(sizeof(env_t));
    memset(env_ptr, 0, sizeof(env_t));

    fd = open(env_mtd,O_RDWR);
    if (fd == -1){
        printf("Error: cannot open %s \n", env_mtd);
    }
    
    read_bytes = read(fd,env_ptr,ENV_SIZE);
    close(fd);    

    env_data = env_ptr->data;

    /* nothing on partition */ 
    if (env_data[0] == (char)(~0) && env_data[1] == (char)(~0) )
    {
        printf("Env partition in Flash is empty!\n");
        sprintf(env_ptr->data, "%s%c%s%c%s%c", "console=yes", '\0', "bootcmd=nand read 0x81000000 0x2e4000 0x19c000; bootm 0x81000000", '\0', 
                "bootargs=root=/dev/mtdblock2 panic=1", '\0');

        //sprintf(env_data, "%s%s%c", "console=", console_on?"yes":"no", '\0');    
        goto writefl; /* write mtd partition directly */
    }

    /* find the key word */
    for (env=env_data; *env; env=nxt+1)
    {
        for (nxt=env; *nxt; ++nxt)
            ;
        if ( strncmp(env, "console", 7) == 0)
            break;
    }

    if(!env)
    {   
        //printf("Cannot find console in env, pointer is at: %d\n", env-env_data);
        printf("No console variable found in Flash memory; will create a new one.\n");
    }
    else 
    {
        //printf("find console para\n");
        env+=8;
        if( ( strlen( env ) == 3 ) &&
                ( env[0] == 'Y' || env[0] == 'y' ) &&
                ( env[1] == 'E' || env[1] == 'e' ) &&
                ( env[2] == 'S' || env[2] == 's' )
          )
        {
            if(console_on ==1 )
            {   
                printf("Kernel serial console was already enabled!\n");
                return ;  
            } 
        } 
        else if( ( strlen( env ) == 2 ) &&
                ( env[0] == 'N' || env[0] == 'n' ) &&
                ( env[1] == 'O' || env[1] == 'o' )
               )
        {   
            if(console_on==0)
            {   
                printf("Kernel serial console was already disabled!\n");
                return ;  
            }  
        }
       
        /* move the remaining to front */
        remain = env+ strlen(env) +1;
        //printf("remaining starts from %d\n", remain-env_data);
        memmove(env-8, remain, env_data+ENV_SIZE-1- remain); 
    }
     
     /*
     * Append new definition at the end
     */
    for (env=env_data; *env || *(env+1); ++env)
        ;
    if (env > env_data)
        ++env;
    //printf("end is %d\n", env - env_data);

    /* end is marked with double '\0' */
    if(console_on == 1)
    {
        printf("Enabling kernel serial console...\n");
        sprintf(env, "%s%c", "console=yes", '\0');
    } 
    else if(console_on == 0)
    {
        printf("Disabling kernel serial console...\n");
        sprintf(env, "%s%c", "console=no", '\0');
    }
    
    /* erase partition */
    //printf("begin to erase partition\n");
    fd = open(env_mtd,O_RDWR);
    if (fd == -1){
        printf("Error: cannot open Flash partition %s\n", env_mtd);
    }
    ioctl(fd, MEMGETINFO, &mtd_info); 
    erase_partition(&mtd_info, fd);
    close(fd);

writefl:
    /* write partition */    
    //printf("write partition\n");

    fd = open(env_mtd, O_RDWR);
    if (fd == -1){
        printf("Error: cannot open Flash partition %s\n", env_mtd);
    }

    env_ptr->crc = crc32(0, env_ptr->data, ENV_SIZE);
    rt = write(fd, env_ptr, sizeof(env_t));
	if (rt <= 0)
		printf("Error: Writing to Flash memory fails!\n");
	else
		printf("Kernel console setting saved to Flash memory.\nReboot device for the setting to be effective!\n");
    
    free (env_ptr);
    close(fd);
}
